/**
 * @file    ModuleHelper_iOS.h
 
 * @brief   iOS 통합 모듈 개발 편의를 위한 클래스들의 헤더
 
 * @author  Byeong Yeol Jeon
 * @ingroup iOS Helper
 * @date    2014/04/25
 */
#ifndef _MODULEHELPER_IOS_H_
#define _MODULEHELPER_IOS_H_
/**
 * @defgroup Helper
 */
#if __OBJC__
#import <Foundation/Foundation.h>
#import <UIKit/UIImage.h>
/**
 * @interface C2SSerializable
 
 * @brief 상속받은 클래스의 변수, 프로퍼티를 map으로 변경 가능하도록 만든다.
 * @ingroup iOS Helper
 * @author Byeong Yeol Jeon
 
 */
@interface C2SSerializable : NSObject
/**
 * @brief 클래스를 map으로 변환한다.
 
 * @return 클래스의 변수, 프로퍼티를 키로 하고 그 데이터를 값으로 하는 map을 제공
 
 * @author Byeong Yeol Jeon
 
 */
-(NSDictionary*)toDictionary;
@end
/**
 * @interface C2SSerializableSocial_Initailize_Push
 
 * @brief 푸쉬 모듈 초기화 편의 클래스
 * @ingroup iOS Helper Social
 * @author Byeong Yeol Jeon
 
 */
@interface C2SSerializableSocial_Initailize_Push : C2SSerializable
/** YES일 경우 staging 서버를 사용. NO일 경우 production 서버를 사용한다. */
@property (nonatomic, assign) BOOL useStaging;
@end
/**
 * @interface C2SSerializableSocial_Initailize_Promotion
 
 * @brief 프로모션 모듈 초기화 편의 클래스
 * @ingroup iOS Helper Social
 * @author Byeong Yeol Jeon
 
 */
@interface C2SSerializableSocial_Initailize_Promotion : C2SSerializable
/** YES일 경우 stagind 서버를 사용. NO일 경우 production 서버를 사용한다. */
@property (nonatomic, assign) BOOL useStaging;
/** YES일 경우 로그를 출력한다. */
@property (nonatomic, assign) BOOL showLog;
@end
/**
 * @interface C2SSerializableSocial_Initailize
 
 * @brief 소셜 모듈 초기화 편의 클래스
 * @ingroup iOS Helper Social
 * @author Byeong Yeol Jeon
 
 */
@interface C2SSerializableSocial_Initailize : C2SSerializable
/** YES일 경우 유저 동의 창을 출력한다.. */
@property (nonatomic, assign) BOOL checkAgreement;
/** YES일 경우 로그를 출력한다. */
@property (nonatomic, assign) BOOL showLog;
/** YES일 경우 stagind 서버를 사용. NO일 경우 production 서버를 사용한다. */
@property (nonatomic, assign) BOOL useStaging;
/** 앱 실행시 받은 iOS app launch option */
@property (nonatomic, strong) NSDictionary* launchOptions;
/** 푸쉬 모듈 초기화 인자 */
@property (nonatomic, strong) C2SSerializableSocial_Initailize_Push* Push;
/** 프로모션 모듈 초기화 인자 */
@property (nonatomic, strong) C2SSerializableSocial_Initailize_Promotion* Promotion;
/**
 * @brief C2SSerializableSocial_Initailize의 인스턴스를 구한다.
 
 * @return C2SSerializableSocial_Initailize의 인스턴스
 * @author Byeong Yeol Jeon
 
 */
+(C2SSerializableSocial_Initailize*)instance;
@end
/**
 * @interface C2SSerializableSocial_Show
 
 * @brief Social Show 명령 실행을 위한 편의 클래스
 * @ingroup iOS Helper Social
 * @author Byeong Yeol Jeon
 
 */
@interface C2SSerializableSocial_Show : C2SSerializable
/** show 종류 */
@property (nonatomic, strong) NSString* type;
/** type이 terms일 경우 약관 페이지 리다이렉션 url*/
@property (nonatomic, strong) NSString* redirectURL;
/**
 * @brief C2SSerializableSocial_Show의 인스턴스를 구한다.
 
 * @return C2SSerializableSocial_Show의 인스턴스
 
 * @author Byeong Yeol Jeon
 
 */
+(C2SSerializableSocial_Show*)instance;
@end
/**
 * @interface C2SSerializableSocial_Show_Terms
 
 * @brief Social Show: type이 terms일때의 편의 클래스
 * @ingroup iOS Helper Social
 * @author Byeong Yeol Jeon
 
 */
@interface C2SSerializableSocial_Show_Terms : C2SSerializable
/** type이 terms일 경우 약관 페이지 리다이렉션 url*/
@property (nonatomic, strong) NSString* redirectURL;
/**
 * @brief C2SSerializableSocial_Show_Terms의 인스턴스를 구한다.
 
 * @return C2SSerializableSocial_Show_Terms의 인스턴스
 
 * @author Byeong Yeol Jeon
 
 */
+(C2SSerializableSocial_Show_Terms*)instance;
@end
/**
 * @interface C2SSerializableSns_Login
 
 * @brief Sns Login 명령 실행을 위한 편의 클래스
 * @ingroup iOS Helper Sns
 * @author Byeong Yeol Jeon
 
 */
@interface C2SSerializableSns_Login : C2SSerializable
/** 로그인의 종류 */
@property (nonatomic, strong) NSString* type;
/**
 * @brief C2SSerializableSns_Login의 인스턴스를 구한다.
 
 * @return C2SSerializableSns_Login의 인스턴스
 
 * @author Byeong Yeol Jeon
 
 */
+(C2SSerializableSns_Login*)instance;
@end
/**
 * @interface C2SSerializableSns_Login_Guest
 
 * @brief 게스트 로그인 편의 클래스
 * @ingroup iOS Helper Sns
 * @author Byeong Yeol Jeon
 
 */
@interface C2SSerializableSns_Login_Guest : C2SSerializable
/**
 * @brief C2SSerializableSns_Login_Guest의 인스턴스를 구한다.
 
 * @return C2SSerializableSns_Login_Guest의 인스턴스
 
 * @author Byeong Yeol Jeon
 
 */
+(C2SSerializableSns_Login_Guest*)instance;
@end

/**
 * @interface C2SSerializableSns_Login_Select
 
 * @brief 유저 업데이트 편의 클래스
 * @ingroup iOS Helper Sns
 * @author Byeong Yeol Jeon
 
 */
@interface C2SSerializableSns_Login_Select : C2SSerializable
/** 게스트 데이터 */
@property (nonatomic, strong) NSDictionary* currentVidData;
/** 정식 유저 데이터 */
@property (nonatomic, strong) NSDictionary* usedVidData;
/**
 * @brief C2SSerializableSns_Login_Select의 인스턴스를 구한다.
 * @return C2SSerializableSns_Login_Select의 인스턴스
 
 * @author Byeong Yeol Jeon
 
 */
+(C2SSerializableSns_Login_Select*)instance;
@end
/**
 * @interface C2SSerializableSns_User
 
 * @brief Sns User 명령 실행 편의 클래스
 * @ingroup iOS Helper Sns
 * @author Byeong Yeol Jeon
 
 */
@interface C2SSerializableSns_User : C2SSerializable
/** 대상 vid */
@property (nonatomic, strong) NSArray* vid;
/**
 * @brief C2SSerializableSns_User의 인스턴스를 구한다.
 
 * @return C2SSerializableSns_User의 인스턴스
 * @author Byeong Yeol Jeon
 
 */
+(C2SSerializableSns_User*)instance;
@end
/**
 * @interface C2SSerializableSns_User_Me
 
 * @brief 자신의 정보 요청 편의 클래스
 * @ingroup iOS Helper Sns
 * @author Byeong Yeol Jeon
 
 */
@interface C2SSerializableSns_User_Me : C2SSerializable
/**
 * @brief C2SSerializableSns_User_Me의 인스턴스를 구한다.
 
 * @return C2SSerializableSns_User_Me의 인스턴스
 
 * @author Byeong Yeol Jeon
 
 */
+(C2SSerializableSns_User_Me*)instance;
@end
/**
 * @interface C2SSerializableSns_Friends
 
 * @brief 친구 데이터 요청 편의 클래스
 * @ingroup iOS Helper Sns
 * @author Byeong Yeol Jeon
 
 */
@interface C2SSerializableSns_Friends : C2SSerializable
/** 요청할 서비스의 타입 */
@property (nonatomic, strong) NSString* service;
/** 요청할 친구리스트의 타입 */
@property (nonatomic, strong) NSString* type;
/**
 * @brief C2SSerializableSns_Friends의 인스턴스를 구한다.
 
 * @param C2SSerializableSns_Friends의 인스턴스
 */
+(C2SSerializableSns_Friends*)instance;
@end
/**
 * @interface C2SSerializableSns_Friends_All
 
 * @brief 전체 친구 요청 편의 클래스
 * @ingroup iOS Helper Sns
 * @author Byeong Yeol Jeon
 
 */
@interface C2SSerializableSns_Friends_All : C2SSerializable
/**
 * @brief C2SSerializableSns_Friends_All의 인스턴스를 구한다.
 
 * @param C2SSerializableSns_Friends_All의 인스턴스
 
 * @author Byeong Yeol Jeon
 
 */
+(C2SSerializableSns_Friends_All*)instance;
@end
/**
 * @interface C2SSerializableSns_Friends_Game
 
 * @brief 게임 친구 요청 편의 클래스
 * @ingroup iOS Helper Sns
 * @author Byeong Yeol Jeon
 
 */
@interface C2SSerializableSns_Friends_Game : C2SSerializable
/**
 * @brief C2SSerializableSns_Friends_Game의 인스턴스를 구한다.
 
 * @return C2SSerializableSns_Friends_Game의 인스턴스
 * @author Byeong Yeol Jeon
 
 */
+(C2SSerializableSns_Friends_Game*)instance;
@end
/**
 * @interface C2SSerializableSns_Friends_Other
 
 * @brief 게임하지 않는 친구 요청 편의 클래스
 * @ingroup iOS Helper Sns
 * @author Byeong Yeol Jeon
 
 */
@interface C2SSerializableSns_Friends_Other : C2SSerializable
/**
 * @brief C2SSerializableSns_Friends_Other의 인스턴스를 구한다.
 
 * @return C2SSerializableSns_Friends_Other의 인스턴스
 * @author Byeong Yeol Jeon
 
 */
+(C2SSerializableSns_Friends_Other*)instance;
@end
/**
 * @interface C2SSerializableSns_Friends_Other_Facebook
 
 * @brief 게임하지 않는 페이스북 친구 리스트 요청 편의 클래스
 * @ingroup iOS Helper Sns
 * @author Byeong Yeol Jeon
 
 */
@interface C2SSerializableSns_Friends_Other_Facebook : C2SSerializable
/**
 * @brief C2SSerializableSns_Friends_Other_Facebook의 인스턴스를 구한다.
 
 * @return C2SSerializableSns_Friends_Other_Facebook의 인스턴스
 
 * @author Byeong Yeol Jeon
 
 */
+(C2SSerializableSns_Friends_Other_Facebook*)instance;
@end
/**
 * @interface C2SSerializableSns_Message_Kakao
 
 * @brief 카카오 플랫폼으로 친구에게 메시지를 보낸다.
 * @ingroup iOS Helper Sns
 * @author Byeong Yeol Jeon
 
 */
@interface C2SSerializableSns_Message_Kakao : C2SSerializable
/** 메시지를 받을 대상의 vid */
@property (nonatomic, strong) NSString* vid;
/** 메시지 내용 */
@property (nonatomic, strong) NSString* message;
/** default가 아닌 앱의 링크 */
@property (nonatomic, strong) NSString* appLinkURL;
/**
 * @brief C2SSerializableSns_Message_Kakao의 인스턴스를 구한다.
 
 * @return C2SSerializableSns_Message_Kakao의 인스턴스
 * @author Byeong Yeol Jeon
 
 */
+(C2SSerializableSns_Message_Kakao*)instance;
@end

@class UIImage;
/**
 * @interface C2SSerializableSns_Message_Kakao_CustomImage
 
 * @brief 카카오 플랫폼 친구에게 이미지 메시지를 보낸다.
 * @ingroup iOS Helper Sns
 * @author Byeong Yeol Jeon
 
 */
@interface C2SSerializableSns_Message_Kakao_CustomImage : C2SSerializable
/** 메시지를 받을 대상의 vid */
@property (nonatomic, strong) NSString* vid;
/** 카카오 형식에 맞는 json형태의 키 밸류 메타 데이터 문자열 */
@property (nonatomic, strong) NSString* jsonMetaData;
/** default가 아닌 앱의 링크를 사용할 때 설정 */
@property (nonatomic, strong) NSString* appLinkURL;
/** 보낼 이미지 데이터 */
@property (nonatomic, strong) UIImage* image;
/** 메시지의 템플릿 id */
@property (nonatomic, strong) NSString* templateId;
/**
 * @brief C2SSerializableSns_Message_Kakao_CustomImage의 인스턴스를 구한다.
 
 * @return C2SSerializableSns_Message_Kakao_CustomImage의 인스턴스
 * @author Byeong Yeol Jeon
 
 */
+(C2SSerializableSns_Message_Kakao_CustomImage*)instance;
@end
/**
 * @interface C2SSerializableSns_Message_Hub
 
 * @brief 허브 플랫폼 친구에게 메시지를 보낸다.
 * @ingroup iOS Helper Sns Hub
 * @author Byeong Yeol Jeon
 
 */
@interface C2SSerializableSns_Message_Hub : C2SSerializable
/** 메시지를 받을 대상의 vid */
@property (nonatomic, strong) NSString* vid;
/** 보낼 메시지 */
@property (nonatomic, strong) NSString* message;
/** 보낼 이미지의 url */
@property (nonatomic, strong) NSString* imageURL;
/** 이미지 썸네일의 url */
@property (nonatomic, strong) NSString* thumbnailURL;
/** 메시지를 받는 대상에게 푸쉬를 보낸다. */
@property (nonatomic, assign) BOOL usePush;
/** YES일 경우 초대메시지로 보낸다. */
@property (nonatomic, assign) BOOL invite;
/**
 * @brief C2SSerializableSns_Message_Hub의 인스턴스를 구한다.
 
 * @return C2SSerializableSns_Message_Hub의 인스턴스
 
 * @author Byeong Yeol Jeon
 
 */
+(C2SSerializableSns_Message_Hub*)instance;
@end
/**
 * @interface C2SSerializableSns_Message_Facebook
 
 * @brief 페이스북 친구에게 메시지를 보낸다.
 * @ingroup iOS Helper Sns
 * @author Byeong Yeol Jeon
 
 */
@interface C2SSerializableSns_Message_Facebook : C2SSerializable
/** 메시지를 받을 대상의 페이스북 아이디 */
@property (nonatomic, strong) NSString* facebookId;
/** 보낼 메시지의 문자열 */
@property (nonatomic, strong) NSString* message;
/**
 * @brief C2SSerializableSns_Message_Facebook의 인스턴스를 구한다.
 
 * @return C2SSerializableSns_Message_Facebook의 인스턴스
 
 * @author Byeong Yeol Jeon
 
 */
+(C2SSerializableSns_Message_Facebook*)instance;
@end
/**
 * @interface C2SSerializableSns_Message_GooglePlus
 
 * @brief 구글 플러스 친구에게 메시지를 보낸다.
 * @ingroup iOS Helper Sns
 * @author Byeong Yeol Jeon
 
 */
@interface C2SSerializableSns_Message_GooglePlus : C2SSerializable
/** 메시지를 받을 대상의 구글플러스 아이디 */
@property (nonatomic, strong) NSString* googleplusId;
/** 보낼 메시지의 문자열 */
@property (nonatomic, strong) NSString* message;
/**
 * @brief C2SSerializableSns_Message_GooglePlus의 인스턴스를 구한다.
 
 * @return C2SSerializableSns_Message_GooglePlus의 인스턴스
 
 * @author Byeong Yeol Jeon
 
 */
+(C2SSerializableSns_Message_GooglePlus*)instance;
@end
/**
 * @interface C2SSerializableSns_Message_SinaWeibo
 
 * @brief 시나 웨이보 친구에게 메시지를 보낸다.
 * @ingroup iOS Helper Sns
 * @author Byeong Yeol Jeon
 
 */
@interface C2SSerializableSns_Message_SinaWeibo : C2SSerializable
/** 메시지를 받을 대상의 시나 웨이보 아이디 */
@property (nonatomic, strong) NSString* sinaweiboId;
/** 보낼 메시지의 문자열 */
@property (nonatomic, strong) NSString* message;
/**
 * @brief C2SSerializableSns_Message_SinaWeibo의 인스턴스를 구한다.
 
 * @return C2SSerializableSns_Message_SinaWeibo의 인스턴스
 
 * @author Byeong Yeol Jeon
 
 */
+(C2SSerializableSns_Message_SinaWeibo*)instance;
@end
/**
 * @interface C2SSerializablePush_Register
 
 * @brief 푸쉬 등록 편의 클래스
 * @ingroup iOS Helper Push
 * @author Byeong Yeol Jeon
 
 */
@interface C2SSerializablePush_Register : C2SSerializable
/** 푸쉬 아이디 */
@property (nonatomic, assign) int32_t pushId;
/** 푸쉬 타이틀 */
@property (nonatomic, strong) NSString* title;
/** 푸쉬 메시지 */
@property (nonatomic, strong) NSString* message;
/** 사운드 */
@property (nonatomic, strong) NSString* sound;
/** 푸쉬 알림 시간 */
@property (nonatomic, assign) int64_t after;
/**
 * @brief C2SSerializablePush_Register의 인스턴스를 구한다.
 * @return C2SSerializablePush_Register의 인스턴스
 * @author Byeong Yeol Jeon
 
 */
+(C2SSerializablePush_Register*)instance;
@end
/**
 * @interface C2SSerializablePush_Register_DeviceToken
 
 * @brief 푸쉬 디바이스 토큰 등록 편의 클래스
 * @ingroup iOS Helper Push
 * @author Byeong Yeol Jeon
 
 */
@interface C2SSerializablePush_Register_DeviceToken : C2SSerializable
/** 푸쉬 디바이스 토큰 */
@property (nonatomic, strong) NSData* deviceToken;
/**
 * @brief C2SSerializablePush_Register_DeviceToken의 인스턴스를 구한다.
 
 * @return C2SSerializablePush_Register_DeviceToken의 인스턴스
 
 * @author Byeong Yeol Jeon
 
 */
+(C2SSerializablePush_Register_DeviceToken*)instance;
@end
/**
 * @interface C2SSerializablePush_Register_PromotionCode
 
 * @brief 푸쉬 프로모션 코드 등록 편의 클래스
 * @ingroup iOS Helper Push
 * @author Byeong Yeol Jeon
 
 */
@interface C2SSerializablePush_Register_PromotionCode : C2SSerializable
/** 푸쉬 프로모션 코드 */
@property (nonatomic, strong) NSDictionary* promotionCode;
/**
 * @brief C2SSerializablePush_Register_PromotionCode의 인스턴스를 구한다.
 
 * @return C2SSerializablePush_Register_PromotionCode의 인스턴스
 
 * @author Byeong Yeol Jeon
 
 */
+(C2SSerializablePush_Register_PromotionCode*)instance;
@end
/**
 * @interface C2SSerializablePromotion_Show
 
 * @brief Promotion Show의 편의 클래스
 * @ingroup iOS Helper Promotion
 * @author Byeong Yeol Jeon
 
 */
@interface C2SSerializablePromotion_Show : C2SSerializable
/** 보여줄 뷰의 타입 */
@property (nonatomic, strong) NSString* type;
/** YES일 경우 24시간 다시 보여주지 않기를 무시한다. */
@property (nonatomic, assign) BOOL forced;
/**
 * @brief C2SSerializablePromotion_Show의 인스턴스를 구한다.
 * @return C2SSerializablePromotion_Show의 인스턴스
 * @author Byeong Yeol Jeon
 
 */
+(C2SSerializablePromotion_Show*)instance;
@end
/**
 * @interface C2SSerializablePromotion_Show_Notice
 
 * @brief 공지 보여주기 편의 클래스
 * @ingroup iOS Helper Promotion
 * @author Byeong Yeol Jeon
 
 */
@interface C2SSerializablePromotion_Show_Notice : C2SSerializable
/** YES일 경우 24시간 다시 보여주지 않기를 무시한다. */
@property (nonatomic, assign) BOOL forced;
/**
 * @brief C2SSerializablePromotion_Show_Notice의 인스턴스를 구한다.
 
 * @return C2SSerializablePromotion_Show_Notice의 인스턴스
 * @author Byeong Yeol Jeon
 
 */
+(C2SSerializablePromotion_Show_Notice*)instance;
@end
/**
 * @interface C2SSerializablePromotion_Show_Event
 
 * @brief 이벤트 보여주기 편의 클래스
 * @ingroup iOS Helper Promotion
 * @author Byeong Yeol Jeon
 
 */
@interface C2SSerializablePromotion_Show_Event : C2SSerializable
/** YES일 경우 24시간 다시 보여주지 않기를 무시한다. */
@property (nonatomic, assign) BOOL forced;
/**
 * @brief C2SSerializablePromotion_Show_Event의 인스턴스를 구한다.
 
 * @return C2SSerializablePromotion_Show_Event의 인스턴스
 * @author Byeong Yeol Jeon
 
 */
+(C2SSerializablePromotion_Show_Event*)instance;
@end
#endif // #if __OBJC__

#endif
