//
//  AnalyticsTest.cpp
//  hivesdk
//
//  Created by paikjongman on 2017. 2. 28..
//
//

#include "AnalyticsTest.hpp"

using namespace hive;
using namespace cocos2d;
using namespace cocos2d::ui;

const std::vector<const char*> eventTypeLists = {"Install","Update","Login","Purchase","TutorialComplete"};
const std::vector<const char*> eventLogTypeLists = {"Account_Join","Account_Login"};

bool AnalyticsTest::init()
{
    if(!BaseView::init())
        return false;
    setUseShowLogButton(true);
    Logger::setLoggerListener([=](std::string tag, std::string level, std::string logMessage){
        addLogMessage(logMessage);
    });

    addCombobox("Event Name : ", std::move(eventTypeLists), 0, COMBOBOX_EVENT_NAME);
    addButton("Analytics.sendEvent", BUTTON_SEND_EVENT);
    
    addCombobox("Analytics Log : ",std::move(eventLogTypeLists), 0, COMBOBOX_ANALYTICS_LOG);

    // TODO: 초당 120개 로그 전송 기능에 대한 기능 구현 필요
    /*
    BeginHorizontal();
    addEditBox("Log Amount : ", "10", "", {screenSize.width/2, DEFAULT_CONTENT_HEIGHT}, EditBox::InputMode::SINGLE_LINE, 20, EDITBOX_LOG_AMOUNT);
    addEditBox("Cycle(sec) : ", "1", "", {screenSize.width/2, DEFAULT_CONTENT_HEIGHT}, EditBox::InputMode::SINGLE_LINE, 20, EDITBOX_CYCLE);
    EndHorizontal();

    BeginHorizontal();
    addEditBox("Send limit : ", "100", "", {screenSize.width/2, DEFAULT_CONTENT_HEIGHT}, EditBox::InputMode::SINGLE_LINE, 20, EDITBOX_SEND_LIMIT);
    addEditBox("Queue limit : ", "500", "", {screenSize.width/2, DEFAULT_CONTENT_HEIGHT}, EditBox::InputMode::SINGLE_LINE, 20, EDITBOX_QUEUE_LIMIT);
    EndHorizontal();
     */

    addButton("Analytics.sendAnalyticsLog", BUTTON_SEND_ANALYTICS_LOG);
    
    setUseBackButton(true, "- Analytics -");
    
    return true;
}


void AnalyticsTest::onButtonEvent(int buttonTag)
{
    switch (buttonTag) {
        case BUTTON_SEND_EVENT:
        {
            std::string currentEventType = eventTypeLists[getSelectedComboboxIndex(COMBOBOX_EVENT_NAME)];
            Logger::log("\n\n=== 서드파티 이벤트 전송 테스트 ===\n");
            Logger::log("Analytics::sendEvent(currentEventType = %s) Called\n", currentEventType.c_str());
            
            Analytics::sendEvent(currentEventType);
            break;
        }
        case BUTTON_SEND_ANALYTICS_LOG:
        {
            Logger::log("\n\n=== Analytics Log 전송 테스트 ===\n");

            int getSelectedIndex = getSelectedComboboxIndex(COMBOBOX_ANALYTICS_LOG);
            std::string currentEventType = eventLogTypeLists[getSelectedIndex];

            if(currentEventType.compare("Account_Join") == 0)
            {
                picojson::object logData;
                logData["logId"] = picojson::value((double)1);          // 계정
                logData["logSubId"] = picojson::value((double)1);       // 가입
                logData["userId"] = picojson::value("user001");
                logData["nickname"] = picojson::value("게임돌이");
                
                Analytics::sendAnalyticsLog(logData);
                
                Logger::log("Analytics::sendAnalyticsLog(logData) Called\n");
                Logger::log("logData = %s\n", picojson::value(logData).serialize().c_str());
            }
            else if(currentEventType.compare("Account_Login") == 0)
            {
                picojson::object logData;
                logData["logId"] = picojson::value((double)1);          // 계정
                logData["logSubId"] = picojson::value((double)2);       // 로그인
                logData["userId"] = picojson::value("user001");
                logData["currentGold"] = picojson::value((double)1000);
                
                Analytics::sendAnalyticsLog(logData);
                
                Logger::log("Analytics::sendAnalyticsLog(logData) Called\n");
                Logger::log("logData = %s\n", picojson::value(logData).serialize().c_str());
            }
            break;
        }
        default:
            break;
    }
}
