//
//  AppInvitationInfo.cpp
//  HIVE_SDK_Cocos_Sample
//
//  Created by paikjongman on 2017. 7. 24..
//  Copyright © 2017년 GCP. All rights reserved.
//

#include "AppInvitationInfo.hpp"
#include "UICombobox.hpp"
#include "ResultUtils.hpp"
#if (CC_TARGET_PLATFORM == CC_PLATFORM_ANDROID)
#include "platform/android/jni/JniHelper.h"
#endif

std::string getSharableDirectory(){
#if (CC_TARGET_PLATFORM == CC_PLATFORM_ANDROID)
    return cocos2d::JniHelper::callStaticStringMethod("org/cocos2dx/cpp/AppActivity","getExternalStorage")+"/";
#else
    return cocos2d::FileUtils::getInstance()->getWritablePath();
#endif
}

static const char* qrcode_tempfile_name = "hive_temp_invitation_qrcode.png";

AppInvitaionView* AppInvitaionView::create(const hive::AppInvitationData& data)
{
    AppInvitaionView *pRet = new(std::nothrow) AppInvitaionView();
    if (pRet && pRet->init(data))
    {
        pRet->autorelease();
        return pRet;
    }
    else
    {
        delete pRet;
        pRet = nullptr;
        return nullptr;
    }
}

bool AppInvitaionView::init(const hive::AppInvitationData& data) {
    if(!BaseView::init())
        return false;
    
    std::vector<char> bytes(data.qrCode.begin(), data.qrCode.end());
    bytes.push_back('\0');
    unsigned char *c = reinterpret_cast<unsigned char*>(&bytes[0]);
    
    cocos2d::Data imageData;
    imageData.copy(c, bytes.size());
    
    auto savePath = getSharableDirectory() + qrcode_tempfile_name;
    
    
    cocos2d::FileUtils::getInstance()->writeDataToFile(imageData, savePath);
    
    addLabel(":: AppInvitationInfo ::");
    addButtonImage(savePath.c_str(), BUTTON_QR_CODE);
    addButton("inviteCommonLink",BUTTON_COMMON_LINK);
    addButton("inviteHivemsgLink",BUTTON_HIVEMSG_LINK);
    addButton("inviteFacebookLink",BUTTON_FACEBOOK_LINK);
    addButton("inviteMessage",BUTTON_INVITE_MSG);
    if(data.eachCampaignList.size() > 0) {
        addButton("Show eachCampaignList", BUTTON_SHOW_EACH_CAMPAIGNLIST);
        for(auto it=data.eachCampaignList.begin();it!=data.eachCampaignList.end();++it) {
            eachlist.push_back(it->title);
        }
        appInvitationCampaignList = data.eachCampaignList;
    }
    if(data.stageCampaignList.size() > 0){
        addButton("Show stageCampaignList", BUTTON_SHOW_STAGE_CAMPAIGNLIST);
        for(auto it=data.stageCampaignList.begin();it!=data.stageCampaignList.end();++it) {
            stagelist.push_back(it->title);
        }
        appInvitationCampaignStageList = data.stageCampaignList;
    }
    
#if (CC_TARGET_PLATFORM == CC_PLATFORM_ANDROID)
    //android의 경우 fileprovider로 추가로 경로를 얻어야 합니다.
    savePath = cocos2d::JniHelper::callStaticStringMethod("org/cocos2dx/cpp/AppActivity","getShareMediaPathFileUri",savePath);
#else
    savePath = "file://"+savePath;
#endif
    media.insert(std::pair<UIS,std::string>(BUTTON_QR_CODE,savePath));
    media.insert(std::pair<UIS,std::string>(BUTTON_COMMON_LINK,data.inviteCommonLink));
    media.insert(std::pair<UIS,std::string>(BUTTON_HIVEMSG_LINK,data.inviteHivemsgLink));
    media.insert(std::pair<UIS,std::string>(BUTTON_FACEBOOK_LINK,data.inviteFacebookLink));
    media.insert(std::pair<UIS,std::string>(BUTTON_INVITE_MSG,data.inviteMessage));

    setUseBackButton(true);
    return true;
}

void AppInvitaionView::onButtonEvent(int buttonTag){
    auto proc = [=](bool isSuccess){
        if(isSuccess) {
            cocos2d::MessageBox("share success","success");
        }else{
            cocos2d::MessageBox("share fail","fail");
        }
    };
    UIS uis=static_cast<UIS>(buttonTag);
    if(media.find(uis) != media.end()) {
        if(uis == BUTTON_QR_CODE) {
//            hive::PlatformHelper::shareMedia({media[uis]}, "Sharing",proc);
            
            hive::PlatformShare* platformShare = new hive::PlatformShare();
            platformShare->setShareType(hive::PlatformShareType::MEDIA);
            platformShare->setMedia({media[uis]});
            
            hive::PlatformHelper::share(*platformShare, proc);
        }
        else {
//            hive::PlatformHelper::shareText("Sharing : "+media[uis], proc);
            
            hive::PlatformShare* platformShare = new hive::PlatformShare();
            platformShare->setShareType(hive::PlatformShareType::TEXT);
            platformShare->setText("Sharing : "+media[uis]);
            
            hive::PlatformHelper::share(*platformShare, proc);
        }
    } else {
        switch (buttonTag) {
            case BUTTON_SHOW_EACH_CAMPAIGNLIST:
                pushView(Combobox::create(":: EACH CAMPAIGN LIST ::", {screenSize.width, DEFAULT_CONTENT_HEIGHT}, eachlist, true, [=](const char *selectItem,int index){
                    std::string message = ResultUtils::getAppInvitationCampaignString(appInvitationCampaignList[index]);
                    cocos2d::MessageBox(message.c_str(),"each campaign info");
                }));
                break;
            case BUTTON_SHOW_STAGE_CAMPAIGNLIST:
                pushView(Combobox::create(":: STAGE CAMPAIGN LIST ::", {screenSize.width, DEFAULT_CONTENT_HEIGHT}, eachlist, true, [=](const char *selectItem,int index){
                    std::string message = ResultUtils::getAppInvitationCampaignStageString(appInvitationCampaignStageList[index]);
                    cocos2d::MessageBox(message.c_str(),"each campaign info");
                }));
                break;
            default:
                break;
        }
    }
}
