//
//  AuthV4HelperTest.cpp
//  HIVE_SDK_Cocos_Sample
//
//  Created by paikjongman on 2018. 3. 12..
//  Copyright © 2018년 GCP. All rights reserved.
//

#include "AuthV4HelperTest.hpp"
#include "AuthV4Test.hpp"
#include "GameManager.hpp"
#include "ResultUtils.hpp"
#include "Postbox.hpp"

using namespace cocos2d;
using namespace cocos2d::ui;
using namespace hive;

const std::vector<const char*> providerList ={
    "GUEST"
    ,"HIVE"
    ,"FACEBOOK"
    ,"GOOGLE"
    ,"QQ"
    ,"WEIBO"
    ,"VK"
    ,"WECHAT"
    ,"APPLE"
    ,"SIGNIN_APPLE"
    ,"LINE"
    ,"TWITTER"
    ,"WEVERSE"
    ,"HUAWEI"
    ,"AUTO"
};

bool AuthV4HelperTest::init() {
    if(!BaseView::init())
        return false;
    setUseShowLogButton(true);
    Logger::setLoggerListener([=](std::string tag, std::string level, std::string logMessage){
        addLogMessage(logMessage);
    });

    addButton("AuthV4Helper.getIDPLIst", BUTTON_GET_IDPLIST);

    BeginHorizontal();
    addButton("AuthV4Helper.signIn", {screenSize.width/3, DEFAULT_CONTENT_HEIGHT}, BUTTON_SIGNIN);
    addButton("AuthV4Helper.signOut", {screenSize.width/3, DEFAULT_CONTENT_HEIGHT}, BUTTON_SIGNOUT);
    addButton("AuthV4Helper.playerDelete", {screenSize.width/3, DEFAULT_CONTENT_HEIGHT}, BUTTON_PLAYERDELETE);
    EndHorizontal();

    BeginHorizontal();
    addButton("AuthV4Helper.connect", {screenSize.width/2, DEFAULT_CONTENT_HEIGHT}, BUTTON_CONNECT);
    addButton("AuthV4Helper.disconnect", {screenSize.width/2, DEFAULT_CONTENT_HEIGHT}, BUTTON_DISCONNECT);
    EndHorizontal();

    addButton("AuthV4Helper.syncAccount", BUTTON_SYNC_ACCOUNT);
    addButton("AuthV4Helper.showGameCenterLoginCancelDialog",BUTTON_SHOW_GAME_CENTER_LOGINCANCELDIALOG);

    BeginHorizontal();
    addButton("AuthV4Helper.showConflict", {screenSize.width/2, DEFAULT_CONTENT_HEIGHT}, BUTTON_SHOW_CONFLICT_1);
    addButton("AuthV4Helper.showConflict ForGame", {screenSize.width/2, DEFAULT_CONTENT_HEIGHT}, BUTTON_SHOW_CONFLICT_2);
    EndHorizontal();

    BeginHorizontal();
    addButton("AuthV4Helper.resolveConflict", {screenSize.width/2, DEFAULT_CONTENT_HEIGHT}, BUTTON_RESOLVE_CONFLICT);
    addButton("AuthV4Helper.switchConflict", {screenSize.width/2, DEFAULT_CONTENT_HEIGHT}, BUTTON_SWITCH_CONFLICT);
    EndHorizontal();

    BeginHorizontal();
    addButton("AuthV4Helper.showAchievements", {screenSize.width/2, DEFAULT_CONTENT_HEIGHT}, BUTTON_SHOW_ACHIVEMENTS);
    addButton("AuthV4Helper.showLeaderboard", {screenSize.width/2, DEFAULT_CONTENT_HEIGHT}, BUTTON_SHOW_LEADERBOARD);
    EndHorizontal();
    
    addEditBox("ID : ", "ID", EDITBOX_ID);
    addEditBox("VALUE : ", "VALUE", EDITBOX_VALUE);
    addButton("achievementsReveal", BUTTON_ACHIVEMENTS_REVEAL);
    addButton("achievementsUnlock", BUTTON_ACHIVEMENTS_UNLOCK);
    addButton("achievementsIncrements", BUTTON_ACHIVEMENTS_INCREMENTS);
    addButton("leaderboardsSubmitScore", BUTTON_LEADERBOARDS_SUBMIT_SCORE);
    addButton("AuthV4Helper.showSignIn", BUTTON_SHOW_SIGNIN);

    setUseBackButton(true, "- AuthV4Helper -");
    
    return true;
}

void AuthV4HelperTest::onButtonEvent(int buttonTag) {
    switch(buttonTag) {
        case BUTTON_SIGNIN: {
            AuthV4::Helper::signIn([=](ResultAPI const & result,std::shared_ptr<PlayerInfo> playerInfo) {
                if(result.isSuccess()) {
                    GameManager::getInstance()->SetCurrentPlayerInfo(*playerInfo);
                    
                    Postbox::setLoginType(Postbox::v4);
                    Postbox::login([=](const Postbox::Login *login, Postbox::Error error){
                        if( login != nullptr )
                        {
                            int gold = login->gold;
                            int gem = login->gem;
                            std::string nickname = login->nickname;
                            Logger::log("nick %s, gold %d, gem %d", nickname.c_str(), gold, gem);
                        }
                        else
                        {
                            Logger::log("Postbox login failed.");
                            Logger::log("result code = %s, msg = %s", error.result_code.c_str(), error.result_msg.c_str());
                        }
                    });
                }
                logResult("signIn", result, playerInfo);
            });
        }
            break;
        case BUTTON_SIGNOUT:
            AuthV4::Helper::signOut([=](ResultAPI const & result,std::shared_ptr<PlayerInfo> playerInfo) {
                logResult("signOut", result, playerInfo);
            });
            break;
        case BUTTON_PLAYERDELETE:
            AuthV4::Helper::playerDelete([=](ResultAPI const & result,std::shared_ptr<PlayerInfo> playerInfo) {
                logResult("playerDelete", result, playerInfo);
            });
            break;
        case BUTTON_SHOW_SIGNIN:
            AuthV4::showSignIn([=](ResultAPI const & result, PlayerInfo const & playerInfo){
                Postbox::setLoginType(Postbox::v4);
                GameManager::getInstance()->SetCurrentPlayerInfo(playerInfo);

                logResult("showSignIn", result, playerInfo);
            });
            break;

        case BUTTON_GET_IDPLIST:
            logResult("getIDPList", getProviderTypeStringListStr());
            break;
        case BUTTON_CONNECT:
            showProviderSelectView(toStringVector(providerList), [=](hive::ProviderType type) {
                AuthV4::Helper::connect(type, [=](ResultAPI const & result, std::shared_ptr<PlayerInfo> playerInfo) {
                    logResult("connect", result, playerInfo);

                    if(result.getErrorCode() == ResultAPI::CONFLICT_PLAYER ) {
                        GameManager::getInstance()->SetConflictPlayerInfo(*playerInfo);
                    }
                });
            });
            break;
        case BUTTON_DISCONNECT:
            showProviderSelectView(toStringVector(providerList), [=](hive::ProviderType type) {
                AuthV4::Helper::disconnect(type, [=](ResultAPI const & result,std::shared_ptr<PlayerInfo> playerInfo) {
                    logResult("disconnect", result, playerInfo);
                });
            });
            break;
        case BUTTON_RESOLVE_CONFLICT:
            //current user
            AuthV4::Helper::resolveConflict([=](ResultAPI const & result,std::shared_ptr<PlayerInfo> playerInfo) {
                conflictProcess(result,playerInfo);
                logResult("resolveConflict", result, playerInfo);
            });
            break;
        case BUTTON_SWITCH_CONFLICT:
            //change user
            AuthV4::Helper::switchAccount([=](ResultAPI const & result,std::shared_ptr<PlayerInfo> playerInfo) {
                conflictProcess(result,playerInfo);
                logResult("switchAccount", result, playerInfo);
            });
            break;
        case BUTTON_SHOW_CONFLICT_1: {
            AuthV4::Helper::showConflict([=](ResultAPI const & result,std::shared_ptr<PlayerInfo> playerInfo){
                conflictProcess(result,playerInfo);
                logResult("showConflict(listener)", result, playerInfo);
            });
        }
            break;
        case BUTTON_SHOW_CONFLICT_2: {
            PlayerInfo conflictPlayer = GameManager::getInstance()->GetConflictPlayerInfo();

            ConflictSingleViewInfo* _info = new ConflictSingleViewInfo(conflictPlayer.playerId);
            _info->SetValue("ID", conflictPlayer.playerId);
            _info->SetValue("Name", conflictPlayer.playerName.c_str());
            _info->SetValue("Token", conflictPlayer.playerToken.c_str());

            AuthV4::Helper::showConflict(*_info, [=](ResultAPI const & result, std::shared_ptr<PlayerInfo> playerInfo){
                conflictProcess(result, playerInfo);
                logResult("showConflict(conflictData, listener)", result, playerInfo);
            });
        }
            break;
        case BUTTON_SYNC_ACCOUNT: {
            showProviderSelectView(getProviderTypeStringListStr(), [=](hive::ProviderType type) {
                AuthV4::Helper::syncAccount(type, [=](ResultAPI const & result,std::shared_ptr<PlayerInfo> playerInfo) {
                    logResult("syncAccount",result,playerInfo);
                });
            });
        }
            break;
        case BUTTON_SHOW_GAME_CENTER_LOGINCANCELDIALOG:
            AuthV4::Helper::showGameCenterLoginCancelDialog([=](bool isDismiss){
                Logger::log("showGameCenterLoginCancelDialog isDismiss : %s",isDismiss ? "true" : "false");
            });
            break;
        case BUTTON_SHOW_ACHIVEMENTS:
            AuthV4::Helper::showAchievements([=](ResultAPI const & result,std::shared_ptr<PlayerInfo> playerInfo) {
                logResult("showAchievements", result, playerInfo);
            });
            break;
        case BUTTON_SHOW_LEADERBOARD:
            AuthV4::Helper::showLeaderboard([=](ResultAPI const & result,std::shared_ptr<PlayerInfo> playerInfo) {
                logResult("showLeaderboard", result, playerInfo);
            });
            break;
        case BUTTON_ACHIVEMENTS_REVEAL:
        {
            std::string achievementsID = getEditboxValue(EDITBOX_ID);
            AuthV4::Helper::achievementsReveal(achievementsID, [=](ResultAPI const & result,std::shared_ptr<PlayerInfo> playerInfo) {
                logResult("achievementsReveal", result, playerInfo);
            });
            break;
        }
        case BUTTON_ACHIVEMENTS_UNLOCK:
        {
            std::string achievementsID = getEditboxValue(EDITBOX_ID);
            AuthV4::Helper::achievementsUnlock(achievementsID, [=](ResultAPI const & result,std::shared_ptr<PlayerInfo> playerInfo) {
                logResult("achievementsUnlock", result, playerInfo);
            });
            break;
        }
        case BUTTON_ACHIVEMENTS_INCREMENTS:
        {
            std::string achievementsID = getEditboxValue(EDITBOX_ID);
            int value = std::stoi(getEditboxValue(EDITBOX_VALUE));
            AuthV4::Helper::achievementsIncrement(achievementsID, value, [=](ResultAPI const & result,std::shared_ptr<PlayerInfo> playerInfo) {
                logResult("achievementsIncrement", result, playerInfo);
            });
            break;
        }
        case BUTTON_LEADERBOARDS_SUBMIT_SCORE:
        {
            std::string achievementsID = getEditboxValue(EDITBOX_ID);
            int value = std::stoi(getEditboxValue(EDITBOX_VALUE));
            AuthV4::Helper::leaderboardsSubmitScore(achievementsID, value, [=](ResultAPI const & result,std::shared_ptr<PlayerInfo> playerInfo) {
                logResult("leaderboardsSubmitScore", result, playerInfo);
            });
            break;
        }
    }
}

void AuthV4HelperTest::conflictProcess(hive::ResultAPI const & result,std::shared_ptr<hive::PlayerInfo> playerInfo) {
    if(result.isSuccess() && playerInfo && (GameManager::getInstance()->GetCurrentPlayerInfo().playerId != playerInfo->playerId)) {
        Logger::log("Change Player : %ld",playerInfo->playerId);
        GameManager::getInstance()->SetCurrentPlayerInfo(*playerInfo);
    }
}

void AuthV4HelperTest::logResult(const char* funcName,const hive::ResultAPI &result,std::shared_ptr<PlayerInfo> playerInfo) {
    ResultUtils::logHead("AuthV4HelperTest", funcName,result);
    if(playerInfo) {
        ResultUtils::logPlayerInfo("AuthV4HelperTest", funcName, *playerInfo);
    }
}

void AuthV4HelperTest::logResult(const char* funcName, const hive::ResultAPI &result, const hive::PlayerInfo &playerInfo) {
    ResultUtils::logHead("AuthV4HelperTest", funcName,result);
    ResultUtils::logPlayerInfo("AuthV4HelperTest", funcName, playerInfo);
}

void AuthV4HelperTest::logResult(const char* funcName, std::vector<std::string> idpList) {
    ResultUtils::logHead("AuthV4HelperTest", funcName);
    std::stringstream ss;
    for(int i=0; i<idpList.size(); i++) {
        ss << idpList[i] << '\n';
    }
    Logger::log("%s", ss.str().c_str());
}

std::vector<const char*>& AuthV4HelperTest::getProviderTypeStringList() {
    std::vector<ProviderType> list = AuthV4::Helper::getIDPList();
    providerTypeStringListStr.clear();
    providerTypeStringList.clear();
    for(auto iter : list) {
        providerTypeStringListStr.push_back(ProviderInfo::stringProviderType(iter).c_str());
        providerTypeStringList.push_back(providerTypeStringListStr.back().c_str());
    }
    return providerTypeStringList;
}
std::vector<std::string>& AuthV4HelperTest::getProviderTypeStringListStr() {
    getProviderTypeStringList();
    return providerTypeStringListStr;
}
