//
//  IAPTest.cpp
//  hivesdk
//
//  Created by paikjongman on 2017. 2. 28..
//
//

#include "IAPTest.hpp"
#include "ResultUtils.hpp"
#include "PostBoxView.hpp"

using namespace hive;

std::string IAPReceiptToString(IAPReceipt const & receipt){
    return "additionalInfo : "+receipt.additionalInfo+"\niapTransactionId : "+receipt.iapTransactionId+"\nproducts : " + receipt.product.toString();
}

static std::string IAPTypeToString(IAPType const type){
    std::string typeString;
    switch(type){
        case IAPType::APPLE_APPSTORE:
            typeString = "Apple";
            break;
        case IAPType::GOOGLE_PLAYSTORE:
            typeString = "Google";
            break;
        case IAPType::HIVE_LEBI:
            typeString = "Lebi";
            break;
        case IAPType::ONESTORE:
            typeString = "OneStore";
            break;
        default:
            typeString = "None";
            break;
    }
    return typeString;
}

bool IAPTest::init()
{
    
    if(!BaseView::init())
        return false;
//    LABEL_SERVER,
//    LABEL_GEM,
//    LABEL_GOLD,
    
    setUseShowLogButton(true);
    Logger::setLoggerListener([=](std::string tag, std::string level, std::string logMessage){
        addLogMessage(logMessage);
    });
    addEditBox("Location Code : ","main","main", EDITBOX_LOCATION_CODE);
#if (CC_TARGET_PLATFORM == CC_PLATFORM_ANDROID)
    addButton("IAP.showMarketSelection",BUTTON_MARKET_SELECT);
#endif
    addButton("IAP.initialize", BUTTON_INITIALIZE);
    addButton("IAP.getShopInfo", BUTTON_GET_SHOP_INFO);
    addCombobox("PID : ", {"getShopInfoFirst"}, 0, COMBO_PID);
    addEditBox("INPUT Character Name",EDITBOX_CHARACTER_KEY);
    addButton("IAP.purchase", BUTTON_PURCHASE);
    addButton("IAP.restore", BUTTON_RESTORE);
    addButton("IAP.restoreReceipt",BUTTON_RESTORE_RECEIPT);
#if (CC_TARGET_PLATFORM == CC_PLATFORM_ANDROID)
    addButton("IAP.getBalanceInfo",BUTTON_BALANCE_INFO);
    addButton("IAP.showCharge",BUTTON_BALANCE_CHARGE);
#endif
#if (CC_TARGET_PLATFORM == CC_PLATFORM_IOS)
    addButton("IAP.checkPromotePurchase",BUTTON_CHECK_PROMOTE_PURHCASE);
#endif
    addButton("Show Postbox", BUTTON_SHOW_POSTBOX);

    
    setUseBackButton(true, "- IAP -");
    
    return true;
}

void IAPTest::onButtonEvent(int buttonTag)
{
    switch (buttonTag) {
        case BUTTON_INITIALIZE:
        {
            Logger::log("\n\n=== IAP 결재 API 초기화 요청 ===\n");
            Logger::log("IAP::initialize(onIAPMarketInfo) Called\n");
            IAP::initialize(std::bind(&IAPTest::onIAPMarketInfoCB, this, std::placeholders::_1, std::placeholders::_2));
            break;
        }
        case BUTTON_GET_SHOP_INFO:
        {
            std::string locationCode = getEditboxValue(EDITBOX_LOCATION_CODE);
            Logger::log("\n\n=== 상점 정보 조회 ===\n");
            Logger::log("IAP::getShopInfo(locationCode = %s, onIAPShopInfo) Called\n", locationCode.c_str());
            IAP::getShopInfo(locationCode,
                             std::bind(&IAPTest::onIAPShopInfoCB, this, std::placeholders::_1, std::placeholders::_2, std::placeholders::_3));
            break;
        }
        case BUTTON_PURCHASE:
        {
            std::string pid = getComboboxValue(COMBO_PID);
            Logger::log("\n\n=== HIVE IAP 상품에 대한 구매를 요청. ===\n");
            Logger::log("IAP::purchase(pid = %s, onIAPPurchase) Called\n", pid.c_str());
            
            std::string additionalInfo = getEditboxValue(EDITBOX_CHARACTER_KEY);
            
            StartActivityIndicator();
            if(additionalInfo.size() > 0){
                IAP::purchase(pid, "{\"character_key\":\""+additionalInfo+"\"}", [=](ResultAPI const & result, IAPReceipt const & receipt){
                    Logger::log("IAPTestView::onIAPPurchaseReceiptCB() Callback\nresult = %s\n", result.toString().c_str());
                    Logger::log("IAPTestView::onIAPPurchaseReceiptCB() receipts\n %s", IAPReceiptToString(receipt).c_str());
                    EndActivityIndicator();
                });
            }
            else{
                //additionalInfo 없이 작업
				IAP::purchase(pid, "", [=](ResultAPI const & result, IAPReceipt const & receipt){
					Logger::log("IAPTestView::onIAPPurchaseReceiptCB() Callback\nresult = %s\n", result.toString().c_str());
					Logger::log("IAPTestView::onIAPPurchaseReceiptCB() receipts\n %s", IAPReceiptToString(receipt).c_str());
					EndActivityIndicator();
				});
            }
            
            
            break;
        }
        case BUTTON_RESTORE_RECEIPT:
        case BUTTON_RESTORE:
        {
            Logger::log("\n\n=== HIVE IAP 상품에 대한 미지급된 아이템 지급을 요청. ===\n");
            StartActivityIndicator();
                Logger::log("IAP::restore(onIAPRestore) Called\n");
                IAP::restoreReceipt([=](ResultAPI const & result, std::vector<IAPReceipt> const & receipts){
                    Logger::log("IAPTestView::onIAPRestoreCB() Callback\nresult = %s\n", result.toString().c_str());
                    Logger::log("IAPTestView::onIAPPurchaseReceiptCB() receipts\n");
                    
                    for(int i=0;i<receipts.size();++i){
                        Logger::log("%s\n",IAPReceiptToString(receipts[i]).c_str());
                    }
                    EndActivityIndicator();
                });
			
            break;
        }
        case BUTTON_CHECK_PROMOTE_PURHCASE:
        {
            IAP::checkPromotePurchase([=](ResultAPI const & result,std::string gamePid){
                Logger::log("\n\n=== checkPromotePurchase. ===\n");
                Logger::log("gamePid : %s",gamePid.c_str());
                ResultUtils::showResult(result);
            });
        }
            break;
        case BUTTON_SHOW_POSTBOX:
            pushView(PostBoxView::create());
            break;
        case BUTTON_MARKET_SELECT:
            IAP::showMarketSelection([=](ResultAPI const & result, std::vector<IAPType> const & iapTypeList) {
                ResultUtils::logHead("IAP", "showMarketSelection",result);
                Logger::log("=== IAPType list ===");
                for(auto it = iapTypeList.begin(); it != iapTypeList.end(); ++it) {
                    Logger::log(IAPTypeToString(*it).c_str());
                    Logger::log("\n");
                }
            });
            break;
        case BUTTON_BALANCE_INFO:
            IAP::getBalanceInfo([=](ResultAPI const & result, int balance){
                ResultUtils::logHead("IAP", "getBalanceInfo",result);
                Logger::log("balance : %d\n",balance);
            });
            break;
        case BUTTON_BALANCE_CHARGE:
            IAP::showCharge([=](ResultAPI const & result, int balance){
                ResultUtils::logHead("IAP", "showCharge",result);
                Logger::log("balance : %d\n",balance);
            });
            break;
        default:
            break;
    }
}

// Apple, Google, OneStore 와 같은 마켓에 등록한 마켓 정보 통지 리스너 - IAP::initialize()
void IAPTest::onIAPMarketInfoCB(ResultAPI result, std::vector<IAPType> marketId)
{
    Logger::log("IAPTestView::onIAPMarketInfoCB() Callback\nresult = %s\n", result.toString().c_str());
    
   	Logger::log("marketId List = \n");
    
    for(int i = 0; i < (int)marketId.size(); i++)
    {
        Logger::log("%d\n", static_cast<int>(marketId[i]));
    }
}


// Hive IAP 백오피스에서 구성한 상점 정보 통지 리스너 - IAP::getShopInfo()
void IAPTest::onIAPShopInfoCB(ResultAPI result, IAPShop iapShop, int balance)
{
    Logger::log("IAPTestView::onIAPShopInfoCB() Callback\nresult = %s\n", result.toString().c_str());
    
    Logger::log("IAP::IAPShop = %s, balance = %d\n", iapShop.toString().c_str(), balance);
    
    if( iapShop.productList.size() > 0 )
    {
        std::vector<const char*> pidLists;
        std::vector<IAPProduct>::iterator it;
        
        for( it = iapShop.productList.begin() ; it != iapShop.productList.end() ; ++it )
        {
            pidLists.push_back((*it).gamePid.c_str());
            CCLOG("%s",(*it).gamePid.c_str());
        }
        
        changeComboboxList(std::move(pidLists), 0, COMBO_PID);
    }
    
}


// 결제 시도 결과 통지 리스너 - IAP::purchase()
void IAPTest::onIAPPurchaseCB(ResultAPI result)
{
    Logger::log("IAPTestView::onIAPPurchaseCB() Callback\nresult = %s\n", result.toString().c_str());
}


// HIVE IAP 상품에 대한 미지급된 아이템 지급을 요청 결과 통지 리스너 - IAP::restore()
void IAPTest::onIAPRestoreCB(ResultAPI result)
{
    Logger::log("IAPTestView::onIAPRestoreCB() Callback\nresult = %s\n", result.toString().c_str());
}


// 러비 잔액 확인 결과 통지 리스너 - IAP::balance()
void IAPTest::onIAPBalanceInfoCB(ResultAPI result, int balance)
{
    Logger::log("IAPTestView::onIAPBalanceInfoCB() Callback\nresult = %s\n", result.toString().c_str());
    
    Logger::log("balance = %d\n", balance);
}
