//
//  IAPV4Test.cpp
//  HIVE_SDK_Cocos_Sample
//
//  Created by paikjongman on 2017. 8. 31..
//  Copyright © 2017년 GCP. All rights reserved.
//

#include "IAPV4Test.hpp"
#include "ResultUtils.hpp"
#include "UICombobox.hpp"
#include "UIMultiselect.hpp"
#include "PostBoxView.hpp"
#include "IAPV4SubscriptionProductInfoView.hpp"

USING_NS_HIVE;

bool useIAPV4Postbox() {
    //return Configuration::getZone() == ZoneType::TEST;
    return true; //4.7.0 기준 모든 서버에 적용.
}

bool IAPV4Test::init() {
    if(!BaseView::init())
        return false;
    
    setUseShowLogButton(true);
    Logger::setLoggerListener([=](std::string tag, std::string level, std::string logMessage){
        addLogMessage(logMessage);
    });

#if (CC_TARGET_PLATFORM == CC_PLATFORM_ANDROID)
    addButton("IAPV4.showMarketSelection",BUTTON_MARKET_SELECT);
#elif (CC_TARGET_PLATFORM == CC_PLATFORM_IOS)
    addButton("FinishAllTransactions (iOS Only)", BUTTON_FINISH_ALL_TRANSACTIONS);
#endif
    addButton("IAPV4.marketConnect", BUTTON_MARKET_CONNECT);
    addButton("IAPV4.getSelectedMarket", BUTTON_GET_SELECTED_MARKET);
    addButton("IAPV4.getAccountUuid", BUTTON_GET_ACCOUNT_UUID);
#if (CC_TARGET_PLATFORM == CC_PLATFORM_ANDROID)
    addButton("IAPV4.getBalanceInfo",BUTTON_BALANCE_INFO);
    addButton("IAPV4.showCharge",BUTTON_BALANCE_CHARGE);
#endif
    addButton("IAPV4.getMarketProductInfo", BUTTON_GET_MARKET_PRODUCT_INFO);
#if (CC_TARGET_PLATFORM == CC_PLATFORM_ANDROID)
    addEditBox("marketPidList : ", "item01", "", EDITBOX_MARKET_PID_LIST);
#elif (CC_TARGET_PLATFORM == CC_PLATFORM_IOS)
    addEditBox("marketPidList : ", "com.com2us.hivesdk.normal.freefull.apple.global.ios.universal.cs01", "", EDITBOX_MARKET_PID_LIST);
#endif
    addButton("IAPV4.getProductInfo", BUTTON_GET_PRODUCT_INFO);
    addEditBox("AdditionalInfo", EDITBOX_ADDITIONAL_INFO);
    addButton("IAPV4.purchase", BUTTON_PURCHASE);
    addButton("IAPV4.restore", BUTTON_RESTORE);
    
    // Subscription
    addButton("IAPV4.getSubscriptionProductInfo", BUTTON_GET_SUBSCRIPTION_PRODUCT_INFO);
    addButton("IAPV4.purchaseSubscriptionUpdate", BUTTON_PURCHASE_SUBSCRIPTION);
    addButton("IAPV4.restoreSubscription", BUTTON_RESTORE_SUBSCRIPTION);
    
    addButton("IAPV4.checkPromotePurchase", BUTTON_CHECK_PROMOTE_PURCHASE);
    
    if(!useIAPV4Postbox()) {
        addButton("IAPV4.transactionFinish" ,BUTTON_TRANSACTION_FINISH);
        addButton("IAPV4.transactionMultiFinish" ,BUTTON_TRANSACTIONMULTI_FINISH);
    } else {
        addButton("Show Postbox", BUTTON_SHOW_POSTBOX);
    }
    
    setUseBackButton(true, "- IAPV4 - ");
    
    return true;
}

void IAPV4Test::logHead(const char* funcName){
    ResultUtils::logHead("IAPV4", funcName);
}
void IAPV4Test::logHead(const char* funcName,hive::ResultAPI const & result){
    ResultUtils::logHead("IAPV4", funcName, result);
}

static std::string IAPV4TypeToString(IAPV4Type const type){
    std::string typeString;
    switch(type){
        case IAPV4Type::NotSelected:
            typeString = "NotSelected";
            break;
        case IAPV4Type::AppleAppStore:
            typeString = "Apple";
            break;
        case IAPV4Type::GooglePlayStore:
            typeString = "Google";
            break;
        case IAPV4Type::HIVELebi:
            typeString = "Lebi";
            break;
        case IAPV4Type::OneStore :
            typeString = "OneStore";
            break;
        case IAPV4Type::AmazonAppStore :
            typeString = "AMAZON_APPSTORE";
            break;
        case IAPV4Type::SamsungGalaxyStore :
            typeString = "SAMSUNG_GALAXYSTORE";
            break;
        case IAPV4Type::HuaweiAppGallery :
            typeString = "HUAWEI_APPGALLERY";
            break;
        default:
            typeString = "None";
            break;
    }
    return typeString;
}

static std::string IAPV4ProductToString(IAPV4Product const & product) {
    
    std::string LoggerString = cocos2d::StringUtils::format("marketPid: %s, currency: %s, price: %f, title: %s, displayPrice: %s, productDescription: %s",
                                                            product.marketPid.c_str()
                                                            ,product.currency.c_str()
                                                            ,product.price
                                                            ,product.title.c_str()
                                                            ,product.displayPrice.c_str()
                                                            ,product.productDescription.c_str());
    return LoggerString;
}

static std::string getReceiptToString(IAPV4Receipt const & receipt) {
    std::string logString = "{";
    
    logString += "\ntype : "+IAPV4TypeToString(receipt.type);
    logString += "\nproduct : "+IAPV4ProductToString(receipt.product);
#if (CC_TARGET_PLATFORM == CC_PLATFORM_IOS)
    // #GCPHQ-3511 영수증 데이터가 너무 길어 로그로 표현시 크래시 발생 가능성 존재
    logString += "\nreceipt : "+receipt.hiveiapReceipt.substr(0, IAPV4Test::iOS_RECEIPT_LENGTH_LIMIT) + "...";
#else
    logString += "\nreceipt : "+receipt.hiveiapReceipt;
#endif
    logString += "\nadditionalInfo : "+receipt.additionalInfo;
    
    //필요할경우 케스팅하여 사용합니다.
    switch(receipt.type){
        case IAPV4Type::AppleAppStore:{
            //애플 객체에는 추가로 주는 정보가 없습니다.
            IAPV4ReceiptApple const & castReceipt = dynamic_cast<IAPV4ReceiptApple const &>(receipt);
            break;
        }
        case IAPV4Type::GooglePlayStore:{
            IAPV4ReceiptGoogle const & castReceipt = dynamic_cast<IAPV4ReceiptGoogle const &>(receipt);
            logString += "\npurchaseData : "+castReceipt.purchaseData;
            
            logString += "\nsignature : "+castReceipt.signature;
            logString += "\nitemType : "+castReceipt.itemType;
            logString += "\norderId : "+castReceipt.orderId;
            logString += "\npackageName : "+castReceipt.packageName;
            logString += "\nsku : "+castReceipt.sku;
            logString += "\ndeveloperPayload : "+castReceipt.developerPayload;
            logString += "\ntoken : "+castReceipt.token;
            logString += "\nvid : "+castReceipt.vid;
            logString += "\ngameCurrency : "+castReceipt.gameCurrency;
            logString += cocos2d::StringUtils::format("\ngamePrice : %f",castReceipt.gamePrice);
            logString += "\nserverId : "+castReceipt.serverId;
            break;
        }
        case IAPV4Type::HIVELebi:{
            IAPV4ReceiptLebi const & castReceipt = dynamic_cast<IAPV4ReceiptLebi const &>(receipt);
            logString += cocos2d::StringUtils::format("\ntradeno : %lld",castReceipt.tradeno);
            logString += cocos2d::StringUtils::format("\nuid : %lld",castReceipt.uid);
            logString += cocos2d::StringUtils::format("\nvid : %lld",castReceipt.vid);
            logString += "\nbillItemId"+castReceipt.billItemId;
            logString += "\ngameName"+castReceipt.gameName;
            logString += "\nappId"+castReceipt.appId;
            logString += cocos2d::StringUtils::format("\ntradeMoney : %d",castReceipt.tradeMoney);
            logString += "\ntradeDate"+castReceipt.tradeDate;
            logString += "\nitemName"+castReceipt.itemName;
            logString += "\noriginalJson"+castReceipt.originalJson;
            break;
        }
        case IAPV4Type::OneStore:{
            IAPV4ReceiptOneStore const & castReceipt = dynamic_cast<IAPV4ReceiptOneStore const &>(receipt);
            logString+="\npurchaseData : " + castReceipt.purchaseData;
            logString+="\nsignature : " + castReceipt.signature;
            logString+="\norderId : " + castReceipt.orderId;
            logString+="\npackageName : " + castReceipt.packageName;
            logString+="\nproductId : " + castReceipt.productId;
            cocos2d::StringUtils::format("\npurchaseTime : %ld",castReceipt.purchaseTime);
            logString+="\nacknowledgeState : " + castReceipt.acknowledgeState;
            logString+="\npurchaseToken : " + castReceipt.purchaseToken;
            logString+="\nbillingKey : " + castReceipt.billingKey;
            cocos2d::StringUtils::format("\npurchaseState : %d",castReceipt.purchaseState);
            cocos2d::StringUtils::format("\nrecurringState : %d",castReceipt.recurringState);
            logString+="\nstoreCode : " + castReceipt.storeCode;
            break;
        }
        case IAPV4Type::AmazonAppStore:{
            IAPV4ReceiptAmazon const & castReceipt = dynamic_cast<IAPV4ReceiptAmazon const &>(receipt);
            logString+="\nuserId : " + castReceipt.userId;
            logString+="\nmarketPlace : " + castReceipt.marketPlace;
            logString+="\nreceiptId : " + castReceipt.receiptId;
            logString+="\nsku : " + castReceipt.sku;
            logString+="\nproductType : " + castReceipt.productType;
            logString+="\npurchaseDate : " + castReceipt.purchaseDate;
            logString+="\ncancelDate : " + castReceipt.cancelDate;
            break;
        }
        case IAPV4Type::SamsungGalaxyStore:{
            IAPV4ReceiptSamsung const & castReceipt = dynamic_cast<IAPV4ReceiptSamsung const &>(receipt);
            logString+="\npurchaseData : " + castReceipt.purchaseData;
            break;
        }
        case IAPV4Type::HuaweiAppGallery:{
            IAPV4ReceiptHuawei const & castReceipt = dynamic_cast<IAPV4ReceiptHuawei const &>(receipt);
            logString+="\npurchaseData : " + castReceipt.purchaseData;
            logString+="\nsignature : " + castReceipt.signature;
            logString+="\norderId : " + castReceipt.orderId;
            logString+="\nproductId : " + castReceipt.productId;
            logString+="\npriceType : " + castReceipt.priceType;
            break;
        }
    }
    
    return logString + "\n}";
}

void IAPV4Test::receiptVerify(std::vector<hive::IAPV4Receipt> vec,int startIndex,Postbox::ItemType itemType, std::function<void()> callback) {
    if (vec.size() > startIndex) {
        const hive::IAPV4Receipt& receipt = vec[startIndex];
        Postbox::verifyReceipt(receipt.bypassInfo,itemType , [=](const Postbox::VerifyReceipt *data,Postbox::Error error)mutable{
            if(data != nullptr) {
                //영수증 검증 성공, 피니쉬 처리.
                IAPV4::transactionFinish(receipt.product.marketPid,[=](ResultAPI const & result,std::string marketPid)mutable{
                    logHead("transactionFinish", result);
                    Logger::log("marketPid  : %s",marketPid.c_str());
                    receiptVerify(vec, startIndex+1,itemType,callback);
                });
            } else {
                Logger::log("receiptVerify fail marketPid : %s",receipt.product.marketPid.c_str());
                receiptVerify(vec, startIndex+1,itemType,callback);
            }
        });
    } else {
        if(callback)
            callback();
    }
}

void IAPV4Test::receiptVerify(hive::IAPV4Receipt receipt,Postbox::ItemType itemType,std::function<void()> callback) {
    std::vector<hive::IAPV4Receipt> vec;
    vec.push_back(receipt);
    receiptVerify(vec, 0,itemType, callback);
}

void IAPV4Test::onEnter() {
    BaseView::onEnter();
    //게임서버 로그인시킨다.
    StartActivityIndicator();
    Postbox::login([=](const Postbox::Login *login, Postbox::Error loginError){
        EndActivityIndicator();
        if (login == nullptr) {
            ShowPostboxErrorMessage(loginError);
        }
    });
}

static void showLogIAPV4MarketConnectResult(ResultAPI const & result,std::vector<IAPV4Type> const & marketIDs) {
    if(ResultUtils::errorCheckResultAPI(result)){
        std::string logString = "";
        logString = "IAPV4Types = {\n";
        
        for(auto it : marketIDs){
            logString += IAPV4TypeToString(it)+"\n";
        }
        logString += "}\n";
        Logger::log(logString.c_str());
        cocos2d::MessageBox(logString.c_str(),"marketConnect");
    }
}

// for string delimiter
vector<string> split (string s, string delimiter) {
    size_t pos_start = 0, pos_end, delim_len = delimiter.length();
    string token;
    vector<string> res;

    while ((pos_end = s.find (delimiter, pos_start)) != string::npos) {
        token = s.substr (pos_start, pos_end - pos_start);
        pos_start = pos_end + delim_len;
        res.push_back (token);
    }

    res.push_back (s.substr (pos_start));
    return res;
}

void IAPV4Test::onButtonEvent(int buttonTag) {
    
    switch(buttonTag){
#if (CC_TARGET_PLATFORM == CC_PLATFORM_IOS)
        case BUTTON_FINISH_ALL_TRANSACTIONS: {
            int result = finishAllTransactions();

            std::string logString = cocos2d::StringUtils::format("%d transactions finished.", result);
            cocos2d::MessageBox(logString.c_str(), "finishAllTransactions (iOS Only)");
        }
            break;
#endif
        case BUTTON_MARKET_CONNECT:{
            IAPV4::marketConnect([=](ResultAPI const & result,std::vector<IAPV4Type> const & marketIDs){
                logHead("marketConnect", result);
                showLogIAPV4MarketConnectResult(result,marketIDs);
            });
        }
            break;
        case BUTTON_GET_MARKET_PRODUCT_INFO:{
            string marketPidListString = getEditboxValue(EDITBOX_MARKET_PID_LIST);

            vector<string> marketPidList = split(marketPidListString, ",");
            
            IAPV4::getMarketProductInfo(marketPidList, [=](ResultAPI const & result,std::vector<IAPV4Product> const & productInfoList,unsigned int balance){
                logHead("getMarketProductInfo", result);
                if(ResultUtils::errorCheckResultAPI(result)){
                    pidList.clear();
                    productCache.clear();
                    productCache.assign(productInfoList.begin(),productInfoList.end());
                    std::string logString = "productInfoList = {";
                    for(auto it : productInfoList){
                        logString += IAPV4ProductToString(it) +"\n";
                        pidList.push_back(it.marketPid);
                    }
                    logString += "}\n";
                    logString = cocos2d::StringUtils::format("%sbalance : %d\n",logString.c_str(),balance);
                    
                    Logger::log(logString.c_str());
                    cocos2d::MessageBox(logString.c_str(),"getMarketProductInfo");
                }
            });
        }
            break;
        case BUTTON_GET_PRODUCT_INFO:{
            IAPV4::getProductInfo([=](ResultAPI const & result,std::vector<IAPV4Product> const & productInfoList,unsigned int balance){
                logHead("getProductInfo", result);
                if(ResultUtils::errorCheckResultAPI(result)){
                    pidList.clear();
                    productCache.clear();
                    productCache.assign(productInfoList.begin(),productInfoList.end());
                    std::string logString = "productInfoList = {";
                    for(auto it : productInfoList){
                        logString += IAPV4ProductToString(it) +"\n";
                        pidList.push_back(it.marketPid);
                    }
                    logString += "}\n";
                    logString = cocos2d::StringUtils::format("%sbalance : %d\n",logString.c_str(),balance);
                    
                    Logger::log(logString.c_str());
                    cocos2d::MessageBox(logString.c_str(),"getProductInfo");
                }
            });
        }
            break;
        case BUTTON_PURCHASE: {
            
            pushView(Combobox::create("Select Item : ", {screenSize.width, DEFAULT_CONTENT_HEIGHT}, std::move(pidList), true, [=](const char *selectItem,int index){
                IAPV4::purchase(pidList[index], getEditboxValue(EDITBOX_ADDITIONAL_INFO), [=](ResultAPI const & result,IAPV4Receipt const & receipt){
                    logHead("purchase", result);
                    if(ResultUtils::errorCheckResultAPI(result)){
                        Logger::log(getReceiptToString(receipt).c_str());
                        
                        if(useIAPV4Postbox()){
                            StartActivityIndicator();
                            receiptVerify(receipt,Postbox::consumable,[=]() {
                                EndActivityIndicator();
                            });
                        }
                    }
                });
            }));
            
        }
            break;
        case BUTTON_RESTORE: {
            IAPV4::restore([=](ResultAPI const & result,std::vector<std::reference_wrapper<IAPV4Receipt>> receiptList){
                logHead("restore", result);
                if(ResultUtils::errorCheckResultAPI(result)){
                    std::vector<std::string> pidList;
                    std::string logString = "receiptList = {";
                    for(auto it : receiptList){
                        logString += getReceiptToString(it)+"\n";
                        pidList.push_back(it.get().product.marketPid);
                    }
                    logString += "}\n";
                    Logger::log(logString.c_str());
                    cocos2d::MessageBox(logString.c_str(),"restore");
                    
                    if(useIAPV4Postbox()) {
                        std::vector<IAPV4Receipt> receipts;
                        receipts.assign(receiptList.begin(), receiptList.end());
                        StartActivityIndicator();
                        receiptVerify(receipts,0,Postbox::consumable,[=](){
                            EndActivityIndicator();
                        });
                    }
                }
            });
            break;
        }
            
        case BUTTON_GET_SUBSCRIPTION_PRODUCT_INFO:{
            IAPV4::getSubscriptionProductInfo([=](ResultAPI const & result,std::vector<IAPV4Product> const & productInfoList,unsigned int balance){
                logHead("getSubscriptionProductInfo", result);
                if(ResultUtils::errorCheckResultAPI(result)){
                    pidList.clear();
                    productCache.clear();
                    productCache.assign(productInfoList.begin(),productInfoList.end());
                    std::string logString = "productInfoList = {";
                    for(auto it : productInfoList){
                        logString += IAPV4ProductToString(it) +"\n";
                        pidList.push_back(it.marketPid);
                    }
                    logString += "}\n";
                    logString = cocos2d::StringUtils::format("%sbalance : %d\n",logString.c_str(),balance);
                    
                    Logger::log(logString.c_str());
                    cocos2d::MessageBox(logString.c_str(),"getSubscriptionProductInfo");
                }
            });
        }
            break;
        case BUTTON_PURCHASE_SUBSCRIPTION: {

            pushView(IAPV4SubscriptionProductInfoView::create(pidList));
            
        }
            break;
        case BUTTON_RESTORE_SUBSCRIPTION: {
            IAPV4::restoreSubscription([=](ResultAPI const & result,std::vector<std::reference_wrapper<IAPV4Receipt>> receiptList){
                logHead("restoreSubscription", result);
                if(ResultUtils::errorCheckResultAPI(result)){
                    std::vector<std::string> pidList;
                    std::string logString = "receiptList = {";
                    for(auto it : receiptList){
                        logString += getReceiptToString(it)+"\n";
                        pidList.push_back(it.get().product.marketPid);
                    }
                    logString += "}\n";
                    Logger::log(logString.c_str());
                    cocos2d::MessageBox(logString.c_str(),"restoreSubscription");
                    
                    if(useIAPV4Postbox()) {
                        std::vector<IAPV4Receipt> receipts;
                        receipts.assign(receiptList.begin(), receiptList.end());
                        StartActivityIndicator();
                        receiptVerify(receipts,0,Postbox::subscription,[=](){
                            EndActivityIndicator();
                        });
                    }
                }
            });
            break;
        }
            
        case BUTTON_TRANSACTION_FINISH:{
            pushView(Combobox::create("Select Finish Pid", {screenSize.width, DEFAULT_CONTENT_HEIGHT}, std::move(pidList), true, [=](const char *selectItem,int index){
                IAPV4::transactionFinish(pidList[index],[=](ResultAPI const & result,std::string marketPid){
                    logHead("transactionFinish", result);
                    Logger::log("marketPid : %s",marketPid.c_str());
                    showResult(result);
                });
            }));
        }
            break;
        case BUTTON_TRANSACTIONMULTI_FINISH:{
            pushView(MultiSelect::create("Select Finish Pids",std::move(pidList),[=](MultiSelect* select,const std::vector<int>& selectedIndexes){
                std::vector<std::string> multiPid;
                for(auto it : selectedIndexes){
                    multiPid.push_back(pidList[it]);
                }
                IAPV4::transactionMultiFinish(multiPid, [=](std::vector<ResultAPI> const & resultList,std::vector<std::string> const & marketPidList){
                    logHead("transactionMultiFinish",resultList[0]);
                    if(resultList.size() > 0 && resultList[0].isSuccess()) {
                        Logger::log("resultList count : %d, marketPidList count : %d",resultList.size(),marketPidList.size());
                        std::string logString = "";
                        for(int i=0;i<resultList.size();++i){
                            logString += "marketPid : " + marketPidList[i] + "\n";
                            logString += ResultUtils::getResultAPIString(resultList[i]) + "\n";
                        }
                        Logger::log(logString.c_str());
                    }
                    
                });
            }));
        }
            break;
        case BUTTON_MARKET_SELECT: {
            IAPV4::showMarketSelection([=](ResultAPI const & result,std::vector<IAPV4Type> const & marketIDs){
                logHead("showMarketSelection", result);
                showLogIAPV4MarketConnectResult(result,marketIDs);
            });
        }
            break;
        case BUTTON_BALANCE_CHARGE: {
            IAPV4::showCharge([=](ResultAPI const & result, int balance){
                logHead("showCharge", result);
                Logger::log("balance : %d",balance);
                showResult(result);
            });
        }
            break;
        case BUTTON_BALANCE_INFO:{
            IAPV4::getBalanceInfo([=](ResultAPI const & result, int balance){
                logHead("getBalanceInfo", result);
                Logger::log("balance : %d",balance);
                showResult(result);
            });
        }
            break;
        case BUTTON_CHECK_PROMOTE_PURCHASE:{
            IAPV4::checkPromotePurchase([=](ResultAPI const & result,std::string marketPid){
                logHead("checkPromotePurchase", result);
                Logger::log("marketPid : %s",marketPid.c_str());
                showResult(result);
            });
        }
            break;
        case BUTTON_SHOW_POSTBOX: {
            pushView(PostBoxView::create());
        }
            break;
        case BUTTON_GET_SELECTED_MARKET: {
            IAPV4Type selectedMarket = IAPV4::getSelectedMarket();
            logHead("getSelectedMarket");
            std::string logString = "selectedMarket : " + IAPV4TypeToString(selectedMarket);
            Logger::log(logString.c_str());
        }
        case BUTTON_GET_ACCOUNT_UUID: {
            std::string accountUuid = IAPV4::getAccountUuid();
            logHead("getAccountUuid");
            std::string logString = "accountUuid : " + accountUuid;
            Logger::log(logString.c_str());
        }
    }
}

void IAPV4Test::showResult(hive::ResultAPI const & api) {
    scheduleOnce([=](float dt){
        ResultUtils::showResult(api);
    }, 0.5f, "showResult");
    
}
