//
//  PromotionTest.cpp
//  hivesdk
//
//  Created by paikjongman on 2017. 2. 28..
//
//

#include "PromotionTest.hpp"
#include "AppInvitationInfo.hpp"
#include "ResultUtils.hpp"
#include "PromotionBannerView.hpp"
#include "Configuration.hpp"
#include <sstream>

using namespace hive;

const int BANNER_TYPEINDEX = 0;
const int NEWS_TYPEINDEX = 1;
const int NOTICE_TYPEINDEX = 2;
const int BANNERLEGACY_TYPEINDEX = 3;

const std::vector<const char*> promotionTypeLists = {"BANNER","NEWS","NOTICE","BANNERLEGACY"};
const std::vector<const char*> customTypeList = {"VIEW","BOARD","SPOT","DIRECT"};
const std::vector<const char*> offersStateList = {"ENABLED","DISABLED","UNKNOWN"};
const std::vector<const char*> campaignTypeLists = {"EVENT", "NOTICE", "ALL", "CROSS"};
const std::vector<const char*> bannerTypeLists = {"GREAT", "SMALL", "ROLLING"};

std::string inviteHIVEMessage = "Invite you";
std::string inviteHIVELink = "http://m.withhive.com";

PromotionType strToPromotionType(const char *typeStr) {
    if (strcmp(typeStr, promotionTypeLists[BANNER_TYPEINDEX]) == 0) {
        return PromotionType::BANNER;
    } else if (strcmp(typeStr,promotionTypeLists[NEWS_TYPEINDEX]) == 0) {
        return PromotionType::NEWS;
    } else if (strcmp(typeStr, promotionTypeLists[NOTICE_TYPEINDEX]) == 0) {
        return PromotionType::NOTICE;
    }
    return PromotionType::BANNERLegacy;
}

bool PromotionTest::init()
{
    cocos2d::UserDefault* instance = cocos2d::UserDefault::getInstance();

    if(!BaseView::init())
        return false;
    
    setUseShowLogButton(true);
    Logger::setLoggerListener([=](std::string tag, std::string level, std::string logMessage){
        addLogMessage(logMessage);
    });

    addCombobox("PromotionType : ", std::move(promotionTypeLists), 0, COMBOBOX_PROMOTION_TYPE);
    addTrueFalseToggleButton("Promotion Forece Type : ", 1, {screenSize.width, DEFAULT_CONTENT_HEIGHT}, TOGGLE_BUTTON_PROMOTION_FORCE);
    addButton("Promotion.showPromotion()", BUTTON_SHOW_PROMOTION);

    addEditBox("Menu : ", "", "Enter Menu",EDITBOX_NEWS_MENU);
    addEditBox("GiftPidList : ", "", "Enter GiftPidList ex) 1, 2, 3, 4 ...", EDITBOX_NEWS_GIFTPIDLIST);
    addButton("Random Gift Pid", BUTTON_NEWS_GIFTPIDLIST_RANDOM);
    addButton("Promotion.showNews()", BUTTON_SHOW_NEWS);

    addCombobox("CustomType : ", std::move(customTypeList), 0, COMBOBOX_CUSTOM_TYPE);
    addEditBox("ContentsKey : ", "", "Enter ContentsKey",EDITBOX_CONTENTS_KEY);
    addButton("Promotion.showCustomContents()", BUTTON_SHOW_CUSTOM_CONTENTS);

    addButton("Promotion.showCustomContentsOnGameWindow()", BUTTON_SHOW_CUSTOM_CONTENTS_ON_GAME_WINDOW);

    BeginHorizontal();
    addButton("Promotion.getOfferwalState()", {screenSize.width/2, DEFAULT_CONTENT_HEIGHT}, BUTTON_GET_OFFERWALL_STATE);
    addButton("Promotion.showOfferwall()", {screenSize.width/2, DEFAULT_CONTENT_HEIGHT}, BUTTON_SHOW_OFFERWALL);
    EndHorizontal();
    addButton("Promotion.showNativeReview()", BUTTON_SHOW_NATIVE_REVIEW);
    addButton("Promotion.showReview()", BUTTON_SHOW_REVIEW);

    if( cocos2d::Application::getInstance()->getTargetPlatform() == cocos2d::Application::Platform::OS_ANDROID)
        addButton("Promotion.showExit", BUTTON_SHOW_EXIT);//Android Only

    addButton("Promotion.getViewInfo()", BUTTON_GET_VIEW_INFO);
    addButton("Promotion.getBadgeInfo()", BUTTON_GET_BADGE_INFO);

    addCombobox("CampaignType : ", std::move(campaignTypeLists), 0, COMBOBOX_CAMPAIGN_TYPE);
    addCombobox("BannerType : ", std::move(bannerTypeLists), 0, COMBOBOX_BANNER_TYPE);
    addButton("Promotion.getBannerInfo()", BUTTON_GET_BANNER_INFO);

    BeginHorizontal();
    addEditBox("CampaignType: ", "event", "", {screenSize.width/2, DEFAULT_CONTENT_HEIGHT}, cocos2d::ui::EditBox::InputMode::SINGLE_LINE, 20, EDITBOX_CAMPAIGN_TYPE);
    addEditBox("BannerType: ", "rolling", "", {screenSize.width/2, DEFAULT_CONTENT_HEIGHT}, cocos2d::ui::EditBox::InputMode::SINGLE_LINE, 20, EDITBOX_BANNER_TYPE);
    EndHorizontal();
    addButton("Promotion.getBannerInfoString()", BUTTON_GET_BANNER_INFO_STRING);

    addButton("Promotion.getAppInvitationData()", BUTTON_GET_APP_INVITATION_DATA);

    // \\ -> '\', \" -> '"'
    addEditBox("AdditionalInfo : ", "{\\\"complete\\\":[202533,202514,202498]}", "", EDITBOX_ADDITIONAL_INFO);
    addButton("Promotion.setAdditionalInfo", BUTTON_SET_ADDITIONAL_INFO);
    
    addButton("Promotion.updatePromotionData()",BUTTON_UPDATE_PROMOTIN_DATA);
    
    std::string gameLanguage = instance->getStringForKey(keyLanguage, "en");
    addEditBox("Game Language : ", gameLanguage.c_str(), "", EDITBOX_UPDATE_GAME_LANGUAGE);
    addButton("Promotion.updateGameLanguage()" ,BUTTON_UPDATE_GAME_LANGUAGE);
    
    std::string serverId = instance->getStringForKey(keyServerId, "kr");
    addEditBox("Game Server ID : ",serverId.c_str(), "", EDITBOX_UPDATE_SERVER_ID);
    addButton("Promotion.updateServerID()",BUTTON_UPDATE_SERVER_ID);

    BeginHorizontal();
    addButton("Random Giftbox", {screenSize.width/2, DEFAULT_CONTENT_HEIGHT}, BUTTON_RANDOM_ADDITIONAL_INFO);
    addButton("Reset AdditionalInfo", {screenSize.width/2, DEFAULT_CONTENT_HEIGHT}, BUTTON_RESET_ADDITIONAL_INFO);
    EndHorizontal();

    addButton("Promotion.showUAShare()", BUTTON_SHOW_UA_SHARE);

    addEditBox("uri : ", "interwork://hive/socialinquiry", "Enter EngagementURI", EDITBOX_USER_ENGAGEMENT_URI);
    addButton("Promotion.processURI()", BUTTON_PROCESS_URI);

    addButton("Promotion.getAppInvitationSenderInfo()", BUTTON_GET_APP_INVITATION_SENDER_INFO);
    
    setUseBackButton(true, "- Promotion -");
    
    return true;
}

void PromotionTest::onButtonEvent(int buttonTag)
{
    cocos2d::UserDefault* instance = cocos2d::UserDefault::getInstance();
    switch (buttonTag) {
        case BUTTON_SHOW_PROMOTION:
        {
            int index = getSelectedComboboxIndex(COMBOBOX_PROMOTION_TYPE);

            PromotionType currentPromotionType = strToPromotionType(promotionTypeLists[index]);
            bool currentIsForced = getSelectedToggleButtonIndex(TOGGLE_BUTTON_PROMOTION_FORCE);
            
            Logger::log("\n\n=== 이벤트, 프로모션, 공지 화면 노출. ===\n");
            Logger::log("Promotion::showPromotion(PromotionType = %s, isForced = %s, onPromotionView) Called\n",
                        promotionTypeLists[index],
                        currentIsForced ? "true" : "false");
            
            Promotion::showPromotion(currentPromotionType, currentIsForced,
                                     std::bind(&PromotionTest::onPromotionView, this, std::placeholders::_1, std::placeholders::_2));
            break;
        }
        case BUTTON_NEWS_GIFTPIDLIST_RANDOM:
        {
            Promotion::getBannerInfo(PromotionCampaignType::ALL, PromotionBannerType::SMALL, [&](ResultAPI const& result, std::vector<PromotionBannerInfo> const& list) {
                std::random_device rd;
                std::mt19937 eng(rd());
                std::uniform_int_distribution<> distr_size(0, static_cast<int>(list.size()) - 1);
                int pid = list[distr_size(eng)].pid;
                std::string pidString = std::to_string(pid);
                
                setEditboxValue(EDITBOX_NEWS_GIFTPIDLIST, pidString);
                
                Logger::log("\n\n=== Gift Pid List 랜덤 선택 ===\n");
                Logger::log("giftPidList Random Select One: %s Called\n", pidString.c_str());
            });
            
            break;
        }
        case BUTTON_SHOW_NEWS:
        {
            std::string menu = getEditboxValue(EDITBOX_NEWS_MENU);
            
            std::string giftPidListString = getEditboxValue(EDITBOX_NEWS_GIFTPIDLIST);
            stringstream ss(giftPidListString);
            
            std::vector<int> giftPidList;
            int giftPid;
            
            while(ss >> giftPid)
            {
                giftPidList.push_back(giftPid);
                if (ss.peek() == ',') ss.ignore();
            }

            Logger::log("\n\n=== 새소식 특정 페이지 노출 ===\n");
            Logger::log("Promotion::showNews(menu = %s, giftPidList = %s, onPromotionView) Called\n", menu.c_str(), giftPidListString.c_str());

            Promotion::showNews(menu, giftPidList, std::bind(&PromotionTest::onPromotionView, this, std::placeholders::_1, std::placeholders::_2));
        }
        case BUTTON_SHOW_CUSTOM_CONTENTS:
        {
            int index = getSelectedComboboxIndex(COMBOBOX_CUSTOM_TYPE);
            PromotionCustomType customType = static_cast<PromotionCustomType>(index);
            std::string contentsKey = getEditboxValue(EDITBOX_CONTENTS_KEY);
            
            Logger::log("\n\n=== Custom 뷰 화면 노출. ===\n");
            Logger::log("Promotion::showCustomView(CustomTYpe = %s, contentsKey = %s, onPromotionView) Called\n", customTypeList[index], contentsKey.c_str());
            
            Promotion::showCustomContents(customType, contentsKey,
                                      std::bind(&PromotionTest::onPromotionView, this, std::placeholders::_1, std::placeholders::_2));
            break;
        }
        case BUTTON_SHOW_CUSTOM_CONTENTS_ON_GAME_WINDOW:
        {
            int index = getSelectedComboboxIndex(COMBOBOX_CUSTOM_TYPE);
            PromotionCustomType customType = static_cast<PromotionCustomType>(index);
            std::string contentsKey = getEditboxValue(EDITBOX_CONTENTS_KEY);

            Logger::log("\n\n=== Custom 뷰 화면 노출. (OnGameWindow) ===\n");
            Logger::log("Promotion::showCustomContentsOnGameWindow(CustomTYpe = %s, contentsKey = %s, onPromotionView) Called\n", customTypeList[index], contentsKey.c_str());

            Promotion::showCustomContentsOnGameWindow(customType, contentsKey,
                                          std::bind(&PromotionTest::onPromotionView, this, std::placeholders::_1, std::placeholders::_2));
            break;
        }
        case BUTTON_GET_OFFERWALL_STATE:
        {
            OfferwallState state = Promotion::getOfferwallState();
            Logger::log("\n\n=== 게임내 오퍼월(무료 충전소) 노출 가능 여부 조회 (ENABLE, DISABLED, UNKONOWN) ===\n");
            Logger::log("Promotion::getOfferwallState(), state = %s", offersStateList[static_cast<int>(state)]);
            break;
        }
        case BUTTON_SHOW_OFFERWALL:
        {
            Logger::log("\n\n=== 게임내 오퍼월(무료 충전소) 화면 노출. (Android only) ===\n");
            Logger::log("Promotion::showOfferwall(onPromotionView) Called\n");
            
            Promotion::showOfferwall(std::bind(&PromotionTest::onPromotionView, this, std::placeholders::_1, std::placeholders::_2));
            break;
        }
        case BUTTON_SHOW_NATIVE_REVIEW:
        {
            Logger::log("\n\n=== 네이티브 리뷰 팝업 대화 상자 노출. ===\n");
            Logger::log("Promotion::showNativeReview(onPromotionView) Called\n");
            
            Promotion::showNativeReview(std::bind(&PromotionTest::onPromotionView, this, std::placeholders::_1, std::placeholders::_2));
            break;
        }
        case BUTTON_SHOW_REVIEW:
        {
            Logger::log("\n\n=== Hive 리뷰 팝업 노출. ===\n");
            Logger::log("Promotion::showReview(onPromotionView) Called\n");

            Promotion::showReview(std::bind(&PromotionTest::onPromotionView, this, std::placeholders::_1, std::placeholders::_2));
            break;
        }
        case BUTTON_SHOW_EXIT:
        {
            Logger::log("\n\n=== 게임 종료 팝업 대화 상자 노출. ===\n");
            Logger::log("Promotion::showExit(onPromotionView) Called\n");
            
            Promotion::showExit(std::bind(&PromotionTest::onPromotionView, this, std::placeholders::_1, std::placeholders::_2));
            break;
        }
        case BUTTON_GET_VIEW_INFO:
        {
            int index = getSelectedComboboxIndex(COMBOBOX_CUSTOM_TYPE);
            PromotionCustomType customType = static_cast<PromotionCustomType>(index);
            std::string contentsKey = getEditboxValue(EDITBOX_CONTENTS_KEY);
            
            Logger::log("\n\n=== viewId에 해당하는 프로모션 컨텐츠 조회. ===\n");
            Logger::log("Promotion::getViewInfo(CustomTYpe = %s, contentsKey = %s, onPromotionView) Called\n",customTypeList[index], contentsKey.c_str());
            
            Promotion::getViewInfo(customType, contentsKey, std::bind(&PromotionTest::onPromotionViewInfo, this, std::placeholders::_1, std::placeholders::_2));
            break;
        }
        case BUTTON_GET_BADGE_INFO:
        {
            Logger::log("\n\n=== 게임내 프로모션 버튼의 부각을 위한 정보 (Badge) 조회 ===\n");
            Logger::log("Promotion::getBadgeInfo(onPromotionView) Called\n");
            
            Promotion::getBadgeInfo(std::bind(&PromotionTest::onPromotionBadgeInfo, this, std::placeholders::_1, std::placeholders::_2));
            break;
        }
        case BUTTON_GET_BANNER_INFO:
        {
            int campaignTypeIndex = getSelectedComboboxIndex(COMBOBOX_CAMPAIGN_TYPE);
            int bannerTypeIndex = getSelectedComboboxIndex(COMBOBOX_BANNER_TYPE);

            PromotionCampaignType campaignType = static_cast<PromotionCampaignType>(campaignTypeIndex);
            PromotionBannerType bannerType = static_cast<PromotionBannerType>(bannerTypeIndex);

            Logger::log("\n\n=== 게임내 롤링 프로모션 배너 정보 조회 ===\n");
            Logger::log("Promotion::getBannerInfo(campaignType = %s, bannerType = %s, onPromotionBadgeInfo) Called\n", campaignTypeLists[campaignTypeIndex], bannerTypeLists[bannerTypeIndex]);

            Promotion::getBannerInfo(campaignType, bannerType, std::bind(&PromotionTest::onPromotionBannerInfo, this, std::placeholders::_1, std::placeholders::_2));
            break;
        }
        case BUTTON_GET_BANNER_INFO_STRING:
        {
            std::string campaignType = getEditboxValue(EDITBOX_CAMPAIGN_TYPE);
            std::string bannerType = getEditboxValue(EDITBOX_BANNER_TYPE);

            Logger::log("\n\n=== 프로모션 배너 정보 조회 ===\n");
            Logger::log("Promotion::getBannerInfoString(campaignType = %s, bannerType = %s, onPromotionBadgeInfo) Called\n", campaignType.c_str(), bannerType.c_str());
            Promotion::getBannerInfoString(campaignType, bannerType, std::bind(&PromotionTest::onPromotionBannerInfo, this, std::placeholders::_1, std::placeholders::_2));
            break;
        }
        case BUTTON_GET_APP_INVITATION_DATA:
        {
            Logger::log("\n\n=== 앱 초대를 위한 데이터 조회 (qrCode, inviteLink) ===\n");
            Logger::log("Promotion::getAppInvitationData(onPromotionView) Called\n");
            
            Promotion::getAppInvitationData(std::bind(&PromotionTest::onAppInvitationData, this, std::placeholders::_1, std::placeholders::_2));
            break;
        }
		case BUTTON_SHOW_UA_SHARE:
		{
			Logger::log("\n\n=== 앱 초대를 공유 기능 수행  ===\n");
			Logger::log("Promotion::showUAShare(onUAShare) Called\n");
			// TODO : UA 공유 기능 작업 진행
			
            std::string message = this->invitationData.inviteMessage;
            std::string link = this->invitationData.inviteHivemsgLink;
			Promotion::showUAShare(message, link,std::bind(&PromotionTest::onUAShare, this, std::placeholders::_1));
		}
        case BUTTON_SET_ADDITIONAL_INFO:
        {
            std::string additionalInfo = getEditboxValue(EDITBOX_ADDITIONAL_INFO);
            
            Logger::log("\n\n=== GMS 등을 통해 별도의 가공 없이 게임 서버 API 또는 게임 서버의 중계용 API 등 그에 준하는 API에 전달될 값 설정 ===\n");
            Logger::log("Promotion::setAdditionalInfo(additionalInfo = %s) Called\n", additionalInfo.c_str());
            
            Promotion::setAdditionalInfo(additionalInfo.c_str());
            break;
        }
        case BUTTON_PROCESS_URI:
        {
            std::string uri = getEditboxValue(EDITBOX_USER_ENGAGEMENT_URI);

            bool result = Promotion::processURI(uri);

            Logger::log("\n\n=== URI 입력을 통해 UserEngagement 이벤트를 추가 ===\n");
            Logger::log("Promotion::processURI() Called\n");
            Logger::log("result = %s\n",result == true ? "true" : "false");
            break;
        }
        case BUTTON_UPDATE_PROMOTIN_DATA:
        {
            Logger::log("\n\n=== Update Promotion Data called ===\n");
            Promotion::updatePromotionData();
            break;
            
        }
        case BUTTON_UPDATE_GAME_LANGUAGE:
        {
            std::string updateGameLanguage = getEditboxValue(EDITBOX_UPDATE_GAME_LANGUAGE);
            Logger::log("\n\n=== Update Promotion Game Language called ===\n");
            
            hive::Configuration::updateGameLanguage(updateGameLanguage);
            instance->setStringForKey(keyLanguage, updateGameLanguage);
            break;
        }
        case BUTTON_UPDATE_SERVER_ID:
        {
            std::string updateServerId = getEditboxValue(EDITBOX_UPDATE_SERVER_ID);
            Logger::log("\n\n=== Update ServerID Server ID called ===\n");
            
            hive::Configuration::updateServerId(updateServerId);
            instance->setStringForKey(keyServerId, updateServerId);
            break;
        }
        case BUTTON_RESET_ADDITIONAL_INFO:
        {
            // additional Info 값 초기화
            std::string additionalInfo = setEditboxValue(EDITBOX_ADDITIONAL_INFO, "");
            
            Promotion::setAdditionalInfo(additionalInfo);
            break;
            
        }
        case BUTTON_RANDOM_ADDITIONAL_INFO:
        {
            // 전체 배너 리스트 중 랜덤하게 선물상자 노출
            int campaignTypeIndex = 2;
            int bannerTypeIndex = 1;

            PromotionCampaignType campaignType = static_cast<PromotionCampaignType>(campaignTypeIndex);
            PromotionBannerType bannerType = static_cast<PromotionBannerType>(bannerTypeIndex);
            Promotion::getBannerInfo(campaignType, bannerType,
                                     [this](ResultAPI result, std::vector<PromotionBannerInfo> bannerInfoList) {
                int selectBanner = rand()%(int)bannerInfoList.size();
                int selectedBannerPID = bannerInfoList[selectBanner].pid;
                
                std::string pids = "{\"complete\":[";
                pids.append(std::to_string(selectedBannerPID));
                pids.append("]}");

                cocos2d::Director::getInstance()->getScheduler()->performFunctionInCocosThread([=]() {
                    setEditboxValue(EDITBOX_ADDITIONAL_INFO, pids);
                });
                
                Promotion::setAdditionalInfo(pids);
            });
            break;
        }
        case BUTTON_GET_APP_INVITATION_SENDER_INFO:
        {
            Promotion::getAppInvitationSenderInfo([this](ResultAPI result, AppInvitationSenderInfo senderInfo) {
                Logger::log("\n\n=== getAppInvitationSenderInfo called ===\n");

                stringstream log;
                log << "senderInfo : " << '\n';
                log << "{" << '\n';
                if (senderInfo.vid != "") {
                    log << "vid : " << senderInfo.vid << '\n';
                }
                log << "}" << '\n';
                log << "result : " << result.toString().c_str() << '\n';

                Logger::log(log.str().c_str());
            });
            break;
        }
        default:
            break;
    }
}


// 프로모션 뷰 API 결과 통지 리스너 - Promotion::showXXX()
void PromotionTest::onPromotionView(ResultAPI result, PromotionEventType promotionEventType)
{
    std::string eventType = "";
    switch (promotionEventType) {
        case hive::PromotionEventType::OPEN:
            //프로모션 창이 열림
            eventType = "open";
            break;
        case hive::PromotionEventType::CLOSE:
            //프로모션 창이 닫힘
            eventType = "close";
            break;
        case hive::PromotionEventType::START_PLAYBACK:
            //동영상 재생 시작
            eventType = "start_playback";
            break;
        case hive::PromotionEventType::FINISH_PLAYBACK:
            //동영상 재생 끝
            eventType = "finish_playback";
            break;
        case hive::PromotionEventType::EXIT:
            eventType = "exit";
            break;
        default:
            break;
    }
    Logger::log("PromotionTestView::onPromotionView()\n %s\n Callback\nresult = %s\n",eventType.c_str(), result.toString().c_str());
}


// viewId에 해당하는 컨텐츠 조회의 결과 통지 리스너 - Promotion::getViewInfo()
void PromotionTest::onPromotionViewInfo(ResultAPI result, std::vector<PromotionViewInfo> promotionViewInfo)
{
    Logger::log("PromotionTestView::onPromotionViewInfo() Callback\nresult = %s\n", result.toString().c_str());
    std::vector<PromotionViewInfo>::iterator it;
    for( it = promotionViewInfo.begin() ; it != promotionViewInfo.end() ; ++it )
        addLogMessage((*it).toString().c_str());
}

// 프로모션 배지 표시 정보 조회에 대한 결과 통지 리스너 - Promotion::getBadgeInfo()
void PromotionTest::onPromotionBadgeInfo(ResultAPI result, std::vector<PromotionBadgeInfo> badgeInfoList)
{
    Logger::log("PromotionTestView::onPromotionBadgeInfo() Callback\nresult = %s\n", result.toString().c_str());
    
    Logger::log("Promotion::PromotionBadgeInfo List = \n");
    
    for(int i = 0; i < (int)badgeInfoList.size(); i++)
    {
        PromotionBadgeInfo promotionBadgeInfo = badgeInfoList[i];
        
        Logger::log("%s\n", promotionBadgeInfo.toString().c_str());
    }
}

// 롤링 프로모션 배너 정보 조회 리스너 - Promotion.getBannerInfo()
void PromotionTest::onPromotionBannerInfo(ResultAPI result, std::vector<PromotionBannerInfo> bannerInfoList)
{
    Logger::log("PromotionTestView::onPromotionBannerInfo() Callback\nresult = %s\n", result.toString().c_str());

    Logger::log("Promotion::PromotionBannerInfo List = \n");

    for (int i=0; i< (int)bannerInfoList.size(); i++)
    {
        PromotionBannerInfo promotionBannerInfo = bannerInfoList[i];

        Logger::log("%s\n", promotionBannerInfo.toString().c_str());
    }

    if (!result.isSuccess()) {
        return;
    }
    scheduleOnce([=](float dt){
        pushView(PromotionBannerView::create(bannerInfoList));
    }, 1, "getBannerInfo");
}


// 앱 초대 정보 요청에 대한 결과 통지 리스너 - Promotion::getAppInvitationData()
void PromotionTest::onAppInvitationData(ResultAPI result, AppInvitationData appInvitationData)
{
    Logger::log("PromotionTestView::onAppInvitationData() Callback\nresult = %s\n", result.toString().c_str());
    
    Logger::log("appInvitationData =\ninviteCommonLink = %s\ninviteHivemsgLink = %s\ninviteFacebookLink = %s\n"
                , appInvitationData.inviteCommonLink.c_str()
                , appInvitationData.inviteHivemsgLink.c_str()
                , appInvitationData.inviteFacebookLink.c_str());
    
    // 초대 정보를 글로벌 변수로 저장
    this->invitationData = appInvitationData;
    
    if(appInvitationData.eachCampaignList.size() > 0) {
        Logger::log("== appInvitationData eachCampaignList ==\n");
        for(auto it = appInvitationData.eachCampaignList.begin();it!=appInvitationData.eachCampaignList.end();++it) {
            Logger::log(ResultUtils::getAppInvitationCampaignString(*it).c_str());
            Logger::log("\n");
        }
    }
    
    if(appInvitationData.stageCampaignList.size() > 0) {
        Logger::log("== appInvitationData stageCampaignList ==\n");
        for(auto it = appInvitationData.stageCampaignList.begin();it!=appInvitationData.stageCampaignList.end();++it) {
            Logger::log(ResultUtils::getAppInvitationCampaignStageString(*it).c_str());
            Logger::log("\n");
        }
    }
    
    scheduleOnce([=](float dt){
        pushView(AppInvitaionView::create(appInvitationData));
    }, 1, "openAppInvitation");
}


// UA 공유 기능 요청에 대한 결과 통지 리스너 - Promotion::showUAShare()
void PromotionTest::onUAShare(hive::ResultAPI result)
{
	Logger::log("PromotionTestView::onUAShare() Callback\nresult = %s\n", result.toString().c_str());
	
}

// 앱 초대 정보 요청에 대한 결과 통지 리스너 - Promotion::getAppInvitationSenderInfo()
void PromotionTest::onAppInvitationSenderInfo(ResultAPI result, AppInvitationSenderInfo appInvitationSenderInfo)
{
    Logger::log("PromotionTestView::onAppInvitationSenderInfo() Callback\nresult = %s\n", result.toString().c_str());

//    Logger::log("senderVid = %s\n"
//            , appInvitationSenderInfo.vid.c_str());

}
