//
//  ActiveUserParamScene.cpp
//  HIVE_SDK_Cocos_Sample
//
//  Created by sjyong on 2021/09/29.
//

#include "ActiveUserParamScene.hpp"
#include "ParameterDataManager.hpp"
#include "ui/CocosGUI.h"

using namespace cocos2d;
using namespace cocos2d::ui;
using namespace hive;

#include <map>

using namespace::std;

bool ActiveUserParamScene::init()
{
    if(!BaseView::init())
        return false;

    // Data init
    orientationList = {
        "ALL",
        "PORTRAIT",
        "LANDSCAPE"
    };

    addButton("OK", BUTTON_OK);

    scheduleUpdate();

    return true;
}

void ActiveUserParamScene::update(float dt)
{
    BaseView::update(dt);

    static bool firstRun = true;
    if (firstRun) {
        firstRun = false;
    }
}

void ActiveUserParamScene::onButtonEvent(int buttonTag)
{
    if (buttonTag != BUTTON_OK) {
        return;
    }

    setCompletionHandler();
    this->popView();
}

void ActiveUserParamScene::showParameters(int tag)
{
    api = static_cast<C2SModuleAPIList::ActiveUserAPI>(tag);
    switch (api) {
        case C2SModuleAPIList::ACTIVEUSER_START_EX: {
            showStartExParam([=]() {
                SERVER_STATE serverState = stringToServerState(getComboboxValue(COMBOBOX_SERVER_STATE));
                ParameterDataManager::getInstance()->activeUserParams->setServerState(serverState);
            });
            break;
        }
        case C2SModuleAPIList::ACTIVEUSER_START_WITHOUT_NOTICE: {
            showStartWithoutNoticeParam([=]() {
                SERVER_STATE serverState = stringToServerState(getComboboxValue(COMBOBOX_SERVER_STATE));
                ParameterDataManager::getInstance()->activeUserParams->setServerState(serverState);

                int showNotice = getSelectedToggleButtonIndex(TOGGLE_BUTTON_SHOW_NOTICE);
                ParameterDataManager::getInstance()->activeUserParams->setShowNotice(showNotice);
            });
            break;
        }
        case C2SModuleAPIList::ACTIVEUSER_SET_VID: {
            showSetVidParam([=]() {
                string vid = getEditboxValue(EDITBOX_VID);
                ParameterDataManager::getInstance()->activeUserParams->setVid(vid);
            });
            break;
        }
        case C2SModuleAPIList::ACTIVEUSER_UPDATE_SERVER_ID: {
            showSetServerIdParam(true, [=]() {
                string serverId = getEditboxValue(EDITBOX_SERVER_ID);
                ParameterDataManager::getInstance()->activeUserParams->setServerId(serverId);
            });
            break;
        }
        case C2SModuleAPIList::ACTIVEUSER_SET_SERVER_ID: {
            showSetServerIdParam(false, [=]() {
                string serverId = getEditboxValue(EDITBOX_SERVER_ID);
                ParameterDataManager::getInstance()->activeUserParams->setServerId(serverId);
            });
            break;
        }
        case C2SModuleAPIList::ACTIVEUSER_UPDATE_GAME_LANGUAGE: {
            showSetGameLanguageParam(true, [=]() {
                string gameLanguage = getComboboxValue(COMBOBOX_GAME_LANGUAGE);
                ParameterDataManager::getInstance()->activeUserParams->setGameLanguage(gameLanguage);
            });
            break;
        }
        case C2SModuleAPIList::ACTIVEUSER_SET_GAME_LANGUAGE: {
            showSetGameLanguageParam(false, [=]() {
                string gameLanguage = getComboboxValue(COMBOBOX_GAME_LANGUAGE);
                ParameterDataManager::getInstance()->activeUserParams->setGameLanguage(gameLanguage);
            });
            break;
        }
        case C2SModuleAPIList::ACTIVEUSER_SET_MIN_TIMEOUT_SECONDS: {
            showSetMinTimeoutSecondsParam([=]() {
                int seconds = stoi(getEditboxValue(EDITBOX_MIN_TIMEOUT_SECONDS));
                ParameterDataManager::getInstance()->activeUserParams->setMinTimeoutSeconds(seconds);
            });
            break;
        }
        case C2SModuleAPIList::ACTIVEUSER_SET_LOGGED: {
            showSetLoggedParam([=]() {
                bool isLogged = bool(getSelectedToggleButtonIndex(TOGGLE_BUTTON_IS_LOGGED));
                ParameterDataManager::getInstance()->activeUserParams->setIsLogged(isLogged);
            });
            break;
        }
        case C2SModuleAPIList::ACTIVEUSER_USER_AGREE_SHOW_TERMS_UI: {
            showUserAgreeShowTermsUIExParam([=]() {
                string url = getEditboxValue(EDITBOX_URL);
                ParameterDataManager::getInstance()->activeUserParams->setUrl(url);
            });
            break;
        }
        case C2SModuleAPIList::ACTIVEUSER_USER_AGREE_SHOW_TERMS_UI_EX: {
            showUserAgreeShowTermsUIExParam([=]() {
                string url = getEditboxValue(EDITBOX_URL);
                ParameterDataManager::getInstance()->activeUserParams->setUrl(url);
            });
            break;
        }
        case C2SModuleAPIList::ACTIVEUSER_SET_PERMISSION_VIEW_STATE: {
            showSetPermissionViewStateParam([=]() {
                bool isOn = bool(getSelectedToggleButtonIndex(TOGGLE_BUTTON_IS_ON));
                ParameterDataManager::getInstance()->activeUserParams->setIsOn(isOn);
            });
            break;
        }
        case C2SModuleAPIList::ACTIVEUSER_SEND_FUNNELS_LOGS: {
            showSendFunnelsLogsParam([=]() {
                string funnelsTrack = getEditboxValue(EDITBOX_FUNNELS_TRACK);
                string optionTag = getEditboxValue(EDITBOX_OPTION_TAG);

                ParameterDataManager::getInstance()->activeUserParams->setFunnelsTrack(funnelsTrack);
                ParameterDataManager::getInstance()->activeUserParams->setOptionTag(optionTag);
            });
            break;
        }
        case C2SModuleAPIList::ACTIVEUSER_REQUEST_USER_PERMISSIONS: {
            showRequestUserPermissionsParam([=]() {
                string requests = getEditboxValue(EDITBOX_REQUESTS);
                ParameterDataManager::getInstance()->activeUserParams->setRequests(requests);
            });
            break;
        }
        case C2SModuleAPIList::ACTIVEUSER_SET_ANALYTICS_LOG_SETTINGS: {
            showSetAnalyticsLogSettingsParam([=]() {
                int sendLimit = stoi(getEditboxValue(EDITBOX_SEND_LIMIT));
                int queueLimit = stoi(getEditboxValue(EDITBOX_QUEUE_LIMIT));
                int sendCycleSecond = stoi(getEditboxValue(EDITBOX_SEND_CYCLE_SECOND));

                ParameterDataManager::getInstance()->activeUserParams->setSendLimit(sendLimit);
                ParameterDataManager::getInstance()->activeUserParams->setQueueLimit(queueLimit);
                ParameterDataManager::getInstance()->activeUserParams->setSendCycleSecond(sendCycleSecond);
            });
            break;
        }
        case C2SModuleAPIList::ACTIVEUSER_SEND_ANALYTICS_LOG: {
            showSendAnalyticsLogParam([=]() {

            });
            break;
        }
        case C2SModuleAPIList::ACTIVEUSER_SET_MODULE_ORIENTATION:
        {
            showSetModuleOrientationParam([=]() {
                MODULE_ORIENTATION moduleOrientation = stringToOrientation(getComboboxValue(COMBOBOX_MODULE_ORIENTATION));
                ParameterDataManager::getInstance()->activeUserParams->setModuleOrientation(moduleOrientation);
            });
            break;
        }
        case C2SModuleAPIList::ACTIVEUSER_SET_EMERGENCY_MODE_UID:
        {
            showSetEmergencyModeUid([=]() {
                string uid = getEditboxValue(EDITBOX_UID);
                ParameterDataManager::getInstance()->commonParams->setUid(uid);
            });
        }
        case C2SModuleAPIList::ACTIVEUSER_SET_USE_HERCULES: {
            showSetUseHerculesParam([=]() {
                bool isEnable = bool(getSelectedToggleButtonIndex(TOGGLE_BUTTON_IS_USE_HERCULES));
                ParameterDataManager::getInstance()->activeUserParams->setUseHercules(isEnable);
            });
            break;
        }
        case C2SModuleAPIList::ACTIVEUSER_SET_CHANNEL: {
            showSetChannelParam([=]() {
                string channel = getEditboxValue(EDITBOX_CHANNEL);
                ParameterDataManager::getInstance()->activeUserParams->setChannel(channel);
            });
            break;
        }

        default:
            break;
    }
}

void ActiveUserParamScene::showStartExParam(function<void()> completionHandler)
{
    setUseBackButton(true, "- StartEx -");
    setCompletionHandler = completionHandler;

    vector<const char*> serverList = { "live", "staging", "sandbox" };
    SERVER_STATE serverState = ParameterDataManager::getInstance()->activeUserParams->getServerState();

    addCombobox("serverState : ", serverList, static_cast<int>(serverState), COMBOBOX_SERVER_STATE);
}

void ActiveUserParamScene::showStartWithoutNoticeParam(function<void()> completionHandler)
{
    setUseBackButton(true, "- StartWithoutNotice -");
    setCompletionHandler = completionHandler;

    vector<const char*> serverList = { "live", "staging", "sandbox" };
    SERVER_STATE serverState = ParameterDataManager::getInstance()->activeUserParams->getServerState();

    addCombobox("serverState : ", serverList, static_cast<int>(serverState), COMBOBOX_SERVER_STATE);
    addTrueFalseToggleButton("showNotice : ", 1, {screenSize.width, DEFAULT_CONTENT_HEIGHT}, TOGGLE_BUTTON_SHOW_NOTICE);
}

void ActiveUserParamScene::showSetVidParam(function<void()> completionHandler)
{
    setUseBackButton(true, "- SetVid -");
    setCompletionHandler = completionHandler;

    string vid = ParameterDataManager::getInstance()->activeUserParams->getVid();
    addEditBox("VID : ", vid.c_str(), "", EDITBOX_VID);
}

void ActiveUserParamScene::showSetServerIdParam(bool isUpdate, function<void()> completionHandler)
{
    if (isUpdate)
        setUseBackButton(true, "- UpdateServerId -");
    else
        setUseBackButton(true, "- SetServerId -");

    setCompletionHandler = completionHandler;

    string serverId = ParameterDataManager::getInstance()->activeUserParams->getServerId();
    addEditBox("serverId : ", serverId.c_str(), "", EDITBOX_SERVER_ID);
}

void ActiveUserParamScene::showSetGameLanguageParam(bool isUpdate, function<void()> completionHandler)
{
    if (isUpdate)
        setUseBackButton(true, "- UpdateGameLanguage -");
    else
        setUseBackButton(true, "- SetGameLanguage -");

    setCompletionHandler = completionHandler;

    vector<const char*> languageCode = {
        "en",
        "de",
        "es",
        "fr",
        "id",
        "it",
        "ja",
        "ko",
        "pt",
        "ru",
        "th",
        "tr",
        "vi",
        "zh-Hans",
        "zh-Hant",
        "ar",
        "null"
    };

    string gameLanguage = ParameterDataManager::getInstance()->activeUserParams->getGameLanguage();
    int findIndex = static_cast<int>(find(languageCode.begin(), languageCode.end(), gameLanguage) - languageCode.begin());

    addCombobox("gameLanguage : ", languageCode, findIndex, COMBOBOX_GAME_LANGUAGE);
}

void ActiveUserParamScene::showSetMinTimeoutSecondsParam(function<void()> completionHandler)
{
    setUseBackButton(true, "- SetMinTimeoutSeconds -");
    setCompletionHandler = completionHandler;

    string minTimeoutSeconds = to_string(ParameterDataManager::getInstance()->activeUserParams->getMinTimeoutSeconds());
    addEditBox("seconds : ", minTimeoutSeconds.c_str(), "", EDITBOX_MIN_TIMEOUT_SECONDS);
}

void ActiveUserParamScene::showGetVersionParam(function<void()> completionHandler)
{
    setUseBackButton(true, "- GetVersion -");
    setCompletionHandler = completionHandler;

    string bufSize = to_string(ParameterDataManager::getInstance()->activeUserParams->getBufSize());
    addEditBox("bufSize : ", EDITBOX_BUF_SIZE);
}

void ActiveUserParamScene::showSetLoggedParam(function<void()> completionHandler)
{
    setUseBackButton(true, "- SetLogged -");
    setCompletionHandler = completionHandler;

    int isLogged = int(ParameterDataManager::getInstance()->activeUserParams->getIsLogged());
    addTrueFalseToggleButton("isLogged : ", isLogged, {screenSize.width, DEFAULT_CONTENT_HEIGHT}, TOGGLE_BUTTON_IS_LOGGED);
}

void ActiveUserParamScene::showUserAgreeShowTermsUIExParam(function<void()> completionHandler)
{
    setUseBackButton(true, "- UserAgreeShowTermsUIEx -");
    setCompletionHandler = completionHandler;

    string url = ParameterDataManager::getInstance()->activeUserParams->getUrl();
    addEditBox("url : ", url.c_str(), "", EDITBOX_URL);
}

void ActiveUserParamScene::showSetPermissionViewStateParam(function<void()> completionHandler)
{
    setUseBackButton(true, "- SetPermissionViewState -");
    setCompletionHandler = completionHandler;

    int isOn = int(ParameterDataManager::getInstance()->activeUserParams->getIsOn());
    addTrueFalseToggleButton("isOn : ", isOn, {screenSize.width, DEFAULT_CONTENT_HEIGHT}, TOGGLE_BUTTON_IS_ON);
}

void ActiveUserParamScene::showSendFunnelsLogsParam(function<void()> completionHandler)
{
    setUseBackButton(true, "- SendFunnelsLogs -");
    setCompletionHandler = completionHandler;

    string funnelsTrack = ParameterDataManager::getInstance()->activeUserParams->getFunnelsTrack();
    string optionTag = ParameterDataManager::getInstance()->activeUserParams->getOptionTag();

    addEditBox("funnelsTrack : ", funnelsTrack.c_str(), "", EDITBOX_FUNNELS_TRACK);
    addEditBox("optionTag : ", optionTag.c_str(), "", EDITBOX_OPTION_TAG);
}

void ActiveUserParamScene::showRequestUserPermissionsParam(function<void()> completionHandler)
{
    setUseBackButton(true, "- RequestUserPermissions -");
    setCompletionHandler = completionHandler;

    string requests = ParameterDataManager::getInstance()->activeUserParams->getRequests();
    addEditBox("requests : ", requests.c_str(), "", EDITBOX_REQUESTS);
}

void ActiveUserParamScene::showSetAnalyticsLogSettingsParam(function<void()> completionHandler)
{
    setUseBackButton(true, "- SetAnalyticsLogSettings -");
    setCompletionHandler = completionHandler;

    string sendLimit = to_string(ParameterDataManager::getInstance()->activeUserParams->getSendLimit());
    string queueLimit = to_string(ParameterDataManager::getInstance()->activeUserParams->getQueueLimit());
    string sendCycleSecond = to_string(ParameterDataManager::getInstance()->activeUserParams->getSendCycleSecond());

    addEditBox("sendLimit : ", sendLimit.c_str(), "", EDITBOX_SEND_LIMIT);
    addEditBox("queueLimit : ", queueLimit.c_str(), "", EDITBOX_QUEUE_LIMIT);
    addEditBox("sendCycleSecond : ", sendCycleSecond.c_str(), "", EDITBOX_SEND_CYCLE_SECOND);
}

void ActiveUserParamScene::showSendAnalyticsLogParam(function<void()> completionHandler)
{
    setUseBackButton(true, "- SendAnalyticsLog -");
    setCompletionHandler = completionHandler;

    string logData = ParameterDataManager::getInstance()->activeUserParams->getLogData();

    addEditBox("logData : ", logData.c_str(), "", EDITBOX_LOG_DATA);
}

void ActiveUserParamScene::showSetModuleOrientationParam(function<void()> completionHandler)
{
    setUseBackButton(true, "- SetModuleOrientation -");
    setCompletionHandler = completionHandler;

    int selectedOrientation = static_cast<int>(ParameterDataManager::getInstance()->activeUserParams->getModuleOrientation());

    addCombobox("orientation : ", orientationList, selectedOrientation, COMBOBOX_MODULE_ORIENTATION);
}

void ActiveUserParamScene::showSetEmergencyModeUid(function<void ()> completionHandler)
{
    setUseBackButton(true, "- SetEmergencyModeUid -");
    setCompletionHandler = completionHandler;

    string existUid = ParameterDataManager::getInstance()->commonParams->getUid();

    if (existUid.empty()) {
        // 긴급모드시 uid가 비어있는 상태(로그인 후 앱 재실행)라면 파일에서 읽어온다.
        ParameterDataManager::getInstance()->commonParams->readCommonParametersFromFile();
        existUid = ParameterDataManager::getInstance()->commonParams->getUid();
    }

    addEditBox("uid : ", existUid.c_str(), "", EDITBOX_UID);
}

void ActiveUserParamScene::showSetUseHerculesParam(function<void()> completionHandler)
{
    setUseBackButton(true, "- SetUseHercules -");
    setCompletionHandler = completionHandler;

    int isEnable = int(ParameterDataManager::getInstance()->activeUserParams->getUseHercules());
    addTrueFalseToggleButton("isEnable : ", isEnable, {screenSize.width, DEFAULT_CONTENT_HEIGHT}, TOGGLE_BUTTON_IS_USE_HERCULES);
}

void ActiveUserParamScene::showSetChannelParam(function<void()> completionHandler)
{
    setUseBackButton(true, "- SetChannel -");
    setCompletionHandler = completionHandler;

    string channel = ParameterDataManager::getInstance()->activeUserParams->getChannel();

    addEditBox("channel : ", channel.c_str(), "", EDITBOX_CHANNEL);
}

SERVER_STATE ActiveUserParamScene::stringToServerState(std::string value)
{
    if (value == "staging")
        return STAGING_SERVER;
    else if (value == "sandbox")
        return SANDBOX_SERVER;
    
    return LIVE_SERVER;
}

MODULE_ORIENTATION ActiveUserParamScene::stringToOrientation(std::string value)
{
    if (value == "ALL")
        return ALL;
    else if (value == "PORTRAIT")
        return PORTRAIT;

    return LANDSCAPE;
}

