//
//  ActiveUserScene.cpp
//  HIVE_SDK_Cocos_Sample
//
//  Created by sjyong on 2021/09/27.
//

#include "C2SModuleAPIList.hpp"
#include "ActiveUserScene.hpp"
#include "ActiveUserParamScene.hpp"
#include "ParameterDataManager.hpp"
#include "ui/CocosGUI.h"
#include "Postbox.hpp"

using namespace cocos2d;
using namespace cocos2d::ui;
using namespace hive;

#include <map>
#include <sstream>

using namespace::std;

bool ActiveUserScene::init()
{
    if(!BaseView::init())
        return false;

    setUseShowLogButton(true);

    registerCallback();

    setUseBackButton(true, "- ActiveUser -");


    addButton("Start", C2SModuleAPIList::ACTIVEUSER_START);
    addButton("StartEx", ButtonMode::NEED_PARAM, C2SModuleAPIList::ACTIVEUSER_START_EX);
    addButton("StartWithoutNotice", ButtonMode::NEED_PARAM, C2SModuleAPIList::ACTIVEUSER_START_WITHOUT_NOTICE);
    addButton("SetServerId", ButtonMode::NEED_PARAM, C2SModuleAPIList::ACTIVEUSER_SET_SERVER_ID);
    addButton("SetGameLanguage", ButtonMode::NEED_PARAM, C2SModuleAPIList::ACTIVEUSER_SET_GAME_LANGUAGE);
    addButton("SetMinTimeoutSeconds", ButtonMode::NEED_PARAM, C2SModuleAPIList::ACTIVEUSER_SET_MIN_TIMEOUT_SECONDS);
    addButton("GetVersion", C2SModuleAPIList::ACTIVEUSER_GET_VERSION);
    addButton("SetLogged", ButtonMode::NEED_PARAM, C2SModuleAPIList::ACTIVEUSER_SET_LOGGED);
    addButton("GetDID", C2SModuleAPIList::ACTIVEUSER_GET_DID);
    addButton("UserAgreeShowTermsUI", C2SModuleAPIList::ACTIVEUSER_USER_AGREE_SHOW_TERMS_UI);
    addButton("UserAgreeShowTermsUIEx", ButtonMode::NEED_PARAM, C2SModuleAPIList::ACTIVEUSER_USER_AGREE_SHOW_TERMS_UI_EX);
    addButton("UpdateNoticeShowPopup", C2SModuleAPIList::ACTIVEUSER_UPDATE_NOTICE_SHOW_POPUP);
    addButton("UpdateNoticeGetPopupInfo", C2SModuleAPIList::ACTIVEUSER_UPDATE_NOTICE_GET_POPUP_INFO);
    addButton("GetPermissionViewData", C2SModuleAPIList::ACTIVEUSER_GET_PERMISSION_VIEW_DATA);
    addButton("SetPermissionViewState", ButtonMode::NEED_PARAM, C2SModuleAPIList::ACTIVEUSER_SET_PERMISSION_VIEW_STATE);
    addButton("SendFunnelsLogs", ButtonMode::NEED_PARAM, C2SModuleAPIList::ACTIVEUSER_SEND_FUNNELS_LOGS);
    addButton("RequestUserPermissions", ButtonMode::NEED_PARAM, C2SModuleAPIList::ACTIVEUSER_REQUEST_USER_PERMISSIONS);
    addButton("UpdateServerId", ButtonMode::NEED_PARAM, C2SModuleAPIList::ACTIVEUSER_UPDATE_SERVER_ID);
    addButton("UpdateGameLanguage", ButtonMode::NEED_PARAM, C2SModuleAPIList::ACTIVEUSER_UPDATE_GAME_LANGUAGE);
    addButton("SetAnalyticsLogSettings", ButtonMode::NEED_PARAM, C2SModuleAPIList::ACTIVEUSER_SET_ANALYTICS_LOG_SETTINGS);
    addButton("SendAnalyticsLog", ButtonMode::NEED_PARAM, C2SModuleAPIList::ACTIVEUSER_SEND_ANALYTICS_LOG);
    addButton("SetModuleOrientation", ButtonMode::NEED_PARAM, C2SModuleAPIList::ACTIVEUSER_SET_MODULE_ORIENTATION);
    addButton("SetEmergencyModeUid", ButtonMode::NEED_PARAM, C2SModuleAPIList::ACTIVEUSER_SET_EMERGENCY_MODE_UID);
    addButton("SetUseHercules", ButtonMode::NEED_PARAM, C2SModuleAPIList::ACTIVEUSER_SET_USE_HERCULES);
    addButton("GetHiveCountry", C2SModuleAPIList::ACTIVEUSER_GET_HIVE_COUNTRY);
    addButton("GetChannel",  C2SModuleAPIList::ACTIVEUSER_GET_CHANNEL);
    addButton("SetChannel", ButtonMode::NEED_PARAM, C2SModuleAPIList::ACTIVEUSER_SET_CHANNEL);

    scheduleUpdate();

    return true;
}

void ActiveUserScene::update(float dt)
{
    BaseView::update(dt);

    static bool firstRun = true;
    if (firstRun) {
        firstRun = false;
    }
}

void ActiveUserScene::onButtonEvent(int buttonTag)
{
    // Parameter 세팅 화면 생성 및 이동
    if (buttonTag >= DEFAULT_PARAM_BUTTON_TAG) {
        ActiveUserParamScene* paramScene = ActiveUserParamScene::create();
        BaseView::pushView(paramScene);
        paramScene->showParameters(buttonTag % DEFAULT_PARAM_BUTTON_TAG);
        return;
    }

    // API 호출
    C2SModuleAPIList::ActiveUserAPI api = static_cast<C2SModuleAPIList::ActiveUserAPI>(buttonTag);
    switch (api) {
        case C2SModuleAPIList::ACTIVEUSER_START:
        {
            CS_ActiveUserStart();

            addLogMessage("CS_ActiveUserStart called.");
            break;
        }
        case C2SModuleAPIList::ACTIVEUSER_START_EX:
        {
            SERVER_STATE serverState = ParameterDataManager::getInstance()->activeUserParams->getServerState();

            CS_ActiveUserStartEx(serverState);

            addLogMessage("CS_ActiveUserStartEx called.");
            break;
        }
        case C2SModuleAPIList::ACTIVEUSER_START_WITHOUT_NOTICE:
        {
            SERVER_STATE serverState = ParameterDataManager::getInstance()->activeUserParams->getServerState();
            int showNotice = ParameterDataManager::getInstance()->activeUserParams->getShowNotice();

            CS_ActiveUserStartWithoutNotice(serverState, showNotice);

            addLogMessage("CS_ActiveUserStartWithoutNotice called");
            break;
        }
        case C2SModuleAPIList::ACTIVEUSER_SET_VID:
        {
            string vid = ParameterDataManager::getInstance()->activeUserParams->getVid();
            
            addLogMessage("CS_ActiveUserSetVID, not supported.");
            break;
        }
        case C2SModuleAPIList::ACTIVEUSER_SET_SERVER_ID:
        {
            string serverId = ParameterDataManager::getInstance()->activeUserParams->getServerId();

            CS_ActiveUserSetServerId(serverId.c_str());

            addLogMessage("CS_ActiveUserSetServerId called.");
            addLogMessage(" : " + serverId);
            break;
        }
        case C2SModuleAPIList::ACTIVEUSER_SET_GAME_LANGUAGE:
        {
            string gameLanguage = ParameterDataManager::getInstance()->activeUserParams->getGameLanguage();

            CS_ActiveUserSetGameLanguage(gameLanguage.c_str());

            addLogMessage("CS_ActiveUserSetGameLanguage called.");
            addLogMessage(" : " + gameLanguage);
            break;
        }
        case C2SModuleAPIList::ACTIVEUSER_SET_MIN_TIMEOUT_SECONDS:
        {
            int seconds = ParameterDataManager::getInstance()->activeUserParams->getMinTimeoutSeconds();

            CS_SetMinTimeoutSeconds(seconds);

            stringstream log;
            log << " : " << to_string(seconds);
            addLogMessage("CS_SetMinTimeoutSeconds called.");
            addLogMessage(log.str());
            break;
        }
        case C2SModuleAPIList::ACTIVEUSER_REGISTER_TOKEN:
        {
            addLogMessage("CS_RegisterToken called.");
            addLogMessage("( CS_RegisterToken API는 KS C2S에서 더이상 작동하지 않습니다.(deprecated 예정) )");
            break;
        }
        case C2SModuleAPIList::ACTIVEUSER_GET_VERSION:
        {
            int bufSize = ParameterDataManager::getInstance()->activeUserParams->getBufSize();
            char* buf = new char[bufSize];

            int result = CS_ActiveUserGetVersion(buf, bufSize);

            stringstream log;
            log << "CS_ActiveUserGetVersion called." << '\n';
            log << "result : " << to_string(result) << '\n';
            log << "version : " << buf << '\n';

            addLogMessage(log.str().c_str());
            break;
        }
        case C2SModuleAPIList::ACTIVEUSER_SET_LOGGED:
        {
            int isLogged = int(ParameterDataManager::getInstance()->activeUserParams->getIsLogged());

            CS_ActiveUserSetLogged(isLogged);

            stringstream log;
            log << " : " << to_string(isLogged);
            addLogMessage("CS_ActiveUserSetLogged called.");
            addLogMessage(log.str());       // 1: true, 0: false
            break;
        }
        case C2SModuleAPIList::ACTIVEUSER_GET_DID:
        {
            const char* did = CS_ActiveUserGetDID();

            string didValue = did;
            addLogMessage("CS_ActiveUserGetDID called.");
            addLogMessage(" : " + didValue);
            break;
        }
        case C2SModuleAPIList::ACTIVEUSER_USER_AGREE_SHOW_TERMS_UI:
        {
            CS_UserAgreeShowTermsUI(NULL);

            addLogMessage("CS_UserAgreeShowTermsUI called.");
            break;
        }
        case C2SModuleAPIList::ACTIVEUSER_USER_AGREE_SHOW_TERMS_UI_EX:
        {
            string url = ParameterDataManager::getInstance()->activeUserParams->getUrl();
            CS_UserAgreeShowTermsUIEx(NULL, url.c_str());

            addLogMessage("CS_UserAgreeShowTermsUIEx called.");
            break;
        }
        case C2SModuleAPIList::ACTIVEUSER_SET_ENABLE_REQUEST_STORAGE_PERMISSION:
        {
            CS_ActiveUserSetEnableRequestStoragePermission();

            addLogMessage("CS_ActiveUserSetEnableRequestStoragePermission called.");
            break;
        }
        case C2SModuleAPIList::ACTIVEUSER_UPDATE_NOTICE_SHOW_POPUP:
        {
            CS_UpdateNoticeShowPopup([](POPUP_STATE state){
                stringstream log;

                switch (state) {
                    case CLOSE:
                        log << "CLOSE" <<'\n';
                        break;
                    case UNSHOWN:
                        log << "UNSHOWN" << '\n';
                        break;
                    case POPUP_STATE_ERROR:
                        log << "POPUP_STATE_ERROR" << '\n';
                        break;
                    default:
                        break;
                }
                addCallbackLogMessage(log.str().c_str(), "UpdateNoticePopupCB : ");
            });

            addLogMessage("CS_UpdateNoticeShowPopup called.");
            break;
        }
        case C2SModuleAPIList::ACTIVEUSER_UPDATE_NOTICE_GET_POPUP_INFO:
        {
            CS_UpdateNoticeGetPopupInfo([](SHOW_POPUP show, const char* jsonData){
                stringstream log;

                switch (show) {
                    case SHOW:
                        log << "SHOW" << '\n';
                        break;
                    case NOTSHOW:
                        log << "NOTSHOW" << '\n';
                        break;
                    case SHOW_POPUP_ERROR:
                        log << "SHOW_POPUP_ERROR" << '\n';
                        break;
                    default:
                        break;
                }

                log << "jsonData : ";
                if (jsonData) {
                    log << jsonData << '\n';
                }
                else {
                    log << "NULL" << '\n';
                }

                addCallbackLogMessage(log.str().c_str(), "UpdateNoticeInfoCB : ");
            });

            addLogMessage("CS_UpdateNoticeGetPopupInfo called.");
            break;
        }
        case C2SModuleAPIList::ACTIVEUSER_GET_PERMISSION_VIEW_DATA:
        {
            CS_ActiveUserGetPermissionViewData([](const char* jsonData){
                stringstream log;

                log << "jsonData : ";
                if (jsonData) {
                    log << jsonData << '\n';
                }
                else {
                    log << "NULL" << '\n';
                }

                addCallbackLogMessage(log.str().c_str(), "UpdateNoticeInfoCB : ");
            });

            addLogMessage("CS_ActiveUserGetPermissionViewData called.");
            break;
        }
        case C2SModuleAPIList::ACTIVEUSER_SET_PERMISSION_VIEW_STATE:
        {
            int isOn = int(ParameterDataManager::getInstance()->activeUserParams->getIsOn());

            CS_ActiveUserSetPermissionViewState(isOn);

            stringstream log;
            log << " : " << to_string(isOn);
            addLogMessage("CS_ActiveUserSetPermissionViewState called.");
            addLogMessage(log.str());       // 1: true(want to show), 0: false
            break;
        }
        case C2SModuleAPIList::ACTIVEUSER_REQUEST_USER_PERMISSIONS:
        {
#if (CC_TARGET_PLATFORM == CC_PLATFORM_IOS)
            addLogMessage("CS_ActiveUserRequestUserPermissions, not supported in iOS.");
#elif (CC_TARGET_PLATFORM == CC_PLATFORM_ANDROID)
            string requestPermissions = ParameterDataManager::getInstance()->activeUserParams->getRequests();

            CS_ActiveUserRequestUserPermissions(requestPermissions.c_str(), [](REQUEST_PERMISSION_STATE state, const char* grantedPermissions, const char* deniedPermissions){
                stringstream log;

                switch(state) {
                    case SUCCESS:
                        log << "SUCCESS" << '\n';
                        break;
                    case ERROR:
                        log << "ERROR" << '\n';
                        break;
                    case NOT_SUPPORT_OS_VERSION:
                        log << "NOT_SUPPORT_OS_VERSION" << '\n';
                        break;
                    case WRONG_TYPE_DATA:
                        log << "WRONG_TYPE_DATA" << '\n';
                        break;
                    default:
                        break;
                }

                log << "grantedPermissions : ";
                if (grantedPermissions) {
                    log << grantedPermissions << '\n';
                }
                else {
                    log << "NULL(grantedPermissions)" << '\n';
                }
                log << "deniedPermissions : ";
                if (deniedPermissions) {
                    log << deniedPermissions << '\n';
                }
                else {
                    log << "NULL(deniedPermissions)" << '\n';
                }

                addCallbackLogMessage(log.str().c_str(), "RequestUserPermissionsCB : ");
            });
#endif
            break;
        }
        case C2SModuleAPIList::ACTIVEUSER_SEND_FUNNELS_LOGS:
        {
            string funnelsTrack = ParameterDataManager::getInstance()->activeUserParams->getFunnelsTrack();
            string optionTag = ParameterDataManager::getInstance()->activeUserParams->getOptionTag();

            CS_ActiveUserSendFunnelsLogs(funnelsTrack.c_str(), optionTag.c_str());

            addLogMessage("CS_ActiveUserSendFunnelsLogs called.");
            addLogMessage(" : funnelsTrack = " + funnelsTrack + ", optionTag = " + optionTag);
            break;
        }
        case C2SModuleAPIList::ACTIVEUSER_UPDATE_SERVER_ID:
        {
            string serverId = ParameterDataManager::getInstance()->activeUserParams->getServerId();

            CS_ActiveUserUpdateServerId(serverId.c_str());

            addLogMessage("CS_ActiveUserUpdateServerId called.");
            addLogMessage(" : " + serverId);
            break;
        }
        case C2SModuleAPIList::ACTIVEUSER_UPDATE_GAME_LANGUAGE:
        {
            string gameLanguage = ParameterDataManager::getInstance()->activeUserParams->getGameLanguage();

            CS_ActiveUserUpdateGameLanguage(gameLanguage.c_str());

            addLogMessage("CS_ActiveUserUpdateGameLanguage called.");
            addLogMessage(" : " + gameLanguage);
            break;
        }
        case C2SModuleAPIList::ACTIVEUSER_SET_ANALYTICS_LOG_SETTINGS:
        {
            int sendLimit = ParameterDataManager::getInstance()->activeUserParams->getSendLimit();
            int queueLimit = ParameterDataManager::getInstance()->activeUserParams->getQueueLimit();
            int sendCycleSecond = ParameterDataManager::getInstance()->activeUserParams->getSendCycleSecond();

            CS_SetAnalyticsLogSettings(sendLimit, queueLimit, sendCycleSecond);

            addLogMessage("CS_SetAnalyticsLogSettings called.");
            break;
        }
        case C2SModuleAPIList::ACTIVEUSER_SEND_ANALYTICS_LOG:
        {
            string logData = ParameterDataManager::getInstance()->activeUserParams->getLogData();

            CS_SendAnalyticsLog(logData.c_str());

            addLogMessage("CS_SendAnalyticsLog called.");
            break;
        }
        case C2SModuleAPIList::ACTIVEUSER_SET_MODULE_ORIENTATION:
        {
            MODULE_ORIENTATION moduleOrientation = ParameterDataManager::getInstance()->activeUserParams->getModuleOrientation();

            CS_SetModuleOrientation(moduleOrientation);

            addLogMessage("CS_SetModuleOrientation called");
            break;
        }
        case C2SModuleAPIList::ACTIVEUSER_SET_EMERGENCY_MODE_UID:
        {
            string uid = ParameterDataManager::getInstance()->commonParams->getUid();
            if (uid.empty()) {
                // Read File
                ParameterDataManager::getInstance()->commonParams->readCommonParametersFromFile();
                uid = ParameterDataManager::getInstance()->commonParams->getUid();
            }

            CS_SetEmergencyModeWithUid(uid.c_str());

            // Postbox set login type
            Postbox::setLoginType(Postbox::v0);
            Postbox::login([=](const Postbox::Login *data, Postbox::Error error){
                stringstream postboxLog;

                if (data != NULL) {
                    postboxLog << "nickname : " << data->nickname << '\n';
                    postboxLog << "gem : " << data->gem << '\n';
                    postboxLog << "gold : " << data->gold << '\n';
                }
                postboxLog << "error" << '\n';
                postboxLog << "result_code : " << error.result_code << '\n';
                postboxLog << "result_msg : " << error.result_msg << '\n';

                addCallbackLogMessage(postboxLog.str(), "Postbox::login CB : ");
            });

            addLogMessage("CS_SetEmergencyModeWithUid called.");
            break;
        }
        case C2SModuleAPIList::ACTIVEUSER_SET_USE_HERCULES:
        {
            int isEnable = int(ParameterDataManager::getInstance()->activeUserParams->getUseHercules());

            CS_SetUseHercules(isEnable);

            stringstream log;
            log << " : " << to_string(isEnable);
            addLogMessage("CS_SetUseHercules called.");
            addLogMessage(log.str());       // 1: true, 0: false

            break;
        }
        case C2SModuleAPIList::ACTIVEUSER_GET_HIVE_COUNTRY:
        {
            const char* hiveCountry = CS_GetHiveCountry();

            string hiveCountryValue = hiveCountry;
            addLogMessage("CS_GetHiveCountry called.");
            addLogMessage(" : " + hiveCountryValue);
            break;
        }
        case C2SModuleAPIList::ACTIVEUSER_GET_CHANNEL:
        {
            const char* channel = CS_GetChannel();

            string channelValue;
            if (channel != NULL) {
                channelValue = channel;
            } else {
                channelValue = "";
            }
            addLogMessage("CS_GetChannel called.");
            addLogMessage(" : " + channelValue);
            break;
        }
        case C2SModuleAPIList::ACTIVEUSER_SET_CHANNEL:
        {
            string channel = ParameterDataManager::getInstance()->activeUserParams->getChannel();

            CS_SetChannel(channel.c_str());

            addLogMessage("CS_SetChannel called.");
            addLogMessage(" : channel = " + channel);
            break;
        }
        default:
            break;
    }
}


void ActiveUserScene::registerCallback()
{
    CS_UserAgreeSetCallback([](USER_AGREE_RESULT_TYPE result) {
        stringstream log;

        switch (result) {
            case USER_AGREE_SUCCESS:
                log << "USER_AGREE_SUCCESS" << '\n';
                break;
            case USER_AGREE_TERMS_CLOSE:
                log << "USER_AGREE_TERMS_CLOSE" << '\n';
                break;
            default:
                break;
        }

        addCallbackLogMessage(log.str().c_str(), "UserAgreeCB : ");
    });

    CS_ActiveUserSetCallback([](ACTIVE_USER_RESULT result) {
        stringstream log;

        switch (result) {
            case ACTIVE_USER_GETDID_SUCCESS:
                log << "ACTIVE_USER_GETDID_SUCCESS" << '\n';
                break;
            case ACTIVE_USER_GETDID_FAIL:
                log << "ACTIVE_USER_GETDID_FAIL" << '\n';
                break;
            default:
                break;
        }

        addCallbackLogMessage(log.str().c_str(), "ActiveUserCB : ");
    });
}
