//
//  HivePromotionParamScene.cpp
//  HIVE_SDK_Cocos_Sample
//
//  Created by sjyong on 2021/10/15.
//

#include "HivePromotionParamScene.hpp"
#include "ParameterDataManager.hpp"
#include "ui/CocosGUI.h"

using namespace cocos2d;
using namespace cocos2d::ui;
using namespace hive;

#include <map>

using namespace::std;

bool HivePromotionParamScene::init()
{
    if(!BaseView::init())
        return false;

    viewTypeList = { "bannerLegacy", "banner", "news", "notice" };
    customTypeList = { "view", "board", "spot", "direct" };

    addButton("OK", BUTTON_OK);

    scheduleUpdate();

    return true;
}

void HivePromotionParamScene::update(float dt)
{
    BaseView::update(dt);

    static bool firstRun = true;
    if (firstRun) {
        firstRun = false;
    }
}

void HivePromotionParamScene::onButtonEvent(int buttonTag)
{
    if (buttonTag != BUTTON_OK) {
        return;
    }

    setCompletionHandler();
    setCompletionHandler = NULL;
    this->popView();
}

void HivePromotionParamScene::showParameters(int tag)
{
    api = static_cast<C2SModuleAPIList::HivePromotionAPI>(tag);
    switch (api) {
        case C2SModuleAPIList::PROMOTION_SHOW:
        {
            showPromotionShowParam([=](){
                string viewTypeString = getComboboxValue(COMBOBOX_VIEW_TYPE);
                CSPromotionViewType viewType = stringToPromotionViewType(viewTypeString);
                int isForced = getSelectedToggleButtonIndex(TOGGLE_BUTTON_IS_FORCED);
                string uid = getEditboxValue(EDITBOX_UID);

                ParameterDataManager::getInstance()->hivePromotionParams->setViewType(viewType);
                ParameterDataManager::getInstance()->hivePromotionParams->setIsForced(isForced);
                ParameterDataManager::getInstance()->commonParams->setUid(uid);
            });
            break;
        }
        case C2SModuleAPIList::PROMOTION_SHOW_CUSTOM:
        {
            showPromotionShowCustomParam([=]{
                string customTypeString = getComboboxValue(COMBOBOX_CUSTOM_TYPE);
                CSPromotionCustomType customType = stringToPromotionCustomType(customTypeString);
                string contents = getEditboxValue(EDITBOX_CONTENTS);
                string uid = getEditboxValue(EDITBOX_UID);

                ParameterDataManager::getInstance()->hivePromotionParams->setCustomType(customType);
                ParameterDataManager::getInstance()->hivePromotionParams->setContents(contents);
                ParameterDataManager::getInstance()->commonParams->setUid(uid);
            });
            break;
        }
        case C2SModuleAPIList::PROMOTION_SHOW_OFFERWALL:
        {
            showShowOfferwallParam([=]{
                string uid = getEditboxValue(EDITBOX_UID);
                ParameterDataManager::getInstance()->commonParams->setUid(uid);
            });
            break;
        }
        case C2SModuleAPIList::PROMOTION_SHOW_REVIEW:
        {
            showShowReviewParam([=]{
                string uid = getEditboxValue(EDITBOX_UID);
                ParameterDataManager::getInstance()->commonParams->setUid(uid);
            });
            break;
        }
        case C2SModuleAPIList::PROMOTION_SHOW_EXIT:
        {
            showShowExitParam([=]{
                string uid = getEditboxValue(EDITBOX_UID);
                ParameterDataManager::getInstance()->commonParams->setUid(uid);
            });
            break;
        }
        case C2SModuleAPIList::PROMOTION_GET_VIEW_INFO:
        {
            showGetViewInfoParam([=]{
                string customTypeString =  getComboboxValue(COMBOBOX_CUSTOM_TYPE);
                CSPromotionCustomType customType = stringToPromotionCustomType(customTypeString);
                string contents = getEditboxValue(EDITBOX_CONTENTS);
                string uid = getEditboxValue(EDITBOX_UID);

                ParameterDataManager::getInstance()->hivePromotionParams->setCustomType(customType);
                ParameterDataManager::getInstance()->hivePromotionParams->setContents(contents);
                ParameterDataManager::getInstance()->commonParams->setUid(uid);
            });
            break;
        }
        case C2SModuleAPIList::PROMOTION_GET_BADGE_INFO:
        {
            showGetBadgeInfoParam([=]{
                string uid = getEditboxValue(EDITBOX_UID);
                ParameterDataManager::getInstance()->commonParams->setUid(uid);
            });
            break;
        }
        case C2SModuleAPIList::PROMOTION_GET_BANNER_INFO:
        {
            showPromotionGetBannerInfoParam([=]{
                string campaignType = getEditboxValue(EDITBOX_CAMPAIGN_TYPE);
                string bannerType = getEditboxValue(EDITBOX_BANNER_TYPE);
                string uid = getEditboxValue(EDITBOX_UID);

                ParameterDataManager::getInstance()->hivePromotionParams->setCampaignType(campaignType);
                ParameterDataManager::getInstance()->hivePromotionParams->setBannerType(bannerType);
                ParameterDataManager::getInstance()->commonParams->setUid(uid);
            });
            break;
        }
        case C2SModuleAPIList::PROMOTION_GET_APP_INVITATION_DATA:
        {
            showPromotionGetAppInvitationDataParam([=]{
                string uid = getEditboxValue(EDITBOX_UID);
                string additionalInfo = getEditboxValue(EDITBOX_ADDITIONAL_INFO);

                ParameterDataManager::getInstance()->commonParams->setUid(uid);
                ParameterDataManager::getInstance()->hivePromotionParams->setAdditionalInfo(additionalInfo);
            });
            break;
        }
        case C2SModuleAPIList::PROMOTION_SHOW_UA_SHARE:
        {
            showPromotionShowUAShareParam([=]{
                string inviteMessage = getEditboxValue(EDITBOX_INVITE_MESSAGE);
                string inviteLink = getComboboxValue(COMBOBOX_INVITE_LINK);

                ParameterDataManager::getInstance()->hivePromotionParams->setInviteMessage(inviteMessage);
                ParameterDataManager::getInstance()->hivePromotionParams->setInviteLink(inviteLink);
            });
            break;
        }
        case C2SModuleAPIList::PROMOTION_SET_ADDITIONAL_INFO:
        {
            showSetAdditionalInfoParam([=]{
                string additionalInfo = getEditboxValue(EDITBOX_ADDITIONAL_INFO);
                ParameterDataManager::getInstance()->hivePromotionParams->setAdditionalInfo(additionalInfo);
            });
            break;
        }
        default:
            break;
    }
}

void HivePromotionParamScene::showPromotionShowParam(function<void()> completionHandler)
{
    setUseBackButton(true, "- PromotionShow - ");

    int viewTypeInt = static_cast<int>(ParameterDataManager::getInstance()->hivePromotionParams->getViewType());
    viewTypeInt += 1;   // ViewType enum이 -1(BannerLegacy)부터 시작하여 index를 0부터 맞춰지도록 +1 처리. Mercury 에서 처럼 별도의 list 클래스를 만들어 구현도 가능.

    int isForced = ParameterDataManager::getInstance()->hivePromotionParams->getIsForced();
    string uid = ParameterDataManager::getInstance()->commonParams->getUid();

    addCombobox("viewType : ", viewTypeList, viewTypeInt, COMBOBOX_VIEW_TYPE);
    addTrueFalseToggleButton("isForced : ", isForced, {screenSize.width, DEFAULT_CONTENT_HEIGHT}, TOGGLE_BUTTON_IS_FORCED);
    addEditBox("uid : ", uid.c_str(), "", EDITBOX_UID);

    setCompletionHandler = completionHandler;
}

void HivePromotionParamScene::showPromotionShowCustomParam(function<void()> completionHandler)
{
    setUseBackButton(true, "- PromotionShowCustom -");

    int customTypeInt = static_cast<int>(ParameterDataManager::getInstance()->hivePromotionParams->getCustomType());
    string contents = ParameterDataManager::getInstance()->hivePromotionParams->getContents();
    string uid = ParameterDataManager::getInstance()->commonParams->getUid();

    addCombobox("customType : ", customTypeList, customTypeInt, COMBOBOX_CUSTOM_TYPE);
    addEditBox("contents : ", contents.c_str(), "", EDITBOX_CONTENTS);
    addEditBox("uid : ", uid.c_str(), "", EDITBOX_UID);

    setCompletionHandler = completionHandler;
}

void HivePromotionParamScene::showShowOfferwallParam(function<void()> completionHandler)
{
    setUseBackButton(true, "- ShowOfferwall -");

    string uid = ParameterDataManager::getInstance()->commonParams->getUid();
    addEditBox("uid : ", uid.c_str(), "", EDITBOX_UID);

    setCompletionHandler = completionHandler;
}

void HivePromotionParamScene::showShowReviewParam(function<void()> completionHandler)
{
    setUseBackButton(true, "- ShowReview -");

    string uid = ParameterDataManager::getInstance()->commonParams->getUid();
    addEditBox("uid : ", uid.c_str(), "", EDITBOX_UID);

    setCompletionHandler = completionHandler;
}

void HivePromotionParamScene::showShowExitParam(function<void()> completionHandler)
{
    setUseBackButton(true, "- ShowExit -");

    string uid = ParameterDataManager::getInstance()->commonParams->getUid();
    addEditBox("uid : ", uid.c_str(), "", EDITBOX_UID);

    setCompletionHandler = completionHandler;
}

void HivePromotionParamScene::showGetViewInfoParam(function<void()> completionHandler)
{
    setUseBackButton(true, "- GetViewInfo -");

    int customTypeInt = static_cast<int>(ParameterDataManager::getInstance()->hivePromotionParams->getCustomType());
    string contents = ParameterDataManager::getInstance()->hivePromotionParams->getContents();
    string uid = ParameterDataManager::getInstance()->commonParams->getUid();

    addCombobox("customType : ", customTypeList, customTypeInt, COMBOBOX_CUSTOM_TYPE);
    addEditBox("contents : ", contents.c_str(), "", EDITBOX_CONTENTS);
    addEditBox("uid : ", uid.c_str(), "", EDITBOX_UID);

    setCompletionHandler = completionHandler;
}

void HivePromotionParamScene::showGetBadgeInfoParam(function<void()> completionHandler)
{
    setUseBackButton(true, "- GetBadgeInfo -");

    string uid = ParameterDataManager::getInstance()->commonParams->getUid();
    addEditBox("uid : ", uid.c_str(), "", EDITBOX_UID);

    setCompletionHandler = completionHandler;
}

void HivePromotionParamScene::showPromotionGetBannerInfoParam(function<void()> completionHandler)
{
    setUseBackButton(true, "- PromotionGetBannerInfo -");
    
    string campaignType = ParameterDataManager::getInstance()->hivePromotionParams->getCampaignType();
    string bannerType = ParameterDataManager::getInstance()->hivePromotionParams->getBannerType();
    string uid = ParameterDataManager::getInstance()->commonParams->getUid();

    addEditBox("campaignType : ", campaignType.c_str(), "", EDITBOX_CAMPAIGN_TYPE);
    addEditBox("bannerType : ", bannerType.c_str(), "", EDITBOX_BANNER_TYPE);
    addEditBox("uid : ", uid.c_str(), "", EDITBOX_UID);

    setCompletionHandler = completionHandler;
}

void HivePromotionParamScene::showPromotionGetAppInvitationDataParam(function<void()> completionHandler)
{
    setUseBackButton(true, "- PromotionGetAppInvitationData -");

    string uid = ParameterDataManager::getInstance()->commonParams->getUid();
    string additionalInfo = "{\"uatoken\":\"asdfsafsdfs\"}";

    addEditBox("uid : ", uid.c_str(), "", EDITBOX_UID);
    addEditBox("additionalInfo : ", additionalInfo.c_str(), "", EDITBOX_ADDITIONAL_INFO);

    setCompletionHandler = completionHandler;
}

void HivePromotionParamScene::showPromotionShowUAShareParam(function<void()> completionHandler)
{
    setUseBackButton(true, "- PromotionShowUAShare -");

    string inviteMessage = ParameterDataManager::getInstance()->hivePromotionParams->getInviteMessage();
    vector<string> inviteLinkList = ParameterDataManager::getInstance()->hivePromotionParams->getInviteLinkList();

    vector<const char*> inviteLinks(inviteLinkList.size(), nullptr);

    for (int i=0; i<inviteLinkList.size(); i++) {
        inviteLinks[i] = inviteLinkList[i].c_str();
    }

    addEditBox("inviteMessage : ", inviteMessage.c_str(), "", EDITBOX_INVITE_MESSAGE);
    addCombobox("inviteLink : ", inviteLinks, 0, COMBOBOX_INVITE_LINK);

    setCompletionHandler = completionHandler;
}

void HivePromotionParamScene::showSetAdditionalInfoParam(function<void()> completionHandler)
{
    setUseBackButton(true, "- SetAdditionalInfo -");

    string additionalInfo = ParameterDataManager::getInstance()->hivePromotionParams->getAdditionalInfo();
    addEditBox("additionalInfo : ", additionalInfo.c_str(), "", EDITBOX_ADDITIONAL_INFO);

    setCompletionHandler = completionHandler;
}

CSPromotionViewType HivePromotionParamScene::stringToPromotionViewType(string value)
{
    if (value == "bannerLegacy")
        return CS_PROMOTION_BANNERLEGACY;
    else if (value == "banner")
        return CS_PROMOTION_BANNER;
    else if (value == "news")
        return CS_PROMOTION_NEWS;

    return CS_PROMOTION_NOTICE;
}

CSPromotionCustomType HivePromotionParamScene::stringToPromotionCustomType(string value)
{
    if (value == "view")
        return CS_PROMOTION_VIEW;
    else if (value == "board")
        return CS_PROMOTION_BOARD;
    else if (value == "spot")
        return CS_PROMOTION_SPOT;

    return CS_PROMOTION_DIRECT;
}
