//
//  InAppV4ParamScene.cpp
//  HIVE_SDK_Cocos_Sample
//
//  Created by sjyong on 2021/10/18.
//

#include "InAppV4ParamScene.hpp"
#include "ParameterDataManager.hpp"
#include "ui/CocosGUI.h"

using namespace cocos2d;
using namespace cocos2d::ui;
using namespace hive;

#include <map>

using namespace std;

bool InAppV4ParamScene::init()
{
    if(!BaseView::init())
        return false;

    addButton("OK", BUTTON_OK);

    scheduleUpdate();

    return true;
}

void InAppV4ParamScene::update(float dt)
{
    BaseView::update(dt);

    static bool firstRun = true;
    if (firstRun) {
        firstRun = false;
    }
}

void InAppV4ParamScene::onButtonEvent(int buttonTag)
{
    if (buttonTag != BUTTON_OK) {
        return;
    }

    setCompletionHandler();
    setCompletionHandler = NULL;
    this->popView();
}

void InAppV4ParamScene::showParameters(int tag)
{
    api = static_cast<C2SModuleAPIList::InAppV4API>(tag);
    switch (api) {
        case C2SModuleAPIList::INAPPV4_GET_MARKET_PRODUCT_INFO:
        {
            showMarketProductInfoParam([=](){
                string marketPidList = getEditboxValue(EDITBOX_MARKET_PID_LIST);
                ParameterDataManager::getInstance()->inAppV4Params->setMarketProductInfoPidList(marketPidList);
            });
            break;
        }
        case C2SModuleAPIList::INAPPV4_PURCHSE:
        {
            showPurchaseParam([=](){
                string marketPid = getComboboxValue(COMBOBOX_MARKET_PID);
                ParameterDataManager::getInstance()->inAppV4Params->setMarketPid(marketPid);
            });
            break;
        }
        case C2SModuleAPIList::INAPPV4_PURCHASE_SUBSCRIPTION:
        {
            showPurchaseSubscriptionParam([=](){
                string marketPid = getComboboxValue(COMBOBOX_MARKET_PID);
                ParameterDataManager::getInstance()->inAppV4Params->setMarketPid(marketPid);
            });
            break;
        }
        case C2SModuleAPIList::INAPPV4_TRANSACTION_FINISH:
        {
            showTransactionFinishParam([=](){
                string marketPid = getComboboxValue(COMBOBOX_MARKET_PID);
                ParameterDataManager::getInstance()->inAppV4Params->setMarketPid(marketPid);
            });
            break;
        }
        case C2SModuleAPIList::INAPPV4_SET_UID:
        {
            showSetUidParam([=](){
                string uid = getEditboxValue(EDITBOX_UID);
                ParameterDataManager::getInstance()->commonParams->setUid(uid);
            });
            break;
        }
        default:
            break;
    }
}

void InAppV4ParamScene::showMarketProductInfoParam(function<void()> completionHandler)
{
    setUseBackButton(true, "- MarketProductInfo -");

    string pidList = ParameterDataManager::getInstance()->inAppV4Params->getMarketProductInfoPidList();

    addEditBox("pidList : ", pidList.c_str(), "", EDITBOX_MARKET_PID_LIST);

    setCompletionHandler = completionHandler;
}

void InAppV4ParamScene::showPurchaseParam(function<void()> completionHandler)
{
    setUseBackButton(true, "- Purchase -");

    vector<string> marketPidList = ParameterDataManager::getInstance()->inAppV4Params->getMarketPidList();
    vector<const char*> marketPids(marketPidList.size(), nullptr);

    for (int i=0; i<marketPidList.size(); i++) {
        marketPids[i] = marketPidList[i].c_str();
    }

    addCombobox("marketPid : ", marketPids, 0, COMBOBOX_MARKET_PID);

    setCompletionHandler = completionHandler;
}

void InAppV4ParamScene::showPurchaseSubscriptionParam(function<void()> completionHandler)
{
    setUseBackButton(true, "- PurchaseSubscription -");

    vector<string> marketPidList = ParameterDataManager::getInstance()->inAppV4Params->getMarketPidList();
    vector<const char*> marketPids(marketPidList.size(), nullptr);

    for (int i=0; i<marketPidList.size(); i++) {
        marketPids[i] = marketPidList[i].c_str();
    }

    addCombobox("marketPid : ", marketPids, 0, COMBOBOX_MARKET_PID);

    setCompletionHandler = completionHandler;
}

void InAppV4ParamScene::showTransactionFinishParam(function<void()> completionHandler)
{
    setUseBackButton(true, "- TransactionFinish -");

    vector<string> marketPidList = ParameterDataManager::getInstance()->inAppV4Params->getMarketPidList();
    vector<const char*> marketPids(marketPidList.size(), nullptr);

    for (int i=0; i<marketPidList.size(); i++) {
        marketPids[i] = marketPidList[i].c_str();
    }

    addCombobox("marketPid : ", marketPids, 0, COMBOBOX_MARKET_PID);

    setCompletionHandler = completionHandler;
}

void InAppV4ParamScene::showSetUidParam(function<void()> completionHandler)
{
    setUseBackButton(true, "- SetUid -");

    string uid = ParameterDataManager::getInstance()->commonParams->getUid();
    
    addEditBox("uid : ", uid.c_str(), "", EDITBOX_UID);

    setCompletionHandler = completionHandler;
}

