//
//  MercuryScene.cpp
//  HIVE_SDK_Cocos_Sample
//
//  Created by sjyong on 2021/09/27.
//

#include "MercuryScene.hpp"
#include "MercuryParamScene.hpp"
#include "C2SModuleAPIList.hpp"
#include "ParameterDataManager.hpp"
#include "ui/CocosGUI.h"

using namespace cocos2d;
using namespace cocos2d::ui;
using namespace hive;

#include <map>
#include <sstream>

using namespace::std;

bool MercuryScene::init()
{
    if(!BaseView::init())
        return false;

    setUseShowLogButton(true);

    setUseBackButton(true, "- Mercury -");

    registerCallback();


    addButton("ShowEx", ButtonMode::NEED_PARAM, C2SModuleAPIList::MERCURY_SHOW_EX);
    addButton("ShowEx (custom viewId)", ButtonMode::NEED_PARAM, C2SModuleAPIList::MERCURY_SHOW_EX_CUSTOM_VIEW_ID);
    addButton("GetCustomViewInfo", ButtonMode::NEED_PARAM, C2SModuleAPIList::MERCURY_GET_CUSTOM_VIEW_INFO);
    addButton("GetCustomViewInfo (custom viewId)", ButtonMode::NEED_PARAM, C2SModuleAPIList::MERCURY_GET_CUSTOM_VIEW_INFO_CUSTOM_VIEW_ID);
    addButton("ShowReviewPopup", ButtonMode::NEED_PARAM, C2SModuleAPIList::MERCURY_SHOW_REVIEW_POPUP);
    addButton("ShowNativeReviewPopup", ButtonMode::NEED_PARAM, C2SModuleAPIList::MERCURY_SHOW_NATIVE_REVIEW_POPUP);
    addButton("ShowMoregamesPopup", ButtonMode::NEED_PARAM, C2SModuleAPIList::MERCURY_SHOW_MORE_GAMES_POPUP);
//    addButton("SetUid", ButtonMode::NEED_PARAM, C2SModuleAPIList::MERCURY_SET_UID);   // removed, C2S.12.0
//    addButton("SetIsUsingStaging", ButtonMode::NEED_PARAM, C2SModuleAPIList::MERCURY_SET_IS_USING_STAGING);   // removed, C2S.12.0
//    addButton("SetLog", ButtonMode::NEED_PARAM, C2SModuleAPIList::MERCURY_SET_LOG);   // removed, C2S.12.0
    addButton("GetVersion", C2SModuleAPIList::MERCURY_GET_VERSION);
    addButton("initImageLoader", C2SModuleAPIList::MERCURY_INIT_IMAGE_LOADER);
    addButton("GetBadge", ButtonMode::NEED_PARAM, C2SModuleAPIList::MERCURY_GET_BADGE);
//    addButton("GetBadgeType", ButtonMode::NEED_PARAM, C2SModuleAPIList::MERCURY_GET_BADGE_TYPE);  // removed, C2S.19.6
    addButton("RequestUAInfo", ButtonMode::NEED_PARAM, C2SModuleAPIList::MERCURY_REQUEST_UA_INFO);
    addButton("RequestUAILinkShare", ButtonMode::NEED_PARAM, C2SModuleAPIList::MERCURY_REQUEST_UA_LINK_SHARE);
    addButton("GetBannerInfo", ButtonMode::NEED_PARAM, C2SModuleAPIList::MERCURY_GET_BANNER_INFO);
    addButton("setBannerOnGameWindow(forSmon)", C2SModuleAPIList::MERCURY_SET_BANNER_ON_GAME_WINDOW);

    scheduleUpdate();
    
    return true;
}

void MercuryScene::registerCallback()
{
    // 콜백 등록
    CS_MercurySetCallBack([](MERCURY_RESULT result){
        stringstream log;
        log << "result : " << mercuryResultToString((int)result) << '\n';

        addCallbackLogMessage(log.str(), "MercuryCB : ");
    });
}

void MercuryScene::update(float dt)
{
    BaseView::update(dt);

    static bool firstRun = true;
    if (firstRun) {
        firstRun = false;
    }
}

void MercuryScene::onButtonEvent(int buttonTag)
{
    // Parameter 세팅 화면 생성 및 이동
    if (buttonTag >= DEFAULT_PARAM_BUTTON_TAG) {
        MercuryParamScene* paramScene = MercuryParamScene::create();
        BaseView::pushView(paramScene);
        paramScene->showParameters(buttonTag % DEFAULT_PARAM_BUTTON_TAG);
        return;
    }

    // API 호출
    C2SModuleAPIList::MercuryAPI api = static_cast<C2SModuleAPIList::MercuryAPI>(buttonTag);
    switch (api) {
        case C2SModuleAPIList::MERCURY_SHOW_EX:
        {
            string uid = ParameterDataManager::getInstance()->commonParams->getUid();
            MERCURY_SHOW_TYPE showType = ParameterDataManager::getInstance()->mercuryParams->getShowType();
            string additionalInfo = ParameterDataManager::getInstance()->mercuryParams->getAdditionalInfo();

            CS_MercuryShowEx(uid.c_str(), showType, additionalInfo.c_str());

            addLogMessage("CS_MercuryShowEx called.");
            break;
        }
        case C2SModuleAPIList::MERCURY_SHOW_EX_CUSTOM_VIEW_ID:
        {
            string uid = ParameterDataManager::getInstance()->commonParams->getUid();
            int showType = ParameterDataManager::getInstance()->mercuryParams->getViewID();
            string additionalInfo = ParameterDataManager::getInstance()->mercuryParams->getAdditionalInfo();

            CS_MercuryShowEx(uid.c_str(), (MERCURY_SHOW_TYPE)showType, additionalInfo.c_str()); // viewId(int)를 직접 입력하여 호출

            addLogMessage("CS_MercuryShowEx(custom viewId) called.");
            break;
        }
        case C2SModuleAPIList::MERCURY_GET_CUSTOM_VIEW_INFO:
        {
            string uid = ParameterDataManager::getInstance()->commonParams->getUid();
            MERCURY_SHOW_TYPE showType = ParameterDataManager::getInstance()->mercuryParams->getShowType();

            int result = CS_MercuryGetCustomViewInfo(uid.c_str(), (int)showType, [](const MERCURY_CUSTOM_RESULT result, const char* jsonData){
                stringstream log;
                log << "result : " << mercuryCustomResultToString(result) << '\n';
                log << "jsonData : " << jsonData << '\n';

                addCallbackLogMessage(log.str(), "MercuryCBWithData : ");
            });

            stringstream log;
            log << "CS_MercuryGetCustomViewInfo called. result : " << result << '\n';
            addLogMessage(log.str());
            break;
        }
        case C2SModuleAPIList::MERCURY_GET_CUSTOM_VIEW_INFO_CUSTOM_VIEW_ID:
        {
            string uid = ParameterDataManager::getInstance()->commonParams->getUid();
            int viewID = ParameterDataManager::getInstance()->mercuryParams->getViewID();

            int result = CS_MercuryGetCustomViewInfo(uid.c_str(), viewID, [](const MERCURY_CUSTOM_RESULT result, const char* jsonData){
                stringstream log;
                log << "result : " << mercuryCustomResultToString(result) << '\n';
                log << "jsonData : " << jsonData << '\n';

                addCallbackLogMessage(log.str(), "MercuryCBWithData : ");
            });

            stringstream log;
            log << "CS_MercuryGetCustomViewInfo(custom viewId) called. result : " << result << '\n';
            addLogMessage(log.str());
            break;
        }
        case C2SModuleAPIList::MERCURY_SHOW_REVIEW_POPUP:
        {
            string uid = ParameterDataManager::getInstance()->commonParams->getUid();
            string vid = ParameterDataManager::getInstance()->commonParams->getVid();
            string did = ParameterDataManager::getInstance()->commonParams->getDid();

            int result = CS_MercuryShowReviewPopup(uid.c_str(), vid.c_str(), did.c_str());

            stringstream log;
            log << "CS_MercuryShowReviewPopup called. result : " << result << '\n';

            addLogMessage(log.str());
            break;
        }
        case C2SModuleAPIList::MERCURY_SHOW_NATIVE_REVIEW_POPUP:
        {
            string uid = ParameterDataManager::getInstance()->commonParams->getUid();
            string vid = ParameterDataManager::getInstance()->commonParams->getVid();
            string did = ParameterDataManager::getInstance()->commonParams->getDid();

            int result = CS_MercuryShowNativeReviewPopup(uid.c_str(), vid.c_str(), did.c_str());

            stringstream log;
            log << "CS_MercuryShowNativeReviewPopup called. result : " << result << '\n';

            addLogMessage(log.str());
            break;
        }
        case C2SModuleAPIList::MERCURY_SHOW_MORE_GAMES_POPUP:
        {
            string uid = ParameterDataManager::getInstance()->commonParams->getUid();
            string vid = ParameterDataManager::getInstance()->commonParams->getVid();
            string did = ParameterDataManager::getInstance()->commonParams->getDid();

            int result = CS_MercuryShowMoregamesPopup(uid.c_str(), vid.c_str(), did.c_str());

            stringstream log;
            log << "CS_MercuryShowMoregamesPopup called. result : " << result << '\n';

            addLogMessage(log.str());
            break;
        }
        case C2SModuleAPIList::MERCURY_SET_UID:
        {
            string uid = ParameterDataManager::getInstance()->commonParams->getUid();

//            CS_MercurySetUid(uid.c_str());        // deprecated

            addLogMessage("CS_MercurySetUid called.");
            addLogMessage("( CS_MercurySetUid API는 KS C2S에서 더이상 작동하지 않습니다.(deprecated 예정) )");
            break;
        }
        case C2SModuleAPIList::MERCURY_SET_IS_USING_STAGING:
        {
            int isUsingStaging = ParameterDataManager::getInstance()->mercuryParams->getIsUsingStaging();

//            CS_MercurySetIsUsingStaging(isUsingStaging);      // deprecated

            addLogMessage("CS_MercurySetIsUsingStaging called.");
            addLogMessage("( CS_MercurySetIsUsingStaging API는 KS C2S에서 더이상 작동하지 않습니다.(deprecated 예정) )");
            break;
        }
        case C2SModuleAPIList::MERCURY_SET_LOG:
        {
            int isLog = ParameterDataManager::getInstance()->mercuryParams->getIsLog();

//            CS_MercurySetLog(isLog);      // deprecated

            stringstream log;
            log << " : " << to_string(isLog);
            addLogMessage("CS_MercurySetLog called.");
            addLogMessage("CS_MercurySetLog deprecated.");
            addLogMessage(log.str());       // 1: true, 0: false
            break;
        }
        case C2SModuleAPIList::MERCURY_GET_VERSION:
        {
            string version = CS_MercuryGetVersion();

            stringstream log;
            log << "CS_MercuryGetVersion called. version : " << version << '\n';

            addLogMessage(log.str());
            break;
        }
        case C2SModuleAPIList::MERCURY_INIT_IMAGE_LOADER:
        {
            CS_MercuryInitImageLoader();

            addLogMessage("CS_MercuryInitImageLoader called.");
            break;
        }
        case C2SModuleAPIList::MERCURY_GET_BADGE:
        {
            int badgeTarget = ParameterDataManager::getInstance()->mercuryParams->getBadgeTarget();

            CS_MercuryGetBadge(badgeTarget);

            addLogMessage("CS_MercuryGetBadge called.");
            break;
        }
        case C2SModuleAPIList::MERCURY_GET_BADGE_TYPE:
        {
            // deprecated
//            int badgeTarget = ParameterDataManager::getInstance()->mercuryParams->getBadgeTarget();
//            int badgeType = CS_MercuryGetBadgeType(badgeTarget);
//
            stringstream log;
//            log << "CS_MercuryGetBadgeType called. badgeType : " << badgeType << '\n';
            log << "CS_MercuryGetBadgeType called. deprecated.";

            addLogMessage(log.str());
            break;
        }
        case C2SModuleAPIList::MERCURY_REQUEST_UA_INFO:
        {
            string uid = ParameterDataManager::getInstance()->commonParams->getUid();
            string additionalInfo = ParameterDataManager::getInstance()->mercuryParams->getAdditionalInfo();

            if(additionalInfo == "") {
                // test를 위한 uatoken 값 설정
                additionalInfo = "{\"uatoken\":\"asdfsafsdfs\"}";
            }

            int result = CS_MercuryRequestUAInfo(uid.c_str(), additionalInfo.c_str(), [](const MERCURY_CUSTOM_RESULT result, const char* jsonData){
                stringstream log;
                log << "result : " << mercuryCustomResultToString(result) << '\n';
                log << "jsonData : " << jsonData << '\n';

                addCallbackLogMessage(log.str(), "MercuryCBWithData : ");
            });

            stringstream log;
            log << "CS_MercuryRequestUAInfo called. result : " << result << '\n';
            addLogMessage(log.str());
            break;
        }
        case C2SModuleAPIList::MERCURY_REQUEST_UA_LINK_SHARE:
        {
            string urlLink = ParameterDataManager::getInstance()->mercuryParams->getUrlLink();
            string shareMessage = ParameterDataManager::getInstance()->mercuryParams->getShareMessage();

            int result = CS_MercuryRequestUALinkShare(urlLink.c_str(), shareMessage.c_str());

            stringstream log;
            log << "CS_MercuryRequestUALinkShare called. result : " << result << '\n';
            addLogMessage(log.str());
            break;
        }
        case C2SModuleAPIList::MERCURY_GET_BANNER_INFO:
        {
            string campaignType = ParameterDataManager::getInstance()->mercuryParams->getCampaignType();
            string bannerType = ParameterDataManager::getInstance()->mercuryParams->getBannerType();

            int result = CS_MercuryGetBannerInfo(campaignType.c_str(), bannerType.c_str(), [](const MERCURY_CUSTOM_RESULT result, const char* jsonData){
                stringstream log;
                log << "result : " << mercuryCustomResultToString(result) << '\n';
                log << "jsonData : " << jsonData << '\n';

                addCallbackLogMessage(log.str(), "MercuryCBWithData : ");
            });

            stringstream log;
            log << "CS_MercuryGetBannerInfo called. result : " << result << '\n';
            addLogMessage(log.str());
            break;
        }
        case C2SModuleAPIList::MERCURY_SET_BANNER_ON_GAME_WINDOW:
        {
            int onGameWindow = ParameterDataManager::getInstance()->mercuryParams->getOnGameWindow();

            // toggle
            if(onGameWindow == 0) {
                onGameWindow += 1;
            } else {
                onGameWindow = 0;
            }
            ParameterDataManager::getInstance()->mercuryParams->setOnGameWindow(onGameWindow);

            int result = CS_MercurySetBannerOnGameWindow(onGameWindow);

            stringstream log;
            log << "CS_MercurySetBannerOnGameWindow called. result : " << to_string(result) << '\n';
            addLogMessage(log.str());
            break;
        }
        default:
            break;
    }
}

string MercuryScene::mercuryResultToString(int value)
{
    string data = "";

    switch (value) {
        case 1:
            data = "MERCURY_OPEN";
            break;
        case 2:
            data = "MERCURY_CLOSE";
            break;
        case 3:
            data = "MERCURY_FORCED_OPEN";
            break;
        case 4:
            data = "MERCURY_FORCED_CLOSE";
            break;
        case 5:
            data = "MERCURY_NOTICE_OPEN";
            break;
        case 6:
            data = "MERCURY_NOTICE_CLOSE";
            break;

        case 7:
            data = "MERCURY_CUSTOM_OPEN";
            break;
        case 8:
            data = "MERCURY_CUSTOM_CLOSE";
            break;

        case 9:
            data = "MERCURY_CUSTOM_VIEW_OPEN";
            break;
        case 10:
            data = "MERCURY_CUSTOM_VIEW_CLOSE";
            break;

        case 11:
            data = "MERCURY_REVIEW_OPEN";
            break;
        case 12:
            data = "MERCURY_REVIEW_CLOSE";
            break;

        case 13:
            data = "MERCURY_MOREGAMES_OPEN";
            break;
        case 14:
            data = "MERCURY_MOREGAMES_CLOSE";
            break;
        case 15:
            data = "MERCURY_MOREGAMES_CLOSE_WITH_APP_TERMINATION";
            break;

        case 16:
            data = "MERCURY_NETWORK_DISCONNECT";
            break;

        case 17:
            data = "MERCURY_START_PLAYBACK";
            break;
        case 18:
            data = "MERCURY_FINISH_PLAYBACK";
            break;

        case 19:
            data = "MERCURY_MAIN_GOBACK";
            break;
        case 20:
            data = "MERCURY_NOTICE_GOBACK";
            break;
        case 21:
            data = "MERCURY_CUSTOM_BOARD_GOBACK";
            break;
        case 22:
            data = "MERCURY_CUSTOM_VIEW_GOBACK";
            break;

        // badge for what
        case 0:
            data = "MERCURY_BADGE_FOR_ALL";
            break;
        case -20:
            data = "MERCURY_BADGE_FOR_EVENT";
            break;
        case -21:
            data = "MERCURY_BADGE_FOR_NOTICE";
            break;
        case -22:
            data = "MERCURY_BADGE_FOR_NOTICE_ONLY";
            break;
        case -23:
            data = "MERCURY_BADGE_FOR_NONE";
            break;

        // badge type
        case -24:
            data = "MERCURY_BADGE_TYPE_NONE";
            break;
        case -25:
            data = "MERCURY_BADGE_TYPE_NEW";
            break;
        case -26:
            data = "MERCURY_BADGE_TYPE_MAX";
            break;

        default:
            data = "UNKNOWN";

    }

    stringstream returnValue;
    returnValue << data << "(" << to_string(value) << ")" << '\n';
    return returnValue.str();
}

string MercuryScene::mercuryCustomResultToString(int value)
{
    switch (value) {
        case MERCURY_SUCCESS:
            return "MERCURY_SUCCESS";
        case MERCURY_FAILED:
            return "MERCURY_FAILED";
        case MERCURY_INVALID_PARAM:
            return "MERCURY_INVALID_PARAM";
        default:
            return "UNKNOWN";
    }
}
