//
//  OfferwallParamScene.cpp
//  HIVE_SDK_Cocos_Sample
//
//  Created by sjyong on 2021/10/19.
//

#include "OfferwallParamScene.hpp"
#include "ParameterDataManager.hpp"
#include "ui/CocosGUI.h"

using namespace cocos2d;
using namespace cocos2d::ui;
using namespace hive;

#include <map>

using namespace std;

bool OfferwallParamScene::init()
{
    if (!BaseView::init())
        return false;

    addButton("OK", BUTTON_OK);

    scheduleUpdate();

    return true;
}

void OfferwallParamScene::update(float dt)
{
    BaseView::update(dt);

    static bool firstRun = true;
    if (firstRun) {
        firstRun = false;
    }
}

void OfferwallParamScene::onButtonEvent(int buttonTag)
{
    if (buttonTag != BUTTON_OK) {
        return;
    }

    setCompletionHandler();
    setCompletionHandler = NULL;
    this->popView();
}

void OfferwallParamScene::showParameters(int tag)
{
    api = static_cast<C2SModuleAPIList::OfferwallAPI>(tag);
    switch (api) {
        case C2SModuleAPIList::OFFERWALL_INITIALIZE:
        {
            showInitializeParam([=](){
                string uid = getEditboxValue(EDITBOX_UID);
                int isUsingStaging = getSelectedToggleButtonIndex(TOGGLE_BUTTON_IS_LOG);

                ParameterDataManager::getInstance()->commonParams->setUid(uid);
                ParameterDataManager::getInstance()->offerwallParams->setIsUsingStaging(isUsingStaging);
            });
            break;
        }
        case C2SModuleAPIList::OFFERWALL_INITIALIZE_EX:
        {
            showInitializeExParam([=](){
                string uid = getEditboxValue(EDITBOX_UID);
                int isUsingStaging = getSelectedToggleButtonIndex(TOGGLE_BUTTON_IS_LOG);

                ParameterDataManager::getInstance()->commonParams->setUid(uid);
                ParameterDataManager::getInstance()->offerwallParams->setIsUsingStaging(isUsingStaging);
            });
            break;
        }
        case C2SModuleAPIList::OFFERWALL_SET_LOG:
        {
            showSetLogParam([=](){
                int isLog = getSelectedToggleButtonIndex(TOGGLE_BUTTON_IS_LOG);

                ParameterDataManager::getInstance()->offerwallParams->setIsLog(isLog);
            });
            break;
        }
        case C2SModuleAPIList::OFFERWALL_SHOW_EX:
        {
            showShowExParam([=](){
                string additionalInfo = getEditboxValue(EDITBOX_ADDITIONAL_INFO);

                ParameterDataManager::getInstance()->offerwallParams->setAdditionalInfo(additionalInfo);
            });
            break;
        }
        default:
            break;
    }
}

void OfferwallParamScene::showInitializeParam(function<void()> completionHandler)
{
    setUseBackButton(true, "- Initialize -");

    string uid = ParameterDataManager::getInstance()->commonParams->getUid();
    int isUsingStaging = ParameterDataManager::getInstance()->offerwallParams->getIsUsingStaging();

    addEditBox("uid : ", uid.c_str(), "", EDITBOX_UID);
    addTrueFalseToggleButton("isUsingStaging : ", isUsingStaging, {screenSize.width, DEFAULT_CONTENT_HEIGHT}, TOGGLE_BUTTON_IS_USING_STAGING);

    setCompletionHandler = completionHandler;
}

void OfferwallParamScene::showInitializeExParam(function<void()> completionHandler)
{
    setUseBackButton(true, "- InitializeEx -");

    string uid = ParameterDataManager::getInstance()->commonParams->getUid();
    int isUsingStaging = ParameterDataManager::getInstance()->offerwallParams->getIsUsingStaging();

    addEditBox("uid : ", uid.c_str(), "", EDITBOX_UID);
    addTrueFalseToggleButton("isUsingStaging : ", isUsingStaging, {screenSize.width, DEFAULT_CONTENT_HEIGHT}, TOGGLE_BUTTON_IS_USING_STAGING);

    setCompletionHandler = completionHandler;
}

void OfferwallParamScene::showSetLogParam(function<void()> completionHandler)
{
    setUseBackButton(true, "- SetLog -");

    int isLog = ParameterDataManager::getInstance()->offerwallParams->getIsLog();

    addTrueFalseToggleButton("isLog : ", isLog, {screenSize.width, DEFAULT_CONTENT_HEIGHT}, TOGGLE_BUTTON_IS_LOG);

    setCompletionHandler = completionHandler;
}

void OfferwallParamScene::showShowExParam(function<void()> completionHandler)
{
    setUseBackButton(true, "- ShowEx -");

    string additionalInfo = ParameterDataManager::getInstance()->offerwallParams->getAdditionalInfo();

    addEditBox("additionalInfo : ", additionalInfo.c_str(), "", EDITBOX_ADDITIONAL_INFO);

    setCompletionHandler = completionHandler;
}
