//
//  PushScene.cpp
//  HIVE_SDK_Cocos_Sample
//
//  Created by sjyong on 2021/09/27.
//

#include "PushScene.hpp"
#include "C2SModuleAPIList.hpp"
#include "PushParamScene.hpp"
#include "ParameterDataManager.hpp"
#include "ui/CocosGUI.h"

using namespace cocos2d;
using namespace cocos2d::ui;
using namespace hive;

#include <map>
#include <sstream>

using namespace::std;

// 로컬 푸시 스트레스 테스트. 등록 가능한 최대 개수
const int STRESS_LOCAL_PUSH_COUNT = 64;
// 일정 시간 간격으로 스트레스 로컬 푸시를 수신 받도록 설정(5초)
const int STRESS_LOCAL_PUSH_TIME_INTERVAL = 5;
// 일정 시간 간격으로 현재 시간을 담은 레이블을 갱신 (0.1초)
const int UPDATE_LOCAL_PUSH_LABEL_TIME_INTERVAL = 0.1;

bool PushScene::init()
{
    if(!BaseView::init())
        return false;

    setUseShowLogButton(true);

    setUseBackButton(true, "- Push -");

    // MARK: Common
    addLabel("- Common -", LinearGravity::CENTER_HORIZONTAL, DEFAULT_FONT_SIZE, Node::INVALID_TAG);
//    addButton("StartEx", C2SModuleAPIList::PUSH_START_EX);        // removed, C2S.20.0
    addButton("GetAgreeNotice", C2SModuleAPIList::PUSH_GET_AGREE_NOTICE);
    addButton("GetAgreeNight", C2SModuleAPIList::PUSH_GET_AGREE_NIGHT);
    addButton("RegisterLocalPush", ButtonMode::NEED_PARAM, C2SModuleAPIList::PUSH_REGISTER_LOCAL_PUSH);
    addButton("RegisterLocalPushEx", ButtonMode::NEED_PARAM, C2SModuleAPIList::PUSH_REGISTER_LOCAL_PUSH_EX);
    addButton("UnRegisterLocalPush", ButtonMode::NEED_PARAM, C2SModuleAPIList::PUSH_UNREGISTER_LOCAL_PUSH);
    addButton("SetAgree", ButtonMode::NEED_PARAM, C2SModuleAPIList::PUSH_SET_AGREE);
    addButton("GetVersionEx", C2SModuleAPIList::PUSH_GET_VERSION_EX);
//    addButton("SetUid", ButtonMode::NEED_PARAM, C2SModuleAPIList::PUSH_SET_UID);      // removed, C2S.20.0

    // LocalPush Stress
    addLabel("- Local Push stress test -", LinearGravity::CENTER_HORIZONTAL, DEFAULT_FONT_SIZE, LocalPushStressButton::LABEL);
    addButton("RegisterLocalPush (1~64)", LocalPushStressButton::REGISTER);
    addButton("UnRegisterLocalPush (1~64)", LocalPushStressButton::UNREGISTER);
    addButton("RequestPermission", C2SModuleAPIList::PUSH_REQUEST_PERMISSION);

    // MARK: iOS
#if (CC_TARGET_PLATFORM == CC_PLATFORM_IOS)
    addLabel("- iOS only -", LinearGravity::CENTER_HORIZONTAL, DEFAULT_FONT_SIZE, Node::INVALID_TAG);
    addButton("SetForeground", ButtonMode::NEED_PARAM, C2SModuleAPIList::PUSH_SET_FOREGROUND);
    addButton("GetForegroundRemote", C2SModuleAPIList::PUSH_GET_FOREGROUND_REMOTE);
    addButton("GetForegroundLocal", C2SModuleAPIList::PUSH_GET_FOREGROUND_LOCAL);

    // MARK: Android
#elif (CC_TARGET_PLATFORM == CC_PLATFORM_ANDROID)
    addLabel("- Android only -", LinearGravity::CENTER_HORIZONTAL, DEFAULT_FONT_SIZE, Node::INVALID_TAG);
    addButton("SetPush", ButtonMode::NEED_PARAM, C2SModuleAPIList::PUSH_SET_PUSH);
//    addButton("SetSound", ButtonMode::NEED_PARAM, C2SModuleAPIList::PUSH_SET_SOUND);      // removed, C2S.20.0
//    addButton("SetVib", ButtonMode::NEED_PARAM, C2SModuleAPIList::PUSH_SET_VIB);      // removed, C2S.20.0
    addButton("GetPush", C2SModuleAPIList::PUSH_GET_PUSH);
//    addButton("GetSound", C2SModuleAPIList::PUSH_GET_SOUND);      // removed, C2S.20.0
//    addButton("GetVib", C2SModuleAPIList::PUSH_GET_VIB);      // removed, C2S.20.0
    addButton("RegisterLocalPushBig", ButtonMode::NEED_PARAM, C2SModuleAPIList::PUSH_REGISTER_LOCAL_PUSH_BIG);
    addButton("UnregisterAllLocalPush", C2SModuleAPIList::PUSH_UNREGISTER_ALL_LOCAL_PUSH);
    addButton("RegisterCallbackHandler", C2SModuleAPIList::PUSH_REGISTER_CALLBACK_HANDLER);
    addButton("UnregisterCallbackHandler", C2SModuleAPIList::PUSH_UNREGISTER_CALLBACK_HANDLER);
    addButton("SetOperationLocalPushOnRunning", ButtonMode::NEED_PARAM, C2SModuleAPIList::PUSH_SET_OPERATION_LOCAL_PUSH_ON_RUNNING);
    addButton("SetOperationGCMPushOnRunning", ButtonMode::NEED_PARAM, C2SModuleAPIList::PUSH_SET_OPERATION_GCM_PUSH_ON_RUNNING);
    addButton("GetRegistrationId", C2SModuleAPIList::PUSH_GET_REGISTRATION_ID);
#endif

    scheduleUpdate();
    
    schedule(schedule_selector(PushScene::updateLabelLocalPushStressTest), UPDATE_LOCAL_PUSH_LABEL_TIME_INTERVAL);

    return true;
}

void PushScene::update(float dt)
{
    BaseView::update(dt);

    static bool firstRun = true;
    if (firstRun) {
        firstRun = false;
    }
}

void PushScene::onButtonEvent(int buttonTag)
{
    // Parameter 세팅 화면 생성 및 이동
    if (buttonTag >= DEFAULT_PARAM_BUTTON_TAG) {
        PushParamScene* paramScene = PushParamScene::create();
        BaseView::pushView(paramScene);
        paramScene->showParameters(buttonTag % DEFAULT_PARAM_BUTTON_TAG);
        return;
    }

    // API 호출
    C2SModuleAPIList::PushAPI api = static_cast<C2SModuleAPIList::PushAPI>(buttonTag);
    switch (api) {
        // MARK: Common
        case C2SModuleAPIList::PUSH_START_EX:
        {
//            CS_pushStartEx();     // deprecated

            addLogMessage("CS_pushStartEx called.");
            addLogMessage("CS_pushStartEx deprecated.");
            break;
        }
        case C2SModuleAPIList::PUSH_GET_AGREE_NOTICE:
        {
            Push_BoolType notice = CS_pushGetAgreeNotice();

            stringstream log;
            log << "CS_pushGetAgreeNotice called. notice : " << bool(notice) << '\n';

            addLogMessage(log.str());
            break;
        }
        case C2SModuleAPIList::PUSH_GET_AGREE_NIGHT:
        {
            Push_BoolType night = CS_pushGetAgreeNight();

            stringstream log;
            log << "CS_pushGetAgreeNight called. night : " << bool(night) << '\n';

            addLogMessage(log.str());
            break;
        }
        case C2SModuleAPIList::PUSH_REGISTER_LOCAL_PUSH:
        {
            int pushID = ParameterDataManager::getInstance()->pushParams->getPushID();
            char* title = stringToChar(ParameterDataManager::getInstance()->pushParams->getTitle());
            char* msg = stringToChar(ParameterDataManager::getInstance()->pushParams->getMsg());
            char* ticker = convertNullInputRegisterLocalPush(ParameterDataManager::getInstance()->pushParams->getTicker());
            char* type = convertNullInputRegisterLocalPush(ParameterDataManager::getInstance()->pushParams->getType());
            char* icon = convertNullInputRegisterLocalPush(ParameterDataManager::getInstance()->pushParams->getIcon());
            char* sound = convertNullInputRegisterLocalPush(ParameterDataManager::getInstance()->pushParams->getSound());
            char* active = convertNullInputRegisterLocalPush(ParameterDataManager::getInstance()->pushParams->getActive());
            long after = ParameterDataManager::getInstance()->pushParams->getAfter();

            CS_pushRegisterLocalpush(pushID, title, msg, ticker, type, icon, sound, active, after);

            // char* 메모리 해제
            delete[] title;
            delete[] msg;
            delete[] ticker;
            delete[] type;
            delete[] icon;
            delete[] sound;
            delete[] active;

            addLogMessage("CS_pushRegisterLocalPush called.");
            break;
        }
        case C2SModuleAPIList::PUSH_REGISTER_LOCAL_PUSH_EX:
        {
            int pushID = ParameterDataManager::getInstance()->pushParams->getPushID();
            char* title = stringToChar(ParameterDataManager::getInstance()->pushParams->getTitle());
            char* msg = stringToChar(ParameterDataManager::getInstance()->pushParams->getMsg());
            char* ticker = convertNullInputRegisterLocalPush(ParameterDataManager::getInstance()->pushParams->getTicker());
            char* type = convertNullInputRegisterLocalPush(ParameterDataManager::getInstance()->pushParams->getType());
            char* icon = convertNullInputRegisterLocalPush(ParameterDataManager::getInstance()->pushParams->getIcon());
            char* sound = convertNullInputRegisterLocalPush(ParameterDataManager::getInstance()->pushParams->getSound());
            char* active = convertNullInputRegisterLocalPush(ParameterDataManager::getInstance()->pushParams->getActive());
            long after = ParameterDataManager::getInstance()->pushParams->getAfter();
            char* broadcastAction = convertNullInputRegisterLocalPush(ParameterDataManager::getInstance()->pushParams->getBroadcastAction());

            CS_pushRegisterLocalpushEx(pushID, title, msg, ticker, type, icon, sound, active, after, broadcastAction);

            // char* 메모리 해제
            delete[] title;
            delete[] msg;
            delete[] ticker;
            delete[] type;
            delete[] icon;
            delete[] sound;
            delete[] active;
            delete[] broadcastAction;

            addLogMessage("CS_pushRegisterLocalPushEx called.");
            break;
        }
        case C2SModuleAPIList::PUSH_UNREGISTER_LOCAL_PUSH:
        {
            int pushID = ParameterDataManager::getInstance()->pushParams->getPushID();

            CS_pushUnRegisterLocalpush(pushID);

            addLogMessage("CS_pushUnRegisterLocalpush called.");
            break;
        }
        case C2SModuleAPIList::PUSH_SET_AGREE:
        {
            Push_BoolType notice = ParameterDataManager::getInstance()->pushParams->getNotice();
            Push_BoolType night = ParameterDataManager::getInstance()->pushParams->getNight();

            CS_pushSetAgree(notice, night);

            addLogMessage("CS_pushSetAgree called.");
            break;
        }
        case C2SModuleAPIList::PUSH_GET_VERSION_EX:
        {
            const char* version = CS_PushGetVersion();

            stringstream log;
            log << "CS_PushGetVersion called. version  : " << version << '\n';

            addLogMessage(log.str());
            break;
        }
        case C2SModuleAPIList::PUSH_SET_UID:
        {
            string uid = ParameterDataManager::getInstance()->commonParams->getUid();

//            CS_pushSetUid(uid.c_str());       // dprecated

            addLogMessage("CS_pushSetUid called.");
            addLogMessage("CS_pushSetUid deprecated.");
            break;
        }
        case C2SModuleAPIList::PUSH_REQUEST_PERMISSION:
        {
            CS_pushRequestPermission();

            addLogMessage("CS_pushRequestPermission called.");
            break;
        }
#if (CC_TARGET_PLATFORM == CC_PLATFORM_IOS)
        // MARK: iOS
        case C2SModuleAPIList::PUSH_SET_FOREGROUND:
        {
            Push_BoolType remote = ParameterDataManager::getInstance()->pushParams->getRemote();
            Push_BoolType local = ParameterDataManager::getInstance()->pushParams->getLocal();

            CS_pushSetForeground(remote, local);

            stringstream log;
            log << "CS_pushSetForeground called." << '\n';

            addLogMessage(log.str());
            break;
        }
        case C2SModuleAPIList::PUSH_GET_FOREGROUND_REMOTE:
        {
            Push_BoolType remote = CS_pushGetForegroundRemote();

            stringstream log;
            log << "CS_pushGetForegroundRemote called. remote : " << pushBoolTypeToString(remote) << '\n';

            addLogMessage(log.str());
            break;
        }
        case C2SModuleAPIList::PUSH_GET_FOREGROUND_LOCAL:
        {
            Push_BoolType local = CS_pushGetForegroundLocal();

            stringstream log;
            log << "CS_pushGetForegroundLocal called. local : " << pushBoolTypeToString(local) << '\n';

            addLogMessage(log.str());
            break;
        }
#elif (CC_TARGET_PLATFORM == CC_PLATFORM_ANDROID)
        // MARK: Android
        case C2SModuleAPIList::PUSH_SET_PUSH:
        {
            Push_BoolType isPush = ParameterDataManager::getInstance()->pushParams->getIsPush();
            CS_pushSetPush(isPush);

            addLogMessage("CS_pushSetPush called.");

            // 현재 설정된 값 로그 추가
            stringstream log;
            log << "\n(now SetPush value : " << bool(isPush) << ")" << '\n';

            addLogMessage(log.str());

            break;
        }
        case C2SModuleAPIList::PUSH_SET_SOUND:
        {
            Push_BoolType isSound = ParameterDataManager::getInstance()->pushParams->getIsSound();
//            CS_pushSetSound(isSound);     // deprecated

            addLogMessage("CS_pushSetSound called.");
            addLogMessage("CS_pushSetSound deprecated.");
            break;
        }
        case C2SModuleAPIList::PUSH_SET_VIB:
        {
            Push_BoolType isVib = ParameterDataManager::getInstance()->pushParams->getIsVib();
//            CS_pushSetVib(isVib);         // deprecated

            addLogMessage("CS_pushSetVib called.");
            addLogMessage("CS_pushSetVib deprecated.");
            break;
        }
        case C2SModuleAPIList::PUSH_GET_PUSH:
        {
            Push_BoolType push = CS_pushGetPush();

            stringstream log;
            log << "CS_pushGetPush called. push : "<< bool(push) << '\n';

            addLogMessage(log.str());
            break;
        }
        case C2SModuleAPIList::PUSH_GET_SOUND:
        {
//            Push_UserSetting sound = CS_pushGetSound();       // deprecated

            stringstream log;
//            log << "CS_pushGetSound called. sound : "<< bool(sound) << '\n';
            log << "CS_pushGetSound called. deprecated. ";

            addLogMessage(log.str());
            break;
        }
        case C2SModuleAPIList::PUSH_GET_VIB:                    // deprecated
        {
//            Push_UserSetting vib = CS_pushGetVib();

            stringstream log;
//            log << "CS_pushGetVib called. vib : "<< bool(vib) << '\n';
            log << "CS_pushGetVib called. deprecated. ";

            addLogMessage(log.str());
            break;
        }
        case C2SModuleAPIList::PUSH_REGISTER_LOCAL_PUSH_DATA:
        {
            char* jsonPushData = stringToChar(ParameterDataManager::getInstance()->pushParams->getJsonPushData());

            CS_pushRegisterLocalpushData(jsonPushData);

            addLogMessage("CS_pushRegisterLocalpushData called.");
            break;
        }
        case C2SModuleAPIList::PUSH_REGISTER_LOCAL_PUSH_BIG:
        {
            int pushID = ParameterDataManager::getInstance()->pushParams->getPushID();
            char* title = stringToChar(ParameterDataManager::getInstance()->pushParams->getTitle());
            char* msg = stringToChar(ParameterDataManager::getInstance()->pushParams->getMsg());
            char* bigmsg = stringToChar(ParameterDataManager::getInstance()->pushParams->getBigMsg());
            char* ticker = stringToChar(ParameterDataManager::getInstance()->pushParams->getTicker());
            char* type = stringToChar(ParameterDataManager::getInstance()->pushParams->getType());
            char* icon = stringToChar(ParameterDataManager::getInstance()->pushParams->getIcon());
            char* sound = stringToChar(ParameterDataManager::getInstance()->pushParams->getActive());
            char* active = stringToChar(ParameterDataManager::getInstance()->pushParams->getActive());
            long after = ParameterDataManager::getInstance()->pushParams->getAfter();
            char* broadcastAction = stringToChar(ParameterDataManager::getInstance()->pushParams->getBroadcastAction());

            CS_pushRegisterLocalpushBig(pushID, title, msg, bigmsg, ticker, type, icon, sound, active, after, broadcastAction);

            // char* 메모리 해제
            delete[] title;
            delete[] msg;
            delete[] bigmsg;
            delete[] ticker;
            delete[] type;
            delete[] icon;
            delete[] sound;
            delete[] active;
            delete[] broadcastAction;

            addLogMessage("CS_pushRegisterLocalpushBig called.");
            break;
        }
        case C2SModuleAPIList::PUSH_UNREGISTER_ALL_LOCAL_PUSH:
        {
            CS_pushUnRegisterAllLocalpush();

            addLogMessage("CS_pushUnRegisterAllLocalpush called.");
            break;
        }
        case C2SModuleAPIList::PUSH_REGISTER_CALLBACK_HANDLER:
        {
            CS_pushRegisterCallbackHandler([](Push_CallbackType callbackType, int pushID, int remainPushCallback) {
                stringstream log;

                switch (callbackType) {
                    case onReceivedGCMPush:
                        log << "RemotePush" << '\n';
                        break;
                    case onReceivedLocalPush:
                        log << "LocalPush" << '\n';
                        break;
                    default:
                        break;
                }

                log << "pushId : ";
                log << to_string(pushID) << '\n';

                log << "remainPushCallback : ";
                log << to_string(remainPushCallback) << '\n';

                addCallbackLogMessage(log.str().c_str(), "PushCallback : ");
            });

            addLogMessage("CS_pushRegisterCallbackHandler called.");
            break;
        }
        case C2SModuleAPIList::PUSH_UNREGISTER_CALLBACK_HANDLER:
        {
            CS_pushUnRegisterCallbackHandler();

            addLogMessage("CS_pushUnRegisterCallbackHandler called.");
            break;
        }
        case C2SModuleAPIList::PUSH_SET_OPERATION_LOCAL_PUSH_ON_RUNNING:
        {
            int boolean = ParameterDataManager::getInstance()->pushParams->getBoolean();
            CS_pushSetOperationLocalPushOnRunning(boolean);

            addLogMessage("CS_pushSetOperationLocalPushOnRunning called.");
            break;
        }
        case C2SModuleAPIList::PUSH_SET_OPERATION_GCM_PUSH_ON_RUNNING:
        {
            int boolean = ParameterDataManager::getInstance()->pushParams->getBoolean();
            CS_pushSetOperationGCMPushOnRunning(boolean);

            addLogMessage("CS_pushSetOperationGCMPushOnRunning called.");
            break;
        }
        case C2SModuleAPIList::PUSH_GET_REGISTRATION_ID:
        {
            const char* regestrationId = CS_pushGetRegistrationId();

            stringstream log;
            log << "CS_pushGetRegistrationId called. regestrationId : " << regestrationId;

            addLogMessage(log.str());
            break;
        }
#endif
        default:
            break;
    }

    // Local Push Stress Test
    LocalPushStressButton stressAPI = static_cast<LocalPushStressButton>(buttonTag);
    switch (stressAPI) {
        case LABEL:
        {
            // do nothing
            break;
        }
        case REGISTER:
        {
            for (int i=1; i<=STRESS_LOCAL_PUSH_COUNT; i++) {
                int pushID = i;
                char* title = stringToChar(StringUtils::format("stress_title %d", i));
                char* msg = stringToChar(StringUtils::format("stress_message %d", i));
                char* ticker = stringToChar(ParameterDataManager::getInstance()->pushParams->getTicker());
                char* type = stringToChar(ParameterDataManager::getInstance()->pushParams->getType());
                char* icon = stringToChar(ParameterDataManager::getInstance()->pushParams->getIcon());
                char* sound = stringToChar(ParameterDataManager::getInstance()->pushParams->getSound());
                char* active = stringToChar(ParameterDataManager::getInstance()->pushParams->getActive());
                long after = STRESS_LOCAL_PUSH_TIME_INTERVAL * i;

                CS_pushRegisterLocalpush(pushID, title, msg, ticker, type, icon, sound, active, after);

                // char* 메모리 해제
                delete[] title;
                delete[] msg;
                delete[] ticker;
                delete[] type;
                delete[] icon;
                delete[] sound;
                delete[] active;
            }

            stringstream log;
            log << "CS_pushRegisterLocalPush (stress) 1~" << STRESS_LOCAL_PUSH_COUNT << " called.";

            addLogMessage(log.str());
            break;
        }
        case UNREGISTER:
        {
            for (int i=1; i<=STRESS_LOCAL_PUSH_COUNT; i++) {
                int pushID = i;
                CS_pushUnRegisterLocalpush(pushID);
            }

            stringstream log;
            log << "CS_pushUnRegisterLocalPush (stress) 1~" << STRESS_LOCAL_PUSH_COUNT << " called.";

            addLogMessage(log.str());
            break;
        }
        default:
            break;
    }
}

// 주의: 해당 함수 사용 완료 후 반환된 char*에 대하여 메모리 해제 필요
char* PushScene::stringToChar(string value)
{
    char* cptr = new char[value.size() + 1];
    copy(value.begin(), value.end(), cptr);
    cptr[value.size()] = '\0';

    return cptr;
}

void PushScene::updateLabelLocalPushStressTest(float dt) {
    std::string currentTime = getCurrentDateTime();
    std::string stressLocalPushLabel = StringUtils::format("- Local Push stress test (%s) -", currentTime.c_str());
    changeLabelText(stressLocalPushLabel, LocalPushStressButton::LABEL);
}

string PushScene::pushBoolTypeToString(Push_BoolType type) {
    switch (type) {
        case Push_FALSE:
            return "Push_FALSE";
        case Push_TRUE:
            return "Push_TRUE";
        default:
            return "UNKNOWN";
    }
}

char* PushScene::convertNullInputRegisterLocalPush(string value) {
    if(value == "null") {
        return NULL;
    }
    return stringToChar(value);
}
