#ifdef __ANDROID__
#include <jni.h>
#include <android/log.h>
#include <string.h>
#include <stdio.h>

#include "C2SHub2Android.h"
#include "C2SHub2Errors.h"

#ifdef __cplusplus
extern "C" {
#endif

static JavaVM* g_javaVM = 0;
static jclass g_hubinterfaceClass = 0;
static jint g_jniVersion = JNI_VERSION_1_6;

static char jcshubGetEnv();

int initializeEx(const char *appId, HiveSocialServerState serverType) {
		JNIEnv *env;
		jmethodID method;
		signed int ret;

		if (jcshubGetEnv(&env)) {
			method = (*env)->GetStaticMethodID(env, g_hubinterfaceClass, "initializeEx", "(Ljava/lang/String;I)I");
			if (!method) {
				return -100;
			} else {
				jstring jAppId;

				jAppId = (*env)->NewStringUTF(env, appId);

				ret = (*env)->CallStaticIntMethod(env, g_hubinterfaceClass, method, jAppId, serverType);

				(*env)->DeleteLocalRef(env, jAppId);
			}
		}
		return ret;
}

// apply to 64bit system
int showDialog(const char *relativePath, HubCallbackWithJSON closeCallback, void *callbackParam) {

	JNIEnv *env;
	jmethodID method;
	signed int ret;

	if (jcshubGetEnv(&env)) {

		method = (*env)->GetStaticMethodID(env, g_hubinterfaceClass, "showDialog", "(Ljava/lang/String;JJ)I");

		if (!method) {
			return -100;
		} else {
			jstring jrelativePath;
			jrelativePath = (*env)->NewStringUTF(env, relativePath);

            jlong cparam = (jlong) closeCallback;
			jlong jparam = (jlong) callbackParam;

            ret = (*env)->CallStaticIntMethod(env, g_hubinterfaceClass, method, jrelativePath, cparam, jparam);

			(*env)->DeleteLocalRef(env, jrelativePath);
		}
	}
	return ret;
}

// apply to 64bit system
int showDialogForChatbot(const char *dataParam, HubCallbackWithJSON closeCallback, void *callbackParam) {

	JNIEnv *env;
	jmethodID method;
	signed int ret;

	if (jcshubGetEnv(&env)) {

		method = (*env)->GetStaticMethodID(env, g_hubinterfaceClass, "showDialogForChatbot", "(Ljava/lang/String;JJ)I");

		if (!method) {
			return -100;
		} else {
			jstring jdataparam;
			jdataparam = (*env)->NewStringUTF(env, dataParam);

			jlong cparam = (jlong) closeCallback;
			jlong jparam = (jlong) callbackParam;

			ret = (*env)->CallStaticIntMethod(env, g_hubinterfaceClass, method, jdataparam, cparam, jparam);

			(*env)->DeleteLocalRef(env, jdataparam);
		}
	}
	return ret;
}

// apply to 64bit system
int asyncRequest(const char *requestName, const char *params, HubCallbackWithJSON closeCallback, void *callbackParam) {
	JNIEnv *env;
	jmethodID method;
	signed int ret;

	if ((requestName == NULL) || (sizeof(requestName) == 0)) {
		return HUB_E_INVALID;
	}

	if (jcshubGetEnv(&env)) {
		method = (*env)->GetStaticMethodID(env, g_hubinterfaceClass, "asyncRequest", "(Ljava/lang/String;Ljava/lang/String;JJ)I");

		if (!method) {
			return -100;
		} else {
			jstring jrequestName, jparams;
			jrequestName = (*env)->NewStringUTF(env, requestName);
            
			jparams = (*env)->NewStringUTF(env, params);

			jlong cParam = (jlong) closeCallback;
			jlong jparam = (jlong) callbackParam;

			ret = (*env)->CallStaticIntMethod(env, g_hubinterfaceClass, method, jrequestName, jparams, cParam, jparam);

			(*env)->DeleteLocalRef(env, jrequestName);
			(*env)->DeleteLocalRef(env, jparams);
		}
	}

	return ret;
}

// apply to 64bit system
int auth(HubCallbackWithJSON closeCallback, void *callbackParam) {
	JNIEnv *env;
	jmethodID method;
	signed int ret;

	if (jcshubGetEnv(&env)) {
		method = (*env)->GetStaticMethodID(env, g_hubinterfaceClass, "auth", "(JJ)I");

		if (!method) {
			return -100;
		} else {

			jlong cParam = (jlong) closeCallback;
			jlong jparam = (jlong) callbackParam;

			ret = (*env)->CallStaticIntMethod(env, g_hubinterfaceClass, method, cParam, jparam);
		}
	}

	return ret;
}

// apply to 64bit system (GCPSDK-567)
int authWithParam(const char *params, HubCallbackWithJSON closeCallback, void *callbackParam) {
	JNIEnv *env;
	jmethodID method;
	signed int ret;
	jstring jParams;

	if (jcshubGetEnv(&env)) {
		method = (*env)->GetStaticMethodID(env, g_hubinterfaceClass, "authWithParam", "(Ljava/lang/String;JJ)I");

		jParams = (*env)->NewStringUTF(env, params);

		if (!method) {
			return -100;
		} else {


			jlong cParam = (jlong) closeCallback;
			jlong jparam = (jlong) callbackParam;

			ret = (*env)->CallStaticIntMethod(env, g_hubinterfaceClass, method, jParams, cParam, jparam);

			(*env)->DeleteLocalRef(env, jParams);
		}
	}

	return ret;
}

// apply to 64bit system
int logout(HubCallbackWithJSON closeCallback, void *callbackParam) {
	JNIEnv *env;
	jmethodID method;
	signed int ret;

	if (jcshubGetEnv(&env)) {
		method = (*env)->GetStaticMethodID(env, g_hubinterfaceClass, "logout", "(JJ)I");

		if (!method) {
			return -100;
		} else {

			jlong cParam = (jlong) closeCallback;
            jlong jparam = (jlong) callbackParam;

			ret = (*env)->CallStaticIntMethod(env, g_hubinterfaceClass, method, cParam, jparam);
		}
	}

	return ret;
}

// apply to 64bit system
int userDelete(HubCallbackWithJSON closeCallback, void *callbackParam) {
	JNIEnv *env;
	jmethodID method;
	signed int ret;

	if (jcshubGetEnv(&env)) {
		method = (*env)->GetStaticMethodID(env, g_hubinterfaceClass, "userDelete", "(JJ)I");

		if (!method) {
			return -100;
		} else {

			jlong cParam = (jlong) closeCallback;
			jlong jparam = (jlong) callbackParam;

			ret = (*env)->CallStaticIntMethod(env, g_hubinterfaceClass, method, cParam, jparam);
		}
	}

	return ret;
}

// apply to 64bit system
int guestAcquireUid(HubCallbackWithJSON closeCallback, void *callbackParam) {
	JNIEnv *env;
	jmethodID method;
	signed int ret;

	if (jcshubGetEnv(&env)) {
		method = (*env)->GetStaticMethodID(env, g_hubinterfaceClass, "guestAcquireUid", "(JJ)I");

		if (!method) {
			return -100;
		} else {

			jlong cParam = (jlong) closeCallback;
            jlong jparam = (jlong) callbackParam;

			ret = (*env)->CallStaticIntMethod(env, g_hubinterfaceClass, method, cParam, jparam);
		}
	}

	return ret;
}

// apply to 64bit system
int guestBind(const char *guestUid, const char *candidateUid, HubCallbackWithJSON closeCallback, void *callbackParam) {
	JNIEnv *env;
	jmethodID method;
	signed int ret;
	jstring jGuestUid, jCandidateUid;

	if (jcshubGetEnv(&env)) {
		jGuestUid = (*env)->NewStringUTF(env, guestUid);
		jCandidateUid = (*env)->NewStringUTF(env, candidateUid);
		method = (*env)->GetStaticMethodID(env, g_hubinterfaceClass, "guestBind", "(Ljava/lang/String;Ljava/lang/String;JJ)I");
		if (!method) {
			return -100;
		} else {
			jlong cParam = (jlong) closeCallback;
            jlong jparam = (jlong) callbackParam;

			ret = (*env)->CallStaticIntMethod(env, g_hubinterfaceClass, method, jGuestUid, jCandidateUid, cParam, jparam);

			(*env)->DeleteLocalRef(env, jGuestUid);
			(*env)->DeleteLocalRef(env, jCandidateUid);
		}
	}
	return ret;
}

// apply to 64bit system
int socialRequestWithService(const char *socialService, const char *actionName, const char *params, HubCallbackWithJSON responseCallback, void *callbackParam) {
	JNIEnv *env;
	jmethodID method;
	signed int ret;
	jstring jSocialService, jActionName, jParams;

	if (jcshubGetEnv(&env)) {
		jSocialService = (*env)->NewStringUTF(env, socialService);
		jActionName = (*env)->NewStringUTF(env, actionName);
		jParams = (*env)->NewStringUTF(env, params);
		method = (*env)->GetStaticMethodID(env, g_hubinterfaceClass, "socialRequest", "(Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;JJ)I");
		if (!method) {
			return -100;
		} else {
            jlong cparam = (jlong) responseCallback;
            jlong jparam = (jlong) callbackParam;

            ret = (*env)->CallStaticIntMethod(env, g_hubinterfaceClass, method, jSocialService, jActionName, jParams, cparam, jparam);

			(*env)->DeleteLocalRef(env, jSocialService);
			(*env)->DeleteLocalRef(env, jActionName);
			(*env)->DeleteLocalRef(env, jParams);
		}
	}

	return ret;
}

// 2.9.14 - pgs auto login enable
int setPGSAutoLogin(){
	JNIEnv *env;
	jmethodID method;
	signed int ret;

	if (jcshubGetEnv(&env)) {
		method = (*env)->GetStaticMethodID(env, g_hubinterfaceClass, "setPGSAutoLogin", "()I");
		if (!method) {
			return -100;
		} else {
			ret = (*env)->CallStaticIntMethod(env, g_hubinterfaceClass, method);
		}
	}

	return ret;
}

int setPGSAutoLoginWithCB(HubCallbackWithJSON closeCallback, void *callbackParam) {
	JNIEnv *env;
	jmethodID method;
	signed int ret;

	if (jcshubGetEnv(&env)) {
		method = (*env)->GetStaticMethodID(env, g_hubinterfaceClass, "setPGSAutoLoginWithCB", "(JJ)I");

		if (!method) {
			return -100;
		} else {

			jlong cParam = (jlong) closeCallback;
			jlong jparam = (jlong) callbackParam;

			ret = (*env)->CallStaticIntMethod(env, g_hubinterfaceClass, method, cParam, jparam);
		}
	}

	return ret;
}

// auth by PGS
int authByPGS(HubCallbackWithJSON closeCallback, void *callbackParam) {
	JNIEnv *env;
	jmethodID method;
	signed int ret;

	if (jcshubGetEnv(&env)) {
		method = (*env)->GetStaticMethodID(env, g_hubinterfaceClass, "authByPGS", "(JJ)I");

		if (!method) {
			return -100;
		} else {

			jlong cParam = (jlong) closeCallback;
			jlong jparam = (jlong) callbackParam;

			ret = (*env)->CallStaticIntMethod(env, g_hubinterfaceClass, method, cParam, jparam);
		}
	}

	return ret;
}

int isPGSAvailable() {
	JNIEnv *env;
	jmethodID method;
	signed int ret;

	if (jcshubGetEnv(&env)) {
		method = (*env)->GetStaticMethodID(env, g_hubinterfaceClass, "isPGSAvailable", "()I");

		if (!method) {
			return -100;
		} else {
			ret = (*env)->CallStaticIntMethod(env, g_hubinterfaceClass, method);
		}
	}

	return ret;
}

int setAuthDialogClear() {
	JNIEnv *env;
	jmethodID method;
	signed int ret;

	if (jcshubGetEnv(&env)) {
		method = (*env)->GetStaticMethodID(env, g_hubinterfaceClass, "setAuthDialogClear", "()I");

		if (!method) {
			return -100;
		} else {
			ret = (*env)->CallStaticIntMethod(env, g_hubinterfaceClass, method);
		}
	}

	return ret;
}

static char jcshubGetEnv(JNIEnv **jEnv) {
	if ((g_javaVM == 0) || (g_hubinterfaceClass == 0)) {
		return 0;
	}

	if ((*g_javaVM)->GetEnv(g_javaVM, (void **) jEnv, g_jniVersion) != JNI_OK) {
		return 0;
	}

	return 1;
}

JNIEXPORT void JNICALL
Java_com_com2us_peppermint_HubBridge_callCSHubInitialize(JNIEnv *env, jclass cls) {
	(*env)->GetJavaVM(env, &g_javaVM);

	g_hubinterfaceClass = (*env)->FindClass(env, "com/com2us/peppermint/HubBridge");
	g_hubinterfaceClass = (*env)->NewGlobalRef(env, g_hubinterfaceClass);
}

JNIEXPORT void JNICALL
Java_com_com2us_peppermint_HubBridge_hubCallbackWithJSON(JNIEnv *env, jclass cls, jstring jResponseJSONString, jlong callback, jlong callbackParam) {
	void *userParam = (void*) callbackParam;
	char *jsonString;

	jsonString = (char*) (*env)->GetStringUTFChars(env, jResponseJSONString, NULL);

	((HubCallbackWithJSON) callback)(((void*) jsonString), userParam);
	(*env)->ReleaseStringUTFChars(env, jResponseJSONString, jsonString);
}

#ifdef __cplusplus
}
#endif

#endif
