package org.cocos2dx.cpp;

import android.content.ClipData;
import android.content.ClipboardManager;
import android.content.Intent;
import android.net.Uri;
import android.os.AsyncTask;
import android.text.TextUtils;
import android.util.Log;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.concurrent.ExecutionException;

/**
 * @author kimsang-u PostBox 1.0.0 우편함 기본 기능 구현 Login, Post List, Receive,
 *         Userinfo(Guest->LoginUser VID 선택할 때 사용.)
 * 
 *         PostBox 1.0.1 Protocol World 추가 되어 requestLoginConnection 에서 저장한 뒤 다른
 *         API에서 VID와 sessionToken과 마찬가지로 wolrd 값 재활용.
 */
public class PostBox {
	public enum PostNetworkState {
		CM_POST_NETWORK_STATE_NONE,
		CM_POST_NETWORK_STATE_LOGIN,
		CM_POST_NETWORK_STATE_USERINFO,
		CM_POST_NETWORK_STATE_REFRESH,
		CM_POST_NETWORK_STATE_RECEIVE,
		CM_POST_NETWORK_STATE_VERIFY;
	}

	public enum PostServerState 
	{
		POST_SERVER_STATE_LIVE_SERVER,
		POST_SERVER_STATE_TEST_SERVER,
		POST_SERVER_STATE_SANDBOX_SERVER,
	}
	
	// Network AsyncTask inner-class
	/**
	 * @author kimsang-u parameter : URL , JSONObject 해당 URL로 Post통신. 응답 String을
	 *         return.
	 */
	private class sendPostNetworkTask extends AsyncTask<Object, Void, String> {

		private StringBuilder strResponse = new StringBuilder();

		protected String doInBackground(Object... parms) {

			try {

				URL url = (URL) parms[0];
				HttpURLConnection urlConnection = (HttpURLConnection) url
						.openConnection();
				JSONObject postData;
				if (parms[1] instanceof JSONObject) {
					postData = (JSONObject) parms[1];
					Log.w("request ", "data :" + postData.toString());
				} else {
					return null;
				}

				urlConnection.setRequestMethod("POST");
				urlConnection.setReadTimeout(10000);
				urlConnection.setConnectTimeout(10000);
				urlConnection.setRequestProperty("Content-Type",
						"application/json");
				urlConnection.setDoInput(true);
				urlConnection.setDoOutput(true);
				urlConnection.connect();

				JSONObject postJSON = postData;

				OutputStreamWriter jsonWriter = new OutputStreamWriter(
						urlConnection.getOutputStream());
				jsonWriter.write(postJSON.toString());
				jsonWriter.close();

				int responseCode = urlConnection.getResponseCode();
				if (responseCode == HttpURLConnection.HTTP_OK) {

					BufferedReader dataReader = new BufferedReader(
							new InputStreamReader(
									urlConnection.getInputStream()));

					String line = null;
					while ((line = dataReader.readLine()) != null) {
						strResponse.append(line + "\n");
					}

					dataReader.close();

				}

			} catch (IOException exception) {
				exception.printStackTrace();
			}

			return strResponse.toString();
		}

		@Override
		protected void onPostExecute(String result) {
			// if(CMSearchDataBase.spinner != null &&
			// CMSearchDataBase.spinner.isShowing()){
			// CMSearchDataBase.spinner.dismiss();
			// CMSearchDataBase.spinner = null;
			// }
		}
	}

	public static final String POST_BOX_VERSION = "1.1.0";		// 20.07.09
	private String POST_BOX_TESTPAGE = "http://misample.com2us.net/test";
	
	private String POST_BOX_LIVE_SERVER = "http://misample.com2us.net";
	private String POST_BOX_TEST_SERVER = "http://test-misample.com2us.net";
	private String POST_BOX_SANDBOX_SERVER = "http://sandbox-misample.com2us.net";
	
//	private String POST_BOX_ACCOUNT_LOGIN = "http://misample.com2us.net/account/login";
//	private String POST_BOX_ACCOUNT_USER = "http://misample.com2us.net/account/user";
//	private String POST_BOX_POSTBOX_POSTS = "http://misample.com2us.net/postbox/posts";;
//	private String POST_BOX_POSTBOX_RECEIVE = "http://misample.com2us.net/postbox/receive";
	private PostNetworkState cmNetworkProcessState = PostNetworkState.CM_POST_NETWORK_STATE_NONE;
	private PostBoxListener postBoxListener;
	private String mPostBoxVID;
	private String mPostBoxSessionToken;
	private String mPostBoxWorld;

	private static PostServerState serverState = PostServerState.POST_SERVER_STATE_LIVE_SERVER;
	
	public PostBox(PostServerState state, PostBoxListener listener) {
		this.serverState = state;
		this.postBoxListener = listener;
	}

	public static void setServerState(PostServerState state) {
		serverState = state;
	}
	public static PostServerState getServerState() {
		return serverState;
	}
	
	public String getVersion() {
		return POST_BOX_VERSION;
	}

	private String getServerURL()
	{
		String serverUrl;
		switch(serverState)
		{
		case POST_SERVER_STATE_LIVE_SERVER:
			serverUrl = POST_BOX_LIVE_SERVER;
			break;
		case POST_SERVER_STATE_TEST_SERVER:
			serverUrl = POST_BOX_TEST_SERVER;
			break;
		case POST_SERVER_STATE_SANDBOX_SERVER:
			serverUrl = POST_BOX_SANDBOX_SERVER;
			break;
		default:
			serverUrl = POST_BOX_LIVE_SERVER;
			break;
		}
		
		return serverUrl;
	}
	
	/**
	 * 재화 지급 웹 페이지 이동.
	 * 
	 * @param clipboard
	 *            SessionToken복사를 위한 클립보드 객체
	 * @return startActivity 에 이용하여 웹페이지로 이동.
	 */
	public Intent openTestPage(ClipboardManager clipboard) {
		ClipData clip = ClipData.newPlainText("PostBox SessionToken",
				this.mPostBoxSessionToken);
		clipboard.setPrimaryClip(clip);
		Log.w("PostBox", "PostBox - SessionToken Copy Success! : "
				+ this.mPostBoxSessionToken);
		Intent i = new Intent(Intent.ACTION_VIEW);
		Uri u = Uri.parse(POST_BOX_TESTPAGE);
		i.setData(u);
		return i;

	}

	/**
	 * PostBoxServer 로그인 요청 (account/login)
	 * 
	 * @param appID
	 * @param appVersion
	 * @param country
	 * @param language
	 * @param DID
	 * @param VID
	 * @param UID
	 * @param loginCenterSessionKey
	 * @return
	 */
	public Boolean requestLoginConnection(String appID, String appVersion,
			String country, String language, String DID, String VID,
			String UID, String loginCenterSessionKey,
			String world) {
		Log.d("PostBoxTest", "PostBoxTest requestLoginConnection : 01");
		
		if (VID.isEmpty() && UID.isEmpty()) {
			Log.w("PostBox", "VID and UID are Empty");
			Log.d("PostBoxTest", "PostBoxTest requestLoginConnection : 02");
			return false;
		}
		JSONObject sendDataDictionary = new JSONObject();
		Log.d("PostBoxTest", "PostBoxTest requestLoginConnection : 03");
		try {
			Log.d("PostBoxTest", "PostBoxTest requestLoginConnection : 04");
			if (world == null || world.isEmpty()) {
				Log.d("PostBoxTest", "PostBoxTest requestLoginConnection : 05");
				world = "";
			}
			Log.d("PostBoxTest", "PostBoxTest requestLoginConnection : 06");
			sendDataDictionary.put("world", world);
			sendDataDictionary.put("appid", appID);
			sendDataDictionary.put("appver", appVersion);
			sendDataDictionary.put("country", country);
			sendDataDictionary.put("language", language);
			sendDataDictionary.put("did", DID);
			sendDataDictionary.put(PostBoxListener.POST_KEY_VID, VID);
			sendDataDictionary.put("uid", UID);
//			sendDataDictionary.put("lc_sessionkey", loginCenterSessionKey);
			Log.d("PostBoxTest", "PostBoxTest requestLoginConnection : 07");
		} catch (JSONException e) {
			Log.d("PostBoxTest", "PostBoxTest requestLoginConnection : 08");
			Log.w("PostBox", "requestLoginConnection failed");
			e.printStackTrace();
		}
		// set json data
		// call networkRequest function
		this.mPostBoxVID = VID;
		this.mPostBoxWorld = world;
		this.cmNetworkProcessState = PostNetworkState.CM_POST_NETWORK_STATE_LOGIN;
		Log.d("PostBoxTest", "PostBoxTest requestLoginConnection : 09");
		
		StringBuffer buf = new StringBuffer();
		String url = buf.append(getServerURL()).append("/account/login").toString();
		System.out.println("====>>>> PostBox Server URL : "+url);
		Log.d("PostBoxTest", "PostBoxTest requestLoginConnection : 10");
		
		makeNetworkRequest(url, sendDataDictionary);
//		makeNetworkRequest(POST_BOX_ACCOUNT_LOGIN, sendDataDictionary);

		Log.d("PostBoxTest", "PostBoxTest requestLoginConnection : 11");
		return true;
	}

	/**
	 * 로그인한 계정의 우편에서 아이템 받기 요청. 아이템이 우편->계정 으로 지급. JSONArray를 미리 만들어서 한개 혹은
	 * 여러개(전체)를 받음. (postbox/receive)
	 * 
	 * @param postIDList
	 * @return
	 */
	public Boolean requestReceivePostConnection(JSONArray postIDList) {
		JSONObject sendDataDictionary = new JSONObject();

		try {
			sendDataDictionary.put("world", mPostBoxWorld);
			sendDataDictionary.put(PostBoxListener.POST_KEY_VID, mPostBoxVID);
			sendDataDictionary.put("post_id_list", postIDList);
			sendDataDictionary.put("session_token", mPostBoxSessionToken);

		} catch (JSONException e) {
			Log.w("PostBox", "requestReceiveConnection failed");
			e.printStackTrace();
		}

		Log.w("PostBox", "sendData Json  : " + sendDataDictionary.toString());
		// call networkRequest function
		this.cmNetworkProcessState = PostNetworkState.CM_POST_NETWORK_STATE_RECEIVE;
		
		StringBuffer buf = new StringBuffer();
		String url = buf.append(getServerURL()).append("/postbox/receive").toString();
		System.out.println("====>>>> PostBox Server URL : "+url);
		
		makeNetworkRequest(url, sendDataDictionary);
//		makeNetworkRequest(POST_BOX_POSTBOX_RECEIVE, sendDataDictionary);
		return true;
	}

	/**
	 * 로그인한 계정의 재화 조회 (account/posts)
	 * 
	 * @return
	 */
	public Boolean requestRefreshPostConnection() {
		JSONObject sendDataDictionary = new JSONObject();

		try {
			sendDataDictionary.put("world", mPostBoxWorld);
			sendDataDictionary.put(PostBoxListener.POST_KEY_VID, mPostBoxVID);
			sendDataDictionary.put("session_token", mPostBoxSessionToken);

		} catch (JSONException e) {
			Log.w("PostBox", "requestRefreshConnection failed");
			e.printStackTrace();
		}
		// call networkRequest function
		this.cmNetworkProcessState = PostNetworkState.CM_POST_NETWORK_STATE_REFRESH;
		
		StringBuffer buf = new StringBuffer();
		String url = buf.append(getServerURL()).append("/postbox/posts").toString();
		System.out.println("====>>>> PostBox Server URL : "+url);
		
		makeNetworkRequest(url, sendDataDictionary);
//		makeNetworkRequest(POST_BOX_POSTBOX_POSTS, sendDataDictionary);
		return true;
	}

	/**
	 * VID를 이용하여 해당 계정의 재화 정보 조회.( LoginCenter Select 용 ) (account/login)
	 * 
	 * @param targetVID
	 * @return
	 */
	public Boolean requestUserInfoConnection(String targetVID) {
		JSONObject sendDataDictionary = new JSONObject();

		try {
			sendDataDictionary.put("world", mPostBoxWorld);
			sendDataDictionary.put(PostBoxListener.POST_KEY_VID, targetVID);
		} catch (JSONException e) {
			Log.w("PostBox", "requestUserInfoConnection failed");
			e.printStackTrace();
		}
		Log.w("PostBox", "sendData Json  : " + sendDataDictionary.toString());
		// call networkRequest function
		this.cmNetworkProcessState = PostNetworkState.CM_POST_NETWORK_STATE_USERINFO;
		
		StringBuffer buf = new StringBuffer();
		String url = buf.append(getServerURL()).append("/account/user").toString();
		System.out.println("====>>>> PostBox Server URL : "+url);
		
		makeNetworkRequest(url, sendDataDictionary);
//		makeNetworkRequest(POST_BOX_ACCOUNT_USER, sendDataDictionary);
		return true;
	}

	public Boolean requestVerifyConnection(String verifyRequestData) {
		JSONObject verifyJson = new JSONObject();
		JSONObject sendDataDictionary = new JSONObject();

		try {
			verifyJson = new JSONObject(verifyRequestData);
			// required
			if(TextUtils.isEmpty(PostBoxTest.SESSION_TOKEN))
				sendDataDictionary.put("session_token", verifyJson.getString("vidSession"));
			else
				sendDataDictionary.put("session_token", PostBoxTest.SESSION_TOKEN);
			sendDataDictionary.put("world", verifyJson.getString("world"));							// TODO : cocos code로 부터 수신
			sendDataDictionary.put("vid", verifyJson.getString("vid"));								// TODO : cocos code로 부터 수신
			sendDataDictionary.put("uid", verifyJson.getString("uid"));								// TODO : cocos code로 부터 수신
			sendDataDictionary.put("appid", AppActivity.getContext().getPackageName());
			sendDataDictionary.put("market_id", verifyJson.getString("market_id"));					// TODO : cocos code로 부터 수신
			sendDataDictionary.put("hiveiap_receipt", verifyJson.getString("hiveiap_receipt"));		// TODO : cocos code로 부터 수신
//			sendDataDictionary.put("price", Long.parseLong(verifyJson.getString("price")));							// TODO : cocos code로 부터 수신
			sendDataDictionary.put("price", verifyJson.getLong("price"));
			sendDataDictionary.put("currency", verifyJson.getString("currency"));						// TODO : cocos code로 부터 수신
			sendDataDictionary.put("purchase_bypass_info", verifyJson.getString("purchase_bypass_info"));	// TODO : cocos code로 부터 수신
			sendDataDictionary.put("item_type", verifyJson.getString("item_type"));					// TODO : cocos code로 부터 수신

		} catch (Exception e) {
			e.printStackTrace();
		}

		this.cmNetworkProcessState = PostNetworkState.CM_POST_NETWORK_STATE_VERIFY;

		StringBuffer buf = new StringBuffer();
		String url = buf.append(getServerURL()).append("/hive/iap/verify").toString();
		System.out.println("====>>>> PostBox Server URL : "+url);

		makeNetworkRequest(url, sendDataDictionary);

		return true;
	}

	/**
	 * HTTP 요청 후 응답 해석. 생성자에서 설정한 listener를 이용하여 cmNetworkProcessState 별로 결과 전달.
	 * 
	 * @param destintationUrl
	 * @param sendDataDictionary
	 * @return
	 */
	private Boolean makeNetworkRequest(String destintationUrl,
			JSONObject sendDataDictionary) {
		String strResponse;
		URL url;

		try {
			// Url, dictionary insert.
			Log.d("PostBoxTest", "PostBoxTest makeNetworkRequest : 01");
			url = new URL(destintationUrl);

			// send Post netowrk.
			Log.d("PostBoxTest", "PostBoxTest makeNetworkRequest : 02");
			strResponse = new sendPostNetworkTask().execute(url,
					sendDataDictionary).get();

			// response Post Network.
			Log.d("PostBoxTest", "PostBoxTest makeNetworkRequest : 03");
			Log.w("response", "full response : " + strResponse.toString());
			
			Log.d("PostBoxTest", "PostBoxTest makeNetworkRequest : 03.5, response : " + strResponse.toString());

			Log.d("PostBoxTest", "PostBoxTest makeNetworkRequest : 04");
			JSONObject responseJSON = new JSONObject(strResponse.toString());
			// http post result 분석..
			int resultCode = responseJSON.getInt("result_code");
			Log.d("PostBoxTest", "PostBoxTest makeNetworkRequest : 05");
			if (resultCode == 200) {
				Log.d("PostBoxTest", "PostBoxTest makeNetworkRequest : 06");
				JSONObject resultObjects = responseJSON
						.getJSONObject("result_data");
				this.mPostBoxSessionToken = responseJSON.getString(
						"session_token").toString();
				Log.w("response", responseJSON.get("result_data").toString());
				Log.w("response", "State .. : " + this.cmNetworkProcessState);
				Log.d("PostBoxTest", "PostBoxTest makeNetworkRequest : 07");

				switch (this.cmNetworkProcessState) {
				case CM_POST_NETWORK_STATE_LOGIN:
					Log.d("PostBoxTest", "PostBoxTest makeNetworkRequest : 08");
					postBoxListener.finishLoginProcess(resultObjects);
					if(responseJSON.has("session_token")) {
						PostBoxTest.SESSION_TOKEN = responseJSON.getString("session_token");
					}
					this.cmNetworkProcessState = PostNetworkState.CM_POST_NETWORK_STATE_NONE;
					requestRefreshPostConnection();		// 우편함 정보 갱신
					break;
				case CM_POST_NETWORK_STATE_REFRESH:
					Log.d("PostBoxTest", "PostBoxTest makeNetworkRequest : 09");
					postBoxListener.finishRefreshProcess(resultObjects);
					this.cmNetworkProcessState = PostNetworkState.CM_POST_NETWORK_STATE_NONE;
					break;
				case CM_POST_NETWORK_STATE_RECEIVE:
					Log.d("PostBoxTest", "PostBoxTest makeNetworkRequest : 10");
					postBoxListener.finishReceiveProcess(resultObjects);
					this.cmNetworkProcessState = PostNetworkState.CM_POST_NETWORK_STATE_NONE;
					requestRefreshPostConnection();		// 우편함 정보 갱신
					break;
				case CM_POST_NETWORK_STATE_USERINFO:
					Log.d("PostBoxTest", "PostBoxTest makeNetworkRequest : 11");
					this.cmNetworkProcessState = PostNetworkState.CM_POST_NETWORK_STATE_NONE;
					postBoxListener.finishUserInfoProcess(resultObjects);
					break;
				case CM_POST_NETWORK_STATE_VERIFY:
					Log.d("PostBoxTest", "PostBoxTest makeNetworkRequest : 11.5");
					this.cmNetworkProcessState = PostNetworkState.CM_POST_NETWORK_STATE_VERIFY;
					postBoxListener.finishVerifyProcess(resultObjects);
					requestRefreshPostConnection();		// 우편함 정보 갱신
					break;
				default:
					Log.d("PostBoxTest", "PostBoxTest makeNetworkRequest : 12");
					break;
				}
			} else {
				// postBox network 실패 처리.
				Log.d("PostBoxTest", "PostBoxTest makeNetworkRequest : 13");
				postBoxListener.finishNetworkFailProcess(responseJSON);
				this.cmNetworkProcessState = PostNetworkState.CM_POST_NETWORK_STATE_NONE;

//				AppActivity.showDialogMsg("PostBox Request Fail");
				AppActivity.showToastMsg("PostBox Request Fail");
			}
		} catch (JSONException jsonException) {
			Log.d("PostBoxTest", "PostBoxTest makeNetworkRequest : 14");
			jsonException.printStackTrace();
		} catch (MalformedURLException exception) {
			Log.d("PostBoxTest", "PostBoxTest makeNetworkRequest : 15");
			exception.printStackTrace();
		} catch (InterruptedException interruptException) {
			Log.d("PostBoxTest", "PostBoxTest makeNetworkRequest : 16");
			interruptException.printStackTrace();
		} catch (ExecutionException ExecutionException) {
			Log.d("PostBoxTest", "PostBoxTest makeNetworkRequest : 17");
			ExecutionException.printStackTrace();
		}
		Log.d("PostBoxTest", "PostBoxTest makeNetworkRequest : 18");
		return true;
	}
}