#include "AppDelegate.h"
#include "MainScene.h"
//#include "InitListScene.h"

USING_NS_CC;

static cocos2d::Size designResolutionSize = cocos2d::Size(640, 960);
static cocos2d::Size smallResolutionSize = cocos2d::Size(320, 480);
static cocos2d::Size mediumResolutionSize = cocos2d::Size(768, 1024);
static cocos2d::Size largeResolutionSize = cocos2d::Size(1536, 2048);

int AppDelegate::sampleStateWithSDK = 0;
int AppDelegate::sampleSdkV1WidthSettingValue = 240;
int AppDelegate::sampleSdkV1HeightSettingValue = 320;
int AppDelegate::sampleC2SWidthSettingValue = 320;
int AppDelegate::sampleC2SHeightSettingValue = 480;

AppDelegate::AppDelegate()
{
}

AppDelegate::~AppDelegate() 
{
}

// if you want a different context, modify the value of glContextAttrs
// it will affect all platforms
void AppDelegate::initGLContextAttrs()
{
    // set OpenGL context attributes: red,green,blue,alpha,depth,stencil
    GLContextAttrs glContextAttrs = {8, 8, 8, 8, 24, 8};

    GLView::setGLContextAttrs(glContextAttrs);
}

// if you want to use the package manager to install more packages,  
// don't modify or remove this function
static int register_all_packages()
{
    return 0; //flag for packages manager
}



bool AppDelegate::applicationDidFinishLaunching() {
    // initialize director
    auto director = Director::getInstance();
    auto glview = director->getOpenGLView();
    if(!glview) {
#if (CC_TARGET_PLATFORM == CC_PLATFORM_WIN32) || (CC_TARGET_PLATFORM == CC_PLATFORM_MAC) || (CC_TARGET_PLATFORM == CC_PLATFORM_LINUX)
        glview = GLViewImpl::createWithRect("hivesdk", cocos2d::Rect(0, 0, designResolutionSize.width, designResolutionSize.height));
#else
        glview = GLViewImpl::create("hivesdk");
#endif
        director->setOpenGLView(glview);
    }

    // turn on display FPS
    director->setDisplayStats(false);

    // set FPS. the default value is 1.0/60 if you don't call this
    director->setAnimationInterval(1.0f / 60);

    // Set the design resolution
    // TODO : Switch ResolutionSize v4 or v1
    glview->setDesignResolutionSize(designResolutionSize.width, designResolutionSize.height, ResolutionPolicy::FIXED_WIDTH);//v4
//    glview->setDesignResolutionSize(320, 480, ResolutionPolicy::EXACT_FIT);//v1
    
    auto frameSize = glview->getFrameSize();
    // if the frame's height is larger than the height of medium size.
    if (frameSize.height > mediumResolutionSize.height)
    {        
        director->setContentScaleFactor(MIN(largeResolutionSize.height/designResolutionSize.height, largeResolutionSize.width/designResolutionSize.width));
    }
    // if the frame's height is larger than the height of small size.
    else if (frameSize.height > smallResolutionSize.height)
    {        
        director->setContentScaleFactor(MIN(mediumResolutionSize.height/designResolutionSize.height, mediumResolutionSize.width/designResolutionSize.width));
    }
    // if the frame's height is smaller than the height of medium size.
    else
    {        
        director->setContentScaleFactor(MIN(smallResolutionSize.height/designResolutionSize.height, smallResolutionSize.width/designResolutionSize.width));
    }

    register_all_packages();

    // create a scene. it's an autorelease object
    // TODO : Switch Scene v4 or v1
    auto scene = MainScene::createScene();//v4
//    auto scene = InitListScene::createScene();// v1 QA 샘플앱 초기화 화면으로 시작.

    // run
    director->runWithScene(scene);

    return true;
}

// This function will be called when the app is inactive. Note, when receiving a phone call it is invoked.
void AppDelegate::applicationDidEnterBackground() {
    Director::getInstance()->stopAnimation();

    // if you use SimpleAudioEngine, it must be paused
    // SimpleAudioEngine::getInstance()->pauseBackgroundMusic();
}

// this function will be called when the app is active again
void AppDelegate::applicationWillEnterForeground() {
    Director::getInstance()->startAnimation();

    // if you use SimpleAudioEngine, it must resume here
    // SimpleAudioEngine::getInstance()->resumeBackgroundMusic();
}

void AppDelegate::applicationScreenSizeChanged(int newWidth, int newHeight) {
    if (newWidth <= 0 || newHeight <= 0)
    {
        return;
    }
    
    auto director = Director::getInstance();
    auto glview = director->getOpenGLView();
    if (glview)
    {
        auto size = glview->getFrameSize();
        if (size.equals(Size(newWidth, newHeight)))
        return;
        
        glview->setFrameSize(newWidth, newHeight);
        
        ResolutionPolicy resolutionPolicy = glview->getResolutionPolicy();
        if (resolutionPolicy == ResolutionPolicy::UNKNOWN) {
            resolutionPolicy = ResolutionPolicy::SHOW_ALL;
        }
        if(AppDelegate::sampleStateWithSDK == 1 || AppDelegate::sampleStateWithSDK == 2) {
            // SDK v1, C2S 의 경우
            resolutionPolicy = ResolutionPolicy::EXACT_FIT;
        }
        
        if(newWidth > newHeight) {
            if(AppDelegate::sampleStateWithSDK == 1) {
                // SDK v1 샘플 : 가로/세로 회전 시 기존의 resolution을 갖도록
                glview->setDesignResolutionSize(sampleSdkV1HeightSettingValue, sampleSdkV1WidthSettingValue, resolutionPolicy);
            } else if (AppDelegate::sampleStateWithSDK == 2) {
                glview->setDesignResolutionSize(sampleC2SHeightSettingValue, sampleC2SWidthSettingValue, resolutionPolicy);
            } else {
                glview->setDesignResolutionSize(designResolutionSize.height,
                                                designResolutionSize.width, resolutionPolicy);
            }

        } else {
            if(AppDelegate::sampleStateWithSDK == 1) {
                // SDK v1 샘플 : 가로/세로 회전 시 기존의 resolution을 갖도록
                glview->setDesignResolutionSize(sampleSdkV1WidthSettingValue, sampleSdkV1HeightSettingValue, resolutionPolicy);
            } else if(AppDelegate::sampleStateWithSDK == 2) {
                glview->setDesignResolutionSize(sampleC2SWidthSettingValue, sampleC2SHeightSettingValue, resolutionPolicy);
            } else {
                glview->setDesignResolutionSize(designResolutionSize.width, designResolutionSize.height, resolutionPolicy);
            }

        }
        //glview->setDesignResolutionSize(newWidth, newHeight, resolutionPolicy);
    }
}

// SDK v1에서 버튼 크기를 좀더 확대하기 위해 사용
void AppDelegate::changeResolutionForSdkV1() {
    AppDelegate::sampleStateWithSDK = 1; // SDK v1 Sample

    auto director = Director::getInstance();
    auto glview = director->getOpenGLView();

//    glview->setDesignResolutionSize(sampleSdkV1WidthSettingValue, sampleSdkV1HeightSettingValue, ResolutionPolicy::EXACT_FIT);//v1
    glview->setDesignResolutionSize(320, 480, ResolutionPolicy::EXACT_FIT);//v1
//    glview->setDesignResolutionSize(sampleSdkV1WidthSettingValue, sampleSdkV1HeightSettingValue, ResolutionPolicy::FIXED_WIDTH);//v1

    auto frameSize = glview->getFrameSize();
    // if the frame's height is larger than the height of medium size.
    if (frameSize.height > mediumResolutionSize.height)
    {
        director->setContentScaleFactor(MIN(largeResolutionSize.height/designResolutionSize.height, largeResolutionSize.width/designResolutionSize.width));
    }
        // if the frame's height is larger than the height of small size.
    else if (frameSize.height > smallResolutionSize.height)
    {
        director->setContentScaleFactor(MIN(mediumResolutionSize.height/designResolutionSize.height, mediumResolutionSize.width/designResolutionSize.width));
    }
        // if the frame's height is smaller than the height of medium size.
    else
    {
        director->setContentScaleFactor(MIN(smallResolutionSize.height/designResolutionSize.height, smallResolutionSize.width/designResolutionSize.width));
    }
}

// SDK v1에서 버튼 크기를 좀더 확대하기 위해 사용
void AppDelegate::changeResolutionForC2S() {
    AppDelegate::sampleStateWithSDK = 2; // SDK v1 Sample

    auto director = Director::getInstance();
    auto glview = director->getOpenGLView();

    glview->setDesignResolutionSize(sampleC2SWidthSettingValue, sampleC2SHeightSettingValue, ResolutionPolicy::EXACT_FIT);//v1

    auto frameSize = glview->getFrameSize();
    // if the frame's height is larger than the height of medium size.
    if (frameSize.height > mediumResolutionSize.height)
    {
        director->setContentScaleFactor(MIN(largeResolutionSize.height/designResolutionSize.height, largeResolutionSize.width/designResolutionSize.width));
    }
        // if the frame's height is larger than the height of small size.
    else if (frameSize.height > smallResolutionSize.height)
    {
        director->setContentScaleFactor(MIN(mediumResolutionSize.height/designResolutionSize.height, mediumResolutionSize.width/designResolutionSize.width));
    }
        // if the frame's height is smaller than the height of medium size.
    else
    {
        director->setContentScaleFactor(MIN(smallResolutionSize.height/designResolutionSize.height, smallResolutionSize.width/designResolutionSize.width));
    }
}
