//
//  AuthTestView.cpp
//  hivesdk
//
//  Created by paikjongman on 2017. 2. 22..
//
//

#include "AuthTest.hpp"
#include "ui/CocosGUI.h"
#include "Postbox.hpp"
#include "GameManager.hpp"

using namespace cocos2d;
using namespace cocos2d::ui;
using namespace hive;

const std::vector<const char*> LoginTypeNames = {"GUEST","ACCOUNT","SELECT","AUTO"};

bool AuthTest::init()
{
    if(!BaseView::init())
        return false;
    setUseShowLogButton(true);
    Logger::setLoggerListener([=](std::string tag, std::string level, std::string logMessage){
        addLogMessage(logMessage);
    });

    
    addButton("Auth.requestPermissionViewData()",BUTTON_AUTH_REQUEST_PERMISSION_VIEW_DATA);
//#if (CC_TARGET_PLATFORM == CC_PLATFORM_ANDROID)
    addEditBox(nullptr, "android.permission.READ_PHONE_STATE\nandroid.permission.READ_PHONE_NUMBERS\nandroid.permission.WRITE_EXTERNAL_STORAGE\nandroid.permission.READ_CONTACTS\nandroid.permission.VIBRATE", nullptr, {screenSize.width, 300}, EditBox::InputMode::ANY, 20, EDITBOX_PERMISSION_LIST);
    addButton("PlatformHelper.requestUserPermissions", BUTTON_REQUEST_USER_PERMISSIONS);
//#endif
    addButton("Auth.initialize()",BUTTON_AUTH_INIT);
    addButton("Auth.getLoginType()",BUTTON_AUTH_GET_LOGIN_TYPE);
    addCombobox("LoginType : ", std::move(LoginTypeNames), 0, COMBO_AUTH_LOGIN_TYPE);
    addButton("Auth.login()",BUTTON_AUTH_LOGIN);
    addLabel("Current vid : -",-1);
    addLabel("",LABEL_AUTH_CURRENT_VID);
    addLabel("Used vid : -",-1);
    addLabel("",LABEL_AUTH_USED_VID);
    addButton("Auth.showLoginSelection(_usedVid)",BUTTON_AUTH_LOGIN_SELECT);
    addButton("Auth.bindLogin(_usedVid)",BUTTON_AUTH_BIND);
    addButton("Auth.logout()",BUTTON_AUTH_LOGOUT);
    addButton("Auth.getAccount()",BUTTON_AUTH_GET_ACCOUNT);
    addTrueFalseToggleButton("isReady : ", 1, {screenSize.width, DEFAULT_CONTENT_HEIGHT}, TOGGLE_BUTTON_IS_READY_ENGAGEMENT);
    addButton("Auth.setEngagementReady", BUTTON_ISREADYENGAGEMENT);
    addButton("Auth.showTerms()",BUTTON_AUTH_SHOW_TERMS);
    addTrueFalseToggleButton("isShow : ", 1, {screenSize.width, DEFAULT_CONTENT_HEIGHT}, TOGGLE_BUTTON_AUTH_SHOW_MAINTENANCE);
    addButton("Auth.checkMaintenance()",BUTTON_AUTH_CHECK_MAINTENANCE);
    addButton("Auth.showAdultConfirm()",BUTTON_AUTH_SHOW_ADULT_CONFIRM);
    addButton("Auth.reset() - WARNING - Reset data",BUTTON_AUTH_RESET);
    addButton("Auth.setEmergencyMode()", BUTTON_AUTH_SET_EMERGETNCY_MODE);
    
    setUseBackButton(true, "- Auth -");
    
    return true;
}
void AuthTest::onButtonEvent(int buttonTag)
{
    /*
     BUTTON_AUTH_INIT,
     BUTTON_AUTH_GET_LOGIN_TYPE,
     COMBO_AUTH_LOGIN_TYPE,
     BUTTON_AUTH_LOGIN,
     BUTTON_AUTH_LOGIN_SELECT,
     BUTTON_AUTH_BIND,
     BUTTON_AUTH_LOGOUT,
     BUTTON_AUTH_GET_ACCOUNT,
     BUTTON_AUTH_CHECK_MAINTENANCE,
     BUTTON_AUTH_SHOW_TERMS,
     BUTTON_AUTH_SHOW_ADULT_CONFIRM,
     BUTTON_AUTH_RESET
     */
    switch (buttonTag) {
        case BUTTON_AUTH_REQUEST_PERMISSION_VIEW_DATA:
        {
            Auth::requestPermissionViewData([=](ResultAPI const & result,PermissionViewData const & data){
                Logger::log("result code:%d message:%s\n",result.code,result.message.c_str());

                if (result.isSuccess()) {
                    if (result.code == hive::ResultAPI::AuthV4SkipPermissionView) {
                        Logger::log("Can skip permission notice");
                        MessageBox("Can skip permission notice", "Notice");
                    }
                    else {
                        Logger::log("=== 커스텀 약관고지 로그 ===\n");

                        std::string permissionNotice = "";
                        permissionNotice+=StringUtils::format("제목 : %s\n",data.contents.c_str());

                        if (data.permissions.size() > 0) {
                            permissionNotice+=StringUtils::format("\n\n=== From permissions ===");
                        }
                        for (auto unit : data.permissions) {
                            permissionNotice+=StringUtils::format("\n권한명 : %s\n 타이틀 %s\n 내용 %s\n 권한 카테고리 : %d\n",unit.nativePermissionName.c_str(),unit.title.c_str(),unit.contents.c_str(),unit.permissionCategory);
                        }
                        if (data.commons.size() > 0) {
                            permissionNotice+=StringUtils::format("\n\n=== From commons ===");
                        }
                        for (auto unit : data.commons) {
                            permissionNotice+=StringUtils::format("\n권한명 : %s\n 타이틀 %s\n 내용 %s \n 권한 카테고리 : %d\n", unit.nativePermissionName.c_str(), unit.title.c_str(), unit.contents.c_str(), unit.permissionCategory);
                        }
                        Logger::log((data.contents + "\n").c_str());
                        Logger::log(permissionNotice.c_str());
                        MessageBox(permissionNotice.c_str(), data.contents.c_str());
                    }
                }
                else {
                    //AuthV4InProgress에러는 대기, 그 외 에러는 무시하고 AuthV4.setup을 호출할경우 HIVE SDK의 기본 권한고지 팝업이 사용됩니다.
                }
            });
        }
            break;
        case BUTTON_AUTH_INIT:
        {
            Auth::initialize(// Auth 초기화 결과 통지 리스너 - Auth::initialize()
                             [=](ResultAPI result, AuthInitResult authInitResult){
                                 Logger::log("AuthTestView::onAuthInitialize() Callback\nresult = %s\n", result.toString().c_str());
                                if(result.isSuccess()) {
                                     LoginType loginType = authInitResult.loginType;
                                     
                                     changeComboboxList(std::move(LoginTypeNames), static_cast<int>(loginType), COMBO_AUTH_LOGIN_TYPE);
                                    Logger::log("loginType = %s\n", LoginTypeNames[static_cast<int>(loginType)]);
                                    Logger::log("did = %s\n", authInitResult.did.c_str());
                                }
                             });
            
            Promotion::setEngagementHandler(GameManager::engagemenetResult);
            break;
        }
        case BUTTON_AUTH_GET_LOGIN_TYPE:
        {
            changeComboboxList(std::move(LoginTypeNames), static_cast<int>(Auth::getLoginType()), COMBO_AUTH_LOGIN_TYPE);
            Logger::log("\n\n=== 로그인 가능 형태 조회 (GUEST, ACCOUNT, AUTO, SELECT) ===\n");
            Logger::log("Auth::getLoginType(), loginType = %s", LoginTypeNames[getSelectedComboboxIndex(COMBO_AUTH_LOGIN_TYPE)]);
            break;
        }
            
        case BUTTON_AUTH_LOGIN:
        {
            LoginType loginType = static_cast<LoginType>(getSelectedComboboxIndex(COMBO_AUTH_LOGIN_TYPE));
            Auth::login(loginType, // AUTO or GUEST or ACCOUNT
                        // 로그인 결과 통지 리스너 - Auth::login() / Auth::showLoginSelection()
                        [=](ResultAPI result, LoginType loginType, Account currentAccount, Account usedAccount)
            {
                Logger::log("AuthTestView::onAuthLogin() Callback\nresult = %s\n", result.toString().c_str());
                Logger::log("loginType = %s\n", LoginTypeNames[static_cast<int>(loginType)]);
                Logger::log("currentAccount = %s\n", currentAccount.toString().c_str());
                Logger::log("usedAccount = %s\n", usedAccount.toString().c_str());
                changeLabelText(currentAccount.vid, LABEL_AUTH_CURRENT_VID);
                changeLabelText(usedAccount.vid, LABEL_AUTH_USED_VID);
                Postbox::setLoginType(Postbox::v1);


                if(result.isSuccess()){
                    Postbox::login([=](const Postbox::Login *login, Postbox::Error error){
                        if( login != nullptr )
                        {
                            int gold = login->gold;
                            int gem = login->gem;
                            std::string nickname = login->nickname;
                            Logger::log("nick %s, gold %d, gem %d", nickname.c_str(), gold, gem);
                        }
                        else
                        {
                            Logger::log("Postbox login failed.");
                            Logger::log("result code = %s, msg = %s", error.result_code.c_str(), error.result_msg.c_str());
                        }
                    });
                }


            });
            break;
        }
        case BUTTON_AUTH_LOGIN_SELECT:
        {
            picojson::object currentVidData;
            currentVidData["vid"] = picojson::value(getUIWidget<Text>(LABEL_AUTH_CURRENT_VID)->getString());
            
            // 계정에 따라 저장하고 싶은 커스텀한 게임 데이터
            picojson::object currentVidGameData;
            currentVidGameData["Name"] = picojson::value("Nam");
            currentVidGameData["Level"] = picojson::value((double)20);
            currentVidData["data"] = picojson::value(currentVidGameData);
            
            
            picojson::object usedVidData;
            usedVidData["vid"] = picojson::value(getUIWidget<Text>(LABEL_AUTH_USED_VID)->getString());
            
            // 계정에 따라 저장하고 싶은 커스텀한 게임 데이터
            picojson::object usedVidGameData;
            usedVidGameData["Name"] = picojson::value("An");
            usedVidGameData["Level"] = picojson::value((double)10);
            usedVidData["data"] = picojson::value(usedVidGameData);
     
            Logger::log("\n\n=== Auth::login() 메서드를 호출 후 결과 값이 LoginType.SELECT 이면 이 메서드를 호출 하여 사용자를 선택을 요청 ===\n");
            Logger::log("Auth::showLoginSelection(currentVidData, usedVidData, onAuthLogin) Called\n");
            Logger::log("currentVidData = %s\n", picojson::value(currentVidData).serialize().c_str());
            Logger::log("usedVidData = %s\n", picojson::value(usedVidData).serialize().c_str());
            
            Auth::showLoginSelection(currentVidData,
                                     usedVidData,
                                     // 로그인 결과 통지 리스너 - Auth::login() / Auth::showLoginSelection()
                                     [=](ResultAPI result, LoginType loginType, Account currentAccount, Account usedAccount)
                                     {
                                         Logger::log("AuthTestView::onAuthLogin() Callback\nresult = %s\n", result.toString().c_str());
                                         Logger::log("loginType = %s\n", LoginTypeNames[static_cast<int>(loginType)]);
                                         Logger::log("currentAccount = %s\n", currentAccount.toString().c_str());
                                         Logger::log("usedAccount = %s\n", usedAccount.toString().c_str());
                                         changeLabelText(currentAccount.vid, LABEL_AUTH_CURRENT_VID);
                                         changeLabelText(usedAccount.vid, LABEL_AUTH_USED_VID);
                                     });
            break;
        }
        case BUTTON_AUTH_BIND:
        {
            Logger::log("\n\n=== 계정 바인딩 요청 ===\n");
            Logger::log("Auth::bindLogin(usedAccount.vid) Called\n");

            std::string bindVid = getUIWidget<Text>(LABEL_AUTH_USED_VID)->getString();
            Logger::log("Auth::bindLogin vid: %s", bindVid.c_str());
            Auth::bindLogin(bindVid,
                            // 로그인 결과 통지 리스너 - Auth::login() / Auth::showLoginSelection()
                            [=](ResultAPI result, LoginType loginType, Account currentAccount, Account usedAccount)
                            {
                                Logger::log("AuthTestView::onAuthLogin() Callback\nresult = %s\n", result.toString().c_str());
                                Logger::log("loginType = %s\n", LoginTypeNames[static_cast<int>(loginType)]);
                                Logger::log("currentAccount = %s\n", currentAccount.toString().c_str());
                                Logger::log("usedAccount = %s\n", usedAccount.toString().c_str());
                                changeLabelText(currentAccount.vid, LABEL_AUTH_CURRENT_VID);
                                changeLabelText(usedAccount.vid, LABEL_AUTH_USED_VID);
                            });
            break;
        }
        case BUTTON_AUTH_LOGOUT:
        {
            Logger::log("\n\n=== 로그 아웃 요청 ===\n");
            Logger::log("Auth::logout(onAuthLogout) Called\n");
            Auth::logout(
                         [=](ResultAPI result)
                         {
                             Logger::log("AuthTestView::onAuthLogout() Callback\nresult = %s\n", result.toString().c_str());
                         });
            break;
        }
        case BUTTON_AUTH_GET_ACCOUNT:
        {
//            Auth::getAccount();
            Logger::log("\n\n=== 인증된 사용자 정보 조회 ===\n");
            Logger::log("Auth::getAccount() Called\naccount = %s", Auth::getAccount().toString().c_str());
            break;
        }
        case BUTTON_AUTH_CHECK_MAINTENANCE:
        {
            Logger::log("\n\n=== 점검 상태 표시 및 데이터 반환 ===\n");
            Logger::log("Auth::checkMaintenance(isShow = %s, onAuthMaintenance) Called\n", getSelectedToggleButtonIndex(TOGGLE_BUTTON_AUTH_SHOW_MAINTENANCE) ? "true" : "false");
            Auth::checkMaintenance(getSelectedToggleButtonIndex(TOGGLE_BUTTON_AUTH_SHOW_MAINTENANCE),
                                   [=](ResultAPI result, AuthMaintenanceInfo authMaintenanceInfo)
            {
                Logger::log("AuthTestView::onAuthMaintenance() Callback\nresult = %s\n", result.toString().c_str());
                Logger::log("authMaintenanceInfo = %s\n", authMaintenanceInfo.toString().c_str());
            });
            break;
        }
        case BUTTON_AUTH_SHOW_TERMS:
        {
            Logger::log("\n\n=== 약관 정보 표시 ===\n");
            Logger::log("Auth::showTerms(onAuthShowTerms) Called\n");
            Auth::showTerms([=](ResultAPI result)
                            {
                                Logger::log("AuthTestView::onAuthShowTerms() Callback\nresult = %s\n", result.toString().c_str());
                            });
            break;
        }
        case BUTTON_AUTH_SHOW_ADULT_CONFIRM:
        {
            Logger::log("\n\n=== 성인 인증 화면 호출 ===\n");
            Logger::log("Auth::showAdultConfirm(onAuthAdultConfirm) Called\n");
            Auth::showAdultConfirm([=](ResultAPI result)
                                   {
                                       Logger::log("AuthTestView::onAuthAdultConfirm() Callback\nresult = %s\n", result.toString().c_str());
                                   });
            break;
        }
        case BUTTON_AUTH_RESET:
        {
            Logger::log("\n\n=== 인증 정보를 포함하여 SDK 에서 사용하는 모든 데이터 초기화 (테스트 용) ===\n");
            Logger::log("Auth::reset() Called\n");
            Auth::reset();
            break;
        }
        case BUTTON_ISREADYENGAGEMENT:
        {
            bool isready = this->getSelectedToggleButtonIndex(TOGGLE_BUTTON_IS_READY_ENGAGEMENT);
            Promotion::setEngagementReady(isready);
            auto isreadystring = isready ? "set ready : on" : "set ready : off";
            MessageBox(isreadystring,"Engagement");
        }
            break;
        case BUTTON_REQUEST_USER_PERMISSIONS:
        {
            std::vector<std::string> requests;

            std::string editBoxStr = getEditboxValue(EDITBOX_PERMISSION_LIST);
            std::istringstream ss(editBoxStr);
            std::string unitStr;

            while (std::getline(ss, unitStr, '\n')) {
                requests.push_back(unitStr);
            }

            PlatformHelper::requestUserPermissions(requests, [=](ResultAPI const & result, std::vector<std::string> const & granted, std::vector<std::string> const & denied) {
                Logger::log("\n\n=== OS 권한동의 팝업 재노출 ===\n");
                Logger::log("ResultAPI : %s\n", result.toString().c_str());

                if (result.code == hive::ResultAPI::PlatformHelperOSNotSupported) {
                    Logger::log("Only support Android\n");
                }
                else if (result.code == hive::ResultAPI::PlatformHelperOSVersionNotSupported) {
                    Logger::log("Android OS version not supported.\n");
                }

                if (!granted.empty()) {
                    Logger::log("동의한 권한 목록 : \n");
                    for (std::string name : granted) {
                        Logger::log("%s\n",name.c_str());
                    }
                }

                if (!denied.empty()) {
                    Logger::log("\n거절한 권한 목록: \n");
                    for (std::string name : denied) {
                        Logger::log("%s\n",name.c_str());
                    }
                }
            });
        }
            break;
        case BUTTON_AUTH_SET_EMERGETNCY_MODE:
        {
            bool isEmergencyMode = Auth::setEmergencyMode();

            Logger::log("\n\n=== 긴급 모드 ===\n");
            Logger::log("Auth.setEmergencyMode() Called\nisEmergencyMode : %s", isEmergencyMode? "true":"false");

            if (isEmergencyMode) {
                std::string serverId = hive::Configuration::getServerId();
                Postbox::login([=](const Postbox::Login *login, Postbox::Error error){
                    if( login != nullptr )
                    {
                        int gold = login->gold;
                        int gem = login->gem;
                        std::string nickname = login->nickname;
                        Logger::log("nick %s, gold %d, gem %d", nickname.c_str(), gold, gem);
                    }
                    else
                    {
                        Logger::log("Postbox login failed.");
                        Logger::log("result code = %s, msg = %s", error.result_code.c_str(), error.result_msg.c_str());
                    }
                });
            }
        }
            break;
        default:
            break;
    }
}


