//
//  ConfigurationView.cpp
//  hivesdk
//
//  Created by paikjongman on 2017. 2. 27..
//
//

#include "Configuration.hpp"
#include "ResultUtils.hpp"
#include "InitListScene.h"
#include "C2SMainScene.h"
#include "AppDelegate.h"

//const char* languageTable[][2] = {
//    {"English","en"},
//    {"Deutsch","de"},
//    {"español","es"},
//    {"français","fr"},
//    {"Indonesia","id"},
//    {"italiano","it"},
//    {"日本語","ja"},
//    {"한국어","ko"},
//    {"Bahasa Melayu","ms"},
//    {"português","pt"},
//    {"русский","ru"},
//    {"ไทย","th"},
//    {"Türkçe","tr"},
//    {"Tiếng Việt","vi"},
//    {"简体中文","zh-Hans"},
//    {"繁體中文","zh-Hant"}
//};

std::vector<std::string> hiveOrientationType = {
    "NotSet",
    "undefined",
    "all",
    "portrait",
    "landscape"
};

std::vector<const char*> languageDisplayName = {
    "English",
    "Deutsch",
    "español",
    "français",
    "Indonesia",
    "italiano",
    "日本語",
    "한국어",
//    "Bahasa Melayu",
    "português",
    "русский",
    "ไทย",
    "Türkçe",
    "Tiếng Việt",
    "简体中文",
    "繁體中文",
   "الْعَرَبِيَّةُ",
    "Not set"
};

std::vector<std::string> languageCode = {
    "en",
    "de",
    "es",
    "fr",
    "id",
    "it",
    "ja",
    "ko",
//    "ms",
    "pt",
    "ru",
    "th",
    "tr",
    "vi",
    "zh-Hans",
    "zh-Hant",
    "ar",
    "null"
};

std::vector<const char*> appIdList = {
#if (CC_TARGET_PLATFORM == CC_PLATFORM_ANDROID)
    "com.com2us.hivesdk.normal.freefull.google.global.android.common",
    "com.com2us.misample.normal.freefull.google.global.android.common",
    "com.com2us.hivesdk.normal.freefull.onestore.kr.android.common",
    "com.com2us.hivesdk.android.huawei.global.normal.huawei",
    "com.com2us.hivesdk.android.amazon.global.normal"
#elif (CC_TARGET_PLATFORM == CC_PLATFORM_IOS)
    "com.com2us.hivesdk.normal.freefull.apple.global.ios.universal",
    "com.com2us.misample.normal.freefull.apple.global.ios.universal",
#else
    "Window",
    "Mac"
#endif
};

std::vector<const char*> gameServerIdList = {
    "kr",
    "server_001",
    "server_002",
    "server_003"
};

void Configuration::reset()
{
    cocos2d::UserDefault* instance = cocos2d::UserDefault::getInstance();
    
    // AppId
    instance->deleteValueForKey(keyAppId);
    
    // Zone
    instance->deleteValueForKey(keyZone);
  
    // Language
    instance->deleteValueForKey(keyLanguage);

    // ServerId
    instance->deleteValueForKey(keyServerId);
    
    // UseLog
    instance->deleteValueForKey(keyUseLog);
    
    // UseRemoteLog
    instance->deleteValueForKey(keyUseRemoteLog);
    
    // RemoteLogId
    instance->deleteValueForKey(keyRemoteLogId);
    
    // UseRemoteDetailLog
    instance->deleteValueForKey(keyUseRemoteDetailLog);
    
    // getMetaData
    instance->deleteValueForKey(keyHiveMetaData);
}

void Configuration::apply()
{
    cocos2d::UserDefault* instance = cocos2d::UserDefault::getInstance();
    
    // AppId
    std::string appId = getEditboxValue(EDITBOX_APP_ID);
    hive::Configuration::setAppId(appId);
    instance->setStringForKey(keyAppId, appId);
    
    // HiveCertificationKey
    std::string hiveCerificationKey = getEditboxValue(Configuration::EDITBOX_HIVE_CERTIFICATION_KEY);
    hive::Configuration::setHiveCertificationKey(hiveCerificationKey);
    
    //Zone
    int zoneIndex = getSelectedToggleButtonIndex(Configuration::TOGGLE_BUTTON_ZONE);
    switch (zoneIndex) {
        case 0://Sandbox
            hive::Configuration::setZone(hive::ZoneType::SANDBOX);
            break;
        case 1://Real
            hive::Configuration::setZone(hive::ZoneType::REAL);
            break;
        case 2://Test
            hive::Configuration::setZone(hive::ZoneType::TEST);
            break;
        case 3://Dev
            hive::Configuration::setZone(hive::ZoneType::DEV);
            break;
        default:
            break;
    }
    instance->setIntegerForKey(keyZone, zoneIndex);
    
    // Theme
    int hiveThemeIndex = getSelectedToggleButtonIndex(Configuration::TOGGLE_BUTTON_THEME);
    switch (hiveThemeIndex) {
        case 0: // hiveLight
            hive::Configuration::setHiveTheme(hive::HiveThemeType::hiveLight);
            break;
        case 1: // hiveDark
            hive::Configuration::setHiveTheme(hive::HiveThemeType::hiveDark);
            break;
    }
    instance->setIntegerForKey(keyHiveTheme, hiveThemeIndex);

    // HiveOrientation
    int selectedHiveOrientationIndex = getSelectedToggleButtonIndex(Configuration::TOGGLE_BUTTON_ORIENTATION);
    std::string selectedHiveOrientation = hiveOrientationType[selectedHiveOrientationIndex];

    if(selectedHiveOrientation.compare("NotSet") != 0){
        hive::Configuration::setHiveOrientation(selectedHiveOrientation);
    }
    instance->setStringForKey(keyHiveOrientation, selectedHiveOrientation);
    
    // Language
    int selectedLanguageIndex = getSelectedComboboxIndex(Configuration::LIST_GAME_LANGUAGE);
    std::string selectedLanguage = languageCode[selectedLanguageIndex];
    
    if(selectedLanguage.compare("null") != 0){
        hive::Configuration::setGameLanguage(selectedLanguage);
    }
    instance->setStringForKey(keyLanguage, selectedLanguage);

    // ServerId
    std::string gameServerId = getEditboxValue(EDITBOX_GAME_SERVER_ID);
    hive::Configuration::setServerId(gameServerId);
    instance->setStringForKey(keyServerId, gameServerId);
    
    // UseLog
    bool bUseLog = getSelectedToggleButtonIndex(Configuration::TOGGLE_BUTTON_USE_LOG);
    hive::Configuration::setUseLog(bUseLog);
    instance->setBoolForKey(keyUseLog, bUseLog);
    
	
    bool bUseCoppaAgeGateU13 = getSelectedToggleButtonIndex(Configuration::TOGGLE_BUTTON_USE_COPPA_AGE_GATE_U13);
    hive::Configuration::setAgeGateU13(bUseCoppaAgeGateU13);
    instance->setBoolForKey(keyCoppaAgeGateU13, bUseCoppaAgeGateU13);
    
    // MetaData
    std::string metaData = getEditboxValue(EDITBOX_GET_METADATAKEY);
    instance->setStringForKey(keyHiveMetaData, metaData);

    // LogFilter
    int logFilterIndex = getSelectedToggleButtonIndex(Configuration::TOGGLE_BUTTON_LOG_TYPE);

    if(logFilterIndex != 0) { // 0: NotSet

        auto logType = static_cast<hive::LogType>(logFilterIndex - 1);
        hive::LogFilter logFilter = hive::LogFilter();
        logFilter.coreLog = logType;
        logFilter.serviceLog = logType;
        hive::Logger::setLogFilter(logFilter);
    }

    // Hive Community Url
    std::string hiveCommunityUrl = getEditboxValue(Configuration::EDITBOX_HIVE_COMMUNITY_URL);
    hive::Configuration::setHiveCommunityUrl(hiveCommunityUrl);

    bool bUseHercules = getSelectedToggleButtonIndex(Configuration::TOGGLE_BUTTON_USE_HERCULES);
    hive::Configuration::setUseHercules(bUseHercules);
    instance->setBoolForKey(keyUseHercules, bUseHercules);
    
    // SetChannel
    std::string channel = getEditboxValue(Configuration::EDITBOX_CHANNEL);
    hive::Configuration::setChannel(channel);
}

bool Configuration::init()
{
    if(!BaseView::init())
            return false;

    addButton("start with Hive SDK v1", Configuration::BUTTON_SDK_V1);
    addButton("start with Hive SDK v4", Configuration::BUTTON_SDK_V4);
    addButton("start with C2S", Configuration::BUTTON_SDK_C2S);
    return true;
}

void Configuration::startv4()
{
    cocos2d::UserDefault* instance = cocos2d::UserDefault::getInstance();
    
    // AppId
    std::string currentAppId = instance->getStringForKey(keyAppId, hive::Configuration::getAppId());
    // HiveCertificationKey
    std::string defaultHiveCertificationKey = "eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzI1NiJ9.eyJpc3MiOiJIaXZlIiwiaWF0IjoxNjAyMDU2NzI2LCJqdGkiOiIxODczMTExMzIwIn0.3soFiHTPlObCoqR5xX9ZeOQTSvnHrHDHWmopP3QfWtY"; // GCPSDK4-284
    
    // Zone
    int zoneIndex;
    hive::ZoneType zoneType = hive::Configuration::getZone();
    switch (zoneType) {
        case hive::ZoneType::SANDBOX:
            zoneIndex = 0;
            break;
        case hive::ZoneType::REAL:
            zoneIndex = 1;
            break;
        case hive::ZoneType::TEST:
            zoneIndex = 2;
            break;
        case hive::ZoneType::DEV:
            zoneIndex = 3;
            break;
        default:
            break;
    }
    zoneIndex = instance->getIntegerForKey(keyZone, zoneIndex);

    // HiveTheme
    int hiveTheme = instance->getIntegerForKey(keyHiveTheme, 0);

    // HiveOrientation
    std::string hiveOrientation = instance->getStringForKey(keyHiveOrientation, "all");
    int selectedHiveOrientationIndex = 2;

    std::vector<std::string>::iterator orientationIterator = std::find(hiveOrientationType.begin(), hiveOrientationType.end(), hiveOrientation);
    if(orientationIterator != hiveOrientationType.end())
        selectedHiveOrientationIndex = std::distance(hiveOrientationType.begin(), orientationIterator);
    
    // Language
    std::string selectedLanguage = instance->getStringForKey(keyLanguage, "en");
    int selectedLanguageIndex = 0;

    std::vector<std::string>::iterator it = std::find(languageCode.begin(), languageCode.end(), selectedLanguage);
    if(it != languageCode.end())
        selectedLanguageIndex = std::distance(languageCode.begin(), it);

    // Game ServerId
    std::string selectedGameServerId = instance->getStringForKey(keyServerId, gameServerIdList[0]);
    
    // UseLog
    bool bUseLog = instance->getBoolForKey(keyUseLog, true);
    
    // UseRemoteLog
    bool bUseRemoteLog = instance->getBoolForKey(keyUseRemoteLog, false);
    
    // RemoteLogId
    std::string logingId = instance->getStringForKey(keyRemoteLogId);
    
    // UseRemoteDetailLog
    bool bUseRemoteDetailLog = instance->getBoolForKey(keyUseRemoteDetailLog, false);
    
    bool bUseAgeGateU13 = hive::Configuration::getAgeGateU13();
    
    // MetaData
    std::string metaDataKey = instance->getStringForKey(keyHiveMetaData, "game_server_url");

    // Hive Community Url
    std::string defaultHiveCommunityUrl = "";

    // UI Setup
    // SDK Version
    BeginHorizontal();
    addLabel("Hive SDK v4 : ");
    addLabel(hive::Configuration::getHiveSDKVersion().c_str(), Configuration::LABEL_HIVE_SDK_VERSION);
    EndHorizontal();

    BeginHorizontal();
    addLabel("SDK Ver : ");
    addLabel(hive::Configuration::getReferenceSDKVersion().c_str(), Configuration::LABEL_SDK_VERSION);
    EndHorizontal();
    
#if (CC_TARGET_PLATFORM == CC_PLATFORM_IOS)
    BeginHorizontal();
    addLabel("IDFA : ");
    addLabel(Configuration::getIDFA(), Configuration::LABEL_IDFA);
    addButton("COPY", {screenSize.width/4, DEFAULT_CONTENT_HEIGHT}, Configuration::BUTTON_COPY_IDFA);
    EndHorizontal();
    
    BeginHorizontal();
    addLabel("IDFV : ");
    addLabel(Configuration::getIDFV(), Configuration::LABEL_IDFV);
    addButton("COPY", {screenSize.width/4, DEFAULT_CONTENT_HEIGHT}, Configuration::BUTTON_COPY_IDFV);
    EndHorizontal();
#endif

    addCombobox("AppId\nList : ", std::move(appIdList), 0, Configuration::LIST_APP_ID);
    addEditBox("AppId : ", currentAppId.c_str(), "", EDITBOX_APP_ID);
    addEditBox("HiveCertificationKey : ", defaultHiveCertificationKey.c_str(), "Hive Certification Key", Configuration::EDITBOX_HIVE_CERTIFICATION_KEY);
    addToggleButton("Zone : ", {"Sandbox","Real","Test","Dev"}, zoneIndex, {screenSize.width, DEFAULT_CONTENT_HEIGHT}, Configuration::TOGGLE_BUTTON_ZONE);
    addToggleButton("HiveTheme : ", {"hiveLight", "hiveDark"}, hiveTheme, {screenSize.width, DEFAULT_CONTENT_HEIGHT}, Configuration::TOGGLE_BUTTON_THEME);
    addToggleButton("HiveOrientation : ", {"NotSet", "undefined", "all", "portrait", "landscape"}, selectedHiveOrientationIndex, {screenSize.width, DEFAULT_CONTENT_HEIGHT}, Configuration::TOGGLE_BUTTON_ORIENTATION);
    addCombobox("Game Language : ", std::move(languageDisplayName), selectedLanguageIndex, Configuration::LIST_GAME_LANGUAGE);
    addCombobox("Game Server ID List : ", std::move(gameServerIdList), 0, Configuration::LIST_GAME_SERVER_ID);
    addEditBox("Game Server ID : ", selectedGameServerId.c_str(), "", EDITBOX_GAME_SERVER_ID);

    if(cocos2d::Application::getInstance()->getTargetPlatform() == cocos2d::ApplicationProtocol::Platform::OS_IPHONE)
        addLabel("Market : AP", Configuration::LABEL_MARKET);
    else if(cocos2d::Application::getInstance()->getTargetPlatform() == cocos2d::ApplicationProtocol::Platform::OS_ANDROID)
        addLabel("Market : GO", Configuration::LABEL_MARKET);
    
    addButton("Get Hive Country", BUTTON_GET_HIVE_COUNTRY);
    addButton("Get Hive Timezone", BUTTON_GET_HIVE_TIMEZONE);
    addEditBox("Channel : ", hive::Configuration::getChannel().c_str(), "", EDITBOX_CHANNEL);
    addButton("Get Channel", BUTTON_GET_CHANNEL);

    addOnOffToggleButton("Use Log : ", bUseLog, {screenSize.width, DEFAULT_CONTENT_HEIGHT}, Configuration::TOGGLE_BUTTON_USE_LOG);
    addOnOffToggleButton("Use AgeGateU13 : ", bUseAgeGateU13, {screenSize.width, DEFAULT_CONTENT_HEIGHT}, Configuration::TOGGLE_BUTTON_USE_COPPA_AGE_GATE_U13);
    addOnOffToggleButton("Use Hercules : ", true, {screenSize.width, DEFAULT_CONTENT_HEIGHT}, Configuration::TOGGLE_BUTTON_USE_HERCULES);

    BeginHorizontal();
    addOnOffToggleButton("isOn : " , 1, {screenSize.width/3, DEFAULT_CONTENT_HEIGHT}, TOGGLE_BUTTON_IS_ON_HIVE);
    addButton("Configuration.setHivePermissionViewOn", {screenSize.width*2/3, DEFAULT_CONTENT_HEIGHT}, BUTTON_SET_HIVEPERMISSION_VIEW_ON);
    EndHorizontal();

    addCombobox("Select Language : ", std::move(languageDisplayName), selectedLanguageIndex, LIST_HIVE_LANGUAGE);
    addButton("Configuration.getPermissionViewData", cocos2d::Size(screenSize.width, DEFAULT_CONTENT_HEIGHT), BUTTON_GET_PERMISSION_VIEW_DATA);
    
    BeginHorizontal();
    addEditBox("MetaDataKey : ", metaDataKey.c_str(), "", {screenSize.width*2/5, DEFAULT_CONTENT_HEIGHT},cocos2d::ui::EditBox::InputMode::SINGLE_LINE, 20, EDITBOX_GET_METADATAKEY);
    addOnOffToggleButton("isNetwork : " , 1, {screenSize.width/5, DEFAULT_CONTENT_HEIGHT}, TOGGLE_BUTTON_RADIO_GET_METADATA_NETWORK);
    addButton("Configuration.getMetaData", {screenSize.width*2/5, DEFAULT_CONTENT_HEIGHT}, BUTTON_GET_METADATA);
    EndHorizontal();

    BeginHorizontal();
    addToggleButton("LogFilter : ", {"NotSet", "Verbose", "Debug", "Info", "Warning", "Error", "None"}, 2, {screenSize.width, DEFAULT_CONTENT_HEIGHT}, Configuration::TOGGLE_BUTTON_LOG_TYPE);
    EndHorizontal();

    BeginHorizontal();
    addEditBox("Hive Community Url : ", defaultHiveCommunityUrl.c_str(), "Hive Community Url", Configuration::EDITBOX_HIVE_COMMUNITY_URL);
    EndHorizontal();

    BeginHorizontal();
    addButton("Reset", {screenSize.width/2, DEFAULT_CONTENT_HEIGHT}, Configuration::BUTTON_RESET);
    addButton("OK", {screenSize.width/2, DEFAULT_CONTENT_HEIGHT}, Configuration::BUTTON_OK);
    EndHorizontal();
    
    setUseBackButton(true, "- Configuration -");
    
    for( int tag = Configuration::BUTTON_SDK_V1 ; tag <= Configuration::BUTTON_SDK_C2S ; tag++ ) {
        setUIVisible( false, tag);
    }
}

void Configuration::onButtonEvent(int buttonTag)
{
    switch (buttonTag) {
        case Configuration::BUTTON_OK:
        {
            apply();
            popView();
            break;
        }
        case Configuration::BUTTON_RESET:
        {
            reset();
            break;
        }
        case Configuration::LIST_APP_ID:
        {
            int selectedAppIdIndex = getSelectedComboboxIndex(LIST_APP_ID);
            std::string selectedAppId = appIdList[selectedAppIdIndex];
            setEditboxValue(EDITBOX_APP_ID, selectedAppId);
            break;
        }
        case Configuration::LIST_GAME_SERVER_ID:
        {
            int selectedGameServerIdIndex = getSelectedComboboxIndex(LIST_GAME_SERVER_ID);
            std::string selectedGameServerId = gameServerIdList[selectedGameServerIdIndex];
            setEditboxValue(EDITBOX_GAME_SERVER_ID, selectedGameServerId);
            break;
        }
        case Configuration::BUTTON_GET_HIVE_COUNTRY:
        {
            std::string value = hive::Configuration::getHiveCountry();
            cocos2d::MessageBox(value.c_str(),"getHiveCountry");
            break;
        }
        case Configuration::BUTTON_GET_HIVE_TIMEZONE:
        {
            std::string value = hive::Configuration::getHiveTimeZone();
            cocos2d::MessageBox(value.c_str(),"GetHiveTimezone");
            break;
        }
        case Configuration::BUTTON_GET_CHANNEL:
        {
            std::string value = hive::Configuration::getChannel();
            cocos2d::MessageBox(value.c_str(), "GetChannel");
            break;
        }
        case BUTTON_SET_HIVEPERMISSION_VIEW_ON:
        {
            bool isOnHive = getSelectedToggleButtonIndex(TOGGLE_BUTTON_IS_ON_HIVE);

            hive::Configuration::setHivePermissionViewOn(isOnHive);

            std::string message = cocos2d::StringUtils::format("HIVE SDK 권한고지 노출 설정 : %s\n", isOnHive ? "true" : "false");
            cocos2d::MessageBox(message.c_str(), "setHivePermissionViewOn");

            break;
        }
        case BUTTON_GET_PERMISSION_VIEW_DATA:
        {
            // Select Language
            int hiveLanguageIndex = getSelectedComboboxIndex(LIST_HIVE_LANGUAGE);
            std::string hiveLanguageStr = languageCode[hiveLanguageIndex];

            hive::HIVELanguage language;

            if (hiveLanguageStr.compare("de") == 0) {
                language = hive::HIVELanguage::HIVELanguageDE;
            }
            else if (hiveLanguageStr.compare("en") == 0) {
                language = hive::HIVELanguage::HIVELanguageEN;
            }
            else if (hiveLanguageStr.compare("es") == 0) {
                language = hive::HIVELanguage::HIVELanguageES;
            }
            else if (hiveLanguageStr.compare("fr") == 0) {
                language = hive::HIVELanguage::HIVELanguageFR;
            }
            else if (hiveLanguageStr.compare("id") == 0) {
                language = hive::HIVELanguage::HIVELanguageID;
            }
            else if (hiveLanguageStr.compare("it") == 0) {
                language = hive::HIVELanguage::HIVELanguageIT;
            }
            else if (hiveLanguageStr.compare("ja") == 0) {
                language = hive::HIVELanguage::HIVELanguageJA;
            }
            else if (hiveLanguageStr.compare("ko") == 0) {
                language = hive::HIVELanguage::HIVELanguageKO;
            }
            else if (hiveLanguageStr.compare("pt") == 0) {
                language = hive::HIVELanguage::HIVELanguagePT;
            }
            else if (hiveLanguageStr.compare("ru") == 0) {
                language = hive::HIVELanguage::HIVELanguageRU;
            }
            else if (hiveLanguageStr.compare("th") == 0) {
                language = hive::HIVELanguage::HIVELanguageTH;
            }
            else if (hiveLanguageStr.compare("tr") == 0) {
                language = hive::HIVELanguage::HIVELanguageTR;
            }
            else if (hiveLanguageStr.compare("vi") == 0) {
                language = hive::HIVELanguage::HIVELanguageVI;
            }
            else if (hiveLanguageStr.compare("zh-Hant") == 0) {
                language = hive::HIVELanguage::HIVELanguageZHT;
            }
            else if (hiveLanguageStr.compare("zh-Hans") == 0) {
                language = hive::HIVELanguage::HIVELanguageZHS;
            }
            else if (hiveLanguageStr.compare("ar") == 0) {
                language = hive::HIVELanguage::HIVELanguageAR;
            }

            hive::PermissionViewData data =  hive::Configuration::getPermissionViewData(language);

//            if (data == NULL) {
//                cocos2d::MessageBox("data not found", "getPermissionViewData");
//            }

            std::string permissionNotice = "";
            for (auto unit : data.permissions) {
                permissionNotice += cocos2d::StringUtils::format("권한명 : %s\n 타이틀 %s\n 내용 %s\n 권한 카테고리 : %d\n",unit.nativePermissionName.c_str(),unit.title.c_str(),unit.contents.c_str(),unit.permissionCategory);
            }
            for (auto unit : data.commons) {
                permissionNotice += cocos2d::StringUtils::format("권한명 : %s\n 타이틀 %s\n 내용 %s \n 권한 카테고리 : %d\n", unit.nativePermissionName.c_str(), unit.title.c_str(), unit.contents.c_str(), unit.permissionCategory);
            }
            cocos2d::MessageBox(permissionNotice.c_str(), data.contents.c_str());

            break;
        }
        case BUTTON_GET_METADATA:
        {
            apply();
            std::string metaData = getEditboxValue(EDITBOX_GET_METADATAKEY);
            bool isNetwork = getSelectedToggleButtonIndex(TOGGLE_BUTTON_RADIO_GET_METADATA_NETWORK);
            
            hive::Configuration::getMetaData(metaData,isNetwork,[=](hive::ResultAPI const & result,std::string value)mutable{
                cocos2d::MessageBox(value.c_str(),"getMetaData");
            });

            break;
        }
        case BUTTON_COPY_IDFV: {
            
#if (CC_TARGET_PLATFORM == CC_PLATFORM_IOS)
            Configuration::copyIDFV();
#endif
            break;
        }
        case BUTTON_COPY_IDFA: {
#if (CC_TARGET_PLATFORM == CC_PLATFORM_IOS)
            Configuration::copyIDFA();
#endif
            break;
        }
        case BUTTON_SDK_V1: {
            startv1();
            break;
        }
        case BUTTON_SDK_V4: {
            startv4();
            break;
        }
        case BUTTON_SDK_C2S: {
            startc2s();
            break;
        }
        default:
            break;
    }
}

void Configuration::startv1()
{

    AppDelegate::changeResolutionForSdkV1();    // V1으로 테스트 시작 시 resolution 변경

    auto v1layer = InitListScene::create();
    this->addChild(v1layer);
}

void Configuration::startc2s()
{
    auto c2MainScene = C2SMainScene::create();
    this->addChild(c2MainScene);
}
