//
//  DataStoreTest.cpp
//  HIVE_SDK_Cocos_Sample
//
//  Created by sjyong on 2022/03/03.
//

#include "DataStoreTest.hpp"
#include "ui/CocosGUI.h"
#include "UICombobox.hpp"
#include "UIInputList.hpp"
#include "picojson.h"
#include <sstream>

#define EDITBOX_CONTENT_CUSTOM_HEIGHT 200
#define EXTRA_DATA_PARAM_KEY_VALUE_SIZE 3

using namespace std;
using namespace cocos2d;
using namespace cocos2d::ui;
using namespace hive;


string DataStoreTest::mapToString(map<string, string> map) {
    picojson::object object;
    for (auto it = map.begin(); it != map.end(); it++) {
        object[it->first] = picojson::value(it->second);
    }
    picojson::value value = picojson::value(object);
    string result = value.serialize();

    return result;
}

string DataStoreTest::mapToString(map<long long, string> map) {
    picojson::object object;
    for (auto it = map.begin(); it != map.end(); it++) {
        long long keyLong = it->first;
        object[to_string(keyLong)] = picojson::value(it->second);
    }
    picojson::value value = picojson::value(object);
    string result = value.serialize();

    return result;
}

bool DataStoreTest::init() {

    if(!BaseView::init())
        return false;
    setUseShowLogButton(true);
    Logger::setLoggerListener([=](std::string tag, std::string level, std::string logMessage){
        addLogMessage(logMessage);
    });

    setUseBackButton(true, "- DataStore -");

    BeginHorizontal();
    addEditBox("key : ", "", "", {screenSize.width/2, DEFAULT_CONTENT_HEIGHT}, EditBox::InputMode::SINGLE_LINE, 20, EDITBOX_KEY);
    addEditBox("value : ", "", "", {screenSize.width/2, DEFAULT_CONTENT_HEIGHT}, EditBox::InputMode::SINGLE_LINE, 20, EDITBOX_VALUE);
    EndHorizontal();

    string dataString = this->mapToString(paramDataMap);
    addEditBox("data : ", dataString.c_str(), "", {screenSize.width, EDITBOX_CONTENT_CUSTOM_HEIGHT}, EditBox::InputMode::ANY, 20, EDITBOX_DATA);

    for (int enumTag = EDITBOX_DATA_KEY_1; enumTag <= EDITBOX_DATA_KEY_3; enumTag += 2) {
        BeginHorizontal();
        addEditBox("data-key : ", "", "", {screenSize.width/2, DEFAULT_CONTENT_HEIGHT}, EditBox::InputMode::SINGLE_LINE, 20, enumTag);
        addEditBox("data-value : ", "", "", {screenSize.width/2, DEFAULT_CONTENT_HEIGHT}, EditBox::InputMode::SINGLE_LINE, 20, enumTag+1);
        EndHorizontal();
    }

    addButton("DataStore.set(key, value)", BUTTON_SET_WITH_KEY_VALUE);
    addButton("DataStore.set(data)", BUTTON_SET_WITH_DATA);
    addButton("DataStore.get(key)", BUTTON_GET);
    addButton("DataStore.getMyData()", BUTTON_GET_MY_DATA);
    addButton("DataStore.getUsersData(key)", BUTTON_GET_USERS_DATA);

    addEditBox("callback data : ", "", "", {screenSize.width, EDITBOX_CONTENT_CUSTOM_HEIGHT}, EditBox::InputMode::ANY, 20, EDITBOX_CALLBACK_DATA);

    return true;
}

void DataStoreTest::onButtonEvent(int buttonTag) {

    paramKey = getEditboxValue(EDITBOX_KEY);
    paramValue = getEditboxValue(EDITBOX_VALUE);
    paramData = getEditboxValue(EDITBOX_DATA);

    for (int extraTag = EDITBOX_DATA_KEY_1; extraTag <= EDITBOX_DATA_KEY_3; extraTag += 2) {
        string extraDataKey = getEditboxValue(extraTag);
        string extraDataValue = getEditboxValue(extraTag+1);

        paramDataMap[extraDataKey] = extraDataValue;
    }

    switch(buttonTag) {
        case BUTTON_SET_WITH_KEY_VALUE:
        {
            DataStore::set(paramKey, paramValue, [=](ResultAPI const & result){
                stringstream ss;
                ss << "=== DataStore.set(key, value) callback ===" << '\n';
                ss << "{" << '\n';
                ss << "result = " << result.toString() << '\n';
                ss << "}" << '\n';
                Logger::log(ss.str().c_str());
            });
            Logger::log("DataStore.set(key, value) called.");
            break;
        }
        case BUTTON_SET_WITH_DATA:
        {
            DataStore::set(paramDataMap, [=](ResultAPI const & result){
                stringstream ss;
                ss << "=== DataStore.set(data) callback ===" << '\n';
                ss << "{" << '\n';
                ss << "result = " << result.toString() << '\n';
                ss << "}" << '\n';
                Logger::log(ss.str().c_str());
            });
            Logger::log("DataStore.set(data) called.");
            break;
        }
        case BUTTON_GET:
        {
            DataStore::get(paramKey, [=](ResultAPI const & result, string data){

                cocos2d::Director::getInstance()->getScheduler()->performFunctionInCocosThread([=](){
                    setEditboxValue(EDITBOX_CALLBACK_DATA, data);
                });

                stringstream ss;
                ss << "=== DataStore.get(key) callback ===" << '\n';
                ss << "{" << '\n';
                ss << "result = " << result.toString() << '\n';
                ss << "data = " << data << '\n';
                ss << "}" << '\n';
                Logger::log(ss.str().c_str());
            });
            Logger::log("DataStore.get(key) called.");
            break;
        }
        case BUTTON_GET_MY_DATA:
        {
            DataStore::getMyData([=](ResultAPI const & result, map<string, string> const & myData){
                string myDataString = this->mapToString(myData);
                
                cocos2d::Director::getInstance()->getScheduler()->performFunctionInCocosThread([=](){
                    setEditboxValue(EDITBOX_CALLBACK_DATA, myDataString);
                });

                stringstream ss;
                ss << "=== DataStore.getMyData() callback ===" << '\n';
                ss << "{" << '\n';
                ss << "result = " << result.toString() << '\n';
                ss << "myData = " << myDataString << '\n';
                ss << "}" << '\n';
                Logger::log(ss.str().c_str());
            });
            Logger::log("DataStore.getMyData() called.");
            break;
        }
        case BUTTON_GET_USERS_DATA:
        {
            DataStore::getUsersData(paramKey, [=](ResultAPI const & result, string key, map<long long, string> const & usersData){
                string usersDataString = this->mapToString(usersData);

                cocos2d::Director::getInstance()->getScheduler()->performFunctionInCocosThread([=](){
                    setEditboxValue(EDITBOX_CALLBACK_DATA, usersDataString);
                });

                stringstream ss;
                ss << "=== DataStore.getMyData(key) callback ===" << '\n';
                ss << "{" << '\n';
                ss << "result = " << result.toString() << '\n';
                ss << "key = " << key << '\n';
                ss << "usersData = " << usersDataString << '\n';
                ss << "}" << '\n';
                Logger::log(ss.str().c_str());
            });
            Logger::log("DataStore.getUsersData(key) called.");
            break;
        }
        default:

            break;
    }
}
