//
//  ExternalAuth.cpp
//  HIVE_SDK_Cocos_Sample
//
//  Created by sjyong on 2022/02/07.
//

#include "ExternalAuth.hpp"
#include "GameManager.hpp"
#include "cocos2d.h"

using namespace cocos2d;
using namespace cocos2d::network;

ExternalAuth* ExternalAuth::instance = nullptr;

string ExternalAuth::authTypeToString(AuthType type) {
    switch (type) {
        case AuthType::SIGNINIDP:
            return "SIGNINIDP";
        case AuthType::CONNECT:
            return "CONNECT";
    }
}

void ExternalAuth::requestIdp(ZoneType zoneType, string appId, AuthType authType, string idpId, string idpUserId, function<void(const ExternalAuthResponse* response)> callback)
{
    picojson::object parameters;
    parameters["appid"] = picojson::value(appId);
    parameters["idp_id"] = picojson::value(idpId);
    parameters["idp_user_id"] = picojson::value(idpUserId);
    parameters["auth_type"] = picojson::value(authTypeToString(authType));
    if(authType == CONNECT) {
        parameters["vid"] = picojson::value(GameManager::getInstance()->GetCurrentPlayerInfo().playerId);
    }

    string paramData = picojson::value(parameters).serialize();

    hive::Logger::log("posts : %s", paramData.c_str());
    //std::function<void(HttpClient* client, HttpResponse* response)
    requestAPI(signInURL[zoneType].c_str(), paramData.c_str(), [=](HttpClient* client, HttpResponse* response){

        ExternalAuthResponse* authResponse = new ExternalAuthResponse();

        auto retdata = std::string(response->getResponseData()->begin(), response->getResponseData()->end());
        picojson::value v;
        std::string e = picojson::parse(v,retdata);

        if(!e.empty())
        {
            authResponse->resultCode = -1;
            authResponse->resultMsg = StringUtils::format("json parse error \n(%s)",e.c_str());
            return;
        }

        if (!v.get("result_code").is<picojson::null>()) {
            authResponse->resultCode = (long)v.get("result_code").get<double>();
        }

        if (!v.get("result_msg").is<picojson::null>()) {
            authResponse->resultMsg = v.get("result_msg").get<string>();
        }

        if (!v.get("result_data").is<picojson::null>()) {
            authResponse->resultData = v.get("result_data");

            if (!authResponse->resultData.get("auth_key").is<picojson::null>()) {
                authResponse->authKey = authResponse->resultData.get("auth_key").get<string>();
            }
        }

        if (callback != NULL) {
            callback(authResponse);
        }
    });
    
}

void ExternalAuth::requestAPI(const char *url,const char *data,const ccHttpRequestCallback callback)
{
    auto network = HttpClient::getInstance();
    auto *req = new HttpRequest();
    req->setUrl(url);
    req->setRequestType(HttpRequest::Type::POST);
    req->setHeaders({"Content-Type:application/json",
                    StringUtils::format("Content-Length:%zu",strlen(data))});
    req->setRequestData(data, strlen(data));

    req->setResponseCallback(callback);

    network->send(req);
    req->release();
}


