//
//  GameManager.cpp
//  HIVE_SDK_Cocos_Sample
//
//  Created by paikjongman on 2017. 7. 26..
//  Copyright © 2017년 GCP. All rights reserved.
//

#include "GameManager.hpp"
#include "cocos2d.h"
#include "ResultUtils.hpp"
USING_NS_HIVE;
GameManager::GameManager(){}
GameManager::~GameManager(){}
static GameManager* g_instance = nullptr;

bool GameManager::ErrorCheckResultAPI(hive::ResultAPI const & api) {
    if(!api.isSuccess()) {
        std::string message = getResultString(api);
        cocos2d::MessageBox(message.c_str(), "API Call Fail");
        return false;
    }
    return true;
}
GameManager* GameManager::getInstance() {
    if(g_instance == nullptr){
        g_instance = new GameManager();
    }
    return g_instance;
}
std::string GameManager::getResultString(hive::ResultAPI const & api){
    return ResultUtils::getResultAPIString(api);
}
void GameManager::engagemenetResult(hive::ResultAPI const & result, hive::EngagementEventType engagementEventType, hive::EngagementEventState engagementEventState, picojson::value const & param){
    Logger::log("=== Engagement === \n");
    switch (engagementEventState) {
        case hive::EngagementEventState::BEGIN:
            Logger::log("Engagement BEGIN Action!!!\n");
            if(engagementEventType == hive::EngagementEventType::EVENT_TYPE){
                Logger::log("Engagement EVENT_TYPE Action!!!\n");
            }
            break;
        case hive::EngagementEventState::FINISH:
            Logger::log("Engagement FINISH Action!!!\n");
            if(engagementEventType == hive::EngagementEventType::EVENT_TYPE){
                Logger::log("Engagement EVENT_TYPE Action!!!\n");
            }
            break;
        case hive::EngagementEventState::EVENT_START: {
            Logger::log("Engagement EVENT_START Action!!!\n");
            switch (engagementEventType) {
                case hive::EngagementEventType::AUTH_LOGIN_VIEW:
                    Logger::log("Engagement AUTH_LOGIN_VIEW Action!!!\n");
                    break;
                case hive::EngagementEventType::COUPON:
                    Logger::log("Engagement COUPON Action!!!\n");
                    break;
                case hive::EngagementEventType::EVENT:
                    Logger::log("Engagement EVENT Action!!!\n");
                    break;
                case hive::EngagementEventType::IAP_PROMOTE:
                    Logger::log("Engagement IAP_PROMOTE Action!!!\n");
                    break;
                case hive::EngagementEventType::IAP_PURCHASE:
                    Logger::log("Engagement IAP_PURCHASE Action!!!\n");
                    break;
                case hive::EngagementEventType::IAP_UPDATED:
                    Logger::log("Engagement IAP_UPDATED Action!!!\n");
                    break;
                case hive::EngagementEventType::OFFERWALL_VIEW:
                    Logger::log("Engagement OFFERWALL_VIEW Action!!!\n");
                    break;
                case hive::EngagementEventType::PROMOTION_VIEW:
                    Logger::log("Engagement PROMOTION_VIEW Action!!!\n");
                    break;
                case hive::EngagementEventType::SOCIAL_INQUIRY_VIEW:
                    Logger::log("Engagement SOCIAL_INQUIRY_VIEW Action!!!\n");
                    break;
                case hive::EngagementEventType::USER_ACQUISITION:
                    Logger::log("Engagement USER_ACQUISITION Action!!!\n");
                    break;
                case hive::EngagementEventType::SOCIAL_MYINQUIRY_VIEW:
                    Logger::log("Engagement SOCIAL_MYINQUIRY_VIEW Action!!!\n");
                    break;
                case hive::EngagementEventType::SOCIAL_PROFILE_VIEW:
                    Logger::log("Engagement SOCIAL_PROFILE_VIEW Action!!!\n");
                    break;
                case hive::EngagementEventType::COMMUNITY_VIEW:
                    Logger::log("Engagement COMMUNITY_VIEW Action!!!\n");
                    break;
                default:
                    Logger::log("Engagement ERROR UnKnown Event type\n");
                    break;
            }
        }
            break;
        case hive::EngagementEventState::EVENT_END: {
            Logger::log("Engagement EVENT_END Action!!!\n");
            switch (engagementEventType) {
                case hive::EngagementEventType::AUTH_LOGIN_VIEW:
                    Logger::log("Engagement AUTH_LOGIN_VIEW Action!!!\n");
                    if(isConflict(param)){
                        Logger::log("Engagement AUTH_LOGIN_VIEW isConflict Action!!!\n");
                        auto current_account = param.get("currentAccount");
                        auto used_account = param.get("usedAccount");
                        
                        std::string current_vid = current_account.get("vid").get<std::string>();
                        std::string used_vid = used_account.get("vid").get<std::string>();
                        
                        picojson::object currentVidData;
                        currentVidData["vid"] = picojson::value(current_vid);
                        
                        // 계정에 따라 저장하고 싶은 커스텀한 게임 데이터
                        picojson::object currentVidGameData;
                        currentVidGameData["Name"] = picojson::value("Nam");
                        currentVidGameData["Level"] = picojson::value((double)20);
                        currentVidData["data"] = picojson::value(currentVidGameData);
                        
                        picojson::object usedVidData;
                        usedVidData["vid"] = picojson::value(used_vid);
                        
                        // 계정에 따라 저장하고 싶은 커스텀한 게임 데이터
                        picojson::object usedVidGameData;
                        usedVidGameData["Name"] = picojson::value("An");
                        usedVidGameData["Level"] = picojson::value((double)10);
                        usedVidData["data"] = picojson::value(usedVidGameData);
                        
                        Logger::log("\n\n=== Auth::login() 메서드를 호출 후 결과 값이 LoginType.SELECT 이면 이 메서드를 호출 하여 사용자를 선택을 요청 ===\n");
                        Logger::log("Auth::showLoginSelection(currentVidData, usedVidData, onAuthLogin) Called\n");
                        Logger::log("currentVidData = %s\n", picojson::value(currentVidData).serialize().c_str());
                        Logger::log("usedVidData = %s\n", picojson::value(usedVidData).serialize().c_str());
                        
                        Auth::showLoginSelection(currentVidData,
                                                 usedVidData,
                                                 // 로그인 결과 통지 리스너 - Auth::login() / Auth::showLoginSelection()
                                                 [=](ResultAPI result, LoginType loginType, Account currentAccount, Account usedAccount)
                                                 {
                                                     Logger::log("AuthTestView::onAuthLogin() Callback\nresult = %s\n", result.toString().c_str());
                                                     Logger::log("currentAccount = %s\n", currentAccount.toString().c_str());
                                                     Logger::log("usedAccount = %s\n", usedAccount.toString().c_str());
                                                 });
                        
                    }
                    break;
                case hive::EngagementEventType::COUPON:
                    Logger::log("Engagement COUPON Action!!!\n");
                    break;
                case hive::EngagementEventType::EVENT:
                    Logger::log("Engagement EVENT Action!!!\n");
                    break;
                case hive::EngagementEventType::IAP_PROMOTE:
                    Logger::log("Engagement IAP_PROMOTE Action!!!\n");
                    break;
                case hive::EngagementEventType::IAP_PURCHASE:
                    Logger::log("Engagement IAP_PURCHASE Action!!!\n");
                    break;
                case hive::EngagementEventType::IAP_UPDATED:
                    Logger::log("Engagement IAP_UPDATED Action!!!\n");
                    break;
                case hive::EngagementEventType::OFFERWALL_VIEW:
                    Logger::log("Engagement OFFERWALL_VIEW Action!!!\n");
                    break;
                case hive::EngagementEventType::PROMOTION_VIEW:
                    Logger::log("Engagement PROMOTION_VIEW Action!!!\n");
                    break;
                case hive::EngagementEventType::SOCIAL_INQUIRY_VIEW:
                    Logger::log("Engagement SOCIAL_INQUIRY_VIEW Action!!!\n");
                    break;
                case hive::EngagementEventType::USER_ACQUISITION:
                    Logger::log("Engagement USER_ACQUISITION Action!!!\n");
                    break;
                case hive::EngagementEventType::SOCIAL_MYINQUIRY_VIEW:
                    Logger::log("Engagement SOCIAL_MYINQUIRY_VIEW Action!!!\n");
                    break;
                case hive::EngagementEventType::SOCIAL_PROFILE_VIEW:
                    Logger::log("Engagement SOCIAL_PROFILE_VIEW Action!!!\n");
                    break;
                case hive::EngagementEventType::COMMUNITY_VIEW:
                    Logger::log("Engagement COMMUNITY_VIEW Action!!!\n");
                    break;
                default:
                    Logger::log("Engagement ERROR UnKnown Event type\n");
                    break;
            }
        }
            break;
        default:
            Logger::log("Engagement ERROR UnKnown Event state\n");
            break;
    }
    
    Logger::log("EngagementHandler Callback\nresult = %s\n",result.toString().c_str());
    Logger::log("param = %s\n",param.serialize().c_str());
}

bool GameManager::isConflict(picojson::value const & param){
    return !(param.get("currentAccount").is<picojson::null>()) && !(param.get("usedAccount").is<picojson::null>()) && param.get("usedAccount").contains("vid");
}

void GameManager::SetCurrentAuthModule(AuthModule module){
    authModule = module;
}
GameManager::AuthModule GameManager::GetCurrentAuthModule() const{
    return authModule;
}

hive::PlayerInfo const & GameManager::GetCurrentPlayerInfo() const{
    return currentPlayerInfo;
}
void GameManager::SetCurrentPlayerInfo(hive::PlayerInfo const & playerInfo){
    currentPlayerInfo = playerInfo;
}

hive::PlayerInfo const & GameManager::GetConflictPlayerInfo() const {
    return conflictPlayerInfo;
}
void GameManager::SetConflictPlayerInfo(hive::PlayerInfo const & playerInfo) {
    conflictPlayerInfo = playerInfo;
}
