//
//  PostBoxView.cpp
//  HIVE_SDK_Cocos_Sample
//
//  Created by paikjongman on 2018. 1. 15..
//  Copyright © 2018년 GCP. All rights reserved.
//

#include "PostBoxView.hpp"
#include "ResultUtils.hpp"

using namespace hive;

void ShowPostboxErrorMessage(Postbox::Error error){
    auto msg = cocos2d::StringUtils::format("Error code = %s \n, msg %s",error.result_code.c_str(), error.result_msg.c_str());
    cocos2d::MessageBox(msg.c_str(),"Postbox error");
    Logger::log(msg.c_str());
}

void ShowSubscriptionInfoMessage(Postbox::Subscription& subs) {
    auto msg = cocos2d::StringUtils::format("product_id:%s\nmarket_id:%s\nstart_date:%s\nstart_date_ms:%s\nexpire_date:%s\nexpire_date_ms:%s\ncharacter_key:%s",
                                            subs.product_id.c_str(),
                                            subs.market_id.c_str(),
                                            subs.start_date.c_str(),
                                            subs.start_date_ms.c_str(),
                                            subs.expire_date.c_str(),
                                            subs.expire_date_ms.c_str(),
                                            subs.character_key.c_str());
    cocos2d::MessageBox(msg.c_str(),"Subscription Info");
    Logger::log(msg.c_str());
}

bool PostBoxView::init() {
    if(!BaseView::init())
        return false;
    //    LABEL_SERVER,
    //    LABEL_GEM,
    //    LABEL_GOLD,
    //
    //
    postlist.push_back("empty");
    subscriptionlist.push_back("empty");
    
    setUseShowLogButton(true);
    
    addLabel("POSTBOX");
    addCombobox("Postbox : ", toCharlist(postlist), 0, COMBOBOX_POSTBOX);
    addButton("Get POST", BUTTON_GET_POST);
    addButton("Get All POSTS", BUTTON_GET_ALL_POSTS);
    addDottedLine();
    addLabel("Subscriptions");
    addCombobox("Subscriptions : ", toCharlist(subscriptionlist), 0, COMBOBOX_SUBSCRIPTIONBOX);
    addButton("Show Subscription Info", BUTTON_SHOW_SUBSCRIPTIONINFO);
    addDottedLine();
    addButton("Refresh Postbox", BUTTON_REFRESH_POSTS);
    
    loadGameData();
    
    setUseBackButton(true, "- PostBox -");
    
    return true;
}

void PostBoxView::loadGameData() {
    Postbox::login([=](const Postbox::Login *login, Postbox::Error error){
        if( login != nullptr )
        {
            int gold = login->gold;
            int gem = login->gem;
            std::string nickname = login->nickname;
            Logger::log("nick %s, gold %d, gem %d",nickname.c_str(),gold,gem);
            updatePosts();
        }
        else
        {
            ShowPostboxErrorMessage(error);
        }
    });
}

void PostBoxView::onButtonEvent(int buttonTag) {
    
    switch(buttonTag) {
        case BUTTON_GET_POST:
        {
            if(postlist.size() > 0 && postlist[0] != "empty")
            {
                int postindex = getSelectedComboboxIndex(COMBOBOX_POSTBOX);
                Postbox::postReceive({ postboxlists[postindex].post_id.c_str() }, [=](const Postbox::PostsReceive * r, Postbox::Error e){
                    if(e.result_code != "200") {
                        cocos2d::MessageBox(std::string("Post Receive Fail : " + e.result_msg).c_str(), "Postbox info");
                    }else{
                        cocos2d::MessageBox("Post Receive Success", "Postbox info");
                    }
                    updatePosts();
                });
            }
            else{
                cocos2d::MessageBox("Empty Postbox", "Postbox info");
            }
        }
            break;
        case BUTTON_GET_ALL_POSTS:
        {
            if(postlist.size() > 0 && postlist[0] != "empty")
            {
                std::vector<const char*> v;
                for(int i=0;i<postboxlists.size();++i){
                    v.push_back(postboxlists[i].post_id.c_str());
                }
                Postbox::postReceive(v, [=](const Postbox::PostsReceive * r, Postbox::Error e){
                    if(e.result_code != "200") {
                        cocos2d::MessageBox(std::string("Post Receive Fail : " + e.result_msg).c_str(), "Postbox info");
                    }else{
                        cocos2d::MessageBox("Post Receive Success", "Postbox info");
                    }
                    updatePosts();
                });
            }
            else{
                cocos2d::MessageBox("Empty Postbox", "Postbox info");
            }
            
        }
            break;
        case BUTTON_REFRESH_POSTS:
        {
            updatePosts();
        }
            break;
        case BUTTON_SHOW_SUBSCRIPTIONINFO:
        {
            if(subscriptionlist.size() > 0 && subscriptionlist[0] != "empty")
            {
                int index = getSelectedComboboxIndex(COMBOBOX_SUBSCRIPTIONBOX);
                ShowSubscriptionInfoMessage(subscriptionboxlists[index]);
            }
        }
            break;
        default:
            break;
    }
}

void PostBoxView::updatePosts() {
    StartActivityIndicator();
    Postbox::posts([=](const Postbox::Posts * posts,Postbox::Error error){
        if(posts != nullptr){
            Logger::log("callback hive postbox");
            
            //Post BOX!!
            postboxlists = posts->post_list;
            if(postboxlists.size() > 0){
                postlist.clear();
            } else{
                postlist.clear();
                postlist.push_back("empty");
            }
            
            for(auto post:postboxlists){
                if(post.character_key.size() > 0){
                    postlist.push_back("character:"+post.character_key+"["+post.post_id+"]["+post.date_added+"]");
                }else {
                    postlist.push_back("["+post.post_id+"]["+post.date_added+"]");
                }
            }
            changeComboboxList(toCharlist(postlist), 0, COMBOBOX_POSTBOX);
            
            //Subscription BOX!!
            subscriptionboxlists = posts->subscription_list;
            if(subscriptionboxlists.size() > 0){
                subscriptionlist.clear();
            } else{
                subscriptionlist.clear();
                subscriptionlist.push_back("empty");
            }
            
            for(auto subs:subscriptionboxlists){
                if(subs.character_key.size() > 0){
                    subscriptionlist.push_back("character:"+subs.character_key+"["+subs.product_id+"]["+subs.start_date+"]");
                }else {
                    subscriptionlist.push_back("["+subs.product_id+"]["+subs.start_date+"]");
                }
            }
            changeComboboxList(toCharlist(subscriptionlist), 0, COMBOBOX_SUBSCRIPTIONBOX);
            
        }else {
            ShowPostboxErrorMessage(error);
        }
        EndActivityIndicator();
    });
}

std::vector<const char*> PostBoxView::toCharlist(std::vector<std::string>& strlist)
{
    std::vector<const char*> v;
    
    for(int i=0;i<strlist.size();++i){
        v.push_back(strlist[i].c_str());
    }
    return v;
}
