//
//  Postbox.hpp
//  HIVE_SDK_v4_Cocos_Sample
//
//  Created by paikjongman on 2017. 3. 3..
//  Copyright © 2017년 ryuvsken. All rights reserved.
//

#ifndef Postbox_hpp
#define Postbox_hpp

#include <map>
#include <HIVE_SDK_Plugin/HIVE_CPP.h>
#include "network/HttpClient.h"


//document : http://misample.com2us.net/
class Postbox
{
public:
    
    enum LoginType {
        v0,
        v1,
        v4,
    };
    
    enum ItemType {
        consumable,
        subscription
    };
    
    struct Error {
        std::string result_code;
        std::string result_msg;
    };
    
    struct Login {
        std::string nickname;
        int gold;
        int gem;
        std::string session_token;
    };
    
    struct UserInfo {
        std::string nickname;
        int gold;
        int gem;
    };
    
    struct Post {
        std::string post_id;
        std::string asset_code;
        int asset_amount;
        std::string user_msg;
        std::string character_key;
        std::string date_added;
    };
    
    struct Subscription {
        std::string product_id;
        std::string market_id;
        std::string start_date;
        std::string start_date_ms;
        std::string expire_date;
        std::string expire_date_ms;
        std::string character_key;
    };
    
    typedef std::vector<Post> PostList;
    typedef std::vector<Subscription> SubscriptionList;
    
    struct Posts {
        PostList post_list;
        SubscriptionList subscription_list;
        std::string session_token;
    };
    
    struct PostsReceive {
        std::string nickname;
        int gold;
        int gem;
        PostList post_list;
        std::string session_token;
    };
    
    struct VerifyReceipt {
        std::string product_id;
    };

    enum ColomnState: int {
        CONTINUE = 0,
        END = 1,
    };
    
    static void login(std::function<void(const Login *data,Error error)> callback);
    static void userInfo(std::function<void(const UserInfo *data,Error error)> callback);
    static void posts(std::function<void(const Posts *data,Error error)> callback);
    static void postReceive(std::vector<const char*> postids,std::function<void(const PostsReceive *data,Error error)> callback);
    
    //IAPV4를 위한 영수증 검증 API
    static void verifyReceipt(std::string bypassinfo,ItemType itemType,std::function<void(const VerifyReceipt *data,Error error)> callback);
    
    static void setLoginType(LoginType type);
private:
    static void request(const char *url,const char *data,const cocos2d::network::ccHttpRequestCallback callback);
    static std::string getUrl(const char *api);
    static bool parseAndErrorCheck(cocos2d::network::HttpResponse *response,picojson::value &v,Error &err);
    static PostList getPostData(picojson::value v);
    static SubscriptionList getSubscriptionData(picojson::value v);
    static std::string getVidColumn();
    static std::string getWorldColumn(ColomnState state); // isEnd : 우편함 서버에 전송하는 데이터중 마지막 파라미터에 해당하는지
    static LoginType currentLoginType;
    static std::string cache_session_token;
};

#endif /* Postbox_hpp */
