//
//  SocialHiveTest.cpp
//  hivesdk
//
//  Created by paikjongman on 2017. 2. 28..
//
//

#include "SocialHiveTest.hpp"

const std::vector<const char*> friendTypeLists = {"IN_GAME","OUT_GAME","INVITED","ALL_GAME"};
const std::vector<const char*> hiveDialogTypeLists = {"HOME","GAME","USER","INQUIRY","MESSAGE", "CHATBOT", "MYINQUIRY"};
using namespace hive;

bool SocialHiveTest::init()
{
    if(!BaseView::init())
        return false;
    
    setUseShowLogButton(true);
    Logger::setLoggerListener([=](std::string tag, std::string level, std::string logMessage){
        addLogMessage(logMessage);
    });
    
    addButton("SocialHive.getMyProfile()", BUTTON_GET_MY_PROFILE);
    addEditBox("Comment : ", "Comment", EDITBOX_COMMENT);
    addButton("SocialHive.setMyProfile()", BUTTON_SET_MY_PROFILE);
    
    addCombobox("FriendType : ", std::move(friendTypeLists), 0, COMBOBOX_FRIEND_TYPE);
    addButton("SocialHive.getFriends()", BUTTON_GET_FRIENDS);
    
    // TODO : vid 를 콤보박스로 변경후 친구조회후 리스트 넣어주고 선택지를 자동 설정해주기.
//    addEditBox("uid (uid is HIVE user key)", "uid", EDITBOX_FRIENDS_UID);
    addEditBox("vid (vid is Hive UserKey) : ", "vid", EDITBOX_FRIENDS_VID);
    addEditBox("Message : ", "Message", EDITBOX_FRIENDS_MESSAGE);
    addButton("SocialHive.sendMessage()", BUTTON_SEND_MESSAGE);
    
    addEditBox("Invitation uid (uid is Hive UserKey) : ", "uid", EDITBOX_INVITATION_UID);
    addEditBox("Invitation Message : ", "Message", EDITBOX_INVITATION_MESSAGE);
    addButton("SocialHive.sendInvitationMessage()", BUTTON_SENDINVITATION);
    
    addCombobox("HiveDialogType : ", std::move(hiveDialogTypeLists),0, RADIO_HIVE_DIALOG_TYPE);
    addEditBox("vid (vid is Hive UserKey) : ", "vid", EDITBOX_DIALOG_VID);
    addButton("SocialHive.showHiveDialog()", BUTTON_SHOW_HIVE_DIALOG);
    
    setUseBackButton(true, "- SocialHive -");
    
    return true;
}
/*
 BUTTON_GET_MY_PROFILE = 0,
 EDITBOX_COMMENT,
 BUTTON_SET_MY_PROFILE,
 RADIO_FRIEND_TYPE,
 BUTTON_GET_FRIENDS,
 EDITBOX_FRIENDS_UID,
 EDITBOX_FRIENDS_VID,
 EDITBOX_FRIENDS_MESSAGE,
 BUTTON_SEND_MESSAGE,
 EDITBOX_INVITATION_UID,
 EDITBOX_INVITATION_MESSAGE,
 BUTTON_SENDINVITATION,
 RADIO_HIVE_DIALOG_TYPE,
 EDITBOX_DIALOG_VID,
 BUTTON_SHOW_HIVE_DIALOG,
 */

void SocialHiveTest::onButtonEvent(int buttonTag)
{
    switch (buttonTag) {
        case BUTTON_GET_MY_PROFILE:
            Logger::log("\n\n=== HIVE 인증 사용자의 자기 프로필 조회 ===\n");
            Logger::log("SocialHive::getMyProfile(onProfileHive) Called\n");
            SocialHive::getMyProfile(std::bind(&SocialHiveTest::onProfileHive, this, std::placeholders::_1, std::placeholders::_2));
            break;
        case BUTTON_SET_MY_PROFILE:
        {
            std::string comment = getEditboxValue(EDITBOX_COMMENT);
            Logger::log("\n\n=== HIVE 인증 사용자의 자기 대화명 수정 ===\n");
            Logger::log("SocialHive::setMyProfile(%s, onProfileHive) Called\n",
                        comment.c_str());
            SocialHive::setMyProfile(comment,
                                     std::bind(&SocialHiveTest::onProfileHive, this, std::placeholders::_1, std::placeholders::_2));
            break;
        }
        case BUTTON_GET_FRIENDS:
        {
            int index = getSelectedComboboxIndex(COMBOBOX_FRIEND_TYPE);
            Logger::log("\n\n=== HIVE 인증 사용자의 친구 목록 조회 ===\n");
            Logger::log("SocialHive::getFriends(FriendType = %s, onFriendsHive) Called\n",
                        friendTypeLists[static_cast<int>(index)]);
            SocialHive::getFriends(static_cast<FriendType>(index),
                                   std::bind(&SocialHiveTest::onFriendsHive, this, std::placeholders::_1, std::placeholders::_2));
            break;
        }
        case BUTTON_SEND_MESSAGE:
        {
            MessageContent messageContent;
//            messageContent.uid = getEditboxValue(EDITBOX_FRIENDS_UID);
            messageContent.vid = getEditboxValue(EDITBOX_FRIENDS_VID);
            messageContent.message = getEditboxValue(EDITBOX_FRIENDS_MESSAGE);
            
            Logger::log("\n\n=== Hive 친구에게 메시지 전송 요청 ===\n");
            Logger::log("SocialHive::sendMessage(messageContent = %s) Called\n"
                        , messageContent.toString().c_str());
            
            SocialHive::sendMessage(messageContent,
                                    std::bind(&SocialHiveTest::onSendMessageHive, this, std::placeholders::_1));
            break;
        }
        case BUTTON_SENDINVITATION:
        {
            MessageContent messageContent;
            messageContent.uid = getEditboxValue(EDITBOX_INVITATION_UID);
            messageContent.message = getEditboxValue(EDITBOX_INVITATION_MESSAGE);
            
            Logger::log("\n\n=== Hive 친구에게 초대 메시지 전송 요청 ===\n");
            Logger::log("SocialHive::sendInvitationMessage(messageContent = %s) Called\n"
                        , messageContent.toString().c_str());
            
            SocialHive::sendInvitationMessage(messageContent,
                                              std::bind(&SocialHiveTest::onSendMessageHive, this, std::placeholders::_1));
            break;
        }
        case BUTTON_SHOW_HIVE_DIALOG:
        {
            int index = getSelectedComboboxIndex(RADIO_HIVE_DIALOG_TYPE);
            std::string vid = getEditboxValue(EDITBOX_DIALOG_VID);
            
            Logger::log("\n\n=== HIVE 웹뷰 대화상자 호출 ===\n");
            Logger::log("SocialHive::showHiveDialog(HiveDialogType = %s, vid = %s, onShowHiveDialog) Called\n"
                        , hiveDialogTypeLists[static_cast<int>(index)], vid.c_str());


            HiveDialogType dialogType = static_cast<HiveDialogType>(index);

            if (dialogType == HiveDialogType::CHATBOT) {
                // 챗봇 페이지 바로가기 호출시 전달받기로한 약속된 JSON 형식의 String 데이터
                std::string additionalInfo = "{\"evt_code\":1000,\"p1\":\"random\"}";
                SocialHive::showHiveDialog(static_cast<HiveDialogType>(index), vid, additionalInfo,
                                           std::bind(&SocialHiveTest::onShowHiveDialog, this, std::placeholders::_1));
            } else {
                SocialHive::showHiveDialog(static_cast<HiveDialogType>(index), vid,
                                           std::bind(&SocialHiveTest::onShowHiveDialog, this, std::placeholders::_1));
            }

            break;
        }
        default:
            break;
    }
}

// Hive Social 프로필 정보 통지 결과 리스너 - SocialHive::getMyProfile() / SocialHive::setMyProfile() / SocialHive::getFriends()
void SocialHiveTest::onProfileHive(ResultAPI result, std::vector<ProfileHive> profileList)
{
    Logger::log("SocialHiveTestView::onProfileHive() Callback\nresult = %s", result.toString().c_str());
    
    Logger::log("profileList =");
    
    for (int i = 0; i < (int)profileList.size(); i++) {
        
        ProfileHive profile = profileList[i];
        
        Logger::log("%s\n", profile.toString().c_str());
    }
}


// Hive Social 친구 정보 통지 결과 리스너 - SocialHive::getFriends()
void SocialHiveTest::onFriendsHive(ResultAPI result, std::vector<ProfileHive> profileList)
{
    Logger::log("SocialHiveTestView::onFriendsHive() Callback\nresult = %s", result.toString().c_str());
    
    Logger::log("SocialHive::ProfileHive List =\n");
    
    for (int i = 0; i < (int)profileList.size(); i++) {
        
        ProfileHive profile = profileList[i];
        
        Logger::log("%s\n", profile.toString().c_str());
    }
}


// Hive Social 친구 메시지 전송 통지 결과 리스너 - SocialHive::sendMessage()
void SocialHiveTest::onSendMessageHive(ResultAPI result)
{
    Logger::log("SocialHiveTestView::onSendMessageHive() Callback\nresult = %s\n", result.toString().c_str());
}


// HIVE 웹뷰 대화상자 결과 통지 리스너 - SocialHive::showHiveDialog()
void SocialHiveTest::onShowHiveDialog(ResultAPI result)
{
    Logger::log("SocialHiveTestView::onShowHiveDialog() Callback\nresult = %s\n", result.toString().c_str());
}
