//
//  UIMulltiselect.cpp
//  HIVE_SDK_v4_Cocos_Sample
//
//  Created by paikjongman on 2017. 3. 9..
//  Copyright © 2017년 ryuvsken. All rights reserved.
//

#include "UIMultiselect.hpp"

using namespace cocos2d;
using namespace cocos2d::ui;


MultiSelect* MultiSelect::create(const char* title,const std::vector<std::string>& list,MultiSelectCallback callback)
{
    MultiSelect *pRet = new(std::nothrow) MultiSelect();
    if (pRet && pRet->init(title,std::move(list),callback))
    {
        pRet->autorelease();
        return pRet;
    }
    else
    {
        delete pRet;
        pRet = nullptr;
        return nullptr;
    }
}
bool MultiSelect::init(const char* title,const std::vector<std::string>& list,MultiSelectCallback callback)
{
    if(!BaseView::init())
        return false;
    
    viewTitle = title;
    
    addLabel(title);
    
    data = std::move(list);
    
    int i=0;
    for(const auto& item:data)
    {
        addButton(item.c_str(), i++);
    }
    
    OK_BUTTON_TAG = i+1;
    CANCEL_BUTTON_TAG = i+2;
    addButton("OK", OK_BUTTON_TAG);
    addButton("Cancel",CANCEL_BUTTON_TAG);
    
    
    
    okCallback = callback;
    
    return true;
}

void MultiSelect::selectFromIndex(int i)
{
    assert(!isSelectIndex(i));
    auto button = getUIWidget(i);
    button->setColor(Color3B::GREEN);
    selectedIndex.push_back(i);
}
void MultiSelect::unselectFromIndex(int i)
{
    assert(isSelectIndex(i));
    auto button = getUIWidget(i);
    button->setColor(Color3B::WHITE);
    selectedIndex.erase(std::find(selectedIndex.begin(), selectedIndex.end(), i));
    
}
bool MultiSelect::isSelectIndex(int i)
{
    return std::find(selectedIndex.begin(), selectedIndex.end(), i) != selectedIndex.end();
}
std::vector<int> MultiSelect::getSelectIndexes()
{
    return selectedIndex;
}

void MultiSelect::clearSelect()
{
    auto index = selectedIndex;
    for(auto& i:index)
    {
        unselectFromIndex(i);
    }
}

void MultiSelect::onButtonEvent(int buttonTag)
{
    if(buttonTag == OK_BUTTON_TAG)
    {
        okCallback(this,getSelectIndexes());
        popView();
    }
    else if (buttonTag == CANCEL_BUTTON_TAG)
    {
        popView();
    }
    else
    {
        //Toggle Select
        if(isSelectIndex(buttonTag))
        {
            unselectFromIndex(buttonTag);
        }
        else
        {
            selectFromIndex(buttonTag);
        }
    }
}
