/**
 * @file    C2SModule.cpp
 
 * @brief   os 공통 사용되는 함수들의 소스
 
 * @author  Byeong Yeol Jeon
 * @ingroup iOS Android Cocos2d-x
 * @date    2014/04/30
 */

/**
 * @mainpage 통합 모듈 라이브러리
 * @section intro 소개
 * @section CREATEINFO 작성 정보
 * @section MODIFYINFO 수정 정보
 */


#include "C2SModule.h"

#ifdef __OBJC__
#define FUNCTION_BODY_NOERROR(a, b) \
    NSString* result = (NULL != pstrJson ? [NSString stringWithUTF8String:pstrJson] : nil); \
    NSError* error; \
    BOOL ret = [a b:result error:&error]; \
\
    if(NO == ret) \
    { \
        return nil != error ? (C2SModuleErrorCode)error.code : C2SModuleErrorCode_Fail; \
    } \
    return C2SModuleErrorCode_Success;
#elif __ANDROID__
#include <jni.h>

extern JavaVM* C2SModuleJavaVMInstance;

extern jclass C2SModuleSocialClass;
extern jclass C2SModuleSnsClass;
extern jclass C2SModulePushClass;
extern jclass C2SModulePromotionClass;
extern jclass C2SModuleEngagementClass;
extern jclass C2SModuleTrackerClass; // add 1.5.0 - tracking
extern jclass C2SModuleErrorClass;
extern jclass C2SModuleHivePromotionClass; // add Promotion 2.0
extern jclass C2SModuleInappV4Class; // add InappV4

#define FUNCTION_BODY_NOERROR(a, b) \
    JNIEnv* env; \
    if (JNI_OK != C2SModuleJavaVMInstance->GetEnv((void**) &env, JNI_VERSION_1_6)) \
        return C2SModuleErrorCode_Fail; \
\
    jmethodID methodId = env->GetStaticMethodID(a##Class, #b, "(Ljava/lang/String;Lcom/com2us/module/C2SModuleCompletionHandler;)Lcom/com2us/module/C2SModuleError;"); \
\
    jstring arg = pstrJson ? env->NewStringUTF(pstrJson) : NULL; \
    jobject ret = env->CallStaticObjectMethod(a##Class, methodId, arg, NULL); \
\
    if(NULL != arg)	 { \
        env->DeleteLocalRef(arg); \
    } \
\
    if (env->ExceptionCheck()) { \
        jthrowable exception = env->ExceptionOccurred(); \
        env->ExceptionDescribe(); \
        env->ExceptionClear(); \
        env->DeleteLocalRef(exception); \
        return C2SModuleErrorCode_Fail; \
    } \
\
    if (NULL != ret) { \
        jmethodID methodId_getCode = env->GetMethodID(C2SModuleErrorClass, "getCodeToInteger", "()I"); \
        C2SModuleErrorCode code = (C2SModuleErrorCode)env->CallIntMethod(ret, methodId_getCode); \
        env->DeleteLocalRef(ret); \
        return code; \
    }
#endif // __OBJC__

#ifdef __cplusplus
extern "C" {
#endif
    /**
     * @brief Social Initialize의 c 인터페이스
     * @param[in] pstrJson 입력 json 문자열
     * @return 발생한 에러 코드
     * @ingroup iOS Android Cocos2d-x
     * @author Byeong Yeol Jeon
     */
    C2SModuleErrorCode C2SModuleSocial_Initialize(const char* pstrJson) {
        FUNCTION_BODY_NOERROR(C2SModuleSocial, Initialize);
        return C2SModuleErrorCode_Success;
    }
    /**
     * @brief Social Show의 c 인터페이스
     * @param[in] pstrJson 입력 json 문자열
     * @return 발생한 에러 코드
     * @ingroup iOS Android Cocos2d-x
     * @author Byeong Yeol Jeon
     */
    C2SModuleErrorCode C2SModuleSocial_Show(const char* pstrJson) {
        FUNCTION_BODY_NOERROR(C2SModuleSocial, Show);
        return C2SModuleErrorCode_Success;
    }
    /**
	 * @brief Social Info의 c 인터페이스
	 * @param[in] pstrJson 입력 json 문자열
	 * @return 발생한 에러 코드
	 * @ingroup iOS Android Cocos2d-x
	 * @author Choi ji soo
	 */
    C2SModuleErrorCode C2SModuleSocial_Info(const char* pstrJson) {
    	FUNCTION_BODY_NOERROR(C2SModuleSocial, Info);
    	return C2SModuleErrorCode_Success;
    }
    /**
	 * @brief Social SetGameInfo의 c 인터페이스
	 * @param[in] pstrJson 입력 json 문자열
	 * @return 발생한 에러 코드
	 * @ingroup iOS Android Cocos2d-x
	 * @author Choi ji soo
	 */
    C2SModuleErrorCode C2SModuleSocial_SetGameInfo(const char* pstrJson) {
    	FUNCTION_BODY_NOERROR(C2SModuleSocial, SetGameInfo);
    	return C2SModuleErrorCode_Success;
    }
    /**
     * @brief Social UpdateGameLanguage의 c 인터페이스
     * @param[in] pstrJson 입력 json 문자열
     * @return 발생한 에러 코드
     * @ingroup iOS Android Cocos2d-x
     * @author Seokjin Yong
     */
    C2SModuleErrorCode C2SModuleSocial_UpdateGameLanguage(const char* pstrJson) {
        FUNCTION_BODY_NOERROR(C2SModuleSocial, UpdateGameLanguage);
        return C2SModuleErrorCode_Success;
    }
    /**
     * @brief Social UpdateServerId의 c 인터페이스
     * @param[in] pstrJson 입력 json 문자열
     * @return 발생한 에러 코드
     * @ingroup iOS Android Cocos2d-x
     * @author Seokjin Yong
     */
    C2SModuleErrorCode C2SModuleSocial_UpdateServerId(const char* pstrJson) {
        FUNCTION_BODY_NOERROR(C2SModuleSocial, UpdateServerId);
        return C2SModuleErrorCode_Success;

    }

    /**
     * @brief Social SendData의 c 인터페이스
     * @param[in] pstrJson 입력 json 문자열
     * @return 발생한 에러 코드
     * @ingroup iOS Android Cocos2d-x
     * @author Choi BK
     */
    C2SModuleErrorCode C2SModuleSocial_SendData(const char* pstrJson) {
        FUNCTION_BODY_NOERROR(C2SModuleSocial, SendData);
        return C2SModuleErrorCode_Success;
    }
    /**
     * @brief Sns Session의 c 인터페이스
     * @param[in] pstrJson 입력 json 문자열
     * @return 발생한 에러 코드
     * @ingroup iOS Android Cocos2d-x
     * @author Byeong Yeol Jeon
     */
    C2SModuleErrorCode C2SModuleSns_Session(const char* pstrJson) {
        FUNCTION_BODY_NOERROR(C2SModuleSns, Session);
        return C2SModuleErrorCode_Success;
    }
    /**
     * @brief Sns Login의 c 인터페이스
     * @param[in] pstrJson 입력 json 문자열
     * @return 발생한 에러 코드
     * @ingroup iOS Android Cocos2d-x
     * @author Byeong Yeol Jeon
     */
    C2SModuleErrorCode C2SModuleSns_Login(const char* pstrJson) {
        FUNCTION_BODY_NOERROR(C2SModuleSns, Login);
        return C2SModuleErrorCode_Success;
    }
    /**
     * @brief Sns Logout의 c 인터페이스
     * @param[in] pstrJson 입력 json 문자열
     * @return 발생한 에러 코드
     * @ingroup iOS Android Cocos2d-x
     * @author Byeong Yeol Jeon
     */
    C2SModuleErrorCode C2SModuleSns_Logout(const char* pstrJson) {
        FUNCTION_BODY_NOERROR(C2SModuleSns, Logout);
        return C2SModuleErrorCode_Success;
    }
    /**
     * @brief Sns Unregister의 c 인터페이스
     * @param[in] pstrJson 입력 json 문자열
     * @return 발생한 에러 코드
     * @ingroup iOS Android Cocos2d-x
     * @author Byeong Yeol Jeon
     */
    C2SModuleErrorCode C2SModuleSns_Unregister(const char* pstrJson) {
        FUNCTION_BODY_NOERROR(C2SModuleSns, Unregister);
        return C2SModuleErrorCode_Success;
    }
    /**
     * @brief Sns User의 c 인터페이스
     * @param[in] pstrJson 입력 json 문자열
     * @return 발생한 에러 코드
     * @ingroup iOS Android Cocos2d-x
     * @author Byeong Yeol Jeon
     */
    C2SModuleErrorCode C2SModuleSns_User(const char* pstrJson) {
        FUNCTION_BODY_NOERROR(C2SModuleSns, User);
        return C2SModuleErrorCode_Success;
    }
    /**
     * @brief Sns Friends의 c 인터페이스
     * @param[in] pstrJson 입력 json 문자열
     * @return 발생한 에러 코드
     * @ingroup iOS Android Cocos2d-x
     * @author Byeong Yeol Jeon
     */
    C2SModuleErrorCode C2SModuleSns_Friends(const char* pstrJson) {
        FUNCTION_BODY_NOERROR(C2SModuleSns, Friends);
        return C2SModuleErrorCode_Success;
    }
    /**
     * @brief Sns Message의 c 인터페이스
     * @param[in] pstrJson 입력 json 문자열
     * @return 발생한 에러 코드
     * @ingroup iOS Android Cocos2d-x
     * @author Byeong Yeol Jeon
     */
    C2SModuleErrorCode C2SModuleSns_Message(const char* pstrJson) {
        FUNCTION_BODY_NOERROR(C2SModuleSns, Message);
        return C2SModuleErrorCode_Success;
    }
    /**
     * @brief Sns Post의 c 인터페이스
     * @param[in] pstrJson 입력 json 문자열
     * @return 발생한 에러 코드
     * @ingroup iOS Android Cocos2d-x
     * @author Byeong Yeol Jeon
     */
    C2SModuleErrorCode C2SModuleSns_Post(const char* pstrJson) {
        FUNCTION_BODY_NOERROR(C2SModuleSns, Post);
        return C2SModuleErrorCode_Success;
    }
    /**
     * @brief Sns Show의 c 인터페이스
     * @param[in] pstrJson 입력 json 문자열
     * @return 발생한 에러 코드
     * @ingroup iOS Android Cocos2d-x
     * @author Byeong Yeol Jeon
     */
    C2SModuleErrorCode C2SModuleSns_Show(const char* pstrJson) {
        FUNCTION_BODY_NOERROR(C2SModuleSns, Show);
        return C2SModuleErrorCode_Success;
    }
    /**
     * @brief Sns SetGuestUserInfo의 c 인터페이스
     * @param[in] pstrJson 입력 json 문자열
     * @return 발생한 에러 코드
     * @ingroup iOS Android Cocos2d-x
     * @author Byeong Yeol Jeon
     */
    C2SModuleErrorCode C2SModuleSns_SetGuestUserInfo(const char* pstrJson) {
        FUNCTION_BODY_NOERROR(C2SModuleSns, SetGuestUserInfo);
        return C2SModuleErrorCode_Success;
    }
    /**
     * @brief Sns UserDelete의 c 인터페이스
     * @param[in] pstrJson 입력 json 문자열
     * @return 발생한 에러 코드
     * @ingroup iOS Android Cocos2d-x
     * @author Byeong Yeol Jeon
     */
    C2SModuleErrorCode C2SModuleSns_UserDelete(const char* pstrJson) {
        FUNCTION_BODY_NOERROR(C2SModuleSns, UserDelete);
        return C2SModuleErrorCode_Success;
    }
#if __ANDROID__
    C2SModuleErrorCode C2SModuleSns_PGSLoginProc(const char* pstrJson) {
        FUNCTION_BODY_NOERROR(C2SModuleSns, PGSLoginProc);
        return C2SModuleErrorCode_Success;
    }

    C2SModuleErrorCode C2SModuleSns_Connect(const char* pstrJson) {
        FUNCTION_BODY_NOERROR(C2SModuleSns, Connect);
        return C2SModuleErrorCode_Success;
    }
#endif
    /**
     * @brief Push Register의 c 인터페이스
     * @param[in] pstrJson 입력 json 문자열
     * @return 발생한 에러 코드
     * @ingroup iOS Android Cocos2d-x
     * @author Byeong Yeol Jeon
     */
    C2SModuleErrorCode C2SModulePush_Register(const char* pstrJson) {
        FUNCTION_BODY_NOERROR(C2SModulePush, Register);
        return C2SModuleErrorCode_Success;
    }
    /**
     * @brief Push Unregister의 c 인터페이스
     * @param[in] pstrJson 입력 json 문자열
     * @return 발생한 에러 코드
     * @ingroup iOS Android Cocos2d-x
     * @author Byeong Yeol Jeon
     */
    C2SModuleErrorCode C2SModulePush_Unregister(const char* pstrJson) {
        FUNCTION_BODY_NOERROR(C2SModulePush, Unregister);
        return C2SModuleErrorCode_Success;
    }
    /**
     * @brief Push Description의 c 인터페이스
     * @param[in] pstrJson 입력 json 문자열
     * @return 발생한 에러 코드
     * @ingroup iOS Android Cocos2d-x
     * @author Byeong Yeol Jeon
     */
    C2SModuleErrorCode C2SModulePush_Description(const char* pstrJson) {
        FUNCTION_BODY_NOERROR(C2SModulePush, Description);
        return C2SModuleErrorCode_Success;
    }

    /**
     * @brief Push RequestPermission의 c 인터페이스
     * @param[in] pstrJson 입력 json 문자열
     * @return 발생한 에러 코드
     * @ingroup iOS Android Cocos2d-x
     * @author sjyong
     */
    C2SModuleErrorCode C2SModulePush_RequestPermission(const char* pstrJson) {
        FUNCTION_BODY_NOERROR(C2SModulePush, RequestPermission);
        return C2SModuleErrorCode_Success;
    }
    /**
     * @brief Promotion Show의 c 인터페이스
     * @param[in] pstrJson 입력 json 문자열
     * @return 발생한 에러 코드
     * @ingroup iOS Android Cocos2d-x
     * @author Byeong Yeol Jeon
     */
    C2SModuleErrorCode C2SModulePromotion_Show(const char* pstrJson) {
        FUNCTION_BODY_NOERROR(C2SModulePromotion, Show);
        return C2SModuleErrorCode_Success;
    }
    
    C2SModuleErrorCode C2SModulePromotion_Info(const char* pstrJson) {
        FUNCTION_BODY_NOERROR(C2SModulePromotion, Info);
        return C2SModuleErrorCode_Success;
    }

    /**
    * @brief 프로모션 관련 배너 정보를 얻어오고 completionHandler로 전달 받는다
    * @param[in] arg remark 참조
    * @param[in, out] error 실행시 발생한 에러
    * @param[in] completionHandler 결과를 통지받을 블럭 핸들러
    * @return YES일 경우 호출 성공. NO일 경우 호출 실패.
    * @remark
    * - "campaignType" : NSString*.
    *      - "event"  : 이벤트 타입
    *      - "notice" : 공지 타입
    * - "bannerType" : NSString*.
    *      - "great"   : 대배너
    *      - "small"   : 일반배너 (띠배너)
    *      - "rolling" : 롤링 배너
    * @author daunjoung
    */
    C2SModuleErrorCode C2SModulePromotion_GetBannerInfo(const char* pstrJson) {
        FUNCTION_BODY_NOERROR(C2SModulePromotion, GetBannerInfo);
        return C2SModuleErrorCode_Success;
    }
    
    /**
     * @brief Promotion Acquisition의 c 인터페이스
     * @param[in] pstrJson 입력 json 문자열
     * @return 발생한 에러 코드
     * @ingroup iOS Android Cocos2d-x
     * @author Choi BK
     */
    
    C2SModuleErrorCode C2SModulePromotion_Acquisition(const char* pstrJson) {
        FUNCTION_BODY_NOERROR(C2SModulePromotion, Acquisition);
        return C2SModuleErrorCode_Success;
    }
    
    /**
     * @brief Engagement ProcessScheme의 c 인터페이스
     * @param[in] pstrJson 입력 json 문자열
     * @return 발생한 에러 코드
     * @ingroup iOS Android Cocos2d-x
     * @author Hysterit
     */
    
    C2SModuleErrorCode C2SModuleEngagement_ProcessScheme(const char* pstrJson) {
        FUNCTION_BODY_NOERROR(C2SModuleEngagement, ProcessScheme);
        return C2SModuleErrorCode_Success;
    }
    
    /**
     * @brief Engagement GetEvents의 c 인터페이스
     * @param[in] pstrJson 입력 json 문자열
     * @return 발생한 에러 코드
     * @ingroup iOS Android Cocos2d-x
     * @author Hysterit
     * Deprecated in HIVE SDK 1.11.0
     */
    
//    C2SModuleErrorCode C2SModuleEngagement_GetEvents(const char* pstrJson) {
//        FUNCTION_BODY_NOERROR(C2SModuleEngagement, GetEvents);
//        return C2SModuleErrorCode_Success;
//    }
//    
    /**
     * @brief Engagement ConsumeCoupon의 c 인터페이스
     * @param[in] pstrJson 입력 json 문자열
     * @return 발생한 에러 코드
     * @ingroup iOS Android Cocos2d-x
     * @author Hysterit
     */
    
    C2SModuleErrorCode C2SModuleEngagement_ConsumeCoupon(const char* pstrJson) {
        FUNCTION_BODY_NOERROR(C2SModuleEngagement, ConsumeCoupon);
        return C2SModuleErrorCode_Success;
    }
    
    /**
     * @brief Engagement SetReady의 c 인터페이스
     * @param[in] pstrJson 입력 json 문자열
     * @return 발생한 에러 코드
     * @ingroup iOS Android Cocos2d-x
     * @author DaunJung
     */

    C2SModuleErrorCode C2SModuleEngagement_SetReady(const char* pstrJson) {
    	FUNCTION_BODY_NOERROR(C2SModuleEngagement, SetReady);
    	return C2SModuleErrorCode_Success;
    }

    C2SModuleErrorCode C2SModuleTracker_SendEvent(const char* pstrJson) {
        FUNCTION_BODY_NOERROR(C2SModuleTracker, SendEvent);
        return C2SModuleErrorCode_Success;
    }
    
    C2SModuleErrorCode C2SModuleTracker_SetEnable(const char* pstrJson) {
        FUNCTION_BODY_NOERROR(C2SModuleTracker, SetEnable);
        return C2SModuleErrorCode_Success;
    }
    
    /**
     * @brief Hive Promotion (Promotion2.0) Show의 c 인터페이스
     * @param[in] pstrJson 입력 json 문자열
     * @return 발생한 에러 코드
     * @ingroup iOS Android Cocos2d-x
     * @author pnpsinki
     */
    C2SModuleErrorCode C2SModuleHivePromotion_Show(const char* pstrJson) {
        FUNCTION_BODY_NOERROR(C2SModuleHivePromotion, Show);
        return C2SModuleErrorCode_Success;
    }

    /**
     * @brief Hive Promotion (Promotion2.0) Info의 c 인터페이스
     * @param[in] pstrJson 입력 json 문자열
     * @return 발생한 에러 코드
     * @ingroup iOS Android Cocos2d-x
     * @author pnpsinki
     */
    C2SModuleErrorCode C2SModuleHivePromotion_Info(const char* pstrJson) {
        FUNCTION_BODY_NOERROR(C2SModuleHivePromotion, Info);
        return C2SModuleErrorCode_Success;
    }

    /**
    * @brief 프로모션 2.0 관련 배너 정보를 얻어오고 completionHandler로 전달 받는다 Added in HIVE SDK v1.18.3
    * @param[in] arg remark 참조
    * @param[in, out] error 실행시 발생한 에러
    * @param[in] completionHandler 결과를 통지받을 블럭 핸들러
    * @return YES일 경우 호출 성공. NO일 경우 호출 실패.
    * @remark
    * - "campaignType" : NSString*.
    *      - "event"  : 이벤트 타입
    *      - "notice" : 공지 타입
    * - "bannerType" : NSString*.
    *      - "great"   : 대배너
    *      - "small"   : 일반배너 (띠배너)
    *      - "rolling" : 롤링 배너
    * @author daunjoung
    */
    C2SModuleErrorCode C2SModuleHivePromotion_GetBannerInfo(const char* pstrJson) {
        FUNCTION_BODY_NOERROR(C2SModuleHivePromotion, GetBannerInfo);
        return C2SModuleErrorCode_Success;
    }

    /**
     * @brief Hive Promotion (Promotion2.0) GetOfferwallState의 c 인터페이스
     * @param[in] pstrJson 입력 json 문자열
     * @return 발생한 에러 코드
     * @ingroup iOS Android Cocos2d-x
     * @author pnpsinki
     */
    C2SModuleErrorCode C2SModuleHivePromotion_GetOfferwallState(const char* pstrJson) {
        FUNCTION_BODY_NOERROR(C2SModuleHivePromotion, GetOfferwallState);
        return C2SModuleErrorCode_Success;
    }

    /**
     * @brief Hive Promotion (Promotion2.0) SetAdditionalInfo의 c 인터페이스
     * @param[in] pstrJson 입력 json 문자열
     * @return 발생한 에러 코드
     * @ingroup iOS Android Cocos2d-x
     * @author pnpsinki
     */
    C2SModuleErrorCode C2SModuleHivePromotion_SetAdditionalInfo(const char* pstrJson) {
        FUNCTION_BODY_NOERROR(C2SModuleHivePromotion, SetAdditionalInfo);
        return C2SModuleErrorCode_Success;
    }

    /**
    * @brief Hive Promotion (Promotion2.0) UpdatePromotionData c 인터페이스
    * @param[in] pstrJson 입력 json 문자열
    * @return 발생한 에러 코드
    * @ingroup iOS Android Cocos2d-x
    * @author jschoi
    */
    C2SModuleErrorCode C2SModuleHivePromotion_UpdatePromotionData(const char* pstrJson) {
        FUNCTION_BODY_NOERROR(C2SModuleHivePromotion, UpdatePromotionData);
        return C2SModuleErrorCode_Success;
    }

    /**
     * @brief InappV4 MarketConnect의 c 인터페이스 (HIVE SDK 1.20.0에 추가)
     * @param[in] pstrJson 입력 json 문자열
     * @return 발생한 에러 코드
     * @ingroup iOS Android Cocos2d-x
     * @author pnpsinki
     */
    C2SModuleErrorCode C2SModuleInappV4_MarketConnect(const char* pstrJson) {
        FUNCTION_BODY_NOERROR(C2SModuleInappV4, MarketConnect);
        return C2SModuleErrorCode_Success;
    }

    /**
     * @brief InappV4 ItemInfo의 c 인터페이스 (HIVE SDK 1.18.4에 추가)
     * @param[in] pstrJson 입력 json 문자열
     * @return 발생한 에러 코드
     * @ingroup iOS Android Cocos2d-x
     * @author pnpsinki
     */
    C2SModuleErrorCode C2SModuleInappV4_ItemInfo(const char* pstrJson) {
        FUNCTION_BODY_NOERROR(C2SModuleInappV4, ItemInfo);
        return C2SModuleErrorCode_Success;
    }

    /**
     * @brief InappV4 Purchase의 c 인터페이스 (HIVE SDK 1.18.4에 추가)
     * @param[in] pstrJson 입력 json 문자열
     * @return 발생한 에러 코드
     * @ingroup iOS Android Cocos2d-x
     * @author pnpsinki
     */
    C2SModuleErrorCode C2SModuleInappV4_Purchase(const char* pstrJson) {
        FUNCTION_BODY_NOERROR(C2SModuleInappV4, Purchase);
        return C2SModuleErrorCode_Success;
    }

    /**
     * @brief InappV4 Restore의 c 인터페이스 (HIVE SDK 1.18.4에 추가)
     * @param[in] pstrJson 입력 json 문자열
     * @return 발생한 에러 코드
     * @ingroup iOS Android Cocos2d-x
     * @author pnpsinki
     */
    C2SModuleErrorCode C2SModuleInappV4_Restore(const char* pstrJson) {
        FUNCTION_BODY_NOERROR(C2SModuleInappV4, Restore);
        return C2SModuleErrorCode_Success;
    }

    /**
     * @brief InappV4 TransactionFinish의 c 인터페이스 (HIVE SDK 1.18.4에 추가)
     * @param[in] pstrJson 입력 json 문자열
     * @return 발생한 에러 코드
     * @ingroup iOS Android Cocos2d-x
     * @author pnpsinki
     */
    C2SModuleErrorCode C2SModuleInappV4_TransactionFinish(const char* pstrJson) {
        FUNCTION_BODY_NOERROR(C2SModuleInappV4, TransactionFinish);
        return C2SModuleErrorCode_Success;
    }

    /**
     * @brief InappV4 GetSelectedMarket의 c 인터페이스 (HIVE SDK 1.18.4에 추가)
     * @param[in] pstrJson 입력 json 문자열
     * @return 발생한 에러 코드
     * @ingroup iOS Android Cocos2d-x
     * @author pnpsinki
     */
    C2SModuleErrorCode C2SModuleInappV4_GetSelectedMarket(const char* pstrJson) {
        FUNCTION_BODY_NOERROR(C2SModuleInappV4, GetSelectedMarket);
        return C2SModuleErrorCode_Success;
    }

    /**
     * @brief InappV4 GetAccountUuid의 c 인터페이스 (HIVE SDK 1.20.3에 추가)
     * @param[in] pstrJson 입력 json 문자열
     * @return 발생한 에러 코드
     * @ingroup iOS Android Cocos2d-x
     * @author hife
     */
    C2SModuleErrorCode C2SModuleInappV4_GetAccountUuid(const char* pstrJson) {
        FUNCTION_BODY_NOERROR(C2SModuleInappV4, GetAccountUuid);
        return C2SModuleErrorCode_Success;
    }

#if __ANDROID__
    /**
     * @brief InappV4 SelectMarket 의 c 인터페이스 (HIVE SDK 1.18.4에 추가)
     * @param[in] pstrJson 입력 json 문자열
     * @return 발생한 에러 코드
     * @ingroup iOS Android Cocos2d-x
     * @author pnpsinki
     */
    C2SModuleErrorCode C2SModuleInappV4_SelectMarket(const char* pstrJson) {
        FUNCTION_BODY_NOERROR(C2SModuleInappV4, SelectMarket);
        return C2SModuleErrorCode_Success;
    }

    /**
     * @brief InappV4 RequestBalance 의 c 인터페이스 (HIVE SDK 1.18.4에 추가)
     * @param[in] pstrJson 입력 json 문자열
     * @return 발생한 에러 코드
     * @ingroup iOS Android Cocos2d-x
     * @author pnpsinki
     */
    C2SModuleErrorCode C2SModuleInappV4_RequestBalance(const char* pstrJson) {
        FUNCTION_BODY_NOERROR(C2SModuleInappV4, RequestBalance);
        return C2SModuleErrorCode_Success;
    }

    /**
     * @brief InappV4 ChargeBalance 의 c 인터페이스 (HIVE SDK 1.18.4에 추가)
     * @param[in] pstrJson 입력 json 문자열
     * @return 발생한 에러 코드
     * @ingroup iOS Android Cocos2d-x
     * @author pnpsinki
     */
    C2SModuleErrorCode C2SModuleInappV4_ChargeBalance(const char* pstrJson) {
        FUNCTION_BODY_NOERROR(C2SModuleInappV4, ChargeBalance);
        return C2SModuleErrorCode_Success;
    }
#endif

#if __OBJC__
    /**
     * @brief InappV4 CheckPromotePurchase 의 c 인터페이스 (HIVE SDK 1.18.4에 추가)
     * @param[in] pstrJson 입력 json 문자열
     * @return 발생한 에러 코드
     * @ingroup iOS Android Cocos2d-x
     * @author pnpsinki
     */
    C2SModuleErrorCode C2SModuleInappV4_CheckPromotePurchase(const char* pstrJson) {
        FUNCTION_BODY_NOERROR(C2SModuleInappV4, CheckPromotePurchase);
        return C2SModuleErrorCode_Success;
    }
#endif

#ifdef __cplusplus
}
#endif

#if __OBJC__
#import <Foundation/NSJSONSerialization.h>
/**
 * @interface C2SModuleNativeObject
 
 * @brief c, c++ 델리게이트 등록을 위해 생성하는 객체
 * @ingroup iOS Cocos2d-x
 * @author Byeong Yeol Jeon
 */
@interface C2SModuleNativeObject : NSObject<C2SModuleResultDelegate>
/** 등록된 공통 델리게이트 */
@property (nonatomic, assign) CommonResultDelegate customFn;
/**
 * @brief C2SModuleNativeObject의 인스턴스를 구한다.
 
 * @return C2SModuleNativeObject의 인스턴스
 * @author Byeong Yeol Jeon
 
 */
+ (C2SModuleNativeObject*)sharedInstance;
@end

/** 
 * @brief C2SModuleNativeObject의 싱글턴 인스턴스 변수
 * @ingroup iOS Cocos2d-x
 */
static C2SModuleNativeObject* C2SModuleNativeObjectInstance = nil;

@implementation C2SModuleNativeObject
+ (C2SModuleNativeObject*)sharedInstance
{
	static dispatch_once_t onceToken;
	dispatch_once(&onceToken, ^ {
        C2SModuleNativeObjectInstance = [[C2SModuleNativeObject alloc] init];
    });
	return C2SModuleNativeObjectInstance;
}
/**
 * @brief default constructor
 
 * @author Byeong Yeol Jeon
 
 */
-(id)init
{
	self = [super init];
    
	if(self)
	{
		[C2SModuleSocial SetCommonResultDelegate:self];
	}
	return self;
}
/**
 * @brief 공통 델리게이트의 구현
 
 * @param[in] api 결과 api의 종류
 * @param[in] arg 통지될 인자
 * @param[in] error 발생한 에러
 
 * @author Byeong Yeol Jeon
 
 */
-(void)C2SModuleResult:(C2SModuleApi)api parameter:(NSDictionary *)arg error:(NSError*)error
{
	if(nil != self.customFn)
	{
		NSString* result = nil;
        
		if(nil != arg)
		{
			NSError* serializeArgError = nil;
			NSData* data = [NSJSONSerialization dataWithJSONObject:arg options:kNilOptions error:&serializeArgError];
            
			if(nil != serializeArgError)
			{
				error = [NSError errorWithDomain:serializeArgError.domain code:C2SModuleErrorCode_FailConvertToJsonFormat userInfo:serializeArgError.userInfo];
			}
			else
			{
				result = [[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding];
			}
		}
        
        self.customFn(api, nil != result ? [result UTF8String] : nil, nil != error ? (C2SModuleErrorCode)error.code : C2SModuleErrorCode_Success, nil != error ? [error.domain UTF8String] : nil);
	}
}
@end
#elif __ANDROID__
extern jclass C2SModuleSocialClass;
extern jclass C2SModuleResultDelegateJNIClass;
#endif

#ifdef __cplusplus
extern "C" {
#endif
    /**
     * @brief 공통 델리게이트를 설정한다.
     * @param[in] func 등록될 함수
     * @ingroup iOS Android Cocos2d-x
     * @author Byeong Yeol Jeon
     */
    void C2SModuleSocial_SetCommonResultDelegate(CommonResultDelegate func) {
#if __OBJC__
        [C2SModuleNativeObject sharedInstance].customFn = func;
#elif __ANDROID__
        JNIEnv* env;
        if (JNI_OK
			!= C2SModuleJavaVMInstance->GetEnv((void**) &env, JNI_VERSION_1_6))
            return;
        
        jmethodID init = env->GetMethodID(C2SModuleResultDelegateJNIClass, "<init>", "(J)V");
        jobject jHandlerInstance = env->NewObject(C2SModuleResultDelegateJNIClass, init, (jlong)func);
        jobject jHandler = env->NewGlobalRef(jHandlerInstance);
        env->DeleteLocalRef(jHandlerInstance);
        jmethodID methodId = env->GetStaticMethodID(C2SModuleSocialClass, "SetCommonResultDelegate", "(Lcom/com2us/module/C2SModuleResultDelegate;)V");
        env->CallStaticVoidMethod(C2SModuleSocialClass, methodId, jHandler);
        env->DeleteGlobalRef(jHandler);
#endif
    }
#ifdef __cplusplus
}
#endif
