/**
 * @file    C2SModule.h
 * @brief   사용자가 include해야하는 헤더
 * @author  Byeong Yeol Jeon
 * @ingroup iOS Android Cocos2d-x
 * @date    2014/04/30
 */
#ifndef _C2SMODULE_H_
#define _C2SMODULE_H_

#include "C2SModuleApi.h"
#include "C2SModuleErrorCode.h"

#if __OBJC__
#include "iOS/Module_iOS.h"
#elif __ANDROID__
#include "Android/Module_Android.h"
#endif

#if __cplusplus
extern "C" {
#endif
    /**
     * @brief 공통 델리게이트의 타입 정의
     * @ingroup iOS Android Cocos2d-x
     * @author Byeong Yeol Jeon
     */
    typedef void (*CommonResultDelegate)(C2SModuleApi api, const char* json, C2SModuleErrorCode code, const char* message);
    
    void C2SModuleSocial_SetCommonResultDelegate(CommonResultDelegate func);
#if __cplusplus
}
#endif

#if __cplusplus
#include "cocos2d-x/Module_Cocos2d-x.h"
#else
C2SModuleErrorCode C2SModuleSocial_Initialize(const char* pstrJson);
C2SModuleErrorCode C2SModuleSocial_Show(const char* pstrJson);
C2SModuleErrorCode C2SModuleSocial_Info(const char* pstrJson);
C2SModuleErrorCode C2SModuleSocial_SetGameInfo(const char* pstrJson);
C2SModuleErrorCode C2SModuleSocial_UpdateGameLanguage(const char* pstrJson);
C2SModuleErrorCode C2SModuleSocial_UpdateServerId(const char* pstrJson);
C2SModuleErrorCode C2SModuleSocial_SendData(const char* pstrJson);
C2SModuleErrorCode C2SModuleSns_Session(const char* pstrJson);
C2SModuleErrorCode C2SModuleSns_Login(const char* pstrJson);
C2SModuleErrorCode C2SModuleSns_Logout(const char* pstrJson);
C2SModuleErrorCode C2SModuleSns_Unregister(const char* pstrJson);
C2SModuleErrorCode C2SModuleSns_User(const char* pstrJson);
C2SModuleErrorCode C2SModuleSns_Friends(const char* pstrJson);
C2SModuleErrorCode C2SModuleSns_Message(const char* pstrJson);
C2SModuleErrorCode C2SModuleSns_Post(const char* pstrJson);
C2SModuleErrorCode C2SModuleSns_Show(const char* pstrJson);
C2SModuleErrorCode C2SModuleSns_SetGuestUserInfo(const char* pstrJson);
C2SModuleErrorCode C2SModuleSns_UserDelete(const char* pstrJson);
C2SModuleErrorCode C2SModuleSns_PGSLoginProc(const char* pstrJson);
C2SModuleErrorCode C2SModuleSns_Connect(const char* pstrJson);
C2SModuleErrorCode C2SModulePush_Register(const char* pstrJson);
C2SModuleErrorCode C2SModulePush_Unregister(const char* pstrJson);
C2SModuleErrorCode C2SModulePush_Description(const char* pstrJson);
C2SModuleErrorCode C2SModulePush_RequestPermission(const char* pstrJson);
C2SModuleErrorCode C2SModulePromotion_Show(const char* pstrJson);
C2SModuleErrorCode C2SModulePromotion_Info(const char* pstrJson);
C2SModuleErrorCode C2SModulePromotion_GetBannerInfo(const char* pstrJson);		//	 Added in HIVE SDK v1.18.3
C2SModuleErrorCode C2SModulePromotion_Acquisition(const char* pstrJson);
C2SModuleErrorCode C2SModuleEngagement_ProcessScheme(const char* pstrJson);
//C2SModuleErrorCode C2SModuleEngagement_GetEvents(const char* pstrJson);
C2SModuleErrorCode C2SModuleEngagement_ConsumeCoupon(const char* pstrJson);
C2SModuleErrorCode C2SModuleEngagement_SetReady(const char* pstrJson);
C2SModuleErrorCode C2SModuleTracker_SendEvent(const char* pstrJson);
C2SModuleErrorCode C2SModuleTracker_SetEnable(const char* pstrJson);
//
C2SModuleErrorCode C2SModuleHivePromotion_Show(const char* pstrJson);					// add Promotion2.0
C2SModuleErrorCode C2SModuleHivePromotion_Info(const char* pstrJson);					// add Promotion2.0
C2SModuleErrorCode C2SModuleHivePromotion_GetBannerInfo(const char* pstrJson);			//	 Added in HIVE SDK v1.18.3
C2SModuleErrorCode C2SModuleHivePromotion_GetOfferwallState(const char* pstrJson);		// add Promotion2.0
C2SModuleErrorCode C2SModuleHivePromotion_SetAdditionalInfo(const char* pstrJson);		// add Promotion2.0
C2SModuleErrorCode C2SModuleHivePromotion_UpdatePromotionData(const char* pstrJson);	// add Promotion2.0 (HIVE SDK 1.19.0)
//

C2SModuleErrorCode C2SModuleInappV4_ItemInfo(const char* pstrJson);                     // add InappV4 (HIVE SDK 1.18.4)
C2SModuleErrorCode C2SModuleInappV4_Purchase(const char* pstrJson);                     // add InappV4 (HIVE SDK 1.18.4)
C2SModuleErrorCode C2SModuleInappV4_Restore(const char* pstrJson);                      // add InappV4 (HIVE SDK 1.18.4)
C2SModuleErrorCode C2SModuleInappV4_TransactionFinish(const char* pstrJson);            // add InappV4 (HIVE SDK 1.18.4)
C2SModuleErrorCode C2SModuleInappV4_GetSelectedMarket(const char* pstrJson);            // add InappV4 (HIVE SDK 1.18.4)
C2SModuleErrorCode C2SModuleInappV4_GetAccountUuid(const char* pstrJson);               // add InappV4 (HIVE SDK 1.20.3)
C2SModuleErrorCode C2SModuleInappV4_MarketConnect(const char* pstrJson);                // add InappV4 (Hive SDK 1.20.0)

#ifdef __ANDROID__
C2SModuleErrorCode C2SModuleInappV4_SelectMarket(const char* pstrJson);                 // add InappV4 (HIVE SDK 1.18.4)
C2SModuleErrorCode C2SModuleInappV4_RequestBalance(const char* pstrJson);               // add InappV4 (HIVE SDK 1.18.4)
C2SModuleErrorCode C2SModuleInappV4_ChargeBalance(const char* pstrJson);                // add InappV4 (HIVE SDK 1.18.4)
#endif // __ANDROID__

#if __OBJC__
C2SModuleErrorCode C2SModuleInappV4_CheckPromotePurchase(const char* pstrJson);         // add InappV4 (HIVE SDK 1.18.4)
#endif // __OBJC__

#endif

#endif
