/**
 * @file    Module_Cocos2d-x.cpp
 
 * @brief   c++ 지원 모듈의 소스
 
 * @author  Byeong Yeol Jeon
 * @ingroup Cocos2d-x
 * @date    2014/04/25
 */
#include "Module_Cocos2d-x.h"
#include "../C2SModule.h"
#include <cassert>

#define LOGIN_TYPE @"jkdeodkeodcca"

namespace C2SModule_Cocos2dx {
    /**
     * @class C2SModuleErrorDescImpl
     
     * @brief C2SModuleErrorDesc의 구현
     * @author Byeong Yeol Jeon
     * @ingroup Cocos2d-x
     */
    class C2SModuleErrorDescImpl: public C2SModuleErrorDesc {
    public:
        /**
         * @brief constructor
         
         * @param[in] argCode 에러 코드
         * @param[in] argMessage 에러 메시지
         * @author Byeong Yeol Jeon
         
         */
        C2SModuleErrorDescImpl(C2SModuleErrorCode argCode, const char argMessage[]) :
        code(argCode) {
            
            if (NULL != argMessage) {
                size_t length = strlen(argMessage);
                message = new char[length + 1];
                strcpy(message, argMessage);
            } else {
                message = NULL;
            }
        }
        /**
         * @brief default destructor
         
         * @author Byeong Yeol Jeon
         
         */
        virtual ~C2SModuleErrorDescImpl() {
            delete message;
        }
        
        /**
         * @brief 현재 에러 코드를 구한다.
         * @return 현재 에러 코드
         * @author Byeong Yeol Jeon
         
         */
        virtual C2SModuleErrorCode GetCode() const {
            return code;
        }
        /**
         * @brief 현재 에러 메시지를 구한다.
         
         * @return 현재 에러 메시지
         * @author Byeong Yeol Jeon
         
         */
        virtual const char* GetMessage() const {
            return message ? message : "";
        }
        
    private:
        /** 에러 코드 */
        C2SModuleErrorCode code;
        /** 메시지 */
        char* message;
    };
}
#if __OBJC__
#include "../iOS/Module_iOS.h"
//#import <Foundation/NSJSONSerialization.h>
#import "C2SModuleSnsInterface.h"
#import "C2SModuleSocial.h"
#elif __ANDROID__
#include <jni.h>
#include <android/log.h>

extern JavaVM* C2SModuleJavaVMInstance;

extern jclass C2SModuleSocialClass;
extern jclass C2SModuleSnsClass;
//extern jclass C2SModuleInappClass; // remove 1.19.1
extern jclass C2SModulePushClass;
extern jclass C2SModulePromotionClass;
extern jclass C2SModuleEngagementClass;
extern jclass C2SModuleTrackerClass; // add 1.5.0 - tracking
extern jclass C2SModuleArgumentClass;
extern jclass C2SModuleErrorClass;
extern jclass C2SModuleCompletionHandlerJNIClass;
//extern jclass C2SModuleHiveIAPClass; // add HiveIAP // remove 1.19.1
extern jclass C2SModuleHivePromotionClass; // add Promotion2.0
extern jclass C2SModuleInappV4Class; // add Promotion2.0

extern jclass C2SModuleBooleanClass;
extern jclass C2SModuleCharClass;
extern jclass C2SModuleByteClass;
extern jclass C2SModuleShortClass;
extern jclass C2SModuleIntClass;
extern jclass C2SModuleLongClass;
extern jclass C2SModuleVectorClass;
#endif

namespace C2SModule_Cocos2dx {
    /**
     * @class C2SModuleParameterContainer
     
     * @brief 커스텀 map의 컨테이너에 대한 os별 구현
     * @author Byeong Yeol Jeon
     * @ingroup Cocos2d-x
     */
    class C2SModuleParameterContainer {
    public:
        /**
         * @brief default destructor
         * @author Byeong Yeol Jeon
         
         */
        ~C2SModuleParameterContainer() {
#if __OBJC__
            container = nil;
#elif __ANDROID__
            JNIEnv* env;
            if (NULL != container && JNI_OK
                == C2SModuleJavaVMInstance->GetEnv((void**) &env, JNI_VERSION_1_6))
            {
                env->DeleteGlobalRef(reinterpret_cast<jobject>(container));
            }
#endif
        }
#if __OBJC__
        /**
         * @brief constructor
         * @param[in] arg 입력 objc타입
         * @author Byeong Yeol Jeon
         */
        C2SModuleParameterContainer(id arg) : container(arg)
        {
        }
        id container;
#elif __ANDROID__
        /**
         * @brief constructor
         * @param[in] arg 입력 안드로이드 인스턴스 타입
         * @author Byeong Yeol Jeon
         */
        C2SModuleParameterContainer(jobject arg) : container(arg)
        {
        }
        jobject container;
#endif
    };
}

#if __OBJC__
/**
 * @brief objc문자열을 유니티 문자열로 변환한다.
 
 * @param[in] input 대상 문자열
 
 * @return 변환된 문자열
 * @author Byeong Yeol Jeon
 * @ingroup Unity
 */
const char* C2SModuleUnity_MarshalAsString(NSString* input)
{
    if(nil == input)
        return NULL;
    
    const char* src = [input UTF8String];
    size_t len = strlen(src) + 1;
    char* data = (char*)malloc(len);
    
    strncpy(data, src, len);
    return data;
}
/**
 * @brief c 문자열을 유니티 문자열로 변환한다.
 
 * @param[in] input c 문자열
 
 * @return 유니티 문자열
 * @author Byeong Yeol Jeon
 * @ingroup Unity
 */
const char* C2SModuleUnity_MarshalAsString(const char* input)
{
    if(nil == input)
        return NULL;
    
    size_t len = strlen(input) + 1;
    char* data = (char*)malloc(len);
    
    strncpy(data, input, len);
    return data;
}
/**
 * @interface C2SModuleUnityNativeObject
 
 * @brief 유니티 델리게이트 등록을 위한 오브젝트
 * @author Byeong Yeol Jeon
 * @ingroup Unity
 */
@interface C2SModuleUnityNativeObject : NSObject<C2SModuleResultDelegate>
/** 등록될 실제 델리게이트 */
@property (nonatomic, assign) CommonResultDelegate customFn;
/**
 * @brief C2SModuleUnityNativeObject의 싱글턴 인스턴스를 구한다.
 
 * @return C2SModuleUnityNativeObject의 싱글턴 인스턴스
 * @author Byeong Yeol Jeon
 
 */
+ (C2SModuleUnityNativeObject*)sharedInstance;
@end

/**
 * @brief 싱글턴 인스턴스 변수
 * @author Byeong Yeol Jeon
 * @ingroup Unity
 */
static C2SModuleUnityNativeObject* C2SModuleUnityNativeObjectInstance = nil;

@implementation C2SModuleUnityNativeObject
+ (C2SModuleUnityNativeObject*)sharedInstance
{
	static dispatch_once_t onceToken;
	dispatch_once(&onceToken, ^ {
        C2SModuleUnityNativeObjectInstance = [[C2SModuleUnityNativeObject alloc] init];
    });
	return C2SModuleUnityNativeObjectInstance;
}
/**
 * @brief defualt constructor
 * @return 인스턴스
 * @author Byeong Yeol Jeon
 
 */
-(id)init
{
	self = [super init];
    
	if(self)
	{
        // 현재 인스턴스를 공통 델리게이트로 등록한다.
		[C2SModuleSocial SetCommonResultDelegate:self];
	}
	return self;
}
/**
 * @brief 유니트 공통 델리게이트의 연결 구현 부분
 * @param[in] api 통지받은 api의 종류
 * @param[in] arg 입력 인자
 * @param[in] error 발생한 에러
 * @author Byeong Yeol Jeon
 */
-(void)C2SModuleResult:(C2SModuleApi)api parameter:(NSDictionary *)arg error:(NSError*)error
{
	if(nil != self.customFn)
	{
		NSString* result = nil;
        
        // 인자가 있다면
		if(nil != arg)
		{
            // 문자열로 변환한다.
			NSError* serializeArgError = nil;
			NSData* data = [NSJSONSerialization dataWithJSONObject:arg options:kNilOptions error:&serializeArgError];
            
			if(nil != serializeArgError)
			{
				error = [NSError errorWithDomain:serializeArgError.domain code:C2SModuleErrorCode_FailConvertToJsonFormat userInfo:serializeArgError.userInfo];
			}
			else
			{
				result = [[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding];
			}
		}
        
        // 등록된 함수를 호출한다. 유니티로 문자열을 넘길 때 마샬링한다.
        self.customFn(api, C2SModuleUnity_MarshalAsString(result), error ? (C2SModuleErrorCode)error.code : C2SModuleErrorCode_Success, error ? C2SModuleUnity_MarshalAsString(error.domain) : nil);
	}
}
@end

#define FUNCTION_BODY_FOR_UNITY(a, b) \
    id realArg; \
    if(nil != arg) \
    { \
        C2SModule_Cocos2dx::C2SModuleParameterMap* instance = (C2SModule_Cocos2dx::C2SModuleParameterMap*)arg; \
        C2SModule_Cocos2dx::C2SModuleParameterContainer* container = (C2SModule_Cocos2dx::C2SModuleParameterContainer*)instance->Get(); \
        realArg = container->container; \
        delete instance; \
    } \
    else \
    { \
        realArg = nil; \
    } \
\
    NSError* error; \
    BOOL ret; \
    if(NULL == callback) \
    { \
        ret = [a b:realArg error:&error]; \
    } \
    else\
    { \
        ret = [a b:realArg error:&error completionHandler:^(NSDictionary* argResult, NSError* errorResult) { \
            NSError* sereializationError = nil; \
            NSData* data = (nil != argResult ? [NSJSONSerialization dataWithJSONObject:argResult options:kNilOptions error:&sereializationError] : nil); \
\
            if(nil != sereializationError) \
            { \
                GlobalCommonCustomDelegateFunc(callback, nil, C2SModuleErrorCode_FailConvertToJsonFormat, C2SModuleUnity_MarshalAsString(sereializationError.domain)); \
                return; \
            }\
\
            NSString* resultJson = (nil != data ? [[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding] : nil);\
\
            if(nil != errorResult)\
            {\
                GlobalCommonCustomDelegateFunc(callback, C2SModuleUnity_MarshalAsString(resultJson), (C2SModuleErrorCode)errorResult.code, C2SModuleUnity_MarshalAsString(errorResult.domain));\
            }\
            else\
            {\
                GlobalCommonCustomDelegateFunc(callback, C2SModuleUnity_MarshalAsString(resultJson), C2SModuleErrorCode_Success, nil);\
            }\
        }];\
    }\
\
    if(NO == ret)\
    {\
        if(NULL != message)\
        {\
            *message = C2SModuleUnity_MarshalAsString(error.domain);\
        }\
        return (C2SModuleErrorCode)error.code;\
    }\
    return C2SModuleErrorCode_Success;

#define FUNCTION_BODY_FOR_UNITY2(a, b) \
    id realArg; \
    if(nil != arg) \
    { \
        C2SModule_Cocos2dx::C2SModuleParameterMap* instance = (C2SModule_Cocos2dx::C2SModuleParameterMap*)arg; \
        C2SModule_Cocos2dx::C2SModuleParameterContainer* container = (C2SModule_Cocos2dx::C2SModuleParameterContainer*)instance->Get(); \
        realArg = container->container; \
        delete instance; \
    } \
    else \
    { \
        realArg = nil; \
    } \
\
    NSError* error; \
    BOOL ret; \
    if(NULL == callback) \
    { \
        if(NULL != message)\
        {\
            *message = C2SModuleUnity_MarshalAsString("invalid arg");\
        }\
        return C2SModuleErrorCode_InvalidArg;\
    } \
    else\
    { \
        ret = [a b:realArg error:&error completionHandler:^(NSDictionary* argResult, NSError* errorResult) { \
            NSError* sereializationError = nil; \
            NSData* data = (nil != argResult ? [NSJSONSerialization dataWithJSONObject:argResult options:kNilOptions error:&sereializationError] : nil); \
\
            if(nil != sereializationError) \
            { \
                GlobalCommonCustomDelegateFunc(callback, nil, C2SModuleErrorCode_FailConvertToJsonFormat, C2SModuleUnity_MarshalAsString(sereializationError.domain)); \
                return; \
            }\
\
            NSString* resultJson = (nil != data ? [[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding] : nil);\
\
            if(nil != errorResult)\
            {\
                GlobalCommonCustomDelegateFunc(callback, C2SModuleUnity_MarshalAsString(resultJson), (C2SModuleErrorCode)errorResult.code, C2SModuleUnity_MarshalAsString(errorResult.domain));\
            }\
            else \
            {\
                GlobalCommonCustomDelegateFunc(callback, C2SModuleUnity_MarshalAsString(resultJson), C2SModuleErrorCode_Success, nil);\
            }\
        }];\
    }\
\
    if(NO == ret)\
    {\
        if(NULL != message)\
        {\
            *message = C2SModuleUnity_MarshalAsString(error.domain);\
        }\
        return (C2SModuleErrorCode)error.code;\
    }\
    return C2SModuleErrorCode_Success;

#import <UIKit/UIKit.h>
extern "C" {
    /**
     * @brief 유니티에서 호출하는 공통 델리게이트 설정 함수
     * @param[in] func 설정될 델리게이트
     * @author Byeong Yeol Jeon
     * @ingroup Unity
     */
    void C2SModuleUnity_SetCommonResultDelegate(CommonResultDelegate func) {
        [C2SModuleUnityNativeObject sharedInstance].customFn = func;
    }
    /**
     * @brief 유니티에서 설정한 커스텀 핸들러 처리 함수의 정의
     * @author Byeong Yeol Jeon
     * @ingroup Unity
     */
    typedef void (*CommonCustomDelegate)(void* delegateObject, const char* json, C2SModuleErrorCode code, const char* message);
    /**
     * @brief 유니티에서 설정한 커스텀 핸들러 처리 함수의 변수
     * @author Byeong Yeol Jeon
     * @ingroup Unity
     */
    static CommonCustomDelegate GlobalCommonCustomDelegateFunc = NULL;
    /**
     * @brief 유니티에서 설정한 커스텀 핸들러 처리 함수를 모듈에 설정한다.
     * @param[in] func 유니티에서 설정한 커스텀 핸들러 처리 함수
     * @author Byeong Yeol Jeon
     * @ingroup Unity
     */
    void C2SModuleUnity_SetCommonCustomDelegate(CommonCustomDelegate func)
    {
        GlobalCommonCustomDelegateFunc = func;
    }
    /**
     * @brief 유니티에서 iOS콘솔에 로그를 출력한다.
     * @param[in] log 출력할 문자열
     * @author Byeong Yeol Jeon
     * @ingroup Unity
     */
    void C2SModuleUnity_Log(const char* log)
    {
        NSLog(@"%@", [NSString stringWithUTF8String:log]);
    }
    /**
     * @brief Social Initialize의 유니티 호출 함수
     * @param[in] arg 입력 인자
     * @param[in] message 실행시 발생한 에러의 문자열
     * @param[in] callback 결과 통지 델리게이트 오브젝트의 인스턴스
     * @return 실행시 발생한 에러의 코드
     * @ingroup Unity
     * @author Byeong Yeol Jeon
     */
	C2SModuleErrorCode C2SModuleUnitySocial_Initialize(void* arg, const char** message, void* callback)
	{
        FUNCTION_BODY_FOR_UNITY(C2SModuleSocial, Initialize);
	}
    /**
     * @brief Social Show의 유니티 호출 함수
     * @param[in] arg 입력 인자
     * @param[in] message 실행시 발생한 에러의 문자열
     * @param[in] callback 결과 통지 델리게이트 오브젝트의 인스턴스
     * @return 실행시 발생한 에러의 코드
     * @ingroup Unity
     * @author Byeong Yeol Jeon
     */
	C2SModuleErrorCode C2SModuleUnitySocial_Show(void* arg, const char** message, void* callback)
	{
		FUNCTION_BODY_FOR_UNITY(C2SModuleSocial, Show);
	}
    /**
     * @brief Social Info의 유니티 호출 함수
     * @param[in] arg 입력 인자
     * @param[in] message 실행시 발생한 에러의 문자열
     * @param[in] callback 결과 통지 델리게이트 오브젝트의 인스턴스
     * @return 실행시 발생한 에러의 코드
     * @ingroup Unity
     * @author DaunJoung
     */
    C2SModuleErrorCode C2SModuleUnitySocial_Info(void* arg, const char** message, void* callback)
    {
        FUNCTION_BODY_FOR_UNITY(C2SModuleSocial, Info);
    }
    /**
     * @brief Social SetGameInfo의 유니티 호출 함수
     * @param[in] arg 입력 인자
     * @param[in] message 실행시 발생한 에러의 문자열
     * @param[in] callback 결과 통지 델리게이트 오브젝트의 인스턴스
     * @return 실행시 발생한 에러의 코드
     * @ingroup Unity
     * @author Byeong Yeol Jeon
     */
	C2SModuleErrorCode C2SModuleUnitySocial_SetGameInfo(void* arg, const char** message, void* callback)
	{
		FUNCTION_BODY_FOR_UNITY(C2SModuleSocial, SetGameInfo);
	}

    /**
     * @brief Social UpdateGameLanguage의 유니티 호출 함수
     * @param[in] arg 입력 인자
     * @param[in] message 실행시 발생한 에러의 문자열
     * @param[in] callback 결과 통지 델리게이트 오브젝트의 인스턴스
     * @return 실행시 발생한 에러의 코드
     * @ingroup Unity
     * @author Seokjin Yong
     */
    C2SModuleErrorCode C2SModuleUnitySocial_UpdateGameLanguage(void* arg, const char** message, void* callback)
    {
        FUNCTION_BODY_FOR_UNITY(C2SModuleSocial, UpdateGameLanguage);
    }

    /**
     * @brief Social UpdateServerId의 유니티 호출 함수
     * @param[in] arg 입력 인자
     * @param[in] message 실행시 발생한 에러의 문자열
     * @param[in] callback 결과 통지 델리게이트 오브젝트의 인스턴스
     * @return 실행시 발생한 에러의 코드
     * @ingroup Unity
     * @author Seokjin Yong
     */
    C2SModuleErrorCode C2SModuleUnitySocial_UpdateServerId(void* arg, const char** message, void* callback)
    {
        FUNCTION_BODY_FOR_UNITY(C2SModuleSocial, UpdateServerId);
    }

    /**
     * @brief Social SendData의 유니티 호출 함수
     * @param[in] arg 입력 인자
     * @param[in] message 실행시 발생한 에러의 문자열
     * @param[in] callback 결과 통지 델리게이트 오브젝트의 인스턴스
     * @return 실행시 발생한 에러의 코드
     * @ingroup Unity
     * @author Byeong Yeol Jeon
     */
    C2SModuleErrorCode C2SModuleUnitySocial_SendData(void* arg, const char** message, void* callback)
    {
        FUNCTION_BODY_FOR_UNITY(C2SModuleSocial, SendData);
    }
    /**
     * @brief Sns Session의 유니티 호출 함수
     * @param[in] arg 입력 인자
     * @param[in] message 실행시 발생한 에러의 문자열
     * @param[in] callback 결과 통지 델리게이트 오브젝트의 인스턴스
     * @return 실행시 발생한 에러의 코드
     * @ingroup Unity
     * @author Byeong Yeol Jeon
     */
	C2SModuleErrorCode C2SModuleUnitySns_Session(void* arg, const char** message, void* callback)
	{
		FUNCTION_BODY_FOR_UNITY(C2SModuleSns, Session);
	}
    /**
     * @brief Sns Login의 유니티 호출 함수
     * @param[in] arg 입력 인자
     * @param[in] message 실행시 발생한 에러의 문자열
     * @param[in] callback 결과 통지 델리게이트 오브젝트의 인스턴스
     * @return 실행시 발생한 에러의 코드
     * @ingroup Unity
     * @author Byeong Yeol Jeon
     */
	C2SModuleErrorCode C2SModuleUnitySns_Login(void* arg, const char** message, void* callback)
	{
		FUNCTION_BODY_FOR_UNITY(C2SModuleSns, Login);
	}
    
    /**
     * @brief Sns Logout의 유니티 호출 함수
     * @param[in] arg 입력 인자
     * @param[in] message 실행시 발생한 에러의 문자열
     * @param[in] callback 결과 통지 델리게이트 오브젝트의 인스턴스
     * @return 실행시 발생한 에러의 코드
     * @ingroup Unity
     * @author Byeong Yeol Jeon
     */
	C2SModuleErrorCode C2SModuleUnitySns_Logout(void* arg, const char** message, void* callback)
	{
		FUNCTION_BODY_FOR_UNITY(C2SModuleSns, Logout);
	}
    /**
     * @brief Sns Unregister의 유니티 호출 함수
     * @param[in] arg 입력 인자
     * @param[in] message 실행시 발생한 에러의 문자열
     * @param[in] callback 결과 통지 델리게이트 오브젝트의 인스턴스
     * @return 실행시 발생한 에러의 코드
     * @ingroup Unity
     * @author Byeong Yeol Jeon
     */
	C2SModuleErrorCode C2SModuleUnitySns_Unregister(void* arg, const char** message, void* callback)
	{
		FUNCTION_BODY_FOR_UNITY(C2SModuleSns, Unregister);
	}
    /**
     * @brief Sns User의 유니티 호출 함수
     * @param[in] arg 입력 인자
     * @param[in] message 실행시 발생한 에러의 문자열
     * @param[in] callback 결과 통지 델리게이트 오브젝트의 인스턴스
     * @return 실행시 발생한 에러의 코드
     * @ingroup Unity
     * @author Byeong Yeol Jeon
     */
	C2SModuleErrorCode C2SModuleUnitySns_User(void* arg, const char** message, void* callback)
	{
		FUNCTION_BODY_FOR_UNITY(C2SModuleSns, User);
	}
    /**
     * @brief Sns Friends의 유니티 호출 함수
     * @param[in] arg 입력 인자
     * @param[in] message 실행시 발생한 에러의 문자열
     * @param[in] callback 결과 통지 델리게이트 오브젝트의 인스턴스
     * @return 실행시 발생한 에러의 코드
     * @ingroup Unity
     * @author Byeong Yeol Jeon
     */
	C2SModuleErrorCode C2SModuleUnitySns_Friends(void* arg, const char** message, void* callback)
	{
		FUNCTION_BODY_FOR_UNITY(C2SModuleSns, Friends);
	}
    /**
     * @brief Sns Message의 유니티 호출 함수
     * @param[in] arg 입력 인자
     * @param[in] message 실행시 발생한 에러의 문자열
     * @param[in] callback 결과 통지 델리게이트 오브젝트의 인스턴스
     * @return 실행시 발생한 에러의 코드
     * @ingroup Unity
     * @author Byeong Yeol Jeon
     */
	C2SModuleErrorCode C2SModuleUnitySns_Message(void* arg, const char** message, void* callback)
	{
		FUNCTION_BODY_FOR_UNITY(C2SModuleSns, Message);
	}
    /**
     * @brief Sns Post의 유니티 호출 함수
     * @param[in] arg 입력 인자
     * @param[in] message 실행시 발생한 에러의 문자열
     * @param[in] callback 결과 통지 델리게이트 오브젝트의 인스턴스
     * @return 실행시 발생한 에러의 코드
     * @ingroup Unity
     * @author Byeong Yeol Jeon
     */
	C2SModuleErrorCode C2SModuleUnitySns_Post(void* arg, const char** message, void* callback)
	{
		FUNCTION_BODY_FOR_UNITY(C2SModuleSns, Post);
	}
    /**
     * @brief Sns Show의 유니티 호출 함수
     * @param[in] arg 입력 인자
     * @param[in] message 실행시 발생한 에러의 문자열
     * @param[in] callback 결과 통지 델리게이트 오브젝트의 인스턴스
     * @return 실행시 발생한 에러의 코드
     * @ingroup Unity
     * @author Byeong Yeol Jeon
     */
	C2SModuleErrorCode C2SModuleUnitySns_Show(void* arg, const char** message, void* callback)
	{
		FUNCTION_BODY_FOR_UNITY(C2SModuleSns, Show);
	}
    /**
     * @brief Sns LoadData의 유니티 호출 함수
     * @param[in] arg 입력 인자
     * @param[in] message 실행시 발생한 에러의 문자열
     * @param[in] callback 결과 통지 델리게이트 오브젝트의 인스턴스
     * @return 실행시 발생한 에러의 코드
     * @ingroup Unity
     * @author Byeong Yeol Jeon
     */
	C2SModuleErrorCode C2SModuleUnitySns_LoadData(void* arg, const char** message, void* callback)
	{
        FUNCTION_BODY_FOR_UNITY2(C2SModuleSns, LoadData);
	}
    /**
     * @brief Sns SaveData의 유니티 호출 함수
     * @param[in] arg 입력 인자
     * @param[in] message 실행시 발생한 에러의 문자열
     * @param[in] callback 결과 통지 델리게이트 오브젝트의 인스턴스
     * @return 실행시 발생한 에러의 코드
     * @ingroup Unity
     * @author Byeong Yeol Jeon
     */
	C2SModuleErrorCode C2SModuleUnitySns_SaveData(void* arg, const char** message, void* callback)
	{
		FUNCTION_BODY_FOR_UNITY2(C2SModuleSns, SaveData);
	}
    /**
     * @brief Sns SaveData의 유니티 호출 함수
     * @param[in] arg 입력 인자
     * @param[in] message 실행시 발생한 에러의 문자열
     * @param[in] callback 결과 통지 델리게이트 오브젝트의 인스턴스
     * @return 실행시 발생한 에러의 코드
     * @ingroup Unity
     * @author Byeong Yeol Jeon
     */
    C2SModuleErrorCode C2SModuleUnitySns_SetGuestUserInfo(void* arg, const char** message, void* callback)
    {
        FUNCTION_BODY_FOR_UNITY(C2SModuleSns, SetGuestUserInfo);
    }
    /**
     * @brief Sns UserDelete의 유니티 호출 함수
     * @param[in] arg 입력 인자
     * @param[in] message 실행시 발생한 에러의 문자열
     * @param[in] callback 결과 통지 델리게이트 오브젝트의 인스턴스
     * @return 실행시 발생한 에러의 코드
     * @ingroup Unity
     * @author choiBK
     */
    C2SModuleErrorCode C2SModuleUnitySns_UserDelete(void* arg, const char** message, void* callback)
    {
        FUNCTION_BODY_FOR_UNITY(C2SModuleSns, UserDelete);
    }

    /**
     * @brief Push Register의 유니티 호출 함수
     * @param[in] arg 입력 인자
     * @param[in] message 실행시 발생한 에러의 문자열
     * @param[in] callback 결과 통지 델리게이트 오브젝트의 인스턴스
     * @return 실행시 발생한 에러의 코드
     * @ingroup Unity
     * @author Byeong Yeol Jeon
     */
	C2SModuleErrorCode C2SModuleUnityPush_Register(void* arg, const char** message, void* callback)
	{
		FUNCTION_BODY_FOR_UNITY(C2SModulePush, Register);
	}
    /**
     * @brief Push Unregister의 유니티 호출 함수
     * @param[in] arg 입력 인자
     * @param[in] message 실행시 발생한 에러의 문자열
     * @param[in] callback 결과 통지 델리게이트 오브젝트의 인스턴스
     * @return 실행시 발생한 에러의 코드
     * @ingroup Unity
     * @author Byeong Yeol Jeon
     */
	C2SModuleErrorCode C2SModuleUnityPush_Unregister(void* arg, const char** message, void* callback)
	{
		FUNCTION_BODY_FOR_UNITY(C2SModulePush, Unregister);
	}
    /**
     * @brief Push Desription의 유니티 호출 함수
     * @param[in] arg 입력 인자
     * @param[in] message 실행시 발생한 에러의 문자열
     * @param[in] callback 결과 통지 델리게이트 오브젝트의 인스턴스
     * @return 실행시 발생한 에러의 코드
     * @ingroup Unity
     * @author Byeong Yeol Jeon
     */
	C2SModuleErrorCode C2SModuleUnityPush_Description(void* arg, const char** message, void* callback)
	{
		FUNCTION_BODY_FOR_UNITY(C2SModulePush, Description);
	}

    /**
     * @brief Push RequestPermission의 유니티 호출 함수
     * @param[in] arg 입력 인자
     * @param[in] message 실행시 발생한 에러의 문자열
     * @param[in] callback 결과 통지 델리게이트 오브젝트의 인스턴스
     * @return 실행시 발생한 에러의 코드
     * @ingroup Unity
     * @author sjyong
     */
    C2SModuleErrorCode C2SModuleUnityPush_RequestPermission(void* arg, const char** message, void* callback)
    {
        FUNCTION_BODY_FOR_UNITY(C2SModulePush, RequestPermission);
    }

    /**
     * @brief Promotion Show의 유니티 호출 함수
     * @param[in] arg 입력 인자
     * @param[in] message 실행시 발생한 에러의 문자열
     * @param[in] callback 결과 통지 델리게이트 오브젝트의 인스턴스
     * @return 실행시 발생한 에러의 코드
     * @ingroup Unity
     * @author Byeong Yeol Jeon
     */
	C2SModuleErrorCode C2SModuleUnityPromotion_Show(void* arg, const char** message, void* callback)
	{
		FUNCTION_BODY_FOR_UNITY(C2SModulePromotion, Show);
	}
    
    /**
     * @brief Promotion Info의 유니티 호출 함수
     * @param[in] arg 입력 인자
     * @param[in] message 실행시 발생한 에러의 문자열
     * @param[in] callback 결과 통지 델리게이트 오브젝트의 인스턴스
     * @return 실행시 발생한 에러의 코드
     * @ingroup Unity
     * @author Byeong Yeol Jeon
     */
    C2SModuleErrorCode C2SModuleUnityPromotion_Info(void* arg, const char** message, void* callback)
    {
        FUNCTION_BODY_FOR_UNITY(C2SModulePromotion, Info);
    }
    
    /**
    * @brief 프로모션 관련 배너 정보를 얻어오고 completionHandler로 전달 받는다
    * @param[in] arg remark 참조
    * @param[in, out] error 실행시 발생한 에러
    * @param[in] completionHandler 결과를 통지받을 블럭 핸들러
    * @return YES일 경우 호출 성공. NO일 경우 호출 실패.
    * @remark
    * - "campaignType" : NSString*.
    *      - "event"  : 이벤트 타입
    *      - "notice" : 공지 타입
    * - "bannerType" : NSString*.
    *      - "great"   : 대배너
    *      - "small"   : 일반배너 (띠배너)
    *      - "rolling" : 롤링 배너
    * @author daunjoung
    */
    C2SModuleErrorCode C2SModuleUnityPromotion_GetBannerInfo(void* arg, const char** message, void* callback)
    {
        FUNCTION_BODY_FOR_UNITY(C2SModulePromotion, GetBannerInfo);
    }

    /**
     * @brief Promotion Acquisition의 유니티 호출 함수
     * @param[in] arg 입력 인자
     * @param[in] message 실행시 발생한 에러의 문자열
     * @param[in] callback 결과 통지 델리게이트 오브젝트의 인스턴스
     * @return 실행시 발생한 에러의 코드
     * @ingroup Unity
     * @author Choi BK
     */
    C2SModuleErrorCode C2SModuleUnityPromotion_Acquisition(void* arg, const char** message, void* callback)
    {
        FUNCTION_BODY_FOR_UNITY(C2SModulePromotion, Acquisition);
    }
    
    /**
     * @brief Engagement ProcessScheme의 유니티 호출 함수
     * @param[in] arg 입력 인자
     * @param[in] message 실행시 발생한 에러의 문자열
     * @param[in] callback 결과 통지 델리게이트 오브젝트의 인스턴스
     * @return 실행시 발생한 에러의 코드
     * @ingroup Unity
     * @author Hysterit
     */
    C2SModuleErrorCode C2SModuleUnityEngagement_ProcessScheme(void* arg, const char** message, void* callback)
    {
        FUNCTION_BODY_FOR_UNITY(C2SModuleEngagement, ProcessScheme);
    }
    
    /**
     * @brief Engagement GetEvents의 유니티 호출 함수
     * @param[in] arg 입력 인자
     * @param[in] message 실행시 발생한 에러의 문자열
     * @param[in] callback 결과 통지 델리게이트 오브젝트의 인스턴스
     * @return 실행시 발생한 에러의 코드
     * @ingroup Unity
     * @author Hysterit
     * Deprecated in HIVE SDK 1.11.0
     */
//    C2SModuleErrorCode C2SModuleUnityEngagement_GetEvents(void* arg, const char** message, void* callback)
//    {
//        FUNCTION_BODY_FOR_UNITY(C2SModuleEngagement, GetEvents);
//    }
    
    /**
     * @brief Engagement ConsumeCoupon의 유니티 호출 함수
     * @param[in] arg 입력 인자
     * @param[in] message 실행시 발생한 에러의 문자열
     * @param[in] callback 결과 통지 델리게이트 오브젝트의 인스턴스
     * @return 실행시 발생한 에러의 코드
     * @ingroup Unity
     * @author Hysterit
     */
    C2SModuleErrorCode C2SModuleUnityEngagement_ConsumeCoupon(void* arg, const char** message, void* callback)
    {
        FUNCTION_BODY_FOR_UNITY(C2SModuleEngagement, ConsumeCoupon);
    }
    
    /**
     * @brief Engagement GetEvents의 유니티 호출 함수
     * @param[in] arg 입력 인자
     * @param[in] message 실행시 발생한 에러의 문자열
     * @param[in] callback 결과 통지 델리게이트 오브젝트의 인스턴스
     * @return 실행시 발생한 에러의 코드
     * @ingroup Unity
     * @author Hysterit
     */
    C2SModuleErrorCode C2SModuleUnityEngagement_SetReady(void* arg, const char** message, void* callback)
    {
        FUNCTION_BODY_FOR_UNITY(C2SModuleEngagement, SetReady);
    }
    
    
    /**
     * @brief C2STracker SendEvent의 유니티 호출 함수
     * @param[in] arg 입력 인자
     * @param[in] message 실행시 발생한 에러의 문자열
     * @param[in] callback 결과 통지 델리게이트 오브젝트의 인스턴스
     * @return 실행시 발생한 에러의 코드
     * @ingroup Unity
     * @author cocororo
     */
    C2SModuleErrorCode C2SModuleUnityTracker_SendEvent(void* arg, const char** message, void* callback)
    {
        FUNCTION_BODY_FOR_UNITY(C2SModuleTracker, SendEvent);
    }
    
    /**
     * @brief C2STracker SetEnable의 유니티 호출 함수
     * @param[in] arg 입력 인자
     * @param[in] message 실행시 발생한 에러의 문자열
     * @param[in] callback 결과 통지 델리게이트 오브젝트의 인스턴스
     * @return 실행시 발생한 에러의 코드
     * @ingroup Unity
     * @author cocororo
     */
    C2SModuleErrorCode C2SModuleUnityTracker_SetEnable(void* arg, const char** message, void* callback)
    {
        FUNCTION_BODY_FOR_UNITY(C2SModuleTracker, SetEnable);
    }
    
    /**
     * @brief Hive Promotion(Promotion2.0) Show의 유니티 호출 함수
     * @param[in] arg 입력 인자
     * @param[in] message 실행시 발생한 에러의 문자열
     * @param[in] callback 결과 통지 델리게이트 오브젝트의 인스턴스
     * @return 실행시 발생한 에러의 코드
     * @ingroup Unity
     * @author pnpsinki
     */
    C2SModuleErrorCode C2SModuleUnityHivePromotion_Show(void* arg, const char** message, void* callback)
    {
        FUNCTION_BODY_FOR_UNITY(C2SModuleHivePromotion, Show);
    }

    /**
     * @brief Hive Promotion(Promotion2.0) Info의 유니티 호출 함수
     * @param[in] arg 입력 인자
     * @param[in] message 실행시 발생한 에러의 문자열
     * @param[in] callback 결과 통지 델리게이트 오브젝트의 인스턴스
     * @return 실행시 발생한 에러의 코드
     * @ingroup Unity
     * @author pnpsinki
     */
    C2SModuleErrorCode C2SModuleUnityHivePromotion_Info(void* arg, const char** message, void* callback)
    {
        FUNCTION_BODY_FOR_UNITY(C2SModuleHivePromotion, Info);
    }

    /**
     * @brief Hive Promotion(Promotion2.0) GetOfferwallState의 유니티 호출 함수
     * @param[in] arg 입력 인자
     * @param[in] message 실행시 발생한 에러의 문자열
     * @param[in] callback 결과 통지 델리게이트 오브젝트의 인스턴스
     * @return 실행시 발생한 에러의 코드
     * @ingroup Unity
     * @author pnpsinki
     */
    C2SModuleErrorCode C2SModuleUnityHivePromotion_GetOfferwallState(void* arg, const char** message, void* callback)
    {
        FUNCTION_BODY_FOR_UNITY(C2SModuleHivePromotion, GetOfferwallState);
    }

    /**
     * @brief Hive Promotion(Promotion2.0) SetAdditionalInfo의 유니티 호출 함수
     * @param[in] arg 입력 인자
     * @param[in] message 실행시 발생한 에러의 문자열
     * @param[in] callback 결과 통지 델리게이트 오브젝트의 인스턴스
     * @return 실행시 발생한 에러의 코드
     * @ingroup Unity
     * @author pnpsinki
     */
    C2SModuleErrorCode C2SModuleUnityHivePromotion_SetAdditionalInfo(void* arg, const char** message, void* callback)
    {
        FUNCTION_BODY_FOR_UNITY(C2SModuleHivePromotion, SetAdditionalInfo);
    }

    /**
    * @brief 프로모션 관련 배너 정보를 얻어오고 completionHandler로 전달 받는다
    * @param[in] arg remark 참조
    * @param[in, out] error 실행시 발생한 에러
    * @param[in] completionHandler 결과를 통지받을 블럭 핸들러
    * @return YES일 경우 호출 성공. NO일 경우 호출 실패.
    * @remark
    * - "campaignType" : NSString*.
    *      - "event"  : 이벤트 타입
    *      - "notice" : 공지 타입
    * - "bannerType" : NSString*.
    *      - "great"   : 대배너
    *      - "small"   : 일반배너 (띠배너)
    *      - "rolling" : 롤링 배너
    * @author daunjoung
    */
    C2SModuleErrorCode C2SModuleUnityHivePromotion_GetBannerInfo(void* arg, const char** message, void* callback)
    {
        FUNCTION_BODY_FOR_UNITY(C2SModuleHivePromotion, GetBannerInfo)
    }

    /**
     * @brief Hive Promotion(Promotion2.0) UpdatePromotionData의 유니티 호출 함수
     * @param[in] arg 입력 인자
     * @param[in] message 실행시 발생한 에러의 문자열
     * @param[in] callback 결과 통지 델리게이트 오브젝트의 인스턴스
     * @return 실행시 발생한 에러의 코드
     * @ingroup Unity
     * @author jschoi
     */
    C2SModuleErrorCode C2SModuleUnityHivePromotion_UpdatePromotionData(void* arg, const char** message, void* callback)
    {
        FUNCTION_BODY_FOR_UNITY(C2SModuleHivePromotion, UpdatePromotionData);
    }

    //전에 정식 계정 또는 게스트로 로그인 했는지 정보를 가져오는 함수
    const char *C2SModuleUnitySns_GetLoginType() {
        
        
//        [C2SModuleSnsInterface sharedInstance].loginType = [C2SCommonData getProperty:LOGIN_TYPE];
        
        
        if ([C2SModuleSnsInterface sharedInstance].loginType != nil && [[C2SModuleSnsInterface sharedInstance].loginType length] > 0) {
            return strdup([[C2SModuleSnsInterface sharedInstance].loginType UTF8String]);
        }
        
        return strdup([@"" UTF8String]);
    }
    
    const char *C2SModuleUnitySocial_GetHiveCountry {

        if ([C2SModuleSocial sharedInstance].countryCode != nil && [[C2SModuleSocial sharedInstance].countryCode length] > 0) {
            return strdup([[C2SModuleSocial sharedInstance].countryCode UTF8String]);
        }

        return strdup([@"UNKNOWN" UTF8String]);
    }

     /**
     * @brief InappV4 MarkectConnect 의 유니티 호출 함수
     * @param[in] arg 입력 인자
     * @param[in] message 실행시 발생한 에러의 문자열
     * @param[in] callback 결과 통지 델리게이트 오브젝트의 인스턴스
     * @return 실행시 발생한 에러의 코드
     * @ingroup Unity
     * @author pnpsinki
     */
     C2SModuleErrorCode C2SModuleUnityInappV4_MarketConnect(void* arg, const char** message, void* callback)
     {
         FUNCTION_BODY_FOR_UNITY(C2SModuleInappV4, MarketConnect);
     }
     /**
     * @brief InappV4 ItemInfo 의 유니티 호출 함수
     * @param[in] arg 입력 인자
     * @param[in] message 실행시 발생한 에러의 문자열
     * @param[in] callback 결과 통지 델리게이트 오브젝트의 인스턴스
     * @return 실행시 발생한 에러의 코드
     * @ingroup Unity
     * @author pnpsinki
     */
     C2SModuleErrorCode C2SModuleUnityInappV4_ItemInfo(void* arg, const char** message, void* callback)
     {
         FUNCTION_BODY_FOR_UNITY(C2SModuleInappV4, ItemInfo);
     }
     /**
      * @brief InappV4 Purchase 의 유니티 호출 함수
      * @param[in] arg 입력 인자
      * @param[in] message 실행시 발생한 에러의 문자열
      * @param[in] callback 결과 통지 델리게이트 오브젝트의 인스턴스
      * @return 실행시 발생한 에러의 코드
      * @ingroup Unity
      * @author pnpsinki
      */
    C2SModuleErrorCode C2SModuleUnityInappV4_Purchase(void* arg, const char** message, void* callback)
    {
        FUNCTION_BODY_FOR_UNITY(C2SModuleInappV4, Purchase);
    }
    /**
     * @brief InappV4 Restore 의 유니티 호출 함수
     * @param[in] arg 입력 인자
     * @param[in] message 실행시 발생한 에러의 문자열
     * @param[in] callback 결과 통지 델리게이트 오브젝트의 인스턴스
     * @return 실행시 발생한 에러의 코드
     * @ingroup Unity
     * @author pnpsinki
     */
    C2SModuleErrorCode C2SModuleUnityInappV4_Restore(void* arg, const char** message, void* callback)
    {
        FUNCTION_BODY_FOR_UNITY(C2SModuleInappV4, Restore);
    }
    /**
     * @brief InappV4 TransactionFinish 의 유니티 호출 함수
     * @param[in] arg 입력 인자
     * @param[in] message 실행시 발생한 에러의 문자열
     * @param[in] callback 결과 통지 델리게이트 오브젝트의 인스턴스
     * @return 실행시 발생한 에러의 코드
     * @ingroup Unity
     * @author pnpsinki
     */
    C2SModuleErrorCode C2SModuleUnityInappV4_TransactionFinish(void* arg, const char** message, void* callback)
    {
        FUNCTION_BODY_FOR_UNITY(C2SModuleInappV4, TransactionFinish);
    }
    /**
     * @brief InappV4 GetSelectedMarket 의 유니티 호출 함수
     * @param[in] arg 입력 인자
     * @param[in] message 실행시 발생한 에러의 문자열
     * @param[in] callback 결과 통지 델리게이트 오브젝트의 인스턴스
     * @return 실행시 발생한 에러의 코드
     * @ingroup Unity
     * @author pnpsinki
     */
    C2SModuleErrorCode C2SModuleUnityInappV4_GetSelectedMarket(void* arg, const char** message, void* callback)
    {
        FUNCTION_BODY_FOR_UNITY(C2SModuleInappV4, GetSelectedMarket);
    }

#if __ANDROID__
    /**
     * @brief InappV4 SelectMarket 의 유니티 호출 함수
     * @param[in] arg 입력 인자
     * @param[in] message 실행시 발생한 에러의 문자열
     * @param[in] callback 결과 통지 델리게이트 오브젝트의 인스턴스
     * @return 실행시 발생한 에러의 코드
     * @ingroup Unity
     * @author pnpsinki
     */
    C2SModuleErrorCode C2SModuleUnityInappV4_SelectMarket(void* arg, const char** message, void* callback)
    {
        FUNCTION_BODY_FOR_UNITY(C2SModuleInappV4, SelectMarket);
    }
    /**
     * @brief InappV4 RequestBalance 의 유니티 호출 함수
     * @param[in] arg 입력 인자
     * @param[in] message 실행시 발생한 에러의 문자열
     * @param[in] callback 결과 통지 델리게이트 오브젝트의 인스턴스
     * @return 실행시 발생한 에러의 코드
     * @ingroup Unity
     * @author pnpsinki
     */
    C2SModuleErrorCode C2SModuleUnityInappV4_RequestBalance(void* arg, const char** message, void* callback)
    {
        FUNCTION_BODY_FOR_UNITY(C2SModuleInappV4, RequestBalance);
    }
    /**
     * @brief InappV4 ChargeBalance 의 유니티 호출 함수
     * @param[in] arg 입력 인자
     * @param[in] message 실행시 발생한 에러의 문자열
     * @param[in] callback 결과 통지 델리게이트 오브젝트의 인스턴스
     * @return 실행시 발생한 에러의 코드
     * @ingroup Unity
     * @author pnpsinki
     */
    C2SModuleErrorCode C2SModuleUnityInappV4_ChargeBalance(void* arg, const char** message, void* callback)
    {
        FUNCTION_BODY_FOR_UNITY(C2SModuleInappV4, ChargeBalance);
    }
#endif

#if __OBJC__
    /**
     * @brief InappV4 CheckPromotePurchase 의 유니티 호출 함수
     * @param[in] arg 입력 인자
     * @param[in] message 실행시 발생한 에러의 문자열
     * @param[in] callback 결과 통지 델리게이트 오브젝트의 인스턴스
     * @return 실행시 발생한 에러의 코드
     * @ingroup Unity
     * @author pnpsinki
     */
    C2SModuleErrorCode C2SModuleUnityInappV4_CheckPromotePurchase(void* arg, const char** message, void* callback)
    {
        FUNCTION_BODY_FOR_UNITY(C2SModuleInappV4, CheckPromotePurchase);
    }
#endif
}
namespace C2SModule_Cocos2dx {
    /**
     * @class C2SModuleParameterData
     
     * @brief 유니티의 데이터를 저장하는 중간 클래스
     
     * @author Byeong Yeol Jeon
     * @ingroup Unity
     */
    class C2SModuleParameterData
    {
    public:
        /**
         * @brief constructor
         
         * @param[in] sharedAddr 저장할 데이터의 주소
         * @param[in] length 데이터의 길이
         
         * @author Byeong Yeol Jeon
         
         */
        C2SModuleParameterData(void* sharedAddr, int32_t length)
        {
            data = [NSData dataWithBytes:sharedAddr length:length];
        }
        /** 저장된 데이터 */
        NSData* data;
    };
}

extern "C" {
    /**
     * @brief 유니티로 데이터를 생성하여 넘긴다.
     
     * @param[in] sharedAddr 유니티와 공유되는 데이터
     * @param[in] length 데이터의 길이
     * @return 생성된 데이터
     * @author Byeong Yeol Jeon
     * @ingroup Unity
     */
	void* C2SModuleUnityParameterData_Alloc(void* sharedAddr, int32_t length) {
		return new C2SModule_Cocos2dx::C2SModuleParameterData(sharedAddr, length);
	}
    /**
     * @brief 생성한 데이터를 제거한다.
     
     * @param[in] data 제거할 데이터
     
     * @author Byeong Yeol Jeon
     * @ingroup Unity
     */
	void C2SModuleUnityParameterData_Release(void* data) {
		C2SModule_Cocos2dx::C2SModuleParameterData* instance = (C2SModule_Cocos2dx::C2SModuleParameterData*) data;
		delete instance;
	}
}
#endif

namespace C2SModule_Cocos2dx {
    C2SModuleParameterMap::C2SModuleParameterMap() {
#if __OBJC__
        container = new C2SModuleParameterContainer([[NSMutableDictionary alloc] init]);
#elif __ANDROID__
        JNIEnv* env;
        if (JNI_OK
            != C2SModuleJavaVMInstance->GetEnv((void**) &env,
                                               JNI_VERSION_1_6)) {
                container = NULL;
            } else {
                jmethodID methodId = env->GetMethodID(C2SModuleArgumentClass, "<init>",
                                                      "()V");
                jobject tmp = env->NewObject(C2SModuleArgumentClass, methodId);
                
                container = new C2SModuleParameterContainer(env->NewGlobalRef(tmp));
                
                env->DeleteLocalRef(tmp);
                
                if (env->ExceptionCheck()) {
                    jthrowable exception = env->ExceptionOccurred();
                    env->ExceptionClear();
                    env->DeleteLocalRef(exception);
                }
            }
#endif
    }
    
    C2SModuleParameterMap::~C2SModuleParameterMap() {
        C2SModuleParameterContainer* real = (C2SModuleParameterContainer*) container;
        delete real;
        container = NULL;
    }
    
    void C2SModuleParameterMap::Set(const char key[], bool arg) {
        if (NULL == key || NULL == container)
            return;
        
        C2SModuleParameterContainer* real = (C2SModuleParameterContainer*) container;
#if __OBJC__
        [real->container setObject:[NSNumber numberWithBool:arg] forKey:[NSString stringWithUTF8String:key]];
#elif __ANDROID__
        JNIEnv* env;
        if (JNI_OK
            != C2SModuleJavaVMInstance->GetEnv((void**) &env,
                                               JNI_VERSION_1_6))
            return;
        
        jmethodID methodId = env->GetMethodID(C2SModuleArgumentClass, "putData",
                                              "(Ljava/lang/String;Z)V");
        
        jstring jKey = env->NewStringUTF(key);
        jobject ret = env->CallObjectMethod(reinterpret_cast<jobject>(real->container),
                                            methodId, jKey, arg);
        
        env->DeleteLocalRef(ret);
        env->DeleteLocalRef(jKey);
        
        if (env->ExceptionCheck()) {
            jthrowable exception = env->ExceptionOccurred();
            env->ExceptionClear();
            env->DeleteLocalRef(exception);
        }
#endif
    }
    
    void C2SModuleParameterMap::Set(const char key[], int8_t arg) {
        if (NULL == key || NULL == container)
            return;
        
        C2SModuleParameterContainer* real = (C2SModuleParameterContainer*) container;
#if __OBJC__
        [real->container setObject:[NSNumber numberWithChar:arg] forKey:[NSString stringWithUTF8String:key]];
#elif __ANDROID__
        JNIEnv* env;
        if (JNI_OK
            != C2SModuleJavaVMInstance->GetEnv((void**) &env,
                                               JNI_VERSION_1_6))
            return;
        
        jmethodID methodId = env->GetMethodID(C2SModuleArgumentClass, "putData",
                                              "(Ljava/lang/String;C)V");
        
        jstring jKey = env->NewStringUTF(key);
        jobject ret = env->CallObjectMethod(reinterpret_cast<jobject>(real->container),
                                            methodId, jKey, arg);
        
        env->DeleteLocalRef(ret);
        env->DeleteLocalRef(jKey);
        
        if (env->ExceptionCheck()) {
            jthrowable exception = env->ExceptionOccurred();
            env->ExceptionClear();
            env->DeleteLocalRef(exception);
        }
#endif
    }
    
    void C2SModuleParameterMap::Set(const char key[], uint8_t arg) {
        if (NULL == key || NULL == container)
            return;
        
        C2SModuleParameterContainer* real = (C2SModuleParameterContainer*) container;
#if __OBJC__
        [real->container setObject:[NSNumber numberWithUnsignedChar:arg] forKey:[NSString stringWithUTF8String:key]];
#elif __ANDROID__
        JNIEnv* env;
        if (JNI_OK
            != C2SModuleJavaVMInstance->GetEnv((void**) &env,
                                               JNI_VERSION_1_6))
            return;
        
        jmethodID methodId = env->GetMethodID(C2SModuleArgumentClass, "putData",
                                              "(Ljava/lang/String;C)V");
        
        jstring jKey = env->NewStringUTF(key);
        jobject ret = env->CallObjectMethod(reinterpret_cast<jobject>(real->container),
                                            methodId, jKey, arg);
        
        env->DeleteLocalRef(ret);
        env->DeleteLocalRef(jKey);
        
        if (env->ExceptionCheck()) {
            jthrowable exception = env->ExceptionOccurred();
            env->ExceptionClear();
            env->DeleteLocalRef(exception);
        }
#endif
    }
    
    void C2SModuleParameterMap::Set(const char key[], int16_t arg) {
        if (NULL == key || NULL == container)
            return;
        
        C2SModuleParameterContainer* real = (C2SModuleParameterContainer*) container;
#if __OBJC__
        [real->container setObject:[NSNumber numberWithShort:arg] forKey:[NSString stringWithUTF8String:key]];
#elif __ANDROID__
        JNIEnv* env;
        if (JNI_OK
            != C2SModuleJavaVMInstance->GetEnv((void**) &env,
                                               JNI_VERSION_1_6))
            return;
        
        jmethodID methodId = env->GetMethodID(C2SModuleArgumentClass, "putData",
                                              "(Ljava/lang/String;S)V");
        
        jstring jKey = env->NewStringUTF(key);
        jobject ret = env->CallObjectMethod(reinterpret_cast<jobject>(real->container),
                                            methodId, jKey, arg);
        
        env->DeleteLocalRef(ret);
        env->DeleteLocalRef(jKey);
        
        if (env->ExceptionCheck()) {
            jthrowable exception = env->ExceptionOccurred();
            env->ExceptionClear();
            env->DeleteLocalRef(exception);
        }
#endif
    }
    
    void C2SModuleParameterMap::Set(const char key[], uint16_t arg) {
        if (NULL == key || NULL == container)
            return;
        
        C2SModuleParameterContainer* real = (C2SModuleParameterContainer*) container;
#if __OBJC__
        [real->container setObject:[NSNumber numberWithUnsignedShort:arg] forKey:[NSString stringWithUTF8String:key]];
#elif __ANDROID__
        Set(key, static_cast<int16_t>(arg));
#endif
    }
    
    void C2SModuleParameterMap::Set(const char key[], int32_t arg) {
        if (NULL == key || NULL == container)
            return;
        
        C2SModuleParameterContainer* real = (C2SModuleParameterContainer*) container;
#if __OBJC__
        [real->container setObject:[NSNumber numberWithUnsignedInt:arg] forKey:[NSString stringWithUTF8String:key]];
#elif __ANDROID__
        JNIEnv* env;
        if (JNI_OK
            != C2SModuleJavaVMInstance->GetEnv((void**) &env,
                                               JNI_VERSION_1_6))
            return;
        
        jmethodID methodId = env->GetMethodID(C2SModuleArgumentClass, "putData",
                                              "(Ljava/lang/String;I)V");
        
        jstring jKey = env->NewStringUTF(key);
        jobject ret = env->CallObjectMethod(reinterpret_cast<jobject>(real->container),
                                            methodId, jKey, arg);
        
        env->DeleteLocalRef(ret);
        env->DeleteLocalRef(jKey);
        
        if (env->ExceptionCheck()) {
            jthrowable exception = env->ExceptionOccurred();
            env->ExceptionClear();
            env->DeleteLocalRef(exception);
        }
#endif
    }
    
    void C2SModuleParameterMap::Set(const char key[], uint32_t arg) {
        if (NULL == key || NULL == container)
            return;
        
        C2SModuleParameterContainer* real = (C2SModuleParameterContainer*) container;
#if __OBJC__
        [real->container setObject:[NSNumber numberWithUnsignedInt:arg] forKey:[NSString stringWithUTF8String:key]];
#elif __ANDROID__
        Set(key, static_cast<int32_t>(arg));
#endif
    }
    
    void C2SModuleParameterMap::Set(const char key[], int64_t arg) {
        if (NULL == key || NULL == container)
            return;
        
        C2SModuleParameterContainer* real = (C2SModuleParameterContainer*) container;
#if __OBJC__
        [real->container setObject:[NSNumber numberWithLongLong:arg] forKey:[NSString stringWithUTF8String:key]];
#elif __ANDROID__
        JNIEnv* env;
        if (JNI_OK
            != C2SModuleJavaVMInstance->GetEnv((void**) &env,
                                               JNI_VERSION_1_6))
            return;
        
        jmethodID methodId = env->GetMethodID(C2SModuleArgumentClass, "putData",
                                              "(Ljava/lang/String;J)V");
        
        jstring jKey = env->NewStringUTF(key);
        jobject ret = env->CallObjectMethod(reinterpret_cast<jobject>(real->container),
                                            methodId, jKey, arg);
        
        env->DeleteLocalRef(ret);
        env->DeleteLocalRef(jKey);
        
        if (env->ExceptionCheck()) {
            jthrowable exception = env->ExceptionOccurred();
            env->ExceptionClear();
            env->DeleteLocalRef(exception);
        }
#endif
    }
    
    void C2SModuleParameterMap::Set(const char key[], uint64_t arg) {
        if (NULL == key || NULL == container)
            return;
        
        C2SModuleParameterContainer* real = (C2SModuleParameterContainer*) container;
#if __OBJC__
        [real->container setObject:[NSNumber numberWithUnsignedLongLong:arg] forKey:[NSString stringWithUTF8String:key]];
#elif __ANDROID__
        Set(key, static_cast<int64_t>(arg));
#endif
    }
    
    void C2SModuleParameterMap::Set(const char key[], const char arg[]) {
        if (NULL == key || NULL == arg || NULL == container)
            return;
        
        C2SModuleParameterContainer* real = (C2SModuleParameterContainer*) container;
#if __OBJC__
        [real->container setObject:[NSString stringWithUTF8String:arg] forKey:[NSString stringWithUTF8String:key]];
#elif __ANDROID__
        JNIEnv* env;
        if (JNI_OK
            != C2SModuleJavaVMInstance->GetEnv((void**) &env,
                                               JNI_VERSION_1_6))
            return;
        
        jmethodID methodId = env->GetMethodID(C2SModuleArgumentClass, "putData",
                                              "(Ljava/lang/String;Ljava/lang/String;)V");
        
        jobject jValue = env->NewStringUTF(arg);
        jstring jKey = env->NewStringUTF(key);
        jobject ret = env->CallObjectMethod(reinterpret_cast<jobject>(real->container),
                                            methodId, jKey, jValue);
        
        env->DeleteLocalRef(ret);
        env->DeleteLocalRef(jKey);
        env->DeleteLocalRef(jValue);
        
        if (env->ExceptionCheck()) {
            jthrowable exception = env->ExceptionOccurred();
            env->ExceptionClear();
            env->DeleteLocalRef(exception);
        }
#endif
    }
    
    void C2SModuleParameterMap::Set(const char key[], C2SModuleImage& arg) {
        if (NULL == key || NULL == arg.get() || NULL == arg->Get()
            || NULL == container)
            return;
        
        C2SModuleParameterContainer* real = (C2SModuleParameterContainer*) container;
#if __OBJC__
        [real->container setObject:(__bridge id)arg->Get() forKey:[NSString stringWithUTF8String:key]];
#elif __ANDROID__
        JNIEnv* env;
        if (JNI_OK
            != C2SModuleJavaVMInstance->GetEnv((void**) &env,
                                               JNI_VERSION_1_6))
            return;
        
        jmethodID methodId = env->GetMethodID(C2SModuleArgumentClass, "putData",
                                              "(Ljava/lang/String;Ljava/lang/Object;)V");
        
        jstring jKey = env->NewStringUTF(key);
        jobject ret = env->CallObjectMethod(reinterpret_cast<jobject>(real->container),
                                            methodId, jKey, arg->Get());
        
        env->DeleteLocalRef(ret);
        env->DeleteLocalRef(jKey);
        
        if (env->ExceptionCheck()) {
            jthrowable exception = env->ExceptionOccurred();
            env->ExceptionClear();
            env->DeleteLocalRef(exception);
        }
#endif
    }
    
#if __OBJC__
    void C2SModuleParameterMap::Set(const char key[], id arg) {
        if (NULL == key || nil == arg || NULL == container)
            return;
        
        C2SModuleParameterContainer* real = (C2SModuleParameterContainer*)container;
        [real->container setObject:arg forKey:[NSString stringWithUTF8String:key]];
    }
#elif __ANDROID__
    void C2SModuleParameterMap::Set(const char key[], jobject arg) {
        if (NULL == key || NULL == arg || NULL == container)
            return;
        
        JNIEnv* env;
        if (JNI_OK
            != C2SModuleJavaVMInstance->GetEnv((void**) &env,
                                               JNI_VERSION_1_6))
            return;
        
        C2SModuleParameterContainer* real = (C2SModuleParameterContainer*)container;
        jmethodID methodId = env->GetMethodID(C2SModuleArgumentClass, "putData",
                                              "(Ljava/lang/String;Ljava/lang/Object;)V");
        
        jstring jKey = env->NewStringUTF(key);
        jobject ret = env->CallObjectMethod(reinterpret_cast<jobject>(real->container),
                                            methodId, jKey, arg);
        
        env->DeleteLocalRef(ret);
        env->DeleteLocalRef(jKey);
        
        if (env->ExceptionCheck()) {
            jthrowable exception = env->ExceptionOccurred();
            env->ExceptionClear();
            env->DeleteLocalRef(exception);
        }
    }
#endif
    
    void C2SModuleParameterMap::Set(const char key[],
                                    const C2SModuleParameterMap& arg) {
        if (NULL == key || NULL == container || NULL == arg.Get())
            return;
        
        C2SModuleParameterContainer* realSrc =
        (C2SModuleParameterContainer*) arg.Get();
        Set(key, realSrc->container);
    }
    
    void C2SModuleParameterMap::Set(const char key[],
                                    const C2SModuleParameterArray& arg) {
        if (NULL == key || NULL == container || NULL == arg.Get())
            return;
        
        C2SModuleParameterContainer* realSrc =
        (C2SModuleParameterContainer*) arg.Get();
#if __OBJC__
        Set(key, realSrc->container);
#elif __ANDROID__
        jobject target = reinterpret_cast<jobject>(realSrc->container);
        
        if (NULL != target) {
            JNIEnv* env;
            if (JNI_OK
                != C2SModuleJavaVMInstance->GetEnv((void**) &env,
                                                   JNI_VERSION_1_6))
                return;
            
            jmethodID methodId = env->GetMethodID(C2SModuleVectorClass, "toArray",
                                                  "()[Ljava/lang/Object;");
            jobject ret = env->CallObjectMethod(
                                                target, methodId);
            Set(key, ret);
            env->DeleteLocalRef(ret);
            
            if (env->ExceptionCheck()) {
                jthrowable exception = env->ExceptionOccurred();
                env->ExceptionClear();
                env->DeleteLocalRef(exception);
            }
        }
#endif
    }
    
    void* C2SModuleParameterMap::Get() const {
        return container;
    }
}
#if __OBJC__
extern "C" {
    /**
     * @brief 유니티 맵 파라미터를 생성한다.
     
     * @return 생성된 맵 파라미터
     * @author Byeong Yeol Jeon
     * @ingroup Unity
     */
	void* C2SModuleUnityParameterMap_Alloc() {
		return new C2SModule_Cocos2dx::C2SModuleParameterMap();
	}
    /**
     * @brief 맵 파라미터를 제거한다.
     
     * @param[in] map 맵 파라미터의 주소
     * @author Byeong Yeol Jeon
     * @ingroup Unity
     */
	void C2SModuleUnityParameterMap_Release(void* map) {
		C2SModule_Cocos2dx::C2SModuleParameterMap* instance = (C2SModule_Cocos2dx::C2SModuleParameterMap*) map;
		delete instance;
	}
    /**
     * @brief 맵 파라미터의 키에 bool값을 설정한다.
     * @param[in] map 맵 파라미터
     * @param[in] key 맵의 키
     * @param[in] value 입력 값
     * @ingroup Unity
     * @author Byeong Yeol Jeon
     */
	void C2SModuleUnityParameterMap_SetBool(void* map, const char* key,
                                            bool value) {
		C2SModule_Cocos2dx::C2SModuleParameterMap* instance = (C2SModule_Cocos2dx::C2SModuleParameterMap*) map;
		instance->Set(key, value);
	}
    /**
     * @brief 맵 파라미터의 키에 8bit signed값을 설정한다.
     * @param[in] map 맵 파라미터
     * @param[in] key 맵의 키
     * @param[in] value 입력 값
     * @ingroup Unity
     * @author Byeong Yeol Jeon
     */
	void C2SModuleUnityParameterMap_SetChar(void* map, const char* key,
                                            int8_t value) {
		C2SModule_Cocos2dx::C2SModuleParameterMap* instance = (C2SModule_Cocos2dx::C2SModuleParameterMap*) map;
		instance->Set(key, value);
	}
    /**
     * @brief 맵 파라미터의 키에 8bit unsigned값을 설정한다.
     * @param[in] map 맵 파라미터
     * @param[in] key 맵의 키
     * @param[in] value 입력 값
     * @ingroup Unity
     * @author Byeong Yeol Jeon
     */
	void C2SModuleUnityParameterMap_SetUChar(void* map, const char* key,
                                             uint8_t value) {
		C2SModule_Cocos2dx::C2SModuleParameterMap* instance = (C2SModule_Cocos2dx::C2SModuleParameterMap*) map;
		instance->Set(key, value);
	}
    /**
     * @brief 맵 파라미터의 키에 16bit signed값을 설정한다.
     * @param[in] map 맵 파라미터
     * @param[in] key 맵의 키
     * @param[in] value 입력 값
     * @ingroup Unity
     * @author Byeong Yeol Jeon
     */
	void C2SModuleUnityParameterMap_SetShort(void* map, const char* key,
                                             int16_t value) {
		C2SModule_Cocos2dx::C2SModuleParameterMap* instance = (C2SModule_Cocos2dx::C2SModuleParameterMap*) map;
		instance->Set(key, value);
	}
    /**
     * @brief 맵 파라미터의 키에 16bit unsigned값을 설정한다.
     * @param[in] map 맵 파라미터
     * @param[in] key 맵의 키
     * @param[in] value 입력 값
     * @ingroup Unity
     * @author Byeong Yeol Jeon
     */
	void C2SModuleUnityParameterMap_SetUShort(void* map, const char* key,
                                              uint16_t value) {
		C2SModule_Cocos2dx::C2SModuleParameterMap* instance = (C2SModule_Cocos2dx::C2SModuleParameterMap*) map;
		instance->Set(key, value);
	}
    /**
     * @brief 맵 파라미터의 키에 32bit signed값을 설정한다.
     * @param[in] map 맵 파라미터
     * @param[in] key 맵의 키
     * @param[in] value 입력 값
     * @ingroup Unity
     * @author Byeong Yeol Jeon
     */
	void C2SModuleUnityParameterMap_SetInt(void* map, const char* key, int32_t value) {
		C2SModule_Cocos2dx::C2SModuleParameterMap* instance = (C2SModule_Cocos2dx::C2SModuleParameterMap*) map;
		instance->Set(key, value);
	}
    /**
     * @brief 맵 파라미터의 키에 32bit unsigned값을 설정한다.
     * @param[in] map 맵 파라미터
     * @param[in] key 맵의 키
     * @param[in] value 입력 값
     * @ingroup Unity
     * @author Byeong Yeol Jeon
     */
    
    void C2SModuleUnityParameterMap_SetUInt(void* map, const char* key,
                                            uint32_t value) {
		C2SModule_Cocos2dx::C2SModuleParameterMap* instance = (C2SModule_Cocos2dx::C2SModuleParameterMap*) map;
		instance->Set(key, value);
	}
    /**
     * @brief 맵 파라미터의 키에 64bit signed값을 설정한다.
     * @param[in] map 맵 파라미터
     * @param[in] key 맵의 키
     * @param[in] value 입력 값
     * @ingroup Unity
     * @author Byeong Yeol Jeon
     */
	void C2SModuleUnityParameterMap_SetLong(void* map, const char* key,
                                            int64_t value) {
		C2SModule_Cocos2dx::C2SModuleParameterMap* instance = (C2SModule_Cocos2dx::C2SModuleParameterMap*) map;
		instance->Set(key, value);
	}
    /**
     * @brief 맵 파라미터의 키에 64bit unsigned값을 설정한다.
     * @param[in] map 맵 파라미터
     * @param[in] key 맵의 키
     * @param[in] value 입력 값
     * @ingroup Unity
     * @author Byeong Yeol Jeon
     */
	void C2SModuleUnityParameterMap_SetULong(void* map, const char* key,
                                             uint64_t value) {
		C2SModule_Cocos2dx::C2SModuleParameterMap* instance = (C2SModule_Cocos2dx::C2SModuleParameterMap*) map;
		instance->Set(key, value);
	}
    /**
     * @brief 맵 파라미터의 키에 문자열을 설정한다.
     * @param[in] map 맵 파라미터
     * @param[in] key 맵의 키
     * @param[in] value 입력 값
     * @ingroup Unity
     * @author Byeong Yeol Jeon
     */
	void C2SModuleUnityParameterMap_SetString(void* map, const char* key,
                                              const char* value) {
		C2SModule_Cocos2dx::C2SModuleParameterMap* instance = (C2SModule_Cocos2dx::C2SModuleParameterMap*) map;
		instance->Set(key, value);
	}
    /**
     * @brief 맵 파라미터의 키에 맵 파라미터를 설정한다.
     * @param[in] map 맵 파라미터
     * @param[in] key 맵의 키
     * @param[in] value 입력 값
     * @ingroup Unity
     * @author Byeong Yeol Jeon
     */
	void C2SModuleUnityParameterMap_SetMap(void* map, const char* key,
                                           void* value) {
		C2SModule_Cocos2dx::C2SModuleParameterMap* instance = (C2SModule_Cocos2dx::C2SModuleParameterMap*) map;
		C2SModule_Cocos2dx::C2SModuleParameterMap* child = (C2SModule_Cocos2dx::C2SModuleParameterMap*) value;
		instance->Set(key, *child);
	}
    /**
     * @brief 맵 파라미터의 키에 배열 파라미터를 설정한다.
     * @param[in] map 맵 파라미터
     * @param[in] key 맵의 키
     * @param[in] value 입력 값
     * @ingroup Unity
     * @author Byeong Yeol Jeon
     */
	void C2SModuleUnityParameterMap_SetArray(void* map, const char* key,
                                             void* value) {
		C2SModule_Cocos2dx::C2SModuleParameterMap* instance = (C2SModule_Cocos2dx::C2SModuleParameterMap*) map;
		C2SModule_Cocos2dx::C2SModuleParameterArray* child = (C2SModule_Cocos2dx::C2SModuleParameterArray*) value;
		instance->Set(key, *child);
	}
    /**
     * @brief 맵 파라미터의 키에 데이터 파라미터를 설정한다.
     * @param[in] map 맵 파라미터
     * @param[in] key 맵의 키
     * @param[in] value 입력 값
     * @ingroup Unity
     * @author Byeong Yeol Jeon
     */
	void C2SModuleUnityParameterMap_SetData(void* map, const char* key,
                                            void* value) {
		C2SModule_Cocos2dx::C2SModuleParameterMap* instance = (C2SModule_Cocos2dx::C2SModuleParameterMap*) map;
		C2SModule_Cocos2dx::C2SModuleParameterData* child = (C2SModule_Cocos2dx::C2SModuleParameterData*) value;
		instance->Set(key, child->data);
	}
}
#endif
namespace C2SModule_Cocos2dx {
    C2SModuleParameterArray::C2SModuleParameterArray() {
#if __OBJC__
        container = new C2SModuleParameterContainer([[NSMutableArray alloc] init]);
#elif __ANDROID__
        JNIEnv* env;
        if (JNI_OK
            != C2SModuleJavaVMInstance->GetEnv((void**) &env,
                                               JNI_VERSION_1_6)) {
                container = NULL;
            } else {
                jmethodID methodId = env->GetMethodID(C2SModuleVectorClass, "<init>",
                                                      "()V");
                jobject tmp = env->NewObject(C2SModuleVectorClass, methodId);
                
                container = new C2SModuleParameterContainer(env->NewGlobalRef(tmp));
                
                env->DeleteLocalRef(tmp);
                
                if (env->ExceptionCheck()) {
                    jthrowable exception = env->ExceptionOccurred();
                    env->ExceptionClear();
                    env->DeleteLocalRef(exception);
                }
            }
#endif
    }
    
    C2SModuleParameterArray::~C2SModuleParameterArray() {
        C2SModuleParameterContainer* real = (C2SModuleParameterContainer*) container;
        delete real;
        container = NULL;
    }
    
    void C2SModuleParameterArray::Add(bool arg) {
        if (NULL == container)
            return;
        
        C2SModuleParameterContainer* real = (C2SModuleParameterContainer*) container;
#if __OBJC__
        [real->container addObject:[NSNumber numberWithBool:arg]];
#elif __ANDROID__
        JNIEnv* env;
        if (JNI_OK
            != C2SModuleJavaVMInstance->GetEnv((void**) &env,
                                               JNI_VERSION_1_6))
            return;
        
        jmethodID methodId = env->GetMethodID(C2SModuleVectorClass, "addElement",
                                              "(Ljava/lang/Object;)V");
        
        jmethodID methodIdCtor = env->GetMethodID(C2SModuleBooleanClass, "<init>",
                                                  "(Z)V");
        
        jobject jValue = env->NewObject(C2SModuleBooleanClass, methodIdCtor, arg);
        
        env->CallVoidMethod(reinterpret_cast<jobject>(real->container), methodId, jValue);
        
        env->DeleteLocalRef(jValue);
        
        if (env->ExceptionCheck()) {
            jthrowable exception = env->ExceptionOccurred();
            env->ExceptionClear();
            env->DeleteLocalRef(exception);
        }
#endif
    }
    
    void C2SModuleParameterArray::Add(int8_t arg) {
        if (NULL == container)
            return;
        
        C2SModuleParameterContainer* real = (C2SModuleParameterContainer*) container;
#if __OBJC__
        [real->container addObject:[NSNumber numberWithChar:arg]];
#elif __ANDROID__
        JNIEnv* env;
        if (JNI_OK
            != C2SModuleJavaVMInstance->GetEnv((void**) &env,
                                               JNI_VERSION_1_6))
            return;
        
        jmethodID methodId = env->GetMethodID(C2SModuleVectorClass, "addElement",
                                              "(Ljava/lang/Object;)V");
        
        jmethodID methodIdCtor = env->GetMethodID(C2SModuleCharClass, "<init>",
                                                  "(C)V");
        
        jobject jValue = env->NewObject(C2SModuleCharClass, methodIdCtor, arg);
        
        env->CallVoidMethod(reinterpret_cast<jobject>(real->container), methodId, jValue);
        
        env->DeleteLocalRef(jValue);
        
        if (env->ExceptionCheck()) {
            jthrowable exception = env->ExceptionOccurred();
            env->ExceptionClear();
            env->DeleteLocalRef(exception);
        }
#endif
    }
    
    void C2SModuleParameterArray::Add(uint8_t arg) {
        if (NULL == container)
            return;
        
        C2SModuleParameterContainer* real = (C2SModuleParameterContainer*) container;
#if __OBJC__
        [real->container addObject:[NSNumber numberWithUnsignedChar:arg]];
#elif __ANDROID__
        JNIEnv* env;
        if (JNI_OK
            != C2SModuleJavaVMInstance->GetEnv((void**) &env,
                                               JNI_VERSION_1_6))
            return;
        
        jmethodID methodId = env->GetMethodID(C2SModuleVectorClass, "addElement",
                                              "(Ljava/lang/Object;)V");
        
        jmethodID methodIdCtor = env->GetMethodID(C2SModuleByteClass, "<init>",
                                                  "(B)V");
        
        jobject jValue = env->NewObject(C2SModuleByteClass, methodIdCtor, arg);
        
        env->CallVoidMethod(reinterpret_cast<jobject>(real->container), methodId, jValue);
        
        env->DeleteLocalRef(jValue);
        
        if (env->ExceptionCheck()) {
            jthrowable exception = env->ExceptionOccurred();
            env->ExceptionClear();
            env->DeleteLocalRef(exception);
        }
#endif
    }
    
    void C2SModuleParameterArray::Add(int16_t arg) {
        if (NULL == container)
            return;
        
        C2SModuleParameterContainer* real = (C2SModuleParameterContainer*) container;
#if __OBJC__
        [real->container addObject:[NSNumber numberWithShort:arg]];
#elif __ANDROID__
        JNIEnv* env;
        if (JNI_OK
            != C2SModuleJavaVMInstance->GetEnv((void**) &env,
                                               JNI_VERSION_1_6))
            return;
        
        jmethodID methodId = env->GetMethodID(C2SModuleVectorClass, "addElement",
                                              "(Ljava/lang/Object;)V");
        
        jmethodID methodIdCtor = env->GetMethodID(C2SModuleShortClass, "<init>",
                                                  "(S)V");
        
        jobject jValue = env->NewObject(C2SModuleShortClass, methodIdCtor, arg);
        
        env->CallVoidMethod(reinterpret_cast<jobject>(real->container), methodId, jValue);
        
        env->DeleteLocalRef(jValue);
        
        if (env->ExceptionCheck()) {
            jthrowable exception = env->ExceptionOccurred();
            env->ExceptionClear();
            env->DeleteLocalRef(exception);
        }
#endif
    }
    
    void C2SModuleParameterArray::Add(uint16_t arg) {
        if (NULL == container)
            return;
        
#if __OBJC__
        C2SModuleParameterContainer* real = (C2SModuleParameterContainer*) container;
        [real->container addObject:[NSNumber numberWithUnsignedShort:arg]];
#elif __ANDROID__
        Add(static_cast<int16_t>(arg));
#endif
    }
    
    void C2SModuleParameterArray::Add(int32_t arg) {
        if (NULL == container)
            return;
        
        C2SModuleParameterContainer* real = (C2SModuleParameterContainer*) container;
#if __OBJC__
        [real->container addObject:[NSNumber numberWithInt:arg]];
#elif __ANDROID__
        JNIEnv* env;
        if (JNI_OK
            != C2SModuleJavaVMInstance->GetEnv((void**) &env,
                                               JNI_VERSION_1_6))
            return;
        
        jmethodID methodId = env->GetMethodID(C2SModuleVectorClass, "addElement",
                                              "(Ljava/lang/Object;)V");
        
        jmethodID methodIdCtor = env->GetMethodID(C2SModuleIntClass, "<init>",
                                                  "(I)V");
        
        jobject jValue = env->NewObject(C2SModuleIntClass, methodIdCtor, arg);
        
        env->CallVoidMethod(reinterpret_cast<jobject>(real->container), methodId, jValue);
        
        env->DeleteLocalRef(jValue);
        
        if (env->ExceptionCheck()) {
            jthrowable exception = env->ExceptionOccurred();
            env->ExceptionClear();
            env->DeleteLocalRef(exception);
        }
#endif
    }
    
    void C2SModuleParameterArray::Add(uint32_t arg) {
        if (NULL == container)
            return;
        
#if __OBJC__
        C2SModuleParameterContainer* real = (C2SModuleParameterContainer*) container;
        [real->container addObject:[NSNumber numberWithUnsignedInt:arg]];
#elif __ANDROID__
        Add(static_cast<int32_t>(arg));
#endif
    }
    
    void C2SModuleParameterArray::Add(int64_t arg) {
        if (NULL == container)
            return;
        
        C2SModuleParameterContainer* real = (C2SModuleParameterContainer*) container;
#if __OBJC__
        [real->container addObject:[NSNumber numberWithLongLong:arg]];
#elif __ANDROID__
        JNIEnv* env;
        if (JNI_OK
            != C2SModuleJavaVMInstance->GetEnv((void**) &env,
                                               JNI_VERSION_1_6))
            return;
        
        jmethodID methodId = env->GetMethodID(C2SModuleVectorClass, "addElement",
                                              "(Ljava/lang/Object;)V");
        
        jmethodID methodIdCtor = env->GetMethodID(C2SModuleLongClass, "<init>",
                                                  "(J)V");
        
        jobject jValue = env->NewObject(C2SModuleLongClass, methodIdCtor, arg);
        
        env->CallVoidMethod(reinterpret_cast<jobject>(real->container), methodId, jValue);
        
        env->DeleteLocalRef(jValue);
        
        if (env->ExceptionCheck()) {
            jthrowable exception = env->ExceptionOccurred();
            env->ExceptionClear();
            env->DeleteLocalRef(exception);
        }
#endif
    }
    
    void C2SModuleParameterArray::Add(uint64_t arg) {
        if (NULL == container)
            return;
        
#if __OBJC__
        C2SModuleParameterContainer* real = (C2SModuleParameterContainer*) container;
        [real->container addObject:[NSNumber numberWithUnsignedLongLong:arg]];
#elif __ANDROID__
        Add(static_cast<int64_t>(arg));
#endif
    }
    
    void C2SModuleParameterArray::Add(const char arg[]) {
        if (NULL == arg || NULL == container)
            return;
        
        C2SModuleParameterContainer* real = (C2SModuleParameterContainer*) container;
#if __OBJC__
        [real->container addObject:[NSString stringWithUTF8String:arg]];
#elif __ANDROID__
        JNIEnv* env;
        if (JNI_OK
            != C2SModuleJavaVMInstance->GetEnv((void**) &env,
                                               JNI_VERSION_1_6))
            return;
        
        jmethodID methodId = env->GetMethodID(C2SModuleVectorClass, "addElement",
                                              "(Ljava/lang/Object;)V");
        
        jobject jValue = env->NewStringUTF(arg);
        
        env->CallVoidMethod(reinterpret_cast<jobject>(real->container), methodId, jValue);
        
        env->DeleteLocalRef(jValue);
        
        if (env->ExceptionCheck()) {
            jthrowable exception = env->ExceptionOccurred();
            env->ExceptionClear();
            env->DeleteLocalRef(exception);
        }
#endif
    }
    
#if __OBJC__
    void C2SModuleParameterArray::Add(id arg) {
        if (NULL == container)
            return;
        
        C2SModuleParameterContainer* real = (C2SModuleParameterContainer*)container;
        
        [real->container addObject:arg];
    }
#elif __ANDROID__
    void C2SModuleParameterArray::Add(jobject arg) {
        JNIEnv* env;
        if (NULL == container
            || JNI_OK
            != C2SModuleJavaVMInstance->GetEnv((void**) &env,
                                               JNI_VERSION_1_6))
            return;
        
        C2SModuleParameterContainer* real = (C2SModuleParameterContainer*)container;
        
        jmethodID methodId = env->GetMethodID(C2SModuleVectorClass, "addElement",
                                              "(Ljava/lang/Object;)V");
        
        env->CallVoidMethod(reinterpret_cast<jobject>(real->container), methodId, arg);
        
        if (env->ExceptionCheck()) {
            jthrowable exception = env->ExceptionOccurred();
            env->ExceptionClear();
            env->DeleteLocalRef(exception);
        }
    }
#endif
    
    void C2SModuleParameterArray::Add(const C2SModuleParameterMap& arg) {
        if (NULL == container)
            return;
        
        C2SModuleParameterContainer* realSrc =
        (C2SModuleParameterContainer*) arg.Get();
#if __OBJC__
        C2SModuleParameterContainer* real = (C2SModuleParameterContainer*) container;
        [real->container addObject:realSrc->container];
#elif __ANDROID__
        if(NULL != realSrc)
        {
            Add((jobject)realSrc->container);
        }
#endif
    }
    
    void* C2SModuleParameterArray::Get() const {
        return container;
    }
}

#if __OBJC__
extern "C" {
    /**
     * @brief 유니티 배열 파라미터를 생성한다.
     
     * @return 생성된 배열 파라미터
     * @author Byeong Yeol Jeon
     * @ingroup Unity
     */
	void* C2SModuleUnityParameterArray_Alloc() {
		return new C2SModule_Cocos2dx::C2SModuleParameterArray();
	}
    /**
     * @brief 생성된 배열 파라미터를 제거한다.
     
     * @param[in] array 배열 파라미터의 주소
     
     * @author Byeong Yeol Jeon
     * @ingroup Unity
     */
	void C2SModuleUnityParameterArray_Release(void* array) {
		C2SModule_Cocos2dx::C2SModuleParameterArray* instance = (C2SModule_Cocos2dx::C2SModuleParameterArray*) array;
		delete instance;
	}
    /**
     * @brief 배열 파라미터에 bool값을 추가한다.
     * @param[in] value 입력 값
     * @author Byeong Yeol Jeon
     * @ingroup Unity
     */
	void C2SModuleUnityParameterArray_AddBool(void* array, bool value) {
		C2SModule_Cocos2dx::C2SModuleParameterArray* instance = (C2SModule_Cocos2dx::C2SModuleParameterArray*) array;
		instance->Add(value);
	}
    /**
     * @brief 배열 파라미터에 8bit signed값을 추가한다.
     * @param[in] value 입력 값
     * @author Byeong Yeol Jeon
     * @ingroup Unity
     */
	void C2SModuleUnityParameterArray_AddChar(void* array, int8_t value) {
		C2SModule_Cocos2dx::C2SModuleParameterArray* instance = (C2SModule_Cocos2dx::C2SModuleParameterArray*) array;
		instance->Add(value);
	}
    /**
     * @brief 배열 파라미터에 8bit unsigned값을 추가한다.
     * @param[in] value 입력 값
     * @author Byeong Yeol Jeon
     * @ingroup Unity
     */
	void C2SModuleUnityParameterArray_AddUChar(void* array, uint8_t value) {
		C2SModule_Cocos2dx::C2SModuleParameterArray* instance = (C2SModule_Cocos2dx::C2SModuleParameterArray*) array;
		instance->Add(value);
	}
    /**
     * @brief 배열 파라미터에 16bit signed값을 추가한다.
     * @param[in] value 입력 값
     * @author Byeong Yeol Jeon
     * @ingroup Unity
     */
    
	void C2SModuleUnityParameterArray_AddShort(void* array, int16_t value) {
		C2SModule_Cocos2dx::C2SModuleParameterArray* instance = (C2SModule_Cocos2dx::C2SModuleParameterArray*) array;
		instance->Add(value);
	}
    /**
     * @brief 배열 파라미터에 16bit unsigned값을 추가한다.
     * @param[in] value 입력 값
     * @author Byeong Yeol Jeon
     * @ingroup Unity
     */
	void C2SModuleUnityParameterArray_AddUShort(void* array, uint16_t value) {
		C2SModule_Cocos2dx::C2SModuleParameterArray* instance = (C2SModule_Cocos2dx::C2SModuleParameterArray*) array;
		instance->Add(value);
	}
    /**
     * @brief 배열 파라미터에 32bit signed값을 추가한다.
     * @param[in] value 입력 값
     * @author Byeong Yeol Jeon
     * @ingroup Unity
     */
	void C2SModuleUnityParameterArray_AddInt(void* array, int32_t value) {
		C2SModule_Cocos2dx::C2SModuleParameterArray* instance = (C2SModule_Cocos2dx::C2SModuleParameterArray*) array;
		instance->Add(value);
	}
    /**
     * @brief 배열 파라미터에 32bit unsigned값을 추가한다.
     * @param[in] value 입력 값
     * @author Byeong Yeol Jeon
     * @ingroup Unity
     */
	void C2SModuleUnityParameterArray_AddUInt(void* array, uint32_t value) {
		C2SModule_Cocos2dx::C2SModuleParameterArray* instance = (C2SModule_Cocos2dx::C2SModuleParameterArray*) array;
		instance->Add(value);
	}
    /**
     * @brief 배열 파라미터에 64bit signed값을 추가한다.
     * @param[in] value 입력 값
     * @author Byeong Yeol Jeon
     * @ingroup Unity
     */
	void C2SModuleUnityParameterArray_AddLong(void* array, int64_t value) {
		C2SModule_Cocos2dx::C2SModuleParameterArray* instance = (C2SModule_Cocos2dx::C2SModuleParameterArray*) array;
		instance->Add(value);
	}
    /**
     * @brief 배열 파라미터에 64bit unsigned값을 추가한다.
     * @param[in] value 입력 값
     * @author Byeong Yeol Jeon
     * @ingroup Unity
     */
	void C2SModuleUnityParameterArray_AddULong(void* array, uint64_t value) {
		C2SModule_Cocos2dx::C2SModuleParameterArray* instance = (C2SModule_Cocos2dx::C2SModuleParameterArray*) array;
		instance->Add(value);
	}
    /**
     * @brief 배열 파라미터에 문자열 값을 추가한다.
     * @param[in] value 입력 값
     * @author Byeong Yeol Jeon
     * @ingroup Unity
     */
	void C2SModuleUnityParameterArray_AddString(void* array, const char* value) {
		C2SModule_Cocos2dx::C2SModuleParameterArray* instance = (C2SModule_Cocos2dx::C2SModuleParameterArray*) array;
		instance->Add(value);
	}
    /**
     * @brief 배열 파라미터에 맵 파라미터를 추가한다.
     * @param[in] value 입력 값
     * @author Byeong Yeol Jeon
     * @ingroup Unity
     */
	void C2SModuleUnityParameterArray_AddMap(void* array, void* value) {
		C2SModule_Cocos2dx::C2SModuleParameterArray* instance = (C2SModule_Cocos2dx::C2SModuleParameterArray*) array;
		C2SModule_Cocos2dx::C2SModuleParameterMap* child = (C2SModule_Cocos2dx::C2SModuleParameterMap*) value;
		instance->Add(*child);
	}
    /**
     * @brief 배열 파라미터에 데이터 파라미터를 추가한다.
     * @param[in] value 입력 값
     * @author Byeong Yeol Jeon
     * @ingroup Unity
     */
	void C2SModuleUnityParameterArray_AddData(void* map, void* value) {
		C2SModule_Cocos2dx::C2SModuleParameterArray* instance = (C2SModule_Cocos2dx::C2SModuleParameterArray*) map;
		C2SModule_Cocos2dx::C2SModuleParameterData* child = (C2SModule_Cocos2dx::C2SModuleParameterData*) value;
		instance->Add(child->data);
	}
}
#endif

//====================================================================================================================

#if __OBJC__
#define FUNCTION_BODY(a, b) \
    NSString* resultArg = (NULL != pstrJson ? [NSString stringWithUTF8String:pstrJson] : nil); \
    NSError* error; \
    BOOL ret; \
\
    if(NULL == completionHandler) \
    { \
        ret = [a b:resultArg error:&error]; \
    } \
    else \
    { \
        ret = [a b:resultArg error:&error completionHandler:^(NSDictionary* arg, NSError* completeError) { \
            NSString* result; \
            if(nil == arg) \
            { \
                result = nil; \
            } \
            else \
            { \
                NSError* sereializationError; \
                NSData* data = [NSJSONSerialization dataWithJSONObject:arg options:kNilOptions error:&sereializationError]; \
\
                if(nil != sereializationError) \
                { \
                    if(nil == completeError) \
                    { \
                        completeError = [NSError errorWithDomain:sereializationError.domain code:C2SModuleErrorCode_FailConvertToJsonFormat userInfo:sereializationError.userInfo]; \
                    } \
                    result = nil; \
                } \
                else \
                { \
                    result = [[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding]; \
                } \
            } \
\
            C2SModule_Cocos2dx::C2SModuleError resultError; \
\
            if(nil != completeError) \
            { \
                resultError.reset(new C2SModule_Cocos2dx::C2SModuleErrorDescImpl(static_cast<C2SModuleErrorCode>(completeError.code), [completeError.domain UTF8String])); \
            } \
            completionHandler->onComplete([result UTF8String], resultError.get()); \
        }]; \
    } \
\
    if(NO == ret) \
    { \
        if(NULL != pkError && nil != error) \
        { \
            (*pkError).reset(new C2SModule_Cocos2dx::C2SModuleErrorDescImpl(static_cast<C2SModuleErrorCode>(error.code), [error.domain UTF8String])); \
        } \
        return false; \
    }

#define FUNCTION_BODY_ARG(a, b) \
    NSError* error; \
    BOOL ret; \
\
    if(NULL == completionHandler) \
    { \
        ret = [a b:arg ? (__bridge NSMutableDictionary*)arg->Get() : nil  error:&error]; \
    } \
    else \
    { \
        ret = [a b:arg ? (__bridge NSMutableDictionary*)arg->Get() : nil  error:&error completionHandler:^(NSDictionary* arg2, NSError* completeError) { \
            NSString* result; \
            if(nil == arg2) \
            { \
                result = nil; \
            } \
            else \
            { \
                NSError* sereializationError; \
                NSData* data = [NSJSONSerialization dataWithJSONObject:arg2 options:kNilOptions error:&sereializationError]; \
\
                if(nil != sereializationError) \
                { \
                    if(nil == completeError) \
                    { \
                        completeError = [NSError errorWithDomain:sereializationError.domain code:C2SModuleErrorCode_FailConvertToJsonFormat userInfo:sereializationError.userInfo]; \
                    } \
                    result = nil; \
                } \
                else \
                { \
                    result = [[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding]; \
                } \
            } \
\
            C2SModule_Cocos2dx::C2SModuleError resultError; \
\
            if(nil != completeError) \
            { \
                resultError.reset(new C2SModule_Cocos2dx::C2SModuleErrorDescImpl(static_cast<C2SModuleErrorCode>(completeError.code), [completeError.domain UTF8String])); \
            } \
            completionHandler->onComplete([result UTF8String], resultError.get()); \
        }]; \
    } \
\
    if(NO == ret) \
    { \
        if(NULL != pkError && nil != error) \
        { \
            (*pkError).reset(new C2SModule_Cocos2dx::C2SModuleErrorDescImpl(static_cast<C2SModuleErrorCode>(error.code), [error.domain UTF8String])); \
        } \
        return false; \
    }
#elif __ANDROID__
#define FUNCTION_BODY(a, b) \
    JNIEnv* env; \
\
    if (JNI_OK != C2SModuleJavaVMInstance->GetEnv((void**) &env, JNI_VERSION_1_6)) { \
        return false; \
    } \
\
    jstring arg = pstrJson ? env->NewStringUTF(pstrJson) : NULL; \
    jobject ret; \
    if(NULL != completionHandler) { \
        jmethodID init = env->GetMethodID(C2SModuleCompletionHandlerJNIClass, "<init>", "(J)V"); \
        jobject jHandlerInstance = env->NewObject(C2SModuleCompletionHandlerJNIClass, init, (jlong)completionHandler); \
        jobject jHandler = env->NewGlobalRef(jHandlerInstance); \
        env->DeleteLocalRef(jHandlerInstance); \
        jmethodID methodId = env->GetStaticMethodID(a##Class, #b, "(Ljava/lang/String;Lcom/com2us/module/C2SModuleCompletionHandler;)Lcom/com2us/module/C2SModuleError;"); \
        ret = env->CallStaticObjectMethod(a##Class, methodId, arg, jHandler); \
        env->DeleteGlobalRef(jHandler); \
    } \
    else \
    { \
        jmethodID methodId = env->GetStaticMethodID(a##Class, #b, "(Ljava/lang/String;)Lcom/com2us/module/C2SModuleError;"); \
        ret = env->CallStaticObjectMethod(a##Class, methodId, arg); \
    } \
\
    if (NULL != arg) { \
        env->DeleteLocalRef(arg); \
    } \
\
    if (env->ExceptionCheck()) { \
        jthrowable exception = env->ExceptionOccurred(); \
        env->ExceptionDescribe(); \
        env->ExceptionClear(); \
        env->DeleteLocalRef(exception); \
        if(NULL != pkError) { \
            (*pkError).reset(new C2SModule_Cocos2dx::C2SModuleErrorDescImpl(C2SModuleErrorCode_Unknown, "unknown")); \
        } \
        return false; \
    } \
\
    if (NULL != ret) { \
        jmethodID methodId_getCode = env->GetMethodID(C2SModuleErrorClass, "getCodeToInteger", "()I"); \
        C2SModuleErrorCode code = (C2SModuleErrorCode) env->CallIntMethod(ret, methodId_getCode); \
\
        if(NULL != pkError) { \
            jmethodID methodId_getMessage = env->GetMethodID(C2SModuleErrorClass, "getMessage", "()Ljava/lang/String;"); \
            jstring message = (jstring) env->CallObjectMethod(ret, methodId_getMessage); \
            const char* szMessage = env->GetStringUTFChars(message, NULL); \
            (*pkError).reset(new C2SModule_Cocos2dx::C2SModuleErrorDescImpl(code, szMessage)); \
            env->ReleaseStringUTFChars(message, szMessage); \
            env->DeleteLocalRef((jobject) message); \
        } \
\
        env->DeleteLocalRef(ret); \
        return C2SModuleErrorCode_Success == code; \
    }

#define FUNCTION_BODY_ARG(a, b)	\
    JNIEnv* env; \
\
    if (JNI_OK != C2SModuleJavaVMInstance->GetEnv((void**) &env, JNI_VERSION_1_6)) { \
        return false; \
    } \
\
    jobject ret; \
    if(NULL != completionHandler) { \
        jmethodID init = env->GetMethodID(C2SModuleCompletionHandlerJNIClass, "<init>", "(J)V"); \
        jobject jHandlerInstance = env->NewObject(C2SModuleCompletionHandlerJNIClass, init, (jlong)completionHandler); \
        jobject jHandler = env->NewGlobalRef(jHandlerInstance); \
        env->DeleteLocalRef(jHandlerInstance); \
        jmethodID methodId = env->GetStaticMethodID(a##Class, #b, "(Lcom/com2us/module/C2SModuleArgument;Lcom/com2us/module/C2SModuleCompletionHandler;)Lcom/com2us/module/C2SModuleError;"); \
        ret = env->CallStaticObjectMethod(a##Class, methodId, arg ? reinterpret_cast<jobject>(arg->Get()) : NULL, jHandler); \
        env->DeleteGlobalRef(jHandler); \
    } \
    else \
    { \
        jmethodID methodId = env->GetStaticMethodID(a##Class, #b, "(Lcom/com2us/module/C2SModuleArgument;)Lcom/com2us/module/C2SModuleError;"); \
        ret = env->CallStaticObjectMethod(a##Class, methodId, arg ? reinterpret_cast<jobject>(arg->Get()) : NULL); \
    } \
\
    if (env->ExceptionCheck()) { \
        jthrowable exception = env->ExceptionOccurred(); \
        env->ExceptionDescribe(); \
        env->ExceptionClear(); \
        env->DeleteLocalRef(exception); \
        if(NULL != pkError) { \
            (*pkError).reset(new C2SModule_Cocos2dx::C2SModuleErrorDescImpl(C2SModuleErrorCode_Unknown, "unknown")); \
        } \
        return false; \
    } \
\
    if (NULL != ret) { \
        jmethodID methodId_getCode = env->GetMethodID(C2SModuleErrorClass, "getCodeToInteger", "()I"); \
        C2SModuleErrorCode code = (C2SModuleErrorCode) env->CallIntMethod(ret, methodId_getCode); \
\
        if(NULL != pkError) { \
            jmethodID methodId_getMessage = env->GetMethodID(C2SModuleErrorClass, "getMessage", "()Ljava/lang/String;"); \
            jstring message = (jstring) env->CallObjectMethod(ret, methodId_getMessage); \
            const char* szMessage = env->GetStringUTFChars(message, NULL); \
            (*pkError).reset(new C2SModule_Cocos2dx::C2SModuleErrorDescImpl(code, szMessage)); \
            env->ReleaseStringUTFChars(message, szMessage); \
            env->DeleteLocalRef((jobject) message); \
        } \
\
        env->DeleteLocalRef(ret); \
        return C2SModuleErrorCode_Success == code; \
    }
extern "C" {
    /**
     * @brief NDK를 통해서 java 커스텀 핸들러를 등록했을때 호출되는 커스텀 핸들러 실행함수
     * @param[in] env jni env
     * @param[in] object jni class
     * @param[in] handler 호출된 커스텀 핸들러 인스턴스
     * @param[in] jsonArg 결과 json 문자열 인자들
     * @param[in] code 발생한 에러의 코드
     * @param[in] message 발생한 에러의 메시지
     * @author Byeong Yeol Jeon
     * @ingroup Android
     */
	void C2SModuleCompletionHandlerJNI_JNIComplete(JNIEnv* env, jobject object, jlong handler, jstring jsonArg, jint code, jstring message)
	{
		if (0 == handler)
		{
			return;
		}

		jobject newObject = env->NewGlobalRef(object);

		const char* szArg;
		const char* szMessage;
        
		if (NULL != jsonArg) {
			szArg = env->GetStringUTFChars(jsonArg, NULL);
		} else {
			szArg = NULL;
		}
        
		if (NULL != message) {
			szMessage = env->GetStringUTFChars(message, NULL);
		} else {
			szMessage = NULL;
		}
        
		C2SModule_Cocos2dx::C2SModuleCompletionHandler* cHandler = (C2SModule_Cocos2dx::C2SModuleCompletionHandler*)handler;
		C2SModule_Cocos2dx::C2SModuleError onCompleteError;
		if(code != C2SModuleErrorCode_Success)
		{
			onCompleteError.reset(new C2SModule_Cocos2dx::C2SModuleErrorDescImpl((C2SModuleErrorCode) code, szMessage));
		}
        
		cHandler->onComplete(szArg, onCompleteError.get());
        
		if (NULL != jsonArg) {
			env->ReleaseStringUTFChars(jsonArg, szArg);
		}
        
		if (NULL != message) {
			env->ReleaseStringUTFChars(message, szMessage);
		}

		env->DeleteGlobalRef(newObject);
//		env->DeleteGlobalRef(object);
	}
}
#endif // __OBJC__
bool C2SModuleSocial_Initialize(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError,
                                C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler) {
	FUNCTION_BODY(C2SModuleSocial, Initialize);
	return true;
}
bool C2SModuleSocial_Initialize(const C2SModule_Cocos2dx::C2SModuleParameter* arg,
                                C2SModule_Cocos2dx::C2SModuleError* pkError,
                                C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler) {
	FUNCTION_BODY_ARG(C2SModuleSocial, Initialize);
	return ret;
}
bool C2SModuleSocial_Show(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError,
                          C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler) {
	FUNCTION_BODY(C2SModuleSocial, Show);
	return true;
}
bool C2SModuleSocial_Show(const C2SModule_Cocos2dx::C2SModuleParameter* arg,
                          C2SModule_Cocos2dx::C2SModuleError* pkError,
                          C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler) {
	FUNCTION_BODY_ARG(C2SModuleSocial, Show);
	return true;
}
bool C2SModuleSocial_Info(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError,
                          C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler) {
	FUNCTION_BODY(C2SModuleSocial, Info);
	return true;
}
bool C2SModuleSocial_Info(const C2SModule_Cocos2dx::C2SModuleParameter* arg,
                          C2SModule_Cocos2dx::C2SModuleError* pkError,
                          C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler) {
	FUNCTION_BODY_ARG(C2SModuleSocial, Info);
	return true;
}
bool C2SModuleSocial_SetGameInfo(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError,
                          C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler) {
	FUNCTION_BODY(C2SModuleSocial, SetGameInfo);
	return true;
}
bool C2SModuleSocial_SetGameInfo(const C2SModule_Cocos2dx::C2SModuleParameter* arg,
                          C2SModule_Cocos2dx::C2SModuleError* pkError,
                          C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler) {
	FUNCTION_BODY_ARG(C2SModuleSocial, SetGameInfo);
	return true;
}
bool C2SModuleSocial_UpdateGameLanguage(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError,
                                        C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler) {
    FUNCTION_BODY(C2SModuleSocial, UpdateGameLanguage);
    return true;
}
bool C2SModuleSocial_UpdateGameLanguage(const C2SModule_Cocos2dx::C2SModuleParameter* arg,
                                        C2SModule_Cocos2dx::C2SModuleError* pkError,
                                        C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler) {
    FUNCTION_BODY_ARG(C2SModuleSocial, UpdateGameLanguage);
    return true;
}
bool C2SModuleSocial_UpdateServerId(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError,
                                        C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler) {
    FUNCTION_BODY(C2SModuleSocial, UpdateServerId);
    return true;
}
bool C2SModuleSocial_UpdateServerId(const C2SModule_Cocos2dx::C2SModuleParameter* arg,
                                        C2SModule_Cocos2dx::C2SModuleError* pkError,
                                        C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler) {
    FUNCTION_BODY_ARG(C2SModuleSocial, UpdateServerId);
    return true;
}

bool C2SModuleSocial_SendData(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError,
                                 C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler) {
    FUNCTION_BODY(C2SModuleSocial, SendData);
    return true;
}
bool C2SModuleSocial_SendData(const C2SModule_Cocos2dx::C2SModuleParameter* arg,
                                 C2SModule_Cocos2dx::C2SModuleError* pkError,
                                 C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler) {
    FUNCTION_BODY_ARG(C2SModuleSocial, SendData);
    return true;
}

char strLoginTypeBuff[128];
const char* GetLoginType() {
#if __OBJC__
    NSString* loginType = [C2SModuleSns GetLoginType];
    return [loginType UTF8String];
#elif __ANDROID__
	JNIEnv* env;

    if (JNI_OK != C2SModuleJavaVMInstance->GetEnv((void**) &env, JNI_VERSION_1_6)) {
        return NULL;
    }

	jmethodID methodId = env->GetStaticMethodID(C2SModuleSnsClass, "GetLoginType", "()Ljava/lang/String;");
    jstring jstr = (jstring)env->CallStaticObjectMethod(C2SModuleSnsClass, methodId);

	memset(strLoginTypeBuff, '\0',sizeof(strLoginTypeBuff));
	const char *str = env->GetStringUTFChars(jstr, 0);
	strcpy(strLoginTypeBuff, str);
	env->ReleaseStringUTFChars(jstr, str);
	return strLoginTypeBuff;
#endif
}
char strCountryBuff[128];
const char* GetHiveCountry() {
#if __OBJC__
    NSString* country = [C2SModuleSocial GetHiveCountry];
    return [country UTF8String];
#elif __ANDROID__
	JNIEnv* env;

    if (JNI_OK != C2SModuleJavaVMInstance->GetEnv((void**) &env, JNI_VERSION_1_6)) {
        return NULL;
    }

	jmethodID methodId = env->GetStaticMethodID(C2SModuleSocialClass, "GetHiveCountry", "()Ljava/lang/String;");
    jstring jstr = (jstring)env->CallStaticObjectMethod(C2SModuleSocialClass, methodId);

	memset(strCountryBuff, '\0', sizeof(strCountryBuff));
	const char *str = env->GetStringUTFChars(jstr, 0);
	strcpy( strCountryBuff, str );
	env->ReleaseStringUTFChars(jstr, str);

	return strCountryBuff;
#endif
}

char strChannelBuff[128];
const char* GetChannel() {
#if __OBJC__
    NSString* channel = [C2SModuleSocial GetChannel];
    return [channel UTF8String];
#elif __ANDROID__
    JNIEnv* env;

    if (JNI_OK != C2SModuleJavaVMInstance->GetEnv((void**) &env, JNI_VERSION_1_6)) {
        return NULL;
    }

    jmethodID methodId = env->GetStaticMethodID(C2SModuleSocialClass, "GetChannel", "()Ljava/lang/String;");
    jstring jstr = (jstring)env->CallStaticObjectMethod(C2SModuleSocialClass, methodId);

    memset(strChannelBuff, '\0', sizeof(strChannelBuff));
    const char *str = env->GetStringUTFChars(jstr, 0);
    strcpy( strChannelBuff, str );
    env->ReleaseStringUTFChars(jstr, str);

    return strChannelBuff;
#endif
}

void SetChannel(const char* channel) {
#if __OBJC__
    [C2SModuleSocial SetChannel:channel];
#elif __ANDROID__
    JNIEnv* env;

    if (JNI_OK != C2SModuleJavaVMInstance->GetEnv((void**) &env, JNI_VERSION_1_6)) {
        return;
    }

    jstring jchannel = env->NewStringUTF(channel);

	jmethodID methodId = env->GetStaticMethodID(C2SModuleSocialClass, "SetChannel", "(Ljava/lang/String;)V");
    env->CallStaticVoidMethod(C2SModuleSocialClass, methodId, jchannel);

    env->DeleteLocalRef(jchannel);

    if (env->ExceptionCheck()) {
        jthrowable exception = env->ExceptionOccurred();
        env->ExceptionClear();
        env->DeleteLocalRef(exception);
    }
#endif
}

bool C2SModuleSns_Session(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError,
                          C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler) {
	FUNCTION_BODY(C2SModuleSns, Session);
	return true;
}
bool C2SModuleSns_Session(const C2SModule_Cocos2dx::C2SModuleParameter* arg,
                          C2SModule_Cocos2dx::C2SModuleError* pkError,
                          C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler) {
	FUNCTION_BODY_ARG(C2SModuleSns, Session);
	return true;
}
bool C2SModuleSns_Login(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError,
                        C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler) {
	FUNCTION_BODY(C2SModuleSns, Login);
	return true;
}
bool C2SModuleSns_Login(const C2SModule_Cocos2dx::C2SModuleParameter* arg, C2SModule_Cocos2dx::C2SModuleError* pkError,
                        C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler) {
	FUNCTION_BODY_ARG(C2SModuleSns, Login);
	return true;
}
bool C2SModuleSns_Logout(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError,
                         C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler) {
	FUNCTION_BODY(C2SModuleSns, Logout);
	return true;
}
bool C2SModuleSns_Logout(const C2SModule_Cocos2dx::C2SModuleParameter* arg, C2SModule_Cocos2dx::C2SModuleError* pkError,
                         C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler) {
	FUNCTION_BODY_ARG(C2SModuleSns, Logout);
	return true;
}
bool C2SModuleSns_Unregister(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError,
                             C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler) {
	FUNCTION_BODY(C2SModuleSns, Unregister);
	return true;
}
bool C2SModuleSns_Unregister(const C2SModule_Cocos2dx::C2SModuleParameter* arg,
                             C2SModule_Cocos2dx::C2SModuleError* pkError,
                             C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler) {
	FUNCTION_BODY_ARG(C2SModuleSns, Unregister);
	return true;
}
bool C2SModuleSns_User(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError,
                       C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler) {
	FUNCTION_BODY(C2SModuleSns, User);
	return true;
}
bool C2SModuleSns_User(const C2SModule_Cocos2dx::C2SModuleParameter* arg, C2SModule_Cocos2dx::C2SModuleError* pkError,
                       C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler) {
	FUNCTION_BODY_ARG(C2SModuleSns, User);
	return true;
}
bool C2SModuleSns_Friends(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError,
                          C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler) {
	FUNCTION_BODY(C2SModuleSns, Friends);
	return true;
}
bool C2SModuleSns_Friends(const C2SModule_Cocos2dx::C2SModuleParameter* arg,
                          C2SModule_Cocos2dx::C2SModuleError* pkError,
                          C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler) {
	FUNCTION_BODY_ARG(C2SModuleSns, Friends);
	return true;
}
bool C2SModuleSns_Message(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError,
                          C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler) {
	FUNCTION_BODY(C2SModuleSns, Message);
	return true;
}
bool C2SModuleSns_Message(const C2SModule_Cocos2dx::C2SModuleParameter* arg,
                          C2SModule_Cocos2dx::C2SModuleError* pkError,
                          C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler) {
	FUNCTION_BODY_ARG(C2SModuleSns, Message);
	return true;
}
bool C2SModuleSns_Post(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError,
                       C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler) {
	FUNCTION_BODY(C2SModuleSns, Post);
	return true;
}
bool C2SModuleSns_Post(const C2SModule_Cocos2dx::C2SModuleParameter* arg, C2SModule_Cocos2dx::C2SModuleError* pkError,
                       C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler) {
	FUNCTION_BODY_ARG(C2SModuleSns, Post);
	return true;
}
bool C2SModuleSns_Show(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError,
                       C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler) {
	FUNCTION_BODY(C2SModuleSns, Show);
	return true;
}
bool C2SModuleSns_Show(const C2SModule_Cocos2dx::C2SModuleParameter* arg, C2SModule_Cocos2dx::C2SModuleError* pkError,
                       C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler) {
	FUNCTION_BODY_ARG(C2SModuleSns, Show);
	return true;
}
bool C2SModuleSns_SetGuestUserInfo(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError,
                                   C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler) {
    FUNCTION_BODY(C2SModuleSns, SetGuestUserInfo);
    return true;
}
bool C2SModuleSns_SetGuestUserInfo(const C2SModule_Cocos2dx::C2SModuleParameter* arg, C2SModule_Cocos2dx::C2SModuleError* pkError,
                                   C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler) {
    FUNCTION_BODY_ARG(C2SModuleSns, SetGuestUserInfo);
    return true;
}
bool C2SModuleSns_UserDelete(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError,
                         C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler) {
    FUNCTION_BODY(C2SModuleSns, UserDelete);
    return true;
}
bool C2SModuleSns_UserDelete(const C2SModule_Cocos2dx::C2SModuleParameter* arg, C2SModule_Cocos2dx::C2SModuleError* pkError,
                         C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler) {
    FUNCTION_BODY_ARG(C2SModuleSns, UserDelete);
    return true;
}

bool C2SModulePush_Register(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError,
                            C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler) {
	FUNCTION_BODY(C2SModulePush, Register);
	return true;
}
bool C2SModulePush_Register(const C2SModule_Cocos2dx::C2SModuleParameter* arg,
                            C2SModule_Cocos2dx::C2SModuleError* pkError,
                            C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler) {
	FUNCTION_BODY_ARG(C2SModulePush, Register);
	return true;
}
bool C2SModulePush_Unregister(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError,
                              C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler) {
	FUNCTION_BODY(C2SModulePush, Unregister);
	return true;
}
bool C2SModulePush_Unregister(const C2SModule_Cocos2dx::C2SModuleParameter* arg,
                              C2SModule_Cocos2dx::C2SModuleError* pkError,
                              C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler) {
	FUNCTION_BODY_ARG(C2SModulePush, Unregister);
	return true;
}
bool C2SModulePush_Description(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError,
                               C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler) {
	FUNCTION_BODY(C2SModulePush, Description);
	return true;
}
bool C2SModulePush_Description(const C2SModule_Cocos2dx::C2SModuleParameter* arg,
                               C2SModule_Cocos2dx::C2SModuleError* pkError,
                               C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler) {
	FUNCTION_BODY_ARG(C2SModulePush, Description);
	return true;
}

bool C2SModulePush_RequestPermission(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError,
                                     C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler) {
    FUNCTION_BODY(C2SModulePush, RequestPermission);
    return true;
}

bool C2SModulePush_RequestPermission(const C2SModule_Cocos2dx::C2SModuleParameter* arg,
                                     C2SModule_Cocos2dx::C2SModuleError* pkError,
                                     C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler) {
    FUNCTION_BODY_ARG(C2SModulePush, RequestPermission);
    return true;
}

bool C2SModulePromotion_Show(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError,
                             C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler) {
	FUNCTION_BODY(C2SModulePromotion, Show);
	return true;
}
bool C2SModulePromotion_Show(const C2SModule_Cocos2dx::C2SModuleParameter* arg,
                             C2SModule_Cocos2dx::C2SModuleError* pkError,
                             C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler) {
	FUNCTION_BODY_ARG(C2SModulePromotion, Show);
	return true;
}

bool C2SModulePromotion_Info(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError,
                             C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler) {
    FUNCTION_BODY(C2SModulePromotion, Info);
    return true;
}
bool C2SModulePromotion_Info(const C2SModule_Cocos2dx::C2SModuleParameter* arg,
                             C2SModule_Cocos2dx::C2SModuleError* pkError,
                             C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler) {
    FUNCTION_BODY_ARG(C2SModulePromotion, Info);
    return true;
}

bool C2SModulePromotion_GetBannerInfo(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError,
									  C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler) {
	FUNCTION_BODY(C2SModulePromotion, GetBannerInfo);
	return true;
}
bool C2SModulePromotion_GetBannerInfo(const C2SModule_Cocos2dx::C2SModuleParameter* arg,
									  C2SModule_Cocos2dx::C2SModuleError* pkError,
									  C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler) {
	FUNCTION_BODY_ARG(C2SModulePromotion, GetBannerInfo);
	return true;
}
bool C2SModulePromotion_Acquisition(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError,
                             C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler) {
    FUNCTION_BODY(C2SModulePromotion, Acquisition);
    return true;
}
bool C2SModulePromotion_Acquisition(const C2SModule_Cocos2dx::C2SModuleParameter* arg,
                             C2SModule_Cocos2dx::C2SModuleError* pkError,
                             C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler) {
    FUNCTION_BODY_ARG(C2SModulePromotion, Acquisition);
    return true;
}

bool C2SModuleEngagement_ProcessScheme(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler)
{
    FUNCTION_BODY(C2SModuleEngagement, ProcessScheme);
    return true;
}

bool C2SModuleEngagement_ProcessScheme(const C2SModule_Cocos2dx::C2SModuleParameter* arg,
                                       C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler)
{
    FUNCTION_BODY_ARG(C2SModuleEngagement, ProcessScheme);
    return true;
}

// Deprecated in HIVE SDK 1.11.0
//bool C2SModuleEngagement_GetEvents(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler)
//{
//    FUNCTION_BODY(C2SModuleEngagement, GetEvents);
//    return true;
//}

// Deprecated in HIVE SDK 1.11.0
//bool C2SModuleEngagement_GetEvents(const C2SModule_Cocos2dx::C2SModuleParameter* arg,
//                                   C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler)
//{
//    FUNCTION_BODY_ARG(C2SModuleEngagement, GetEvents);
//    return true;
//}

bool C2SModuleEngagement_ConsumeCoupon(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler)
{
    FUNCTION_BODY(C2SModuleEngagement, ConsumeCoupon);
    return true;
}

bool C2SModuleEngagement_ConsumeCoupon(const C2SModule_Cocos2dx::C2SModuleParameter* arg,
                                           C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler)
{
    FUNCTION_BODY_ARG(C2SModuleEngagement, ConsumeCoupon);
    return true;
}

bool C2SModuleEngagement_SetReady(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler)
{
    FUNCTION_BODY(C2SModuleEngagement, SetReady);
    return true;
}

bool C2SModuleEngagement_SetReady(const C2SModule_Cocos2dx::C2SModuleParameter* arg,
                                       C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler)
{
    FUNCTION_BODY_ARG(C2SModuleEngagement, SetReady);
    return true;
}
bool C2SModuleTracker_SendEvent(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError,
        					C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler) {
	FUNCTION_BODY(C2SModuleTracker, SendEvent);
    return true;
}

bool C2SModuleTracker_SendEvent(const C2SModule_Cocos2dx::C2SModuleParameter* arg,
                                C2SModule_Cocos2dx::C2SModuleError* pkError,
                                C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler){
    FUNCTION_BODY_ARG(C2SModuleTracker, SendEvent);
    return true;
}

bool C2SModuleTracker_SetEnable(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError,
        					C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler) {
	FUNCTION_BODY(C2SModuleTracker, SetEnable);
    return true;
}

bool C2SModuleTracker_SetEnable(const C2SModule_Cocos2dx::C2SModuleParameter* arg,
                                C2SModule_Cocos2dx::C2SModuleError* pkError, C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler){
    FUNCTION_BODY_ARG(C2SModuleTracker, SetEnable);
    return true;
}

bool C2SModuleHivePromotion_Show(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError,
                             C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler) {
    FUNCTION_BODY(C2SModuleHivePromotion, Show);
    return true;
}
bool C2SModuleHivePromotion_Show(const C2SModule_Cocos2dx::C2SModuleParameter* arg,
                             C2SModule_Cocos2dx::C2SModuleError* pkError,
                             C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler) {
    FUNCTION_BODY_ARG(C2SModuleHivePromotion, Show);
    return true;
}

bool C2SModuleHivePromotion_Info(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError,
                             C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler) {
    FUNCTION_BODY(C2SModuleHivePromotion, Info);
    return true;
}
bool C2SModuleHivePromotion_Info(const C2SModule_Cocos2dx::C2SModuleParameter* arg,
                             C2SModule_Cocos2dx::C2SModuleError* pkError,
                             C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler) {
    FUNCTION_BODY_ARG(C2SModuleHivePromotion, Info);
    return true;
}
bool C2SModuleHivePromotion_GetBannerInfo(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError,
										  C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler) {
	FUNCTION_BODY(C2SModuleHivePromotion, GetBannerInfo);
	return true;
}
bool C2SModuleHivePromotion_GetBannerInfo(const C2SModule_Cocos2dx::C2SModuleParameter* arg,
										  C2SModule_Cocos2dx::C2SModuleError* pkError,
										  C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler) {
	FUNCTION_BODY_ARG(C2SModuleHivePromotion, GetBannerInfo);
	return true;
}
bool C2SModuleHivePromotion_GetOfferwallState(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError,
                             C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler) {
    FUNCTION_BODY(C2SModuleHivePromotion, GetOfferwallState);
    return true;
}
bool C2SModuleHivePromotion_GetOfferwallState(const C2SModule_Cocos2dx::C2SModuleParameter* arg,
                             C2SModule_Cocos2dx::C2SModuleError* pkError,
                             C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler) {
    FUNCTION_BODY_ARG(C2SModuleHivePromotion, GetOfferwallState);
    return true;
}

bool C2SModuleHivePromotion_SetAdditionalInfo(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError,
                             C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler) {
    FUNCTION_BODY(C2SModuleHivePromotion, SetAdditionalInfo);
    return true;
}
bool C2SModuleHivePromotion_SetAdditionalInfo(const C2SModule_Cocos2dx::C2SModuleParameter* arg,
                             C2SModule_Cocos2dx::C2SModuleError* pkError,
                             C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler) {
    FUNCTION_BODY_ARG(C2SModuleHivePromotion, SetAdditionalInfo);
    return true;
}


bool C2SModuleHivePromotion_UpdatePromotionData(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError,
                                              C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler) {
    FUNCTION_BODY(C2SModuleHivePromotion, UpdatePromotionData);
    return true;
}
bool C2SModuleHivePromotion_UpdatePromotionData(const C2SModule_Cocos2dx::C2SModuleParameter* arg,
                                              C2SModule_Cocos2dx::C2SModuleError* pkError,
                                              C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler) {
    FUNCTION_BODY_ARG(C2SModuleHivePromotion, UpdatePromotionData);
    return true;
}


int GetOfferwallState() {
#if __OBJC__
    //TODO: 어떻게 처리할지 나중에 물어보기. IOS는 오퍼월이 없으니까
    return 0;
#elif __ANDROID__
	JNIEnv* env;

    if (JNI_OK != C2SModuleJavaVMInstance->GetEnv((void**) &env, JNI_VERSION_1_6)) {
        return -15;
    }

	jmethodID methodId = env->GetStaticMethodID(C2SModulePromotionClass, "GetOfferwallState", "()I");
	jint offerstate = env->CallStaticIntMethod(C2SModulePromotionClass, methodId);

	return offerstate;
#endif
    
}

/**
* InappV4 Interface
*/
bool C2SModuleInappV4_MarketConnect(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError,
                                    C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler) {
    FUNCTION_BODY(C2SModuleInappV4, MarketConnect);
    return true;
}
bool C2SModuleInappV4_MarketConnect(const C2SModule_Cocos2dx::C2SModuleParameter* arg,
                                    C2SModule_Cocos2dx::C2SModuleError* pkError,
                                    C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler) {
    FUNCTION_BODY_ARG(C2SModuleInappV4, MarketConnect);
    return true;
}
bool C2SModuleInappV4_ItemInfo(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError,
                             C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler) {
	FUNCTION_BODY(C2SModuleInappV4, ItemInfo);
	return true;
}
bool C2SModuleInappV4_ItemInfo(const C2SModule_Cocos2dx::C2SModuleParameter* arg,
                             C2SModule_Cocos2dx::C2SModuleError* pkError,
                             C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler) {
	FUNCTION_BODY_ARG(C2SModuleInappV4, ItemInfo);
	return true;
}
bool C2SModuleInappV4_Purchase(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError,
                             C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler) {
	FUNCTION_BODY(C2SModuleInappV4, Purchase);
	return true;
}
bool C2SModuleInappV4_Purchase(const C2SModule_Cocos2dx::C2SModuleParameter* arg,
                             C2SModule_Cocos2dx::C2SModuleError* pkError,
                             C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler) {
	FUNCTION_BODY_ARG(C2SModuleInappV4, Purchase);
	return true;
}
bool C2SModuleInappV4_Restore(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError,
                             C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler) {
	FUNCTION_BODY(C2SModuleInappV4, Restore);
	return true;
}
bool C2SModuleInappV4_Restore(const C2SModule_Cocos2dx::C2SModuleParameter* arg,
                             C2SModule_Cocos2dx::C2SModuleError* pkError,
                             C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler) {
	FUNCTION_BODY_ARG(C2SModuleInappV4, Restore);
	return true;
}
bool C2SModuleInappV4_TransactionFinish(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError,
                             C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler) {
	FUNCTION_BODY(C2SModuleInappV4, TransactionFinish);
	return true;
}
bool C2SModuleInappV4_TransactionFinish(const C2SModule_Cocos2dx::C2SModuleParameter* arg,
                             C2SModule_Cocos2dx::C2SModuleError* pkError,
                             C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler) {
	FUNCTION_BODY_ARG(C2SModuleInappV4, TransactionFinish);
	return true;
}
bool C2SModuleInappV4_GetSelectedMarket(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError,
                                        C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler) {
    FUNCTION_BODY(C2SModuleInappV4, GetSelectedMarket);
    return true;
}
bool C2SModuleInappV4_GetSelectedMarket(const C2SModule_Cocos2dx::C2SModuleParameter* arg,
                            C2SModule_Cocos2dx::C2SModuleError* pkError,
                            C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler) {
    FUNCTION_BODY_ARG(C2SModuleInappV4, GetSelectedMarket);
    return true;
}
bool C2SModuleInappV4_GetAccountUuid(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError,
                                        C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler) {
    FUNCTION_BODY(C2SModuleInappV4, GetAccountUuid);
    return true;
}
bool C2SModuleInappV4_GetAccountUuid(const C2SModule_Cocos2dx::C2SModuleParameter* arg,
                                        C2SModule_Cocos2dx::C2SModuleError* pkError,
                                        C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler) {
    FUNCTION_BODY_ARG(C2SModuleInappV4, GetAccountUuid);
    return true;
}

#if __OBJC__
//TODO: IOS는 아래의 PGS 관련 기능 없음
/**
 * InappV4 CheckPromotePurchase
 */
bool C2SModuleInappV4_CheckPromotePurchase(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError,
                             C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler) {
	FUNCTION_BODY(C2SModuleInappV4, CheckPromotePurchase);
	return true;
}
/**
 * InappV4 CheckPromotePurchase
 */
bool C2SModuleInappV4_CheckPromotePurchase(const C2SModule_Cocos2dx::C2SModuleParameter* arg,
                             C2SModule_Cocos2dx::C2SModuleError* pkError,
                             C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler) {
	FUNCTION_BODY_ARG(C2SModuleInappV4, CheckPromotePurchase);
	return true;
}

#elif __ANDROID__
bool C2SModuleSns_PGSLoginProc(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError,
								C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler) {
	FUNCTION_BODY(C2SModuleSns, PGSLoginProc);
	return true;
}

bool C2SModuleSns_PGSLoginProc(const C2SModule_Cocos2dx::C2SModuleParameter* arg,
								C2SModule_Cocos2dx::C2SModuleError* pkError,
								C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler) {
	FUNCTION_BODY_ARG(C2SModuleSns, PGSLoginProc);
	return true;
}

bool C2SModuleSns_Connect(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError,
								C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler) {
	FUNCTION_BODY(C2SModuleSns, Connect);
	return true;
}

bool C2SModuleSns_Connect(const C2SModule_Cocos2dx::C2SModuleParameter* arg,
								C2SModule_Cocos2dx::C2SModuleError* pkError,
								C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler) {
	FUNCTION_BODY_ARG(C2SModuleSns, Connect);
	return true;
}

/**
 * InappV4 SelectMarket
 */
bool C2SModuleInappV4_SelectMarket(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError,
                             C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler) {
	FUNCTION_BODY(C2SModuleInappV4, SelectMarket);
	return true;
}
/**
 * InappV4 SelectMarket
 */
bool C2SModuleInappV4_SelectMarket(const C2SModule_Cocos2dx::C2SModuleParameter* arg,
                             C2SModule_Cocos2dx::C2SModuleError* pkError,
                             C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler) {
	FUNCTION_BODY_ARG(C2SModuleInappV4, SelectMarket);
	return true;
}
/**
 * InappV4 RequestBalance
 */
bool C2SModuleInappV4_RequestBalance(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError,
                             C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler) {
	FUNCTION_BODY(C2SModuleInappV4, RequestBalance);
	return true;
}
/**
 * InappV4 RequestBalance
 */
bool C2SModuleInappV4_RequestBalance(const C2SModule_Cocos2dx::C2SModuleParameter* arg,
                             C2SModule_Cocos2dx::C2SModuleError* pkError,
                             C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler) {
	FUNCTION_BODY_ARG(C2SModuleInappV4, RequestBalance);
	return true;
}
/**
 * InappV4 ChargeBalance
 */
bool C2SModuleInappV4_ChargeBalance(const char* pstrJson, C2SModule_Cocos2dx::C2SModuleError* pkError,
                             C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler) {
	FUNCTION_BODY(C2SModuleInappV4, ChargeBalance);
	return true;
}
/**
 * InappV4 ChargeBalance
 */
bool C2SModuleInappV4_ChargeBalance(const C2SModule_Cocos2dx::C2SModuleParameter* arg,
                             C2SModule_Cocos2dx::C2SModuleError* pkError,
                             C2SModule_Cocos2dx::C2SModuleCompletionHandler* completionHandler) {
	FUNCTION_BODY_ARG(C2SModuleInappV4, ChargeBalance);
	return true;
}
#endif
